package com.comtika.safe.common.action;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.exception.UnOAuthException;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.UserCoreCache;

import org.apache.commons.lang.math.NumberUtils;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.*;

public class BaseController {


	protected HttpServletRequest request;

	protected HttpServletResponse response;

	private static final String DEFAULT_ORDER = " 1 DESC ";// 默认排序

	private static final String DEFAUL_SORT_COLUMN = "1";// 默认排序列-第一列

	private static final String DEFAUL_SORT_TYPE = "DESC";// 默认排序方式 -逆序



	/**
	 * 表示请求该类的每个Action前都会首先执行它 setReqAndRes
	 *
	 * @param request
	 * @param response
	 */
	@ModelAttribute
	public void setReqAndRes(HttpServletRequest request,HttpServletResponse response) {
		this.request = request;
		this.response = response;
	}

	// 获取权限
	public UserCoreCache findUserCache() {
		return findUserCache(true);
	}

	public UserCoreCache findUserCache(boolean isCheckAuth) {
		UserCoreCache userCoreCache = (UserCoreCache) request.getSession().getAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN);
		if (userCoreCache == null) {
			if (isCheckAuth)
				throw new UnOAuthException();
			return null;
		}
		return userCoreCache;
	}

	/**
	 * @Title: getPageQuery
	 * @Description: TODO(基础查询对象，封装分页、排序参数)
	 * @param dataFlag
	 *            :是否开启数据权限验证
	 * @return PageQuery<Map<String,Object>>
	 * @throws
	 */
	protected PageQuery<Map<String, Object>> getPageQuery(boolean dataFlag) {
		PageQuery<Map<String, Object>> pageQuery = new PageQuery<Map<String, Object>>();
		// 数据权限
		help(pageQuery, dataFlag);
		Long start = StringUtil.convertLong(request.getParameter("start"), 1L);
		pageQuery.setPageNumber((start < 1) ? 1L : start);

		Long size = StringUtil.convertLong(request.getParameter("size"), 10L);
		pageQuery.setPageSize((size == -1) ? 99999 : size);// -1代表无限制

		// 排序
		String sortStr = request.getParameter("sortStr");
		String sortType = request.getParameter("sortType");
		pageQuery.setOrderBy(this.getOrderBy(sortStr, sortType));
		// 开始、结束时间
		if (StringUtil.isNotEmpty(request.getParameter("startTime"))) {
			Long startTime = Long.parseLong(request.getParameter("startTime"));
			pageQuery.setPara("startTime", startTime);
		}
		if (StringUtil.isNotEmpty(request.getParameter("endTime"))) {
			Long endTime = Long.parseLong(request.getParameter("endTime"));
			pageQuery.setPara("endTime", endTime);
		}

		return pageQuery;
	}

	/*public PageQuery<Map<String, Object>> getPageQuery1(boolean dataFlag) {
		PageQuery<Map<String, Object>> pageQuery = new PageQuery<Map<String, Object>>();
		// 数据权限
		help(pageQuery, dataFlag);
		pageQuery.setPageSize(99999);
		pageQuery.setTotalRow(10);
		return pageQuery;
	}*/

	public void help(PageQuery<Map<String, Object>> pageQuery, boolean dataFlag) {
		if (dataFlag) {
			UserCoreCache userCoreCache = findUserCache();
			if (userCoreCache != null && !ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())) {
				pageQuery.setPara("personId", userCoreCache.getUserId());
				pageQuery.setPara("merchantData", userCoreCache.getMerchantData());
				pageQuery.setPara("siteData", userCoreCache.getSiteData());
			}
		}
	}
	
	/**
	 * 获取数据权限，返回一个map集合
	 * @return
	 */
	public Map<String,Object> getOrgData() {
		Map<String,Object> map = new HashMap<String, Object>();
		UserCoreCache userCoreCache = findUserCache();
		if (userCoreCache != null && !ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())) {
			if( userCoreCache.getMerchantData() == null || userCoreCache.getMerchantData().size() < 1 ){
				map.put("merchantData", null);
			}else{
				map.put("merchantData", userCoreCache.getMerchantData());
			}
			if( userCoreCache.getSiteData() == null || userCoreCache.getSiteData().size() < 1 ){
				map.put("siteData", null);
			}else{
				map.put("siteData", userCoreCache.getSiteData());
			}
			map.put("personId", userCoreCache.getUserId());
		}
		return map;
	}

	/**
	 * 获取访问者IP 在一般情况下使用Request.getRemoteAddr()即可，但是经过nginx等反向代理软件后，这个方法会失效。
	 * 本方法先从Header中获取X-Real-IP，如果不存在再从X-Forwarded-For获得第一个IP(用,分割)，
	 * 如果还不存在则调用Request .getRemoteAddr()。
	 *
	 * 2018年9月3日12:24:15 修改：获取实际ip地址，有代理的把代理转发ip地址拼接上
	 *
	 * @author xjm
	 * @return
	 */
	public String getIpAddr() {
		// X-Forwarded-For(XFF)是用来识别通过HTTP代理或负载均衡方式连接到Web服务器的客户端最原始的IP地址的HTTP请求头字段
		String realIp = request.getHeader("X-Real-IP");
		String remoteAddr = request.getRemoteAddr();
		String forwarded = request.getHeader("X-Forwarded-For");

		String ip = null;
		if (realIp == null) {
			if (forwarded == null) {
				ip = remoteAddr;
			} else {
				ip = remoteAddr + "/" + forwarded.split(",")[0];
			}
		} else {
			if (realIp.equals(forwarded)) {
				ip = realIp;
			} else {
				if (forwarded != null) {
					forwarded = forwarded.split(",")[0];
				}
				ip = realIp + "/" + forwarded;
			}
		}
		return ip;
	}



	protected String getOrderBy(String sortStr, Integer sortType) {
		String sortColmun = DEFAUL_SORT_COLUMN;
		String sortTypeStr = DEFAUL_SORT_TYPE;
		if (sortType != null && sortType.intValue() == 2) {
			sortTypeStr = "ASC";
		}
		if (StringUtil.isNotEmpty(sortStr)) {
			sortColmun = sortStr;
		}
		return " " + sortColmun + " " + sortTypeStr + " ";
	}

	protected String getOrderBy(String sortStr, String sortType) {
		if (NumberUtils.isNumber(sortType)) {
			return getOrderBy(sortStr, Integer.parseInt(sortType));
		}
		String sortColmun = DEFAUL_SORT_COLUMN;
		String sortTypeStr = DEFAUL_SORT_TYPE;
		if (StringUtil.isNotEmpty(sortType)) {
			sortTypeStr = sortType;
		}
		if (StringUtil.isNotEmpty(sortStr)) {
			sortColmun = sortStr;
		}
		return " " + sortColmun + " " + sortTypeStr + " ";
	}

	protected String getOrderBy() {
		return DEFAULT_ORDER;
	}

}
