package com.comtika.safe.common.cache;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.LoadingCache;

import java.util.Map;
import java.util.concurrent.ConcurrentMap;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 8:44 2019/6/18
 * @DESC: 谷歌 guava 默认的实现
 **/
public class DefaultGuavaCache<T> implements GuavaCache<T> {
    private final LoadingCache<String,  T> googleGuavaCache;
    public DefaultGuavaCache(GoogleGuavaConfig config){
        CacheBuilder<Object, Object> builder = CacheBuilder.newBuilder();
        builder.concurrencyLevel(config.getConcurrencyLevel());
        if(config.isCreateExpire()){
            if(config.isRefreshAfter())
                builder.refreshAfterWrite(config.getExpireTime(),config.getUnit());
            else
                builder.expireAfterWrite(config.getExpireTime(),config.getUnit());
        }else builder.expireAfterAccess(config.getExpireTime(),config.getUnit());
        builder.initialCapacity(config.getInitialCapacity());
        builder.maximumSize(config.getMaximumSize());
        if(config.isRecordStats())
            builder.recordStats();
        if(config.getRemovalListener() != null)
            builder.removalListener(config.getRemovalListener());
        LoadingCache<String, T> build;
        build = builder.build(config.getCacheLoader());
        this.googleGuavaCache = build;
    }
    public DefaultGuavaCache(){
        this(new GoogleGuavaConfig());
    }
    @Override
    public void set(String key, T value) {
        googleGuavaCache.put(key, value);
    }

    @Override
    public void setMap(Map<String, T> map) {
        googleGuavaCache.putAll(map);
    }

    @Override
    public T getIfLoad(String key) {
        return googleGuavaCache.getUnchecked(key);
    }

    @Override
    public T getIfPresent(String key) {
        return googleGuavaCache.getIfPresent(key);
    }

    @Override
    public <T> T get(String key, Class<T> clazz) {
        Object obj = googleGuavaCache.getUnchecked(key);
        if(obj == null){
            return null;
        }
        if(clazz.isAssignableFrom(obj.getClass())){
            return (T) obj;
        }else{
            throw new ClassCastException(obj.getClass().getName()+" cannot be cast to "+clazz.getName());
        }
    }

    @Override
    public void remove(String key) {
        googleGuavaCache.invalidate(key);
    }

    @Override
    public void cleanTotalCache() {
        googleGuavaCache.invalidateAll();
    }

    @Override
    public void cleanUpCache() {
        googleGuavaCache.cleanUp();
    }

    @Override
    public void cleanManyCache(Iterable<Object> keys) {
        googleGuavaCache.invalidateAll(keys);
    }

    @Override
    public ConcurrentMap<String, T> getCacheMap() {
        return googleGuavaCache.asMap();
    }

    @Override
    public void refreshCache(String key) {
        googleGuavaCache.refresh(key);
    }
}
