package com.comtika.safe.common.model;

import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Array;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * JavaBean转Json单向工具,枚举只以toString方法输出,请务必重写枚举toString
 *
 */
public class JsonUtil {
	private static int convertDepth = 10;
	private static String datePattern = "yyyy-MM-dd HH:mm:ss";
	
	
	/**
	 * Set Object Convert Json Depth
	 */
	public static void setConvertDepth(int convertDepth){
		if(convertDepth < 0){
			throw new RuntimeException("convertDepth can not less 0." );
		}
		JsonUtil.convertDepth = convertDepth;
	}
	
	/**
	 * Set Date Format Pattern
	 */
	public static void setDatePattern(String datePattern) {
		if (datePattern == null || "".equals(datePattern.trim())){
			throw new RuntimeException("datePattern can not be empty.");
		}
		JsonUtil.datePattern = datePattern;
	}
	
	
	/**
	 * <b>Object To Json</b><br>
	 * <b>Author：</b>zhousir<br>
	 * <b>Date：</b>2016年8月15日<br>
	 * 
	 * @param obj
	 * @return
	 */
	public static String toJSONString(Object obj) {
		return toJSONString(obj, convertDepth);
	}
	
	/**
	 * <b>Write Object To Json</b><br>
	 * <b>Author：</b>zhousir<br>
	 * <b>Date：</b>2016年8月15日<br>
	 * 
	 * @param out
	 * @param obj
	 */
	public static void writeToJSONString(Writer out, Object obj) {
		String jsonStr = toJSONString(obj, convertDepth);
		try {
			out.write(jsonStr);
		} catch (Exception e) {
			throw new RuntimeException("WriteToJSONString Exception!", e);
		} finally{
			if(out != null){
				try {
					out.close();
				} catch (IOException e) {
				}
			}
		}
		return ;
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	static String toJSONString(Object obj, int depth) {
		if(obj == null || (depth--) < 0){
			return "null";
		}
		if(obj instanceof String){
			return "\"" + escapeStr((String)obj) + "\"";
		}
		if(obj instanceof Double){
			if(((Double)obj).isInfinite() || ((Double)obj).isNaN()){
				return "null";
			}else{
				return obj.toString();
			}
		}
		if(obj instanceof Float){
			if(((Float)obj).isInfinite() || ((Float)obj).isNaN()){
				return "null";
			}else{
				return obj.toString();
			}
		}
		if(obj instanceof Number){
			return obj.toString();
		}
		if(obj instanceof Boolean){
			return obj.toString();
		}
		if (obj instanceof Character) {
			return "\"" + escapeStr(obj.toString()) + "\"";
		}
		if (obj instanceof java.sql.Time){
			return "\"" + obj.toString() + "\"";
		}
		if (obj instanceof java.sql.Date){
			return "\"" + obj.toString() + "\"";
		}
		if (obj instanceof java.util.Date) {
			return "\"" + new SimpleDateFormat(datePattern).format(obj) + "\"";
		}
		if(obj instanceof Map) {
			return MapSerializer.instance.doSerialize(obj, depth);
		}
		if(obj instanceof Collection) {
			return CollectionSerializer.instance.doSerialize(obj, depth);
		}
		if (obj.getClass().isArray()) {
			List list = new ArrayList();
			int length = Array.getLength(obj);
			for (int i = 0; i < length; i++) {
				list.add(Array.get(obj, i));
			}
			return CollectionSerializer.instance.doSerialize(list, depth);
		}
		if (obj instanceof Enum) {
			return ((Enum)obj).toString();
		}
		return JavaBeanSerializer.instance.doSerialize(obj, depth);
	}

	
	/**
	 * Escape quotes, \, /, \r, \n, \b, \f, \t and other control characters (U+0000 through U+001F).
	 */
	static String escapeStr(String s) {
		if(s == null){
			return null;
		}
        StringBuilder sb = new StringBuilder();
        escapeStr(s, sb);
        return sb.toString();
    }
	
	static void escapeStr(String s, StringBuilder builder) {
		for(int i=0; i<s.length(); i++){
			char ch = s.charAt(i);
			switch(ch){
			case '"':
				builder.append("\\\"");
				break;
			case '\\':
				builder.append("\\\\");
				break;
			case '\b':
				builder.append("\\b");
				break;
			case '\f':
				builder.append("\\f");
				break;
			case '\n':
				builder.append("\\n");
				break;
			case '\r':
				builder.append("\\r");
				break;
			case '\t':
				builder.append("\\t");
				break;
			case '/':
				builder.append("\\/");
				break;
			default:
				if((ch >= '\u0000' && ch <= '\u001F') || (ch >= '\u007F' && ch <= '\u009F') || (ch >= '\u2000' && ch <= '\u20FF')) {
					String str = Integer.toHexString(ch);
					builder.append("\\u");
					for(int k=0; k<4-str.length(); k++) {
						builder.append('0');
					}
					builder.append(str.toUpperCase());
				}
				else{
					builder.append(ch);
				}
			}
		}
	}
	
}