package com.comtika.safe.common.request;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Iterator;

public class CookieUtil {
	private final HashMap<String, Cookie> cookieMap;
	private final HttpServletResponse response;
	private final HttpServletRequest request;
	public static String getCookieValue(String name, HttpServletRequest request){
		Cookie[] cookies = request.getCookies();
		if(cookies == null) return null;
		for(Cookie cookie:cookies){
			if(name.equals(cookie.getName()))
				return cookie.getValue();
		}
		return null;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public CookieUtil(HttpServletRequest request, HttpServletResponse response) {
		this.response = response;
		this.request = request;
		cookieMap = new HashMap();
		Cookie[] cookies = request.getCookies();
		if (cookies != null) {
			Cookie[] arrayOfCookie1;
			int j = (arrayOfCookie1 = cookies).length;
			for (int i = 0; i < j; i++) {
				Cookie cookie = arrayOfCookie1[i];
				if (cookie != null) {

					cookieMap.put(cookie.getName(), cookie);
				}
			}
		}
	}

	public Cookie getCookie(String name) {
		return (Cookie) cookieMap.get(name);
	}

	public String[] getCookieNames() {
		Iterator<String> it = cookieMap.keySet().iterator();
		String[] cookieNames = new String[cookieMap.size()];
		int i = 0;
		while (it.hasNext()) {
			cookieNames[i] = ((String) it.next());
			i++;
		}
		return cookieNames;
	}

	public Cookie[] getCookies() {
		Iterator<Cookie> it = cookieMap.values().iterator();
		Cookie[] cookies = new Cookie[cookieMap.size()];
		int i = 0;
		while (it.hasNext()) {
			cookies[i] = ((Cookie) it.next());
			i++;
		}
		return cookies;
	}

	public String getCookieValue(String name) {
		Cookie cookie = (Cookie) cookieMap.get(name);
		return cookie == null ? null : cookie.getValue();
	}

	public void setCookie(String name, String value, String domain, int maxAge,
			String path) {
		Cookie cookie = getCookie(name);
		if (cookie == null) {
			cookie = new Cookie(name, value);
			cookieMap.put(name, cookie);
		} else {
			cookie.setValue(value);
		}

		String serverName = request.getServerName();
		if (!serverName.endsWith(domain)) {
			return;
		}

		cookie.setMaxAge(maxAge);
		cookie.setPath(path);
		cookie.setDomain(domain);
		response.addCookie(cookie);
	}
}
