package com.comtika.safe.common.response;



import com.comtika.safe.common.exception.RespException;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * 数据结构包装器
 */
@SuppressWarnings("serial")
public class RespWrapper<T> implements Serializable
{

    /** 响应码 */
    private int code;

    /** 信息 */
    private String msg;

    /** 返回数据 */
    private T data;



    public RespWrapper() {

    }

    private RespWrapper(int code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }



    /**
     * 创建返回数据
     *
     * @param data
     * @return
     */
    public static <T> RespWrapper<T> makeResp(int code, String msg, T data) {
        return new RespWrapper<T>(code, msg, data);
    }

    /**
     * 创建返回数据
     * @param e
     * @return
     */
    public static <T> RespWrapper<T> makeResp(RespException e) {
        return new RespWrapper<T>(e.getErrcode(), e.getMessage(),null);
    }


    /**
     * 创建返回成功的数据
     *
     * @param data
     * @return
     */
    public static <T> RespWrapper<T> makeSuccessResp(T data) {
        return new RespWrapper<T>(RespCode.SUCCESS, "", data);
    }




    /***
     * 重试
     */
    public static <T> RespWrapper<T> makeRetry(String msg){
    	return new RespWrapper<T>(RespCode.RETRY_CODE, msg,null);
    }
    /**
     * 创建返回失败的数据
     *
     * @param msg
     * @return
     */
    public static <T> RespWrapper<T> makeFailResp(String msg) {
        return new RespWrapper<T>(RespCode.FAIL, msg, null);
    }

    public static <T> RespWrapper<T> makeFailResp(String msg, T data) {
        return new RespWrapper<T>(RespCode.FAIL, msg, data);
    }

    public static <T> RespWrapper<T> makeErrorResp(String msg) {
        return new RespWrapper<T>(RespCode.SERVER_ERROR, msg, null);
    }

    public static <T> RespWrapper<T> makeUnLoginResp(String msg) {
        return new RespWrapper<T>(RespCode.UN_LOGIN, msg, null);
    }

    public static <T> RespWrapper<T> makeInvalidResp(String msg) {
        return new RespWrapper<T>(RespCode.INVALID_TOKEN, msg, null);
    }

    /**
     * 判断返回的是不是成功标识的数据
     *
     * @return
     */
    public boolean isSuccess() {
        return this.code == RespCode.SUCCESS;
    }

    /**
     * 重写后的equals,可直接对比2个RespWrapper的属性是否一样
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean equals(Object o) {
        RespWrapper<T> pr = (RespWrapper<T>) o;
        if (pr == null){
        	return false;
        }
        return code == pr.code && eq(msg, pr.msg) && eq(data, pr.data);
    }

    private static <T> boolean eq(T o1, T o2) {
        return o1 == null ? o2 == null : o1.equals(o2);
    }

    /**
     * 重写后的hashCode,属性相同的RespWrapper的hashCode也一样
     */
    @Override
    public int hashCode() {
        int seed = code;
        seed ^= h(msg) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
        seed ^= h(data) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
        return seed;
    }

    private static int h(Object o) {
        return o == null ? 0 : o.hashCode();
    }

    /**
     * 重写后的toString,会将所有属性列出
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{").append(code).append(",").append(msg).append(",").append(data).append("}");
        return sb.toString();
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }


}
