package com.comtika.safe.common.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @desc  : 日期, 时间戳工具类
 * @author: vwFisher(422578659@qq.com)
 * @date  : 2018/02/02 12:27
 */
public class DateTimeUtils {
	private static final ConcurrentHashMap<String,SimpleDateFormat> SDF = new ConcurrentHashMap<>();
	/**默认时间格式*/
	public static final String FORMAT = "yyyy-MM-dd HH:mm:ss";
	public static final String FORMAT_NETTY = "yyyy-MM-dd HH:mm:ss.SSS";
	// public static final String SIGN_FORMAT = "HH:mm:ss";
	public static final String FORMAT_M = "yyyy-MM-dd HH:mm";
	public static final String FORMAT_YMD = "yyyy-MM-dd";
	public static final String FORMAT_YMD_TASK = "yy-MM-dd";
	public static final String FORMAT_YMD_ZH = "yyyy年MM月";
	public static final String FORMAT_BRIEF = "yyyyMMdd";
	public static final String FORMAT_HOME = "MM-dd";
	public static final String FORMAT_WORKTIME = "HH:mm";
    /** 一秒种时间(毫秒) */
    public static final long ONE_SECOND = 1000;
    /** 一分钟时间(毫秒) */
    public static final long ONE_MINUTE = 60 * 1000;
    /** 一小时时间(毫秒) */
    public static final long ONE_HOUR   = 60 * 60 * 1000;
    /** 一天的时间(毫秒) */
    public static final long ONE_DAY    = 24 * 60 * 60 * 1000;

	public static String currDateFormat(String format){
		return formatDate(new Date(),format);
	}
	/**
	 * 匹配输入的时间字符串是否有效, 返回对应的格式
	 *
	 * @param dateStr 时间字符串
	 * @return
	 */
	public static String matches(String dateStr) {
		String formatStr = "";
		if (dateStr.matches("\\d{4}\\-\\d{1,2}\\-\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}\\:\\d{1,2}\\.\\d{1,3}")) {
			formatStr = "yyyy-MM-dd HH:mm:ss.SSS";
		} else if (dateStr.matches("\\d{4}\\-\\d{1,2}\\-\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}\\:\\d{1,2}")) {
			formatStr = "yyyy-MM-dd HH:mm:ss";
		} else if (dateStr.matches("\\d{4}\\-\\d{1,2}\\-\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}")) {
			formatStr = "yyyy-MM-dd HH:mm";
		} else if (dateStr.matches("\\d{4}\\-\\d{1,2}\\-\\d{1,2}\\s+\\d{1,2}")) {
			formatStr = "yyyy-MM-dd HH";
		} else if (dateStr.matches("\\d{4}\\-\\d{1,2}\\-\\d{1,2}")) {
			formatStr = "yyyy-MM-dd";
		} else if (dateStr.matches("\\d{4}\\/\\d{1,2}\\/\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}\\:\\d{1,2}\\.\\d{1,3}")) {
			formatStr = "yyyy/MM/dd HH:mm:ss.SSS";
		} else if (dateStr.matches("\\d{4}\\/\\d{1,2}\\/\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}\\:\\d{1,2}")) {
			formatStr = "yyyy/MM/dd HH:mm:ss";
		} else if (dateStr.matches("\\d{4}\\/\\d{1,2}\\/\\d{1,2}\\s+\\d{1,2}\\:\\d{1,2}")) {
			formatStr = "yyyy/MM/dd HH:mm";
		} else if (dateStr.matches("\\d{4}\\/\\d{1,2}\\/\\d{1,2}\\s+\\d{1,2}")) {
			formatStr = "yyyy/MM/dd HH";
		} else if (dateStr.matches("\\d{4}\\/\\d{1,2}\\/\\d{1,2}")) {
			formatStr = "yyyy/MM/dd";
		}
		return formatStr;
	}

	/**
	 * 获取当前月 的 后一个月
	 * @return
	 */
	public static int getNowMonth(){
		Calendar calendar = Calendar.getInstance();
		return calendar.get(Calendar.MONTH)+1;
	}

	/**
	 * 获取某个月份第一天的时间戳
	 * @param  month 月份
	 */
	public static Long dateMonthToStamp(Integer month){
		Calendar cal = Calendar.getInstance();
		int year = cal.get(Calendar.YEAR);
		//设置年份
		cal.set(Calendar.YEAR,year);
		//设置月份
		cal.set(Calendar.MONTH, month-1);
		//设置为1号,当前日期既为本月第一天
		cal.set(Calendar.DAY_OF_MONTH, 1);
		//将小时至0
		cal.set(Calendar.HOUR_OF_DAY, 0);
		//将分钟至0
		cal.set(Calendar.MINUTE, 0);
		//将秒至0
		cal.set(Calendar.SECOND,0);
		//将毫秒至0
		cal.set(Calendar.MILLISECOND, 0);
		return cal.getTimeInMillis();
	}

	public static Long dateMonthToLastYear(Integer month){
		Calendar cal = Calendar.getInstance();
		int year = cal.get(Calendar.YEAR);
		//设置年份
		cal.set(Calendar.YEAR,year-1);
		//设置月份
		cal.set(Calendar.MONTH, month-1);
		//设置为1号,当前日期既为本月第一天
		cal.set(Calendar.DAY_OF_MONTH, 1);
		//将小时至0
		cal.set(Calendar.HOUR_OF_DAY, 0);
		//将分钟至0
		cal.set(Calendar.MINUTE, 0);
		//将秒至0
		cal.set(Calendar.SECOND,0);
		//将毫秒至0
		cal.set(Calendar.MILLISECOND, 0);
		return cal.getTimeInMillis();
	}

	public static Long dateMonthLastDayToYear(Integer month){
		Calendar cal = Calendar.getInstance();
		int year = cal.get(Calendar.YEAR);
		//设置年份
		cal.set(Calendar.YEAR,year-1);
		//设置月份
		cal.set(Calendar.MONTH, month-1);
		//获取某月最大天数
		int lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
		//设置日历中月份的最大天数
		cal.set(Calendar.DAY_OF_MONTH, lastDay);
		//将小时至23
		cal.set(Calendar.HOUR_OF_DAY, 23);
		//将分钟至59
		cal.set(Calendar.MINUTE, 59);
		//将秒至59
		cal.set(Calendar.SECOND,59);
		//将毫秒至999
		cal.set(Calendar.MILLISECOND, 999);
		return cal.getTimeInMillis();
	}

	/**
	 * 获取某月最后一天的的时间戳
	 * @param month 月份
	 */
	public static Long dateMonthLastDayToStamp(Integer month){
		Calendar cal = Calendar.getInstance();
		int year = cal.get(Calendar.YEAR);
		//设置年份
		cal.set(Calendar.YEAR,year);
		//设置月份
		cal.set(Calendar.MONTH, month-1);
		//获取某月最大天数
		int lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
		//设置日历中月份的最大天数
		cal.set(Calendar.DAY_OF_MONTH, lastDay);
		//将小时至23
		cal.set(Calendar.HOUR_OF_DAY, 23);
		//将分钟至59
		cal.set(Calendar.MINUTE, 59);
		//将秒至59
		cal.set(Calendar.SECOND,59);
		//将毫秒至999
		cal.set(Calendar.MILLISECOND, 999);
		return cal.getTimeInMillis();
	}

    /**
     * 按照格式, 格式化当前日期(yyyy-MM-dd HH:mm:ss.SSS)
     *
     * @param format	日期格式
     * @return
     */
    public static String formatDate(String format) {
		return formatDate(new Date(), format);
    }

	/**
	 * 传入时分，返回当日年月日+传入时分时间戳
	 */
	public static long returnTimeStamp(String time){
		try {
			return new SimpleDateFormat("yyyy-MM-dd HH:mm").parse(formatDateYMD(new Date())+" "+time).getTime();
		}catch (Exception e){
			e.printStackTrace();
		}
		return 0;
	}



	/**
     * 按照格式, 格式化日期(yyyy-MM-dd HH:mm:ss.SSS)
     *
     * @param date		Date对象
     * @param format	日期格式
     * @return
     */
    public static String formatDate(Date date, String format) {
    	return findSDF(format).format(date);
    }
    private static SimpleDateFormat findSDF(String format){
		SimpleDateFormat simpleDateFormat = SDF.get(format);
		if(simpleDateFormat == null){
			simpleDateFormat = new SimpleDateFormat(format);
			SDF.put(format,simpleDateFormat);
		}
		return simpleDateFormat;
	}

	/**
	 * 获取当天当前时分秒
	 * @return
	 */
	public static String formatDateMS(Date date) {
		return new SimpleDateFormat("HH:mm:ss").format(date);
	}

	/**
	 * 获取当天当前时分秒
	 * @return
	 */
	public static String formatDateYMD(Date date) {
		return new SimpleDateFormat("yyyy-MM-dd").format(date);
	}

	/**
	 * 获取当天当前时分秒
	 * @return
	 */
	public static String formatDateHM(Date date) {
		return new SimpleDateFormat("HH:mm").format(date);
	}

	/**
	 * 比较时间大小 （时分秒）
	 * @return
	 */
	public static boolean formatCompare(String startTimeStr,String endTimeStr,String lockTimeStr) {
		SimpleDateFormat df = new SimpleDateFormat("HH:mm");
		try {
			Date startTime= df.parse(startTimeStr);
			Date endTime= df.parse(endTimeStr);
			Date lockTime= df.parse(lockTimeStr);
			if (endTime.getTime() > lockTime.getTime() && startTime.getTime() < lockTime.getTime()) {
				return true;
			}
			return false;
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return false;
	}


	/**
	 * 格式化日期
	 *
	 * @param dateStr 	输入的日期字符串, 自动匹配输入的日期格式
	 * @param format 	输出日期格式
	 * @return String 格式化后的日期,如果不能格式化则输出原日期字符串
	 */
	public static String formatDate(String dateStr, String format) {
		return formatDate(dateStr, matches(dateStr), format);
	}
	/**
	 * 格式化日期
	 *
	 * @param dateStr 	输入的日期字符串, 自动匹配输入的日期格式
	 * @return String 格式化后的日期,如果不能格式化则输出原日期字符串
	 */
	public static Long formatDateToLong(String dateStr) {
		try {
			if(dateStr == null) return null;
			Date date = findSDF(matches(dateStr)).parse(dateStr);
			return date.getTime();
		}catch (Exception e){
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * 格式化日期
	 *
	 * @param dateStr  		输入的日期字符串
	 * @param inputFormat	输入日期格式
	 * @param format		输出日期格式
	 * @return String 格式化后的日期,如果不能格式化则输出原日期字符串
	 */
	public static String formatDate(String dateStr, String inputFormat, String format) {
		String resultStr = dateStr;
		try {
			Date date = findSDF(inputFormat).parse(dateStr);
			resultStr = formatDate(date, format);
		} catch (ParseException e) {
		}
		return resultStr;
	}


    /**
     * 按格式转换输入字符串为Date对象类型
     *
     * @param dateStr   被格式化的字符串
     * @param format    字符串的格式
     * @return  如果输入格式错误, 返回null
     */
    public static Date formatString(String dateStr, String format) {
        try {
            return findSDF(format).parse(dateStr);
        } catch (Exception e) {
            return null;
        }
    }

	/**
	 * 获取字符串格式日期的时间戳
	 *
	 * @param dateStr  只支持格式, 自动匹配输入的日期格式
	 * @return 时间戳
	 */
	/*public static long getTimeInMillis(String dateStr) {
		SimpleDateFormat sdf = findSDF(matches(dateStr));
		long time = 0;
		try {
			time = sdf.parse(dateStr).getTime();
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return time;
	}*/

    /**
     * 获取几天前或几天后的指定时间
     * @param days    正数=几天前,负数=几天后
     * @param hour    时 0-23
     * @param minute  分 0-59
     * @param second  秒 0-59
     *
     * @return 时间戳
     */
 /*   public static long getTimeInMillis(int days, int hour, int minute, int second ){
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int date = calendar.get(Calendar.DAY_OF_MONTH) - days;
        calendar.set(year, month, date, hour, minute, second);
        long time = calendar.getTimeInMillis();
        // 毫秒数归零
        time = time / 1000L * 1000L;
        return time;
    }*/

	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param years		年, 正值时时间延后，负值时时间提前
	 * @param months	    月, 正值时时间延后，负值时时间提前
	 * @param days		天, 正值时时间延后，负值时时间提前
	 * @param hours		时, 正值时时间延后，负值时时间提前
	 * @param minutes	分, 正值时时间延后，负值时时间提前
	 * @param seconds	秒, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static long getTimeByTodayOffset(int years, int months, int days, int hours, int minutes, int seconds) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		calendar.add(Calendar.YEAR, years);
		calendar.add(Calendar.MONTH, months);

		long calTime = calendar.getTimeInMillis();
		calTime += days * ONE_DAY;
		calTime += hours * ONE_HOUR;
		calTime += minutes * ONE_MINUTE;
		calTime += seconds * ONE_SECOND;
		return calTime;
	}*/

	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param years		年, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static long getTimeByTodayOffsetYears(int years) {
		return getTimeByTodayOffset(years, 0, 0, 0, 0, 0);
	}
	*/
	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param months	月, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static long getTimeByTodayOffsetMonths(int months) {
		return getTimeByTodayOffset(0, months, 0, 0, 0, 0);
	}
	*/
	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param days		天, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static long getTimeByTodayOffsetDays(int days) {
		return getTimeByTodayOffset(0, 0, days, 0, 0, 0);
	}
	*/
	/**
     * 获取距离现在时间, 的偏移时间戳
     *
     * @param hours      小时, 正值时时间延后，负值时时间提前
     * @return
     */
	/*public static long getTimeByTodayOffsetHours(int hours) {
        return getTimeByTodayOffset(0, 0, 0, hours, 0, 0);
    }
	*/
	/**
	 * 获取距离现在时间, 的偏移时间Date
	 *
	 * @param years		年, 正值时时间延后，负值时时间提前
	 * @param months	月, 正值时时间延后，负值时时间提前
	 * @param days		天, 正值时时间延后，负值时时间提前
	 * @param hours		时, 正值时时间延后，负值时时间提前
	 * @param minutes	分, 正值时时间延后，负值时时间提前
	 * @param seconds	秒, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static Date getDateByTodayOffset(int years, int months, int days, int hours, int minutes, int seconds) {
		return new Date(getTimeByTodayOffset(years, months, days, hours, minutes, seconds));
	}*/

	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param years		年, 正值时时间延后，负值时时间提前
	 * @return
	 */
/*	public static Date getDateByTodayOffsetYears(int years) {
		return new Date(getTimeByTodayOffset(years, 0, 0, 0, 0, 0));
	}*/

	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param months	月, 正值时时间延后，负值时时间提前
	 * @return
	 */
/*	public static Date getDateByTodayOffsetMonths(int months) {
		return new Date(getTimeByTodayOffset(0, months, 0, 0, 0, 0));
	}*/

	/**
	 * 获取距离现在时间, 的偏移时间戳
	 *
	 * @param days		天, 正值时时间延后，负值时时间提前
	 * @return
	 */
	/*public static Date getDateByTodayOffsetDays(int days) {
		return new Date(getTimeByTodayOffset(0, 0, days, 0, 0, 0));
	}
*/
    /**
     * 获取本月第一天的指定时间
     *
     * @param hour    时 0-23
     * @param minute  分 0-59
     * @param second  秒 0-59
     * @return
     */
    public static long getMonthTimeByFirstDay(int hour, int minute, int second){
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int date = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(year, month, date, hour, minute, second);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        long time = calendar.getTimeInMillis();
        // 毫秒数归零
        time = time / 1000L * 1000L;
        return time;
    }

    /**
     * 获取本月最后一天的指定时间
     *
     * @param hour    时 0-23
     * @param minute  分 0-59
     * @param second  秒 0-59
     * @return
     */
    public static long getMonthTimeByLastDay(int hour, int minute, int second){
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int date = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(year, month, date, hour, minute, second);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        long time = calendar.getTimeInMillis();
        // 毫秒数归零
        time = time / 1000L * 1000L;
        return time;
    }


	/**
	 * 获取今天0点0分的时间戳
	 * 今天开始时间
	 * @return
	 */
	public static long getStartTimeOfToday() {
		Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT+8"));
		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return calendar.getTimeInMillis();
	}

	/**
	 * 获取指定日期当天0点0分的时间戳
	 *
	 * @return
	 */
	public static long getStartTimeOfDayStr(String dateStr, String format) {
		return getStartTimeOfDay(formatString(dateStr, format));
	}

	/**
	 * 获取指定日期当天0点0分的时间戳
	 *
	 * @return
	 */
	public static long getStartTimeOfDay(Date date) {
		return getStartTimeOfDay(date.getTime());
	}

	/**
	 * 获取指定时间戳当天0点0分的时间戳
	 *
	 * @return
	 */
	public static long 	getStartTimeOfDay(long time) {
        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT+8"));
        calendar.setTimeInMillis(time);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTimeInMillis();
	}
	public static long getDateTimeOfHour(long time){
		Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT+8"));
		calendar.setTimeInMillis(time);
		// calendar.set(Calendar.HOUR, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return calendar.getTimeInMillis();
	}

	/**
	 * 获取今天24点0分的时间戳
	 * 昨天开始时间
	 * @return
	 */
	public static long getEndTimeOfToday() {
		return getStartTimeOfToday() - ONE_DAY;
	}

	/**
	 * 获取指定日期当天24点0分的时间戳
	 *
	 * @return
	 */
	public static long getEndTimeOfDayStr(String dateStr, String format) {
		return getEndTimeOfDay(formatString(dateStr, format));
	}

	/**
	 * 获取指定日期当天24点0分的时间戳
	 *
	 * @return
	 */
	public static long getEndTimeOfDay(Date date) {
		return getStartTimeOfDay(date.getTime()) + ONE_DAY;
	}

	/**
	 * 获取指定时间戳当天24点0分的时间戳
	 *
	 * @return
	 */
	public static long getEndTimeOfDay(long time) {
		return getStartTimeOfDay(time) + ONE_DAY;
	}

	/**
	 * 获取本月第一天的0点时间戳
	 * @return
	 */
	public static long getStartTimeByMonth() {
		Calendar calendar = Calendar.getInstance();
		calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), 1, 0, 0, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return calendar.getTimeInMillis();
	}

	/**
	 * 获取本周星期几的0点时间戳
	 *
	 * @param week	如：Calendar.MONDAY 需要周几就传周几
	 * @return
	 */
	public static long getStartTimeByWeekTime(int week) {
		return getWeekTime(week, 0, 0, 0);
	}

	/**
	 * 获取本周星期几的指定时间戳
	 * @param week		如：Calendar.MONDAY 需要周几就传周几
	 * @param hour  	时 0-23
	 * @param minute	分 0-59
	 * @param second	秒 0-59
	 * @return
	 */
	public static long getWeekTime(int week, int hour, int minute, int second) {
		Calendar calendar = Calendar.getInstance();
		int year = calendar.get(Calendar.YEAR);
		int month = calendar.get(Calendar.MONTH);
		int date = calendar.get(Calendar.DAY_OF_MONTH);
		calendar.set(year, month, date, hour, minute, second);
		calendar.set(Calendar.DAY_OF_WEEK, week);
		long time = calendar.getTimeInMillis();
        // 毫秒数归零
        time = time / 1000L * 1000L;
        return time;
	}

    /**
     * 获取当前年
     * @return
     */
   /* public static int getNowYear(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.YEAR);
    }*/

    /**
     * 获取当前月
     * @return
     */
   /* public static int getNowMonth(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.MONTH)+1;
    }*/

    /**
     * 获取当前日
     * @return
     */
  /*  public static int getNowDayOfMonth(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.DAY_OF_MONTH);
    }*/

    /**
     * 获取当前小时
     * @return
     */
   /* public static int getNowHour(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.HOUR_OF_DAY);
    }*/

    /**
     * 获取当前分
     * @return
     */
  /*  public static int getNowMinute(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.MINUTE);
    }*/

    /**
     * 获取当前秒
     * @return
     */
   /* public static int getNowSecond(){
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.SECOND);
    }*/

    /**
     * 获得两个日期相差的月份
     *
     * @param startDate
     * @param endDate
     * @return
     */
   /* public static int getDiffMonths(Date startDate, Date endDate) {
        int result = 0;
        Calendar startCalender = Calendar.getInstance();
        Calendar endCalender = Calendar.getInstance();
        startCalender.setTime(startDate);
        endCalender.setTime(endDate);
        result = (endCalender.get(Calendar.YEAR) - startCalender.get(Calendar.YEAR)) * 12
                + endCalender.get(Calendar.MONTH) - startCalender.get(Calendar.MONTH);
        return Math.abs(result);
    }*/

	/**
	 * 获取两个日期相差的 时间, 按照field进行分组
	 * @param startDate	开始时间
	 * @param endDate	结束时间
	 * @param field
	 * 		按分钟算：Calendar.MINUTE
	 * 		按小时算：HOUR, HOUR_OF_DAY
	 * 		按天算： DAY_OF_MONTH, DAY_OF_WEEK, DAY_OF_WEEK_IN_MONTH, DAY_OF_YEAR
	 * @return
	 */
    /*public static long getDiffTimeGroupByField(Date startDate, Date endDate, int field) {
        long diffInMillis = endDate.getTime() - startDate.getTime();
        switch (field) {
            case Calendar.MINUTE:
                return TimeUnit.MILLISECONDS.toMinutes(diffInMillis);
            case Calendar.HOUR:
            case Calendar.HOUR_OF_DAY:
                return TimeUnit.MILLISECONDS.toHours(diffInMillis);
            case Calendar.DAY_OF_MONTH:
            case Calendar.DAY_OF_WEEK:
            case Calendar.DAY_OF_WEEK_IN_MONTH:
            case Calendar.DAY_OF_YEAR:
                return TimeUnit.MILLISECONDS.toDays(diffInMillis);
            default:
                throw new UnsupportedOperationException(String.format("Unsupported diff in field %s!", field));
        }
    }*/

	/**
	 * 获取开始时间和结束时间的的天数
	 * @param begin	必须是yyyy-MM-dd的格式
	 * @param end	必须是yyyy-MM-dd的格式
	 * @return
	 */
   	public static long getBetweenDays(String begin, String end) {
   		SimpleDateFormat sdf = new SimpleDateFormat ("yyyy-MM-dd");
   		try {
   			Date dateBegin = sdf.parse(begin);
   	   		Date dateEnd = sdf.parse(end);
   	   		return getBetweenDays(dateBegin, dateEnd);
		} catch (Exception e) {
			return 0;
		}
   	}

	/**
	 * 获取开始时间和结束时间的的天数
	 * @param begin	Date 开始时间
	 * @param end	Date 结束时间
	 * @return
	 */
   	public static long getBetweenDays(Date begin, Date end) {
   		return getBetweenDays(begin, end , false);
   	}

	/**
	 * 获取开始时间和结束时间的的天数
	 * @param begin	Date 开始时间
	 * @param end	Date 结束时间
	 * @return
	 */
   /*	public static long getBetweenDays(long begin, long end) {
   		return getBetweenDays(begin, end , false);
   	}
   	*/
   	/**
   	 * 计算两个日期之间的时间间隔(d1-d2)，可选择是否计算工作日
   	 * @param begin
   	 * @param end
   	 * @param onlyWorkDay 是否只计算工作日
   	 * @return
   	 */
   	public static long getBetweenDays(Date begin, Date end, boolean onlyWorkDay) {
   		return getBetweenDays(begin.getTime(), end.getTime(), false);
   	}

   	/**
   	 * 计算两个日期之间的时间间隔(d1-d2)，可选择是否计算工作日
   	 * @param begin
   	 * @param end
   	 * @param onlyWorkDay 是否只计算工作日
   	 * @return
   	 */
   	public static long getBetweenDays(long begin,long end,boolean onlyWorkDay) {
		if (!onlyWorkDay) {
			return (begin - end) / ONE_DAY + 1;
		} else {
			long days = 0;
			Calendar calendar = Calendar.getInstance();
			try {
				days = (begin - end) / ONE_DAY;
				for (calendar.setTimeInMillis(begin); calendar.getTimeInMillis() < end; calendar.add(Calendar.DAY_OF_YEAR, -1)) {
					int week = calendar.get(Calendar.DAY_OF_WEEK);
					if (week == Calendar.SUNDAY || week == Calendar.SATURDAY) {
						days--;
					}
				}
				if (days < 0) {
					days = 0;
				}
			} catch (Exception e) {
			}
			return days;
		}
   	}

   	/**
   	 * 判断是否是工作日
   	 * @param date
   	 * @return
   	 */
   	/*public static boolean isWorkDay(Date date) {
   		return isWorkDay(date.getTime());
   	}*/

   	/**
   	 * 判断是否是工作日
   	 * @param date
   	 * @return
   	 */
   	public static boolean isWorkDay(long date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(date);
		int week = calendar.get(Calendar.DAY_OF_WEEK);
		if (week == Calendar.SUNDAY || week == Calendar.SATURDAY) {
			return false;
		} else {
			return true;
		}
   	}

   	/**
   	 * 计算两个时间 按什么单位区分的间隔
   	 * @param begin
   	 * @param end
   	 * @param unit	DateUtil.ONEDAT,ONEHOUR,ONEMINUTE,ONESECOND
   	 * @return
   	 */
   /*	public static long getBetweenUnits(Date begin, Date end, long unit) {
   		return getBetweenUnits(begin.getTime(), end.getTime(), unit);
   	}*/

   	/**
   	 * 计算两个时间 按什么单位区分的间隔
   	 * @param begin
   	 * @param end
   	 * @param unit	DateUtil.ONEDAT,ONEHOUR,ONEMINUTE,ONESECOND
   	 * @return
   	 */
   	public static long getBetweenUnits(long begin, long end, long unit) {
		try {
			return (begin - end) / unit;
		} catch (Exception e) {
			return 0;
		}
   	}

	/**
	 * 获取返回的中文时间(中文)
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @return
	 */
	/*public static String getSubtractTimeStr(Date sub, Date min, int length) {
		return getSubtractTimeStr(sub, min, length, 1, "");
	}*/

	/**
	 * 获取返回的中文时间(中文)
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @return
	 */
	/*public static String getSubtractTimeStr(long sub, long min, int length) {
		return getSubtractTimeStr(sub, min, length, 1, "");
	}*/

	/**
	 * 获取返回的中文时间(英文)
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @return
	 */
	/*public static String getSubtractTimeEnStr(Date sub, Date min, int length) {
		return getSubtractTimeStr(sub, min, length, 2, "");
	}*/

	/**
	 * 获取返回的中文时间(英文)
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @return
	 */
	/*public static String getSubtractTimeEnStr(long sub, long min, int length) {
		return getSubtractTimeStr(sub, min, length, 2, "");
	}*/

	/**
	 * 返回自定义间隔的2个时间相减得时间字符串
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @param sep		间隔符
	 * @return
	 */
	/*public static String getSubtractTimeStr(Date sub, Date min, int length, String sep) {
		return getSubtractTimeStr(sub, min, length, 0, sep);
	}*/

	/**
	 * 返回自定义间隔的2个时间相减得时间字符串
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @param sep		间隔符
	 * @return
	 */
	/*public static String getSubtractTimeStr(long sub, long min, int length, String sep) {
		return getSubtractTimeStr(sub, min, length, 0, sep);
	}*/

	/**
	 * 按自定义语言返回相减得时间字符串
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @param language	0自定义 1中文 2英文
	 * @param sep		间隔符
	 * @return
	 */
/*	public static String getSubtractTimeStr(Date sub, Date min, int length, int language, String sep) {
		return getSubtractTimeStr(sub.getTime(), min.getTime(), length, language, sep);
	}*/

	/**
	 * 按自定义语言返回相减得时间字符串
	 * @param sub		减数
	 * @param min		被减数
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒)
	 * @param language	0自定义 1中文 2英文
	 * @param sep		间隔符
	 * @return
	 */
	/*public static String getSubtractTimeStr(long sub, long min, int length, int language, String sep) {
		if (sub - min < 0) {
			throw new RuntimeException("min must bigger than sub!");
		}
		Calendar subC = Calendar.getInstance();
		Calendar minC = Calendar.getInstance();
		subC.setTimeInMillis(sub);
		minC.setTimeInMillis(min);
		int startYear = minC.get(Calendar.YEAR);
		int startMonth = minC.get(Calendar.MONTH);
		int year = subC.get(Calendar.YEAR) - minC.get(Calendar.YEAR);
		int month = subC.get(Calendar.MONTH) - minC.get(Calendar.MONTH);
		int day = subC.get(Calendar.DAY_OF_MONTH) - minC.get(Calendar.DAY_OF_MONTH);
		int hour = subC.get(Calendar.HOUR) - minC.get(Calendar.HOUR);
		int minute = subC.get(Calendar.MINUTE) - minC.get(Calendar.MINUTE);
		int second = subC.get(Calendar.SECOND) - minC.get(Calendar.SECOND);
		int millSecond = subC.get(Calendar.MILLISECOND) - minC.get(Calendar.MILLISECOND);
		if (millSecond < 0) {
			millSecond += 1000;
			second -= 1;
		}
		if (second < 0) {
			second += 60;
			minute -= 1;
		}
		if (minute < 0) {
			minute += 60;
			hour -= 1;
		}
		if (hour < 0) {
			hour += 24;
			day -= 1;
		}
		if (day < 0) {
			day += getDaysOfYearMonth(startYear, startMonth);
			month -= 1;
		}
		if (month < 0) {
			month += 12;
			year -= 1;
		}
		if (year < 0) {
			throw new RuntimeException("error year!");
		}
		return toStr(year, month, day, hour, minute, second, millSecond, length, language, sep);
	}
	*/
	/**
	 * 根据长度 返回中文字符串的日期
	 * @param year
	 * @param month
	 * @param day
	 * @param hour
	 * @param minute
	 * @param second
	 * @param millSecond
	 * @param length	长度(1代表到年, 2代表到月, 3代表到日, 4代表到时, 5代表到分, 6代表到秒, 7代表到毫秒), 默认到秒
	 * @param language  0自定义 1中文 2英文
	 * @param sep		自定义的间隔字符
	 * @return
	 */
	/*private static String toStr(int year, int month, int day, int hour, int minute, int second, int millSecond, int length, int language, String sep) {
		String yearS = year + (language == 1 ? "年" : language == 2 ? "year " : sep);
		String monthS = month + (language == 1 ? "月" : language == 2 ? "month " : sep);
		String dayS = day + (language == 1 ? "日" : language == 2 ? "day " : sep);
		String hourS = hour + (language == 1 ? "时" : language == 2 ? "hour " : sep);
		String minuteS = minute + (language == 1 ? "分" : language == 2 ? "minute " : sep);
		String secondS = second + (language == 1 ? "秒" : language == 2 ? "second " : sep);
		String millSecondS = millSecond + (language == 1 ? "毫秒" : language == 2 ? "millSecond " : sep);
		StringBuilder sb = new StringBuilder();
		switch(length) {
		case 1 :
			sb.append(yearS);
			break;
		case 2:
			sb.append(year > 0 ? yearS + monthS : monthS);
			break;
		case 3:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(dayS);
			break;
		case 4:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(day == 0 ? "" : dayS)
			  .append(hourS);
			break;
		case 5:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(day == 0 ? "" : dayS)
			  .append(hour == 0 ? "" : hourS).append(minuteS);
			break;
		case 6:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(day == 0 ? "" : dayS)
			  .append(hour == 0 ? "" : hourS).append(minute == 0 ? "" : minuteS).append(secondS);
			break;
		case 7:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(day == 0 ? "" : dayS)
			  .append(hour == 0 ? "" : hourS).append(minute == 0 ? "" : minuteS).append(second == 0 ? "" : secondS).append(millSecondS);
			break;
		default:
			sb.append(year == 0 ? "" : yearS).append(month == 0 ? "" : monthS).append(day == 0 ? "" : dayS)
			  .append(hour == 0 ? "" : hourS).append(minute == 0 ? "" : minuteS).append(second == 0 ? "" : secondS);
		}
		return sb.toString();

	}
*/
	private static Map<Integer, Integer> dayOfMonth = new HashMap<Integer, Integer>();
    static {
        // 对应12月的31天
        dayOfMonth.put(0, 31);
        dayOfMonth.put(1, 31);
        // 2月 分闰年(29) 和 常规年(28)
        dayOfMonth.put(2, 28);
        dayOfMonth.put(3, 31);
        dayOfMonth.put(4, 30);
        dayOfMonth.put(5, 31);
        dayOfMonth.put(6, 30);
        dayOfMonth.put(7, 31);
        dayOfMonth.put(8, 31);
        dayOfMonth.put(9, 30);
        dayOfMonth.put(10, 31);
        dayOfMonth.put(11, 30);
        dayOfMonth.put(12, 31);
    }

	/**
	 * 计算当前月份在今年 有多少天
	 * @param year
	 * @param month
	 * @return
	 */
	/*public static int getDaysOfYearMonth(int year, int month) {
		if (month != 2) {
			return dayOfMonth.get(month);
		}
		return isLeapYear(year) ? 29 : dayOfMonth.get(month);
	}*/

	/**
	 * 计算当前年有多少天
	 * @param year
	 * @return
	 */
	/*public static int getDaysOfYear(int year) {
	    return isLeapYear(year) ? 366 : 365;
	}*/

	/**
	 * 判断是否是闰年
	 * @param year
	 * @return
	 */
	/*public static boolean isLeapYear(int year) {
	    return year % 4 == 0 && year % 100 != 0 || year % 400 == 0;
	}
*/
   	/**
   	 * 友好的方式 显示时间
   	 * @param dateLong
   	 * @return
   	 */
   /* public static String friendlyFormat(long dateLong) {
        Date date = new Date(dateLong);
        Calendar now = Calendar.getInstance();
        String time = new SimpleDateFormat("HH:mm").format(date);
        // 第一种情况，日期在同一天
        String curDate = DATE_FORMAT.get().format(now.getTime());
        String paramDate = DATE_FORMAT.get().format(date);
        if (curDate.equals(paramDate)) {
            int hour = (int) ((now.getTimeInMillis() - date.getTime()) / 3600000);
            if (hour > 10) {
                return "今天 " + time;
            } else {
                if (hour > 0) {
                    return hour + "小时前";
                } else {
                    int minute = (int) ((now.getTimeInMillis() - date.getTime()) / 60000);
                    if (minute < 2) {
                        return "刚刚";
                    }
                    if (minute > 30) {
                        return "半小时前";
                    } else {
                        return minute + "分钟前";
                    }
                }
            }
        }
        // 第二种情况，不在同一天
        int days = (int) ((getStartTimeOfToday() - getStartTimeOfDay(date)) / 86400000);
        if (days == 1) {
            return "昨天 " + time;
        }
        if (days == 2) {
            return "前天 " + time;
        }
        if (days <= 7) {
            return days + "天前";
        }
        return DATE_FORMAT.get().format(date);
    }*/

    /**
     * 日期格式
     */
    @SuppressWarnings("unused")
	private final static ThreadLocal<SimpleDateFormat> DATE_FORMAT = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            return new SimpleDateFormat("yyyy-MM-dd");
        }
    };


    /**
     * 获取指定日期字符串，和格式。当前星期几的时间
     * @param dateStr
     * @param format
     * @return
     */
   /* public static String getCurrentWeekOfMonth(String dateStr, String format) {
        return getCurrentWeekOfMonth(formatString(dateStr, format));
    }*/

    /**
     * 获取指定日期。当前星期几的时间
     * @param date
     * @return
     */
  /*  public static String getCurrentWeekOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return getCurrentWeekOfMonth(calendar);
    }*/

   /* public static String getCurrentWeekOfMonth(Calendar calendar) {
        String strWeek = "";
        int dw = calendar.get(Calendar.DAY_OF_WEEK);
        if (dw == 1) {
            strWeek = "星期天";
        } else if (dw == 2) {
            strWeek = "星期一";
        } else if (dw == 3) {
            strWeek = "星期二";
        } else if (dw == 4) {
            strWeek = "星期三";
        } else if (dw == 5) {
            strWeek = "星期四";
        } else if (dw == 6) {
            strWeek = "星期五";
        } else if (dw == 7) {
            strWeek = "星期六";
        }
        return strWeek;
    }*/

    /**
     * 获取指定日期所在月份开始的时间戳
     * @param timestamp 指定日期的时间戳
     * @return
     */
   /* public static Long getMonthBegin(long timestamp) {
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(timestamp);

        //设置为1号,当前日期既为本月第一天
        c.set(Calendar.DAY_OF_MONTH, 1);
        //将小时至0
        c.set(Calendar.HOUR_OF_DAY, 0);
        //将分钟至0
        c.set(Calendar.MINUTE, 0);
        //将秒至0
        c.set(Calendar.SECOND,0);
        //将毫秒至0
        c.set(Calendar.MILLISECOND, 0);
        // 获取本月第一天的时间戳
        return c.getTimeInMillis();
    }*/

    /**
     * 获取指定日期所在月份结束的时间戳
     * @param timestamp 指定日期的时间戳
     * @return
     */
   /* public static Long getMonthEnd(long timestamp) {
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(timestamp);

        //设置为当月最后一天
        c.set(Calendar.DAY_OF_MONTH, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        //将小时至23
        c.set(Calendar.HOUR_OF_DAY, 23);
        //将分钟至59
        c.set(Calendar.MINUTE, 59);
        //将秒至59
        c.set(Calendar.SECOND,59);
        //将毫秒至999
        c.set(Calendar.MILLISECOND, 999);
        // 获取本月最后一天的时间戳
        return c.getTimeInMillis();
    }*/

    /**
     *
     * @Title: getMonthByTimestamp
     * @Description: 获取一个时间戳所在的月份
     * @param timestamp
     * @return
     * String
     * @throws
     */
   /* public static String getMonthByTimestamp(long timestamp) {
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(timestamp);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        return year+"年"+(month+1)+"月";
    }*/
    /**
     * 获取 20180629
     * @Title: createDateFormat
     * @Description: TODO(这里用一句话描述这个方法的作用)
     * @return
     * String
     * @throws
     * @变更记录  2018年6月29日 下午2:16:31 zhaoLuYang create
     */
   /* public static String createDateFormat(){
    	Calendar calendar = Calendar.getInstance();
    	StringBuffer sb = new StringBuffer();
    	calendar.setTime(new Date());
    	sb.append(calendar.get(Calendar.YEAR));
    	int lv2 = calendar.get(Calendar.MONTH)+1;
    	sb.append(lv2<10?("0"+lv2):lv2);
    	int lv5 = calendar.get(Calendar.DAY_OF_MONTH);
    	sb.append(lv5<10?("0"+lv5):lv5);
    	return sb.toString();
    }*/
    //TODO
  /*  public static void main(String[] args) throws ParseException{
    	//"yyyy-MM-dd HH:mm:ss"
    	*//*String[] str = new String[]{"2018-06-22","2018-06-27","2018-07-17","2018-07-23","2018-07-20","2018-07-28","2018-07-28","2018-07-20","2018-08-08","2018-08-07"};
    	SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
    	for(int i=0;i<str.length;i++){
    		System.out.println(simpleDateFormat.parse(str[i]).getTime());
    	}*//*
		String s = friendlyFormat(1539167462000L);
		System.out.println(s);
	}*/

   // public static Long formatDate()
}
