package com.comtika.safe.common.utils;

import jxl.*;
import jxl.format.Alignment;
import jxl.format.Colour;
import jxl.format.UnderlineStyle;
import jxl.format.VerticalAlignment;
import jxl.write.*;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.List;
import java.util.Map;

/**
 * 数据导出、导入使用java execl jxl 导出
 * 查看博客： https://www.cnblogs.com/hanfeihanfei/p/7079210.html
 * @author actor_miss[494977551@qq.com]
 * @date 2019/4/8 14:16
 */
public class JXLUtil {
    public static final String SERIAL_NO = "_serialNo";//序列号
    private static  WritableSheet createTitle(WritableWorkbook wk,String sheetName,int size,String fileName)throws Exception{
        ///创建可写入的Excel工作表
        WritableSheet sheet=wk.createSheet(sheetName, 0);
        //把单元格（column, row）到单元格（column1, row1）进行合并。
        //mergeCells(column, row, column1, row1);
        sheet.mergeCells(0,0, size-1,0);//单元格合并方法
        //创建WritableFont 字体对象，参数依次表示黑体、字号12、粗体、非斜体、不带下划线、亮蓝色
        WritableFont titleFont = new WritableFont(WritableFont.createFont("黑体"),12,WritableFont.BOLD,false, UnderlineStyle.NO_UNDERLINE, Colour.BLACK);
        //创建WritableCellFormat对象，将该对象应用于单元格从而设置单元格的样式
        WritableCellFormat titleFormat=new WritableCellFormat();
        //设置字体格式
        titleFormat.setFont(titleFont);
        //设置文本水平居中对齐
        titleFormat.setAlignment(Alignment.CENTRE);
        //设置文本垂直居中对齐
        titleFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
        //设置背景颜色
        titleFormat.setBackground(Colour.WHITE);
        //设置自动换行
        titleFormat.setWrap(true);
        //添加Label对象，参数依次表示在第一列，第一行，内容，使用的格式
        Label lab_00=new Label(0,0,fileName,titleFormat);
        //将定义好的Label对象添加到工作表上，这样工作表的第一列第一行的内容为‘学员考试成绩一览表’并应用了titleFormat定义的样式
        sheet.addCell(lab_00);
        return sheet;
    }
    private static WritableCellFormat defaultExeclCellFont()throws Exception{
        //加入数据头
        WritableFont headFont = new WritableFont(WritableFont.createFont("宋体"),10,WritableFont.BOLD,false, UnderlineStyle.NO_UNDERLINE, Colour.WHITE);
        WritableCellFormat headFormat=new WritableCellFormat();
        headFormat.setFont(headFont);
        headFormat.setAlignment(Alignment.CENTRE);
        headFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
        headFormat.setBackground(Colour.GRAY_50);
        return headFormat;
    }
    private static WritableCellFormat defaultFileCellFont()throws Exception{
        WritableCellFormat cloumnTitleFormat=new WritableCellFormat();
        cloumnTitleFormat.setFont(new WritableFont(WritableFont.createFont("宋体"),10,WritableFont.BOLD,false));
        cloumnTitleFormat.setAlignment(Alignment.CENTRE);
        return cloumnTitleFormat;
    }

    /**
     * 文件导出
     * @param data
     * @param titleNameAlias
     * @param titleName
     * @param fileName
     * @param out
     */
    private static void execlWrite(List<Map<String,Object>> data, List<String> titleNameAlias, List<String> titleName, String fileName, OutputStream out,String sheetName)throws Exception{
        if(StringUtil.isEmpty(fileName)) fileName = "eloam_"+System.currentTimeMillis();
        if(StringUtil.isEmpty(sheetName)) sheetName = "订单";
        //创建可写入的Excel工作薄，且内容将写入到输出流，并通过输出流输出给客户端浏览
        WritableWorkbook wk= Workbook.createWorkbook(out);
        final int size = titleNameAlias.size();
        ///创建可写入的Excel工作表
        WritableSheet sheet=createTitle(wk,sheetName,size,fileName);
        WritableCellFormat headFormat = defaultExeclCellFont();
        sheet.setRowView(0,400);
        for(int i=0;i<size;i++)sheet.addCell(new Label(i,1,titleNameAlias.get(i),headFormat));
        WritableCellFormat cloumnTitleFormat = defaultFileCellFont();
        Map<String, Object> next;
        Object nextValue;
        int length;
        boolean updateCloumn = true;
        for(int i=0,len=data.size();i<len;i++){
            next = data.get(i);
            for(int k=0;k < size;k++){
                String key = titleName.get(k);
                if(SERIAL_NO.equals(key)){
                    sheet.addCell(new Label(k,i+2,(i+1)+"",cloumnTitleFormat));
                }else {
                    nextValue = next.get(key);
                    if(updateCloumn){
                        String value = nextValue == null ? "" : nextValue.toString();
                        length = value.length();
                        if(length > 6)sheet.setColumnView(k,length*10/6);//根据内容修改长度
                        sheet.addCell(new Label(k,i+2,value,cloumnTitleFormat));
                    }else sheet.addCell(new Label(k,i+2, nextValue == null ? "" : nextValue.toString(),cloumnTitleFormat));
                    nextValue = null;
                }
            }
            updateCloumn = false;
        }
        //将定义的工作表输出到之前指定的介质中（这里是客户端浏览器）
        wk.write();
        //操作完成时，关闭对象，释放占用的内存空间
        wk.close();
        data.clear();
        titleName.clear();
        titleNameAlias.clear();
    }
    public static void execlWrite(List<Map<String,Object>> data, List<String> headTitleName, List<String> contentName, String fileName, HttpServletResponse response,String sheetName)throws Exception{
        OutputStream out = response.getOutputStream();
        response.reset();
        response.setHeader("Content-disposition","attachment;filename="+new String(fileName.getBytes("UTF-8"),"iso-8859-1")+".xls");
        response.setContentType("application/msexcel");
        execlWrite(data,headTitleName,contentName,fileName,out,sheetName);
    }







}
