package com.comtika.safe.common.utils.encrypt;


import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.util.Base64;

/**
 * BASE64工具类,对JAVA原生态BASE64处理做了封装,不抛异常<br>
 */
public class BASE64Coding {

	/**
	 * 按系统默认编码encode字符串
	 *
	 * @param str
	 *
	 * @return String
	 */
	public static String encode(String str) {
		String result = Base64.getEncoder().encodeToString(str.getBytes());
		//String result = new BASE64Encoder().encode(str.getBytes());
		return result.replaceAll("\\r\\n", "");
	}

	/**
	 * <b>按指定编码encode字符串</b><br>
	 *
	 * @param str
	 * @param charsetName
	 *
	 * @return String
	 */
	public static String encode(String str, String charsetName) {
		String result;
		Charset charset = Charset.forName(charsetName);
		if (charset != null) {
			result = Base64.getEncoder().encodeToString(str.getBytes(charset));
			//result = new BASE64Encoder().encode(str.getBytes(charset));
		} else {
			result = Base64.getEncoder().encodeToString(str.getBytes());
			//result = new BASE64Encoder().encode(str.getBytes());
		}
		return result.replaceAll("\\r\\n", "");
	}

	/**
	 * <b>对字节数组进行encode</b><br>
	 *
	 * @param bytes
	 *
	 * @return String
	 */
	public static String encode(byte[] bytes) {
		//String result = new BASE64Encoder().encode(bytes);
		String result = Base64.getEncoder().encodeToString(bytes);
		return result.replaceAll("\\r\\n", "");
	}

	/**
	 * <b>对ByteBuffer进行encode</b><br>
	 *
	 * @param buf
	 *
	 * @return String
	 */
	public static String encode(ByteBuffer buf) {
		return encode(buf.array());
	}

	/**
	 * <b>对BASE64的字符串进行decode,若decode失败返回null</b><br>
	 *
	 * @param str
	 *
	 * @return byte[]
	 */
	public static byte[] decodeBuffer(String str) {
		try {
			return Base64.getDecoder().decode(str);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * <b>按系统默认编码decode字符串,若decode失败返回""</b><br>
	 *
	 * @param str
	 *
	 * @return String
	 */
	public static String decode(String str) {
		byte[] bytes = decodeBuffer(str);
		if(bytes != null) return new String(bytes);
		return null;
	}

	/**
	 *
	 * @Title: decodeNew
	 * @Description: TODO(这里用一句话描述这个方法的作用)
	 * @param str
	 * @return
	 * @throws
	 */
    public static String decodeNew(String str) {
        String result = "";
        try {
            byte[] bytes=Base64.getDecoder().decode(str);
            result = new String(bytes);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

	/**
	 * <b>按指定编码decode字符串,若decode失败返回""</b><br>
	 *
	 * @param str
	 * @param charsetName
	 *
	 * @return String
	 */
	public static String decode(String str, String charsetName) {
		String result = "";
		try {
			Charset cs = Charset.forName(charsetName);
			if (cs != null) {
				byte[] bytes = Base64.getDecoder().decode(str);
				result = new String(bytes, charsetName);
			} else {
				result = decode(str);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	public static void main(String[] args) {
		/*String s = "123456jahahuhai奥哈哈啦啦啦2222222oojsdfdas\nfdsfsd\nfdsfsdsdfdsfdsfs\nszad a 哈哈规划dfsdafasdfsdfasdfasdfsdafsdfsdafsdffsdfdsfsdfsdfsd";
		String str = BASE64Coding.encode("1");
		System.out.println(str);
		System.out.println(BASE64Coding.decode(str));*/
	}

}
