package com.comtika.safe.common.utils.encrypt;

import com.comtika.safe.common.exception.RespException;
import com.comtika.safe.common.response.RespCode;

import java.nio.ByteBuffer;
import java.util.concurrent.ThreadLocalRandom;


/**
 *
 * @desc	:	SID 加密工具
 * @author	:	actorLucky[494977551@qq.com]
 * @date	:	2018年4月27日 下午5:05:22
 */
public class SidUtil {
	private static ThreadLocalRandom random = ThreadLocalRandom.current();
	/** 登录用户SID创建时的加密KEY */
    private static final byte[] KEYS = "(&*%^#$@!~HgUj64Pi5K1s9)".getBytes();
	/** 登录超时时间(毫秒), 为0则永久有效 */
	private static final long LOGIN_TIMEOUT = Integer.MAX_VALUE;

	private static DESCoding des = null;
	static {
		try {
			des = new DESCoding(KEYS);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static class SidInfo {
		public long userId;
		public int skey;
		public long time;

		@Override
		public String toString() {
			return "{userId=" + userId + ",skey=" + skey + ",time=" + time + "}";
		}
	}

	@SuppressWarnings("static-access")
	public static String createSid(long userId) {
		ByteBuffer buf = ByteBuffer.allocate(20);
		buf.putLong(userId);
		buf.putInt(random.nextInt());
		buf.putLong(System.currentTimeMillis());
		buf.flip();
		byte[] encodeBytes = des.encode(buf.array());
		// 不用base64了，增强一些性能
		return HexUtil.bytes2HexStr(encodeBytes);
	}

	@SuppressWarnings("static-access")
	public static String createSidBySessionId(long userId, int skey, long now, String sessionId) {
		ByteBuffer buf = ByteBuffer.allocate(20 + 4 + sessionId.length());
		buf.putLong(userId);
		buf.putInt(skey);
		buf.putLong(now);
		buf.putInt(sessionId.length());
		buf.put(sessionId.getBytes());
		buf.flip();
		byte[] encodeBytes = des.encode(buf.array());
		// 不用base64了，增强一些性能
		return HexUtil.bytes2HexStr(encodeBytes);
	}

	public static void main(String[] args) {
//		String createSid = createSid(4689);
//		System.out.println(createSid);

		String str = "3F572CDF4DB9A65DB26C7DA441C4D917C7BE5A8B527F2424";
		System.out.println(decodeSid(str).userId);
		/*String createSid = createSid(5565557,1235555540,System.currentTimeMillis());
		System.out.println(createSid);
		SidInfo decodeSid = decodeSid(createSid);
		System.out.println(decodeSid.toString());*/
		/*String sid = createSidBySessionId(2, 1230, System.currentTimeMillis(), "aaa-3bcJfVutq6qb-fIew");
		System.out.println(sid.length());
		System.out.println(sid);

		String sid2 = createSidBySessionId(2, 1230, System.currentTimeMillis(), "aaa-kWKpzjzok9sb-fIew");
		System.out.println(sid2.length());
		System.out.println(sid2);*/

	}

	@SuppressWarnings("static-access")
	public static SidInfo decodeSid(String sid) {
		byte[] bs = HexUtil.hexStr2Bytes(sid);
		if (bs == null || bs.length < 10) {
			throw new RespException(RespCode.UN_LOGIN, "SID失效,请重新登录!");
		}
		byte[] bs2 = des.decode(bs);
		if (bs2 == null || bs2.length != 20) {
			throw new RespException(RespCode.UN_LOGIN, "SID失效,请重新登录!");
		}
		ByteBuffer buf = ByteBuffer.wrap(bs2);
		SidInfo info = new SidInfo();
		info.userId = buf.getLong();
		info.skey = buf.getInt();
		info.time = buf.getLong();

		// 需要做登录超时判断
		if (LOGIN_TIMEOUT > 0) {
			long t = System.currentTimeMillis() - info.time;
			if (t > (LOGIN_TIMEOUT)) {
				throw new RespException(RespCode.UN_LOGIN, "登录已过期,请重新登录!");
			}
		}
		return info;
	}
}
