package com.comtika.safe.common.utils.http;


import com.comtika.safe.common.utils.StringUtil;

import javax.net.ssl.*;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;


/**
 * @desc	:	JAVA原生Http通讯工具类,不依赖任何外部JAR包,需要使用真实证书文件的HTTPS请勿用.
 * @author	:	actorLucky[494977551@qq.com]
 */
public class HttpUtil {
	/**
	 * 获取请求连接
	 * @param urlStr 请求url
	 * @return 连接
	 * @throws Exception
	 */
	public static HttpURLConnection createConnection(String urlStr) throws Exception{
		try {
			URL url = new URL(urlStr);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setConnectTimeout(10000);//连接超时
			conn.setReadTimeout(30000);//写超时
			conn.setDoInput(true);//可读设置是否从httpUrlConnection读入，默认情况下是true;
			conn.setDoOutput(true);// 可写http正文内，因此需要设为true, 默认情况下是false;
			conn.setUseCaches(false); // 取消缓存  Post 请求不能使用缓存
			// 设定传送的内容类型是可序列化的java对象
			// (如果不设此项,在传送序列化对象时,当WEB服务默认的不是这种类型时可能抛java.io.EOFException)
			conn.setRequestProperty("Content-type", "application/x-java-serialized-object");
			if("https".equalsIgnoreCase(url.getProtocol())){
				//证书工具,用于不需要证书文件的Https通讯
				TrustManager[] xtm = {new X509TrustManager(){
					@Override
					public void checkClientTrusted(X509Certificate[] arg0,
												   String arg1) throws CertificateException {
						// TODO Auto-generated method stub
					}
					@Override
					public void checkServerTrusted(X509Certificate[] arg0,
												   String arg1) throws CertificateException {
						// TODO Auto-generated method stub
					}
					@Override
					public X509Certificate[] getAcceptedIssuers() {
						// TODO Auto-generated method stub
						return null;
					}}};
				//Https通讯时所需的域名校验
				HostnameVerifier hv = (str,ssl)->true;
				HttpsURLConnection hsConn = (HttpsURLConnection) conn;
				SSLContext sslContext = SSLContext.getInstance("TLS");
				sslContext.init(null, xtm, new SecureRandom());
				hsConn.setSSLSocketFactory(sslContext.getSocketFactory());
				hsConn.setHostnameVerifier(hv);
				return hsConn;
			}else{
				return conn;
			}
		} catch (Exception e) {
			throw new Exception("HttpClient CreateConnection Error!", e);
		}
	}
	/**
	 *
	 * @param url
	 * @return
	 */
	public static String doPost(String url){
		return doPost(url, "", null);
	}
	/**
	 *
	 * @param url
	 * @param params
	 * @return
	 */
	public static String doPost(String url,Map<String,Object> params){
		return doPost(url,params,"");
	}
	/**
	 *
	 * @param url
	 * @param params
	 * @return
	 */
	public static String doPost(String url,String params){
		if("utf-8".equalsIgnoreCase(params) || "gbk".equalsIgnoreCase(params)){
			return doPost(url,"",params);
		}else{
			return doPost(url,params,"");
		}
	}
	/**
	 *
	 * @param url
	 * @param params
	 * @param charset
	 * @return
	 */
	public static String doPost(String url,Map<String,Object> params,String charset){
		StringBuffer sb = new StringBuffer("");
		if(params != null){
			for(Entry<String,Object> entry:params.entrySet()){
				if("".equals(sb.toString())){
					sb.append(entry.getKey()+"="+entry.getValue());
				}else{
					sb.append("&"+entry.getKey()+"="+entry.getValue());
				}
			}
		}
		return doPost(url, sb.toString(), charset);
	}


	/**
	 * 发送post请求
	 * @param url
	 * @param params
	 * @param charsetName
	 * @return
	 */
	public static String doPost(String url,String params,String charsetName){
		HttpURLConnection conn = null;
		try {
			conn = createConnection(url);
			conn.setRequestMethod("POST");
			if(!isEmpty(params)){
				OutputStream out = conn.getOutputStream();
				if(isEmpty(charsetName)){
					out.write(params.getBytes());
				}else{
					out.write(params.getBytes(charsetName));
				}
				out.flush();
				out.close();
			}else{
				// 连接，从上述第2条中url.openConnection()至此的配置必须要在connect之前完成，
				conn.connect();
			}
			// 读取返回信息
			int respCode = conn.getResponseCode();
			String respResult = conn.getResponseMessage();
			//
			if (respCode == 200 || respCode == 302) {
				//发送请求的代码在这里
				InputStream inputStream = conn.getInputStream();
				BufferedReader reader = null;
				if (isEmpty(charsetName)) {
					reader = new BufferedReader(new InputStreamReader(
							inputStream));
				} else {
					reader = new BufferedReader(new InputStreamReader(
							inputStream, charsetName));
				}
				String strMessage = null;
				StringBuffer buffer = new StringBuffer();
				while ((strMessage = reader.readLine()) != null) {
					buffer.append(strMessage);
				}
				respResult = buffer.toString();
			}
			return respResult;
		} catch (Exception e) {
			System.out.println("通讯失败!");
			e.printStackTrace();
		}finally{
			if(conn != null){
				conn.disconnect();
			}
		}
		return null;
	}
	public static String doGet(String url){
		return doGet(url,"");
	}
	/**
	 *
	 * @param url
	 * @param charset
	 * @return
	 */
	public static String doGet(String url,String charset){
		HttpURLConnection conn = null;
		try {
			conn = createConnection(url);
			conn.setRequestMethod("GET");
			conn.connect();
			// 读取返回信息
			int respCode = conn.getResponseCode();
			String respResult = conn.getResponseMessage();
			if(respCode == 200){
				InputStream in = conn.getInputStream();
				BufferedReader reader = null;
				if(isEmpty(charset)){
					reader = new BufferedReader(new InputStreamReader(in));
				}else{
					reader = new BufferedReader(new InputStreamReader(in, charset));
				}
				String str = null;
				StringBuffer sb = new StringBuffer("");
				while((str= reader.readLine()) != null){
					sb.append(str);
				}
				respResult = sb.toString();
			}
			return respResult;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("通讯失败!");
			return "通讯失败!";
		}finally{
			if(conn != null){
				conn.disconnect();
			}
		}
	}
	private static boolean isEmpty(String str){
		if(str == null)
			return true;
		return str.trim().isEmpty();
	}
	@SuppressWarnings("unused")
	public static void main(String[] agrs){
		Map<String,Object> parmas = new HashMap<>();
		parmas.put("xm","王伟");
		parmas.put("sfzhm","362202199008256210");
		parmas.put("method","covid.interface.query");
		String doGet2 = doPost("http://www.bjyqzwfw.com/beiji_tj/api",parmas);


		String doGet = doPost("http://www.bjyqzwfw.com/beiji_tj/api?method=covid.interface.query&xm=王伟&=362202199008256210");
		System.out.println(doGet);
	}

	/**
	 *
	 * @Title: post
	 * @Description: TODO(POST提交数据)
	 * @param conn
	 * @param postData
	 * @return
	 * HttpResult
	 */
	public static HttpResult post(HttpURLConnection conn, String postData){
		return post(conn, postData, null);
	}

	public static HttpResult post(HttpURLConnection conn, String postData, String charsetName){
		HttpResult httpResult = new HttpResult();
		try{
			conn.setRequestMethod("POST");
			OutputStream os = conn.getOutputStream();
			if(StringUtil.isEmpty(charsetName)){
				os.write(postData.getBytes());
			}else{
				os.write(postData.getBytes(charsetName));
			}
			os.flush();
			os.close();

			//读取返回信息
			int respCode = conn.getResponseCode();
			String respResult = conn.getResponseMessage();

			if( respCode == 200 ){
				InputStream inputStream = conn.getInputStream();
				BufferedReader reader = null;
				if(StringUtil.isEmpty(charsetName)){
					reader = new BufferedReader(new InputStreamReader(inputStream));
				}else{
					reader = new BufferedReader(new InputStreamReader(inputStream, charsetName));
				}
				String strMessage = null;
				StringBuffer buffer=new StringBuffer();
				while ((strMessage = reader.readLine()) != null) {
					buffer.append(strMessage);
				}
				respResult = buffer.toString();
			}

			httpResult.setStatus(respCode);
			httpResult.setResult(respResult);
		} catch ( Exception e ) {
			httpResult.setStatus(0);
			httpResult.setResult("Post通讯异常,请检查后重试!");
			e.printStackTrace();
		} finally {
			//关闭连接
			if(conn != null){
				conn.disconnect();
			}
		}
		return httpResult;
	}
}
