package com.comtika.safe.config;

import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.netty.NettyServer;
import com.comtika.safe.core.protocol.repData.TerminalRepDataHandle;
import com.comtika.safe.core.protocol.timer.CheckTerminalHeartbeat;
import lombok.Data;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

/**
 * 待用
 */
@Configuration
@EnableConfigurationProperties({ProtocolConfig.class,WeChatProperties.class})
@Data
public class ProjectConfig {

	//上传url
    @Value("${safe.upload-url}")
    private String uploadUrl;

    //商户管理员角色名称
    @Value("${comtika.safe.parameter.merchantRole}")
	private String merchantRole;

    //市场专员角色名称
	@Value("${comtika.safe.parameter.marketRole}")
	private String marketRole;

    @Autowired
    private TerminalRepDataHandle terminalRepDataHandle;
    @Autowired
    private CheckTerminalHeartbeat checkClientSkip;

    @Autowired
    private NettyServer nettyServer;
    @Autowired
    private ProtocolConfig protocolConfig;

    @PostConstruct
    public void start() throws Exception {
        checkClientSkip.setChceckRate(protocolConfig.getHbrate());
        checkClientSkip.setDelayTimeout(protocolConfig.getHbTimeOut());
        checkClientSkip.start();
        /**netty socket*/
        nettyServer.start();
        terminalRepDataHandle.init();
    }

    @PreDestroy
    public void stop(){
        terminalRepDataHandle.stop();
        nettyServer.stop();
        checkClientSkip.stop();
        LocalProtocolCache.cleanAllCache();
    }

}
