package com.comtika.safe.controller.equip.equip;



import com.comtika.safe.entity.equip.AddEquipListEntity;
import com.comtika.safe.entity.equip.help.TerminalExcelHelp;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.equip.terminal.EquipTerminalAddEntity;
import com.comtika.safe.service.equip.terminal.EquipTerminalService;

/**
 * 集中器控制层
 *
 */
@Api(value = "equipTerminalApi", tags = { "equipTerminalApi" })
@RestController
@RequestMapping("/equip/equipTerminal")
public class EquipTerminalController extends BaseController{

	@Autowired
	private EquipTerminalService equipTerminalService;


	@ApiOperation(value = "新增设备信息", notes = "新增设备信息")
    @ApiImplicitParam(name = "equip", value = "设备参数实体", required = false, dataType = "EquipTerminalAddEntity")
	@PostMapping(value = "/addEquipTerminal")
	public RespWrapper<Object> addEquipTerminal(@RequestBody EquipTerminalAddEntity equip) {
		return equipTerminalService.addEquipTerminal(equip,request);
	}
	
	@ApiOperation(value = "批量新增设备信息", notes = "批量新增设备信息")
    @ApiImplicitParam(name = "equip", value = "批量新增设备参数实体", required = false, dataType = "AddEquipListEntity")
	@PostMapping(value = "/addEquipTerminalList")
	public RespWrapper<Object> addEquipTerminalList(@RequestBody AddEquipListEntity equip) {
		return equipTerminalService.addEquipTerminalList(equip,request);
	}

	@ApiOperation(value = "查询设备列表", notes = "查询设备列表")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "siteName", value = "站点名称", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "equipIccid", value = "物联网卡ICCID", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "equipImei", value = "物联网卡Imei", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "terminalNo", value = "设备编号", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "status", value = "设备状态", required = false, dataType = "Integer",paramType="query"),
        @ApiImplicitParam(name = "equipProductTypeId", value = "设备产品类型id", required = false, dataType = "Long",paramType="query"),
        @ApiImplicitParam(name = "siteId", value = "站点id", required = false, dataType = "Long",paramType="query")
    })
	@PostMapping(value = "/findTerminalList")
	public RespWrapper<Object> findTerminalList() {
		return equipTerminalService.findTerminalList(request,super.getPageQuery(true));
	}
	@ApiOperation(value = "查询设备列表，新接口")
	@PostMapping("fadFindTerminalList")
	public RespWrapper fadFindTerminalList(@RequestBody TerminalExcelHelp help){
		return equipTerminalService.fadFindTerminalList(help,findUserCache());
	}
	@ApiOperation(value = "设备列表导出接口")
	@PostMapping("terminalListExcel")
	public RespWrapper terminalListExcel(TerminalExcelHelp help){
		return equipTerminalService.terminalListExcel(help,findUserCache(),response);
	}


	@ApiOperation(value = "设备详情", notes = "设备详情")
    @ApiImplicitParam(name = "terminalId", value = "设备参数实体", required = false, dataType = "Long")
	@GetMapping(value = "/queryEquipTerminalById")
	public RespWrapper<Object> queryEquipTerminalById(Long terminalId) {
		return equipTerminalService.queryEquipTerminalById(terminalId,request);
	}

	@ApiOperation(value = "修改集中器信息", notes = "修改集中器信息")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "equip", value = "修改参数实体", dataType = "EquipTerminalAddEntity", required = false)
	})
	@PostMapping(value = "/updateEquipTerminal")
	public RespWrapper<Object> updateEquipTerminal(@RequestBody EquipTerminalAddEntity equip) {
		return equipTerminalService.updateEquipTerminal(equip, request);
	}

	@ApiOperation(value = "根据设备编号查询回路数", notes = "根据设备编号查询回路数")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "terminalNo", value = "设备编号", dataType = "String", required = false)
	})
	@GetMapping(value = "/queryLoopNumberByTerminalNo")
	public RespWrapper<Object> queryLoopNumberByTerminalNo(String terminalNo) {
		return equipTerminalService.queryLoopNumberByTerminalNo(terminalNo, request);
	}
	
	@ApiOperation(value = "删除集中器信息", notes = "删除集中器信息")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "terminalId", value = "集中器id", dataType = "Long", required = false)
	})
	@PostMapping(value = "/deleteEquipTerminal")
	public RespWrapper<Object> deleteEquipTerminal(Long terminalId) {
		return equipTerminalService.deleteEquipTerminal(terminalId, request);
	}
}
