package com.comtika.safe.controller.merchant;



import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.metchant.EnableMerchant;
import com.comtika.safe.entity.metchant.RegisterMerchant;
import com.comtika.safe.service.merchant.MerchantService;

@RestController
@RequestMapping("/merchant")
public class MerchantController extends BaseController{
	
	@Autowired
	private MerchantService merchantService;
	
	
	@ApiOperation(value = "新增商户", notes = "新增商户")
	@PostMapping("/add")
	public RespWrapper<Object> addMerchant(@RequestBody RegisterMerchant registerMerchant){
		return merchantService.addMerchant(registerMerchant,super.findUserCache(), request);
	}
	
	@ApiOperation(value = "商户列表", notes = "商户列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "merchantAbbr", value = "商户简称", required = false, dataType = "String",paramType="query"),
            @ApiImplicitParam(name = "cooperatStatus", value = "合作状态", required = false, dataType = "Integer",paramType="query")
    })
	@GetMapping("/list")
	public RespWrapper<Object> queryMerchantList(String merchantAbbr ,Integer cooperatStatus){
		return merchantService.queryMerchantList( merchantAbbr , cooperatStatus,super.getPageQuery(true), request);
	}
	
	@ApiOperation(value = "详情", notes = "获取商户详情")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "merchantId", value = "商户id", required = true, dataType = "Long",paramType="query"),
        @ApiImplicitParam(name = "userFlag", value = "是否需要获取商户下的人员标识，可为空，当值为空或为true时，获取人员列表", required = false, dataType = "boolean",paramType="query")
    })
    @RequestMapping(value = "/merchantDetail", method = RequestMethod.GET)
    public RespWrapper<Object> merchantDetail(Long merchantId,Boolean userFlag) {
		return merchantService.merchantDetail(merchantId,userFlag, request);
    }
	
	@ApiOperation(value = "修改商户", notes = "修改商户")
    @ApiImplicitParam(name = "merchant", value = "参数实体", required = true, dataType = "RegisterMerchant")
    @RequestMapping(value = "/updateMerchant", method = RequestMethod.POST)
    public RespWrapper<Object> updateMerchant(@RequestBody RegisterMerchant merchant) {
		return merchantService.updateMerchant(merchant, request);
    }
	
	@ApiOperation(value = "启用/停用/审核商户", notes = "启用/停用/审核商户")
    @ApiImplicitParam(paramType="query",name = "enableMerchant", value = "审核商户详情", required = false, dataType = "EnableMerchant")
    @PutMapping("/enableMerchant")
    public RespWrapper<Object> enableMerchant(@RequestBody EnableMerchant enableMerchant) {
        return merchantService.enableMerchant(enableMerchant, request);
    }
	
	@ApiOperation(value = "市场专员列表", notes = "获取配置市场专员列表，默认为角色【ROLE_MARKET_USERS】配置的人员，也可传参修改为其他角色")
    @ApiImplicitParam(name = "roleName", value = "市场专员角色名称，为空时默认角色：ROLE_MARKET_USERS", required = false, dataType = "String",paramType="query")
    @RequestMapping(value = "/getMarketUsers", method = RequestMethod.GET)
    public RespWrapper<Object> getMarketUsers(String roleName) {
		return merchantService.getMarketUsers(roleName, request);
    }
	
	@ApiOperation(value = "获取所有商户（商户id，商户全称，商户简称）", notes = "获取所有商户（商户id，商户全称，商户简称）")
    @RequestMapping(value = "/queryMerchantAbbrList", method = RequestMethod.GET)
    public RespWrapper<Object> queryMerchantAbbrList() {
		return merchantService.queryMerchantAbbrList(super.findUserCache(), request);
    }
	
	@ApiOperation(value = "修改商户logo与title", notes = "修改商户logo与title")
    @RequestMapping(value = "/updateMerchantLogoAndTitle", method = RequestMethod.POST)
    public RespWrapper<Object> updateMerchantLogoAndTitle(Long merchantId,String merchantLogo,String merchantTitle) {
		return merchantService.updateMerchantLogoAndTitle(merchantId,merchantLogo,merchantTitle, request);
    }
	
	@ApiOperation(value = "查询商户logo与title", notes = "查询商户logo与title")
    @RequestMapping(value = "/queryMerchantLogoAndTitle", method = RequestMethod.POST)
    public RespWrapper<Object> queryMerchantLogoAndTitle(Long merchantId) {
		return merchantService.queryMerchantLogoAndTitle(merchantId, request);
    }
}
