package com.comtika.safe.controller.message;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.entity.message.FadMessageConf;
import com.comtika.safe.entity.message.help.MessageAcceptHelp;
import com.comtika.safe.entity.wx.WeChatHelp;
import com.comtika.safe.service.message.IMessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController
@RequestMapping("messagepush")
@Api(description = "消息模块，消息配置")
public class MessageController extends BaseController {

    @Autowired
    private IMessageService messageService;

    @GetMapping("fadFindMessageConfig/{siteId}")
    public RespWrapper<Map<String,Object>> fadFindMessageConfig(@PathVariable("siteId")Integer siteId){
        return messageService.fadFindMessageConfig(siteId);
    }
    @PostMapping("addMessageAcceptList")
    public RespWrapper<Boolean> addMessageAcceptList(@RequestBody MessageAcceptHelp help){
        if(help.getSiteId() == null) return RespWrapper.makeFailResp("配置站点不能为空");
        if(help.getUserList() == null || help.getUserList().isEmpty())
            return RespWrapper.makeFailResp("配置用户不能为空");
        return messageService.addMessageAcceptList(help);
    }

    @PostMapping("fadMessageConfig")
    public RespWrapper<Boolean> fadMessageConfig(@RequestBody FadMessageConf help){
        // 处理一些数据
        if(help.getId() == null) return RespWrapper.makeFailResp("配置唯一id 不能为空");
        if(help.getMainSwitch() == null) help.setMainSwitch(false);
        if(help.getOfflineSwitch() == null) help.setOfflineSwitch(false);
        if(help.getWxSwitch() == null) help.setWxSwitch(true);
        return messageService.fadMessageConfig(help);
    }
    @PostMapping("checkPhoneCode/{phoneNo}/{code}")
    public RespWrapper<Boolean> checkPhoneCode(@PathVariable("phoneNo")String phoneNo,@PathVariable("code")String code){
        return messageService.checkPhoneCode(phoneNo,code);
    }

    @DeleteMapping("deleteMessageAcceptUser/{id}")
    public RespWrapper<Boolean> deleteMessageAcceptUser(@PathVariable("id")Integer id){
        return messageService.deleteMessageAcceptUser(id,findUserCache());
    }
    @PostMapping("updateMessageHandleUser/{mesId}/{userId}")
    public RespWrapper<Boolean> updateMessageHandleUser(@PathVariable("mesId")Integer msgId,@PathVariable("userId")Integer userId){
        return messageService.updateMessageHandleUser(msgId,userId);
    }

    @ApiOperation(value ="根据手机号判断该用户是否授权工作号,code=0说明已授权，code=2没有授权")
    @GetMapping("checkWeChatCode")
    public RespWrapper<Boolean> checkWeChatCode(String phoneNo){
        if(StringUtil.isEmpty(phoneNo)) return RespWrapper.makeFailResp("手机号不能为空");
        return messageService.checkWeChatCode(phoneNo);
    }
    @ApiOperation(value = "获取微信公众号openId,使用静默授权方式")
    @PostMapping("weChatAcCodeAuth")
    public RespWrapper<Boolean> weChatAcCodeAuth(@RequestBody WeChatHelp help){
        if(StringUtil.isEmpty(help.getAcCode()))
            return RespWrapper.makeFailResp("静默授权CODE不能为空");
       /* if(StringUtil.isEmpty(help.getPhoneNo()))
            return RespWrapper.makeFailResp("授权手机号不能为空");*/
        /*if(StringUtil.isEmpty(help.getCode()))
            return RespWrapper.makeFailResp("手机验证码不能为空");*/
        return messageService.weChatAcCodeAuth(help,findUserCache());
    }
    @ApiOperation(value = "删除用户自己的微信公众号授权操作")
    @DeleteMapping("delUserAuth")
    public RespWrapper<Boolean> delUserAuth(){
        return messageService.delUserAuth(findUserCache());
    }

    @ApiOperation(value = "发生验证码")
    @PostMapping("sendPhoneCode/{phoneNo}")
    public RespWrapper<Boolean> sendPhoneCode(@PathVariable("phoneNo")String phoneNo){
        return messageService.sendPhoneCode(phoneNo);
    }

}
