package com.comtika.safe.controller.meter;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.entity.meter.CtkWattMeter;
import com.comtika.safe.service.meter.IMeterService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("meter")
@Api(description = "电能API")
public class MeterController extends BaseController {
    @Autowired
    private IMeterService ctkWattMeterService;

    @ApiOperation(value = "根据站点id 查询该站点下配置的电能表")
    @GetMapping("findWattMeterList")
    public RespWrapper<List<CtkWattMeter>> findWattMeterList(Integer siteId){
        if(siteId == null) return RespWrapper.makeFailResp("站点id 不能为空");
        return ctkWattMeterService.findWattMeterList(siteId);
    }
    @ApiOperation("删除站点下的电表记录")
    @DeleteMapping("deleteWattMeter/{id}")
    public RespWrapper<Boolean> deleteWattMeter(@PathVariable("id")Integer id){
        return ctkWattMeterService.deleteWattMeter(id);
    }
    @ApiOperation(value = "新增站点电表记录")
    @PostMapping("addWattMeter")
    public RespWrapper<Boolean> addWattMeter(@RequestBody CtkWattMeter ctkWattMeter){
        RespWrapper<Boolean> ck = checkWattMeter(ctkWattMeter, false);
        if(!ck.isSuccess()) return ck;
        return ctkWattMeterService.addWattMeter(ctkWattMeter);
    }
    @ApiOperation(value = "更新站点电表记录")
    @PostMapping("updateWattMeter")
    public RespWrapper<Boolean> updateWattMeter(@RequestBody CtkWattMeter ctkWattMeter){
        RespWrapper<Boolean> ck = checkWattMeter(ctkWattMeter, true);
        if(!ck.isSuccess()) return ck;
        return ctkWattMeterService.updateWattMeter(ctkWattMeter);
    }
    private RespWrapper<Boolean> checkWattMeter(CtkWattMeter wt,boolean isUpdate){
        if(isUpdate && wt.getId() == null)
            return RespWrapper.makeFailResp("唯一id 不能为空");
        if(wt.getSiteId() == null)
            return RespWrapper.makeFailResp("站点不能为空");
        if(StringUtil.isEmpty(wt.getTerminalNo()))
            return RespWrapper.makeFailResp("设备编号不能为空");
        return RespWrapper.makeSuccessResp(true);
    }
    @ApiOperation(value = "查询站点下的所有设备编码")
    @GetMapping("findSiteAllTerminalNo")
    public RespWrapper<List<String>> findSiteAllTerminalNo(Integer siteId){
        if(siteId == null)
            return RespWrapper.makeFailResp("站点id 不能为空");
        return ctkWattMeterService.findSiteAllTerminalNo(siteId);
    }
}
