/******************************************************************************
* Copyright (C) 2018 comtika Co.,Ltd
* All Rights Reserved.
* 本软件为康体佳智能科技（深圳）有限公司开发研制。未经本公司正式书面同意，其他任何个人、团体不得使用、
* 复制、修改或发布本软件.
*****************************************************************************/
package com.comtika.safe.controller.platform;



import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.SysPermission;
import com.comtika.safe.service.sys.permission.PermissionService;
import com.comtika.safe.service.sys.user.UserService;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @ClassName: MenuPermissionController
 * @Description:菜单按钮-权限控制类 (页面权限控制单位)
 * @version V1.0
 */
@RestController
@RequestMapping("/menu")
public class MenuPermissionController extends BaseController
{
    //private static final Logger log = LoggerFactory.getLogger(MenuPermissionController.class);
    
    @Autowired
    private PermissionService permissionService;
    
    @Autowired
    private UserService userService;
    
    @ApiOperation(value = "当前用户页面权限", notes = "获取当前用户页面权限（菜单及按钮）--登录后菜单初始化")
    @RequestMapping(value = "/pageRole", method = RequestMethod.GET)
    public RespWrapper<Object> pageRole() {
    	return permissionService.pageRole(super.findUserCache(), request);
    }
    
    @ApiOperation(value = "列表", notes = "菜单权限列表信息")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public RespWrapper<Object> getList() {
    	return permissionService.queryList(null, request);
    }

    @ApiOperation(value = "新增菜单权限", notes = "新增菜单权限")
    @ApiImplicitParam(name = "menu", value = "参数实体", required = true, dataType = "SysPermission")
    @RequestMapping(value = "/addMenu", method = RequestMethod.POST)
    public RespWrapper<Object> addMenu(@RequestBody SysPermission menu) {
    	return  permissionService.insertPermission(menu, request);
    }
    
    @ApiOperation(value = "修改菜单权限", notes = "修改菜单权限")
    @ApiImplicitParam(name = "menu", value = "参数实体", required = true, dataType = "SysPermission")
    @RequestMapping(value = "/updateMenu", method = RequestMethod.POST)
    public RespWrapper<Object> updateMenu(@RequestBody SysPermission menu) {
    	return permissionService.updatePermission(menu, request);

    }
    
    @ApiOperation(value = "启用/停用菜单权限", notes = "启用/停用菜单权限")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "menuId", value = "菜单id集合 JSONArray", dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "status", value = "状态：启用--1；停用--2", dataType = "Integer",paramType="query")
    })
    @RequestMapping(value = "/enableMenu", method = RequestMethod.POST)
    public RespWrapper<Object> enableMenu(String menuId,Integer status) {
    	return permissionService.enableMenu(menuId,status, request);
    }
    
    @ApiOperation(value = "获取菜单权限详情", notes = "获取菜单权限详情")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "menuId", value = "菜单id", required = true, dataType = "long",paramType="query")
    })
    @RequestMapping(value = "/menuDetail", method = RequestMethod.GET)
    public RespWrapper<Object> menuDetail(long menuId) {
    	return permissionService.menuDetail(menuId, request);
    }
}
