/******************************************************************************
* Copyright (C) 2018 comtika Co.,Ltd
* All Rights Reserved.
* 本软件为康体佳智能科技（深圳）有限公司开发研制。未经本公司正式书面同意，其他任何个人、团体不得使用、
* 复制、修改或发布本软件.
*****************************************************************************/
package com.comtika.safe.controller.platform;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;




import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.role.ParaSysRole;
import com.comtika.safe.service.sys.role.RoleService;
import com.comtika.safe.service.sys.user.UserService;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 *
 * @ClassName: RoleController
 * @Description:角色控制类
 */
@RestController
@RequestMapping("/role")
public class RoleController extends BaseController
{
    //private static final Logger log = LoggerFactory.getLogger(RoleController.class);

    @Autowired
    private UserService userService;

    @Autowired
    private RoleService roleService;

    @ApiOperation(value = "列表", notes = "分页查角色列表信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "start", value = "页码", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "size", value = "每页大小", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "sortStr", value = "排序列,默认为id", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "sortType", value = "排序方式  逆序：desc 顺序：asc ", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "roleName", value = "角色名称", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "description", value = "描述", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "status", value = "状态", required = false, dataType = "Integer",paramType="query")
    })
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public RespWrapper<Object> getList() {
    	return roleService.getList(super.getPageQuery(true),request);
    }

    @ApiOperation(value = "新增角色", notes = "新增角色")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "role", value = "角色参数实体", dataType = "ParaSysRole")
    })
    @RequestMapping(value = "/addRole", method = RequestMethod.POST)
    public RespWrapper<Object> addRole(@RequestBody ParaSysRole addSysRole) {
    	return roleService.addRole(addSysRole, request);
    }

    @ApiOperation(value = "修改角色", notes = "修改角色")
    /*@ApiImplicitParams({
        @ApiImplicitParam(name = "role", value = "角色参数实体", required = true, dataType = "SysRole"),
        @ApiImplicitParam(name = "userIds", value = "角色下的用户id集合,JSONArray数组", required = false, dataType = "String",paramType="query")
    })*/
    @RequestMapping(value = "/updateRole", method = RequestMethod.POST)
    public RespWrapper<Object> updateRole(@RequestBody ParaSysRole paraSysRole) {
    	return  roleService.updateRole( paraSysRole, request);
    }

    @ApiOperation(value = "启用/停用角色", notes = "启用/停用角色")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id,多个时，逗号分割", dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "status", value = "状态：启用--1；停用--2", dataType = "Integer",paramType="query")
    })
    @RequestMapping(value = "/enableRole", method = RequestMethod.POST)
    public RespWrapper<Object> enableRole(String roleId,Integer status) {
    	return roleService.enableRole(roleId,status, request);
    }

    @ApiOperation(value = "获取角色详情", notes = "获取角色详情")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id", required = true, dataType = "long",paramType="query")
    })
    @RequestMapping(value = "/roleDetail", method = RequestMethod.GET)
    public RespWrapper<Object> roleDetail(Long roleId) {
    	return roleService.roleDetail(roleId, request);
    }

    @ApiOperation(value = "获取角色页面权限详情", notes = "获取角色页面权限详情")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id", required = true, dataType = "long",paramType="query"),
        @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "long",paramType="query")
    })
    @RequestMapping(value = "/rolePageDetail", method = RequestMethod.GET)
    public RespWrapper<Object> rolePageDetail(long roleId,Long userId) {
    	return roleService.rolePageDetail(roleId, userId, request);
    }

    @ApiOperation(value = "角色页面权限配置", notes = "角色页面权限配置")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id", required = true, dataType = "long",paramType="query"),
        @ApiImplicitParam(name = "menuIds", value = "菜单功能id集合", required = true, dataType = "String",paramType="query")
    })
    @RequestMapping(value = "/pageRoleConfig", method = RequestMethod.POST)
    public RespWrapper<Object> pageRoleConfig(long roleId,String menuIds) {
    	return roleService.pageRoleConfig(roleId,menuIds, request);
    }

    @ApiOperation(value = "根据用户id和角色id查询数据权限详情", notes = "根据用户id和角色id查询数据权限详情")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id", required = false, dataType = "Long",paramType="query"),
        @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "Long",paramType="query")
    })
    @RequestMapping(value = "/roleDataDetail", method = RequestMethod.GET)
    public RespWrapper<Object> roleDataDetail(Long roleId,Long userId) {
    	return roleService.roleDataDetail(roleId, userId, request);
    }

    @ApiOperation(value = "角色数据权限配置", notes = "角色数据权限配置")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleId", value = "角色id", required = false, dataType = "Long",paramType="query"),
        @ApiImplicitParam(name = "userId", value = "角色下的用户id", required = false, dataType = "Long",paramType="query"),
        @ApiImplicitParam(name = "orgIds", value = "组织架构id集合,多个id以竖线 | 分割，如：1|2", required = false, dataType = "String",paramType="query")
    })
    @RequestMapping(value = "/dataRoleConfig", method = RequestMethod.POST)
    public RespWrapper<Object> dataRoleConfig(Long roleId,Long userId,String orgIds) {
    	return roleService.dataRoleConfig(roleId,userId,orgIds, request);
    }

    @ApiOperation(value = "根据角色名称、角色id获取角色配置的市场人员列表", notes = "根据角色名称、角色id获取角色配置的人员列表，如商户注册的市场人员配置")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "roleName", value = "角色名称", required = false, dataType = "String",paramType="query"),
        @ApiImplicitParam(name = "roleId", value = "角色id", required = false, dataType = "Long",paramType="query")
    })
    @RequestMapping(value = "/getUserListByParam", method = RequestMethod.GET)
    public RespWrapper<Object> getUserListByParam(@RequestParam(value = "roleName", required = false) String roleName,
        @RequestParam(value = "roleId", required = false) String roleId) {
    	return roleService.getUserListByParam(roleName,roleId, request);
    }

    @ApiOperation(value = "根据用户id获取其配置的角色权限信息", notes = "根据用户id获取其配置的角色权限信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "userId", value = "用户id", required = true, dataType = "Long",paramType="query")
    })
    @RequestMapping(value = "/getUserRoleAuth", method = RequestMethod.GET)
    public RespWrapper<Object> getUserRoleAuth(Long userId) {
    	return roleService.getUserRoleAuth(userId, request);
    }
}
