/******************************************************************************
 * Copyright (C) 2018 comtika Co.,Ltd
 * All Rights Reserved.
 * 本软件为康体佳智能科技（深圳）有限公司开发研制。未经本公司正式书面同意，其他任何个人、团体不得使用、
 * 复制、修改或发布本软件.
 *****************************************************************************/
package com.comtika.safe.controller.platform;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.SysUser;
import com.comtika.safe.service.sys.user.UserService;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;




/**
 * @ClassName: UserController
 */
@RestController
@RequestMapping("/user")
public class UserController extends BaseController {
    //private static final Logger log = LoggerFactory.getLogger(UserController.class);

    @Autowired
    private UserService userService;

    

    @ApiOperation(value = "登录", notes = "登录")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "loginName", value = "登录名", required = true, dataType = "String", paramType = "query"),
    	@ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String", paramType = "query")
    })
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public RespWrapper<Object> login(String loginName,String password) {
    	return userService.login(loginName, password,request,super.getIpAddr());
    }

    @ApiOperation(value = "注销", notes = "注销")
    @RequestMapping(value = "/logout", method = RequestMethod.DELETE)
    public RespWrapper<Object> logout() {
    	return userService.logout(request);
    }

    @ApiOperation(value = "新增用户", notes = "新增用户")
    @ApiImplicitParam(name = "user", value = "参数实体", required = true, dataType = "SysUser")
    @RequestMapping(value = "/addUser", method = RequestMethod.POST)
    public RespWrapper<Object> addUser(@RequestBody SysUser user) {
        return userService.insertUser(user,super.findUserCache().getUserId(), request);
    }

    @ApiOperation(value = "后台管理中心-修改用户", notes = "后台管理中心-修改用户")
    @ApiImplicitParam(name = "user", value = "参数实体", required = true, dataType = "SysUser")
    @RequestMapping(value = "/updateUser", method = RequestMethod.POST)
    public RespWrapper<Object> updateUser(@RequestBody SysUser user) {
    	return userService.updateUser(user, request);
    }

    @ApiOperation(value = "根据参数实体修改用户信息", notes = "根据参数实体修改用户信息，实体id必须要有值，否则修改无效")
    @ApiImplicitParam(name = "user", value = "参数实体", required = true, dataType = "SysUser")
    @RequestMapping(value = "/updateUserByEntity", method = RequestMethod.POST)
    public RespWrapper<Object> updateUserByEntity(@RequestBody SysUser user) {
    	return userService.updateUserByEntity(user, request);
    }

    @ApiOperation(value = "启用/停用", notes = "启用/停用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id,批量时，逗号分割", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "status", value = "状态：启用--1；停用--2", required = false, dataType = "Integer")
    })
    @RequestMapping(value = "/enableUser", method = RequestMethod.POST)
    public RespWrapper<Object> enableUser(Long userId, Integer status) {
    	return userService.enableUser(userId,status, request);
    }

    @ApiOperation(value = "详情", notes = "获取用户详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", required = true, dataType = "Long")
    })
    @RequestMapping(value = "/userDetail", method = RequestMethod.GET)
    public RespWrapper<Object> userDetail(Long userId) {
    	return userService.getUserDetailById(userId, request);
    }

    @ApiOperation(value = "列表", notes = "分页查用户列表信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "start", value = "页码", required = false, dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "size", value = "每页大小", required = false, dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "sortStr", value = "排序列,默认为id", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "sortType", value = "排序方式  1逆序 2顺序 ", required = false, dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "name", value = "姓名", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "loginName", value = "登录名", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "merchantName", value = "商户名", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "status", value = "状态", required = false, dataType = "Integer", paramType = "query")
    })
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public RespWrapper<Object> getList() {
    	return userService.queryList(super.getPageQuery(true),request);
    }

    @ApiOperation(value = "后台密码重置", notes = "后台密码重置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", required = true, dataType = "Long", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "重置的密码", required = true, dataType = "String", paramType = "query")
    })
    @RequestMapping(value = "/resetPassword", method = RequestMethod.POST)
    public RespWrapper<Object> resetPassword(Long userId, String password) {
    	return userService.resetPassword( userId,  password, request);
    }

    @ApiOperation(value = "修改密码", notes = "修改密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "oldPassword", value = "旧密码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "newPassword", value = "新密码", required = true, dataType = "String", paramType = "query")
    })
    @RequestMapping(value = "/updatePassword", method = RequestMethod.POST)
    public RespWrapper<Object> updatePassword(String oldPassword, String newPassword) {
    	return userService.updatePassword(oldPassword,newPassword,super.findUserCache(), request);
    }

    @ApiOperation(value = "用户手机号码唯一性检查", notes = "注册或新增时的用户（商户管理员）手机号码是否已被占用")
    @ApiImplicitParam(name = "phoneNo", value = "输入的手机号码", required = true, dataType = "String", paramType = "query")
    @RequestMapping(value = "/isUniquePhoneNo", method = RequestMethod.GET)
    public RespWrapper<Object> isUniquePhoneNo(String phoneNo) {
    	return userService.isUniquePhoneNo(request);
    }
    
    @ApiOperation(value = "商户中心-用户获取个人信息", notes = "商户中心-用户获取个人信息")
    @RequestMapping(value = "/queryUserPersonalInformation", method = RequestMethod.GET)
    public RespWrapper<Object> queryUserPersonalInformation() {
    	return userService.queryUserPersonalInformation(request,super.findUserCache());
    }

}
