package com.comtika.safe.controller.protocol;

import com.comtika.safe.common.request.RequestUtil;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.protocol.cache.SafeCache;
import com.comtika.safe.core.protocol.entity.*;
import com.comtika.safe.core.protocol.entity.charge.ChargeBase;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.charge.ChargeStart;
import com.comtika.safe.core.protocol.entity.device.EquipNodeParams;
import com.comtika.safe.core.protocol.entity.node.NodeChargeData;
import com.comtika.safe.core.protocol.entity.node.NodeChargeRecord;
import com.comtika.safe.core.protocol.entity.node.NodeInfo;
import com.comtika.safe.core.protocol.entity.node.NodeVersion;
import com.comtika.safe.core.protocol.entity.ota.OtaUpHelp;
import com.comtika.safe.core.protocol.entity.ota.TerminalOTA;
import com.comtika.safe.core.protocol.entity.safe.BitHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskConfHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskHelp;
import com.comtika.safe.core.protocol.entity.safe.ThrParamHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerQueryHelp;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.eunms.charge.ChargeMode;
import com.comtika.safe.core.protocol.eunms.charge.SystemControllerMode;
import com.comtika.safe.core.protocol.eunms.netty.OperationCode;
import com.comtika.safe.core.protocol.eunms.netty.OrderCodeType;
import com.comtika.safe.entity.report.AlwaysDataHelp;
import com.comtika.safe.entity.report.RepAlwaysData;
import com.comtika.safe.service.protocol.ProtocolService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("protocol")
@Api(description = "协议模块")
public class ProtocolController {

	@Autowired
	private ProtocolService protocolService;

	// 判断节点是否是主节点
	@ApiOperation(value = "判断节点是否是主节点 code = 0 为主节点 code=-1 提示msg code=-10 从节点")
	@GetMapping("findDeviceWhiteStatus/{terminalNo}")
	public RespWrapper<Boolean> findDeviceWhiteStatus(@PathVariable("terminalNo")String terminalNo){
		return protocolService.findDeviceWhiteStatus(terminalNo);
	}
	@ApiOperation(value = "查询设备配置的白名单列表")
	@GetMapping("findDeviceWhiteListParams/{terminalNo}")
	public RespWrapper<List<String>> findDeviceWhiteListParams(@PathVariable("terminalNo")String terminalNo){
		return protocolService.findDeviceWhiteListParams(terminalNo);
	}
	@ApiOperation(value = "查询设备白名单组网情况")
	@GetMapping("findDeviceWhiteListOnlineParams/{terminalNo}")
	public RespWrapper<List<String>> findDeviceWhiteListOnlineParams(@PathVariable("terminalNo")String terminalNo){
		return protocolService.findDeviceWhiteListOnlineParams(terminalNo);
	}
	@ApiOperation(value = "配置设备的白名单;入参 设备唯一id")
	@GetMapping("setDeviceWhiteListParams/{id}")
	public RespWrapper<Boolean> setDeviceWhiteListParams(@PathVariable("id")String id){
		return protocolService.setDeviceWhiteListParams(id);
	}


	@ApiOperation(value = "根据设备唯一id 查询iccid imei")
	@GetMapping("fadFindTerminalIccId/{id}")
	public RespWrapper<Map<String,Object>> fadFindTerminalIccId(@PathVariable("id") Integer id){
		return protocolService.fadFindTerminalIccId(id);
	}
	@ApiOperation(value = "查询设备的iccId")
	@PostMapping("fadUpdateTerminalIccId/{id}")
	public RespWrapper<String> fadUpdateTerminalIccId(@PathVariable("id")Integer id){
		return protocolService.fadUpdateTerminalIccId(id);
	}
	@ApiOperation(value = "查询设备的imei")
	@PostMapping("fadUpdateTerminalImei/{id}")
	public RespWrapper<String> fadUpdateTerminalImei(@PathVariable("id")Integer id){
		return protocolService.fadUpdateTerminalImei(id);
	}

	@ApiOperation(value = "查询电能表指令")
    @PostMapping("fadEleTotalStatus/{terminalNo}")
    public RespWrapper<WattMeterHelp> fadEleTotalStatus(@PathVariable("terminalNo") String terminalNo){
	    if(StringUtil.isEmpty(terminalNo))
	        return RespWrapper.makeFailResp("集中器编号不能为空");
	    return protocolService.fadEleTotalStatus(terminalNo);
    }


	@ApiOperation(value = "辅助继电器操作0: 关闭辅助继电器\n" +
			"     * 1：打开辅助继电器\n" +
			"     * 2：查看辅助继电器状态" +
			"bitList 数组true代表开，false代表关")
	@PostMapping("fadAuxHandle")
	public RespWrapper<Object> fadAuxHandle(@RequestBody AuxHandleHelp auxHandleHelp){
		RespWrapper check = auxHandleHelp.check();
		if(!check.isSuccess())
			return check;
		return protocolService.fadAuxHandle(auxHandleHelp);
	}



	@ApiOperation(value = "发送协议到设备操作")
	@PostMapping("fadSendProtocolToTerminal")
	public RespWrapper<Object> fadSendProtocolToTerminal(@RequestBody ProtocolHelp help){
		return protocolService.fadSendProtocolToTerminal(help);
	}

	@ApiOperation(value = "设备复位，重启F2H;post 方式 form 表单提交；参数：deviceCode = 设备编码")
	@PostMapping("fad/deviceRestart")
	public RespWrapper<String> fadDeviceRestart(HttpServletRequest request){
		String deviceCode = RequestUtil.getStringParam(request,"deviceCode");
		if(StringUtil.isEmpty(deviceCode))
			return RespWrapper.makeFailResp("设备编码不能为空");
		return protocolService.fadDeviceRestart(deviceCode);
	}
	@ApiOperation("查询设备mac")
	@GetMapping("fad/findDeviceMac")
	public RespWrapper<String> fadDeviceMac(String deviceCode){
		if(StringUtil.isEmpty(deviceCode))
			return RespWrapper.makeFailResp("设备编码不能为空");
		return protocolService.fadDeviceMac(deviceCode);
	}
	@ApiOperation(value = "设置mac地址；说明:post form 表单提交；参数：1、deviceCode：设备编码 2、macId：设置的mac地址")
	@PostMapping("fad/setDeviceMac")
	public RespWrapper<Boolean> fadSetDeviceMac(HttpServletRequest request){
        String deviceCode = RequestUtil.getStringParam(request, "deviceCode");
        String macId = RequestUtil.getStringParam(request, "macId");
        if(StringUtil.isEmpty(deviceCode))
            return RespWrapper.makeFailResp("设备编码不能为空");
        if(StringUtil.isEmpty(macId))
            return RespWrapper.makeFailResp("设置mac不能为空");
        return protocolService.fadSetDeviceMac(deviceCode,macId);
    }
    @ApiOperation(value = "查询设备时间")
    @GetMapping("fad/findDeviceTime")
    public RespWrapper<String> fadFindDeviceTime(String deviceCode){
	    if(StringUtil.isEmpty(deviceCode))
	        return RespWrapper.makeFailResp("设备编码不能为空");
	    return protocolService.fadFindDeviceTime(deviceCode);
    }
    @ApiOperation(value = "设置设备时间；如果设置时间为null,则以服务器的时间设置")
    @GetMapping("fad/setDeviceTime")
    public RespWrapper<Boolean> fadSetDeviceTime(String deviceCode,Long time){
	    if(StringUtil.isEmpty(deviceCode))
	        return RespWrapper.makeFailResp("设备编码不能为空");
	    if(time == null) time = System.currentTimeMillis();
	    return protocolService.fadSetDeviceTime(deviceCode,time);
    }
    @ApiOperation("查询可配置的服务器地址")
    @GetMapping("fad/findTerminalAddressList")
    public RespWrapper<List<TerminalAddress>> fadFindTerminalAddressList(){
	    return protocolService.fadFindTerminalAddressList();
    }
    @ApiOperation(value = "查询设置的服务地址")
    @GetMapping("fad/findTerminalAddress")
    public RespWrapper<Map<String,String>> fadFindTerminalAddress(String deviceCode){
	    if(StringUtil.isEmpty(deviceCode))
	        return RespWrapper.makeFailResp("设备编码不能为空");
	    return protocolService.fadFindTerminalAddress(deviceCode);
    }
    @ApiOperation(value = "配置服务器地址;说明:post form提交方法；参数：1、deviceCode:设备编码 2、serviceId:服务配置id")
    @PostMapping("fad/setTerminalAddress")
    public RespWrapper<Boolean> fadSetTerminalAddress(HttpServletRequest request){
	    String deviceCode = RequestUtil.getStringParam(request,"deviceCode");
	    if(StringUtil.isEmpty(deviceCode))
	        return RespWrapper.makeFailResp("设备编码不能为空");
        Integer serviceId = RequestUtil.getIntParam(request, "serviceId",Integer.MAX_VALUE);
        if(serviceId == null || serviceId == Integer.MAX_VALUE) return RespWrapper.makeFailResp("配置服务器id 不能为空");
        return protocolService.fadSetTerminalAddress(deviceCode,serviceId);
    }

	@ApiOperation(value = "心电跟踪方法, 入参为12位的设备编码")
	@GetMapping("findProtocolDataList")
	public RespWrapper<List<String>> findProtocolDataList(String terminalNo){
		if(StringUtil.isEmpty(terminalNo)) return RespWrapper.makeFailResp("设备编码不能为空");
		String key = terminalNo.trim();
		if(key.length() != 12) return RespWrapper.makeFailResp("设备编码不是12为的数组");
		return RespWrapper.makeSuccessResp(SafeCache.get(key));
	}
	@ApiOperation(value = "清除当前心电跟踪方法，入参：12位设备编码")
	@GetMapping("clearProtocolDataKList")
	public RespWrapper<Boolean> clearProtocolDataKList(String terminalNo){
		if(StringUtil.isEmpty(terminalNo)) return RespWrapper.makeFailResp("设备编码不能为空");
		String key = terminalNo.trim();
		if(key.length() != 12) return RespWrapper.makeFailResp("设备编码不是12为的数组");
		SafeCache.removeProtocolDataList(key);
		return RespWrapper.makeSuccessResp(true);
	}
	@ApiOperation(value = "安全用电增加：清除节点电能信息 85H")
	@PostMapping("cleanTerminalEle/{terminalNo}")
	public RespWrapper<Boolean> cleanTerminalEle(@PathVariable("terminalNo")String terminalNo,@RequestBody BitHelp bit){
		if(StringUtil.isEmpty(terminalNo))
			return RespWrapper.makeFailResp("集中器编号不能为空");
		if(bit == null)
			return RespWrapper.makeFailResp("Bit0- Bit15 每一位对应一路继电器。不能为空");
		return protocolService.cleanTerminalEle(terminalNo,bit);
	}
	@ApiOperation(value = "安全用电增加：集中器继电器关（节点开关控制）71H;主机控制集中器继电器断开")
	@PostMapping("safeTerminalOff/{terminalNo}")
	public RespWrapper<Boolean> safeTerminalOff(@PathVariable("terminalNo")String terminalNo,@RequestBody BitHelp bit) {
		if(StringUtil.isEmpty(terminalNo))
			return RespWrapper.makeFailResp("集中器编号不能为空");
		if(bit == null)
			return RespWrapper.makeFailResp("Bit0- Bit15 每一位对应一路继电器。不能为空");
		return protocolService.safeTerminalClose(terminalNo,bit);
	}
	@ApiOperation(value = "安全用电增加：集中器继电器开（节点开关控制）70H")
	@PostMapping("safeTerminalOn/{terminalNo}")
	public RespWrapper<Boolean> safeTerminalOn(@PathVariable("terminalNo")String terminalNo,@RequestBody BitHelp bit) {
		if(StringUtil.isEmpty(terminalNo))
			return RespWrapper.makeFailResp("集中器编号不能为空");
		if(bit == null)
			return RespWrapper.makeFailResp("Bit0- Bit15 每一位对应一路继电器。不能为空");
		return protocolService.safeTerminalOn(terminalNo,bit);
	}
	@ApiOperation(value = "安全用电增加：查询定时任务 83H")
	@GetMapping("findSafeTaskFromTerminal/{terminalNo}")
	public RespWrapper<List<SafeTaskHelp>> findSafeTaskFromTerminal(@PathVariable("terminalNo")String terminalNo){
		if(StringUtil.isEmpty(terminalNo))
			return RespWrapper.makeFailResp("设备编号不能为空");
		return protocolService.findSafeTaskFromTerminal(terminalNo);
	}
	@ApiOperation(value = "安全用电增加：下发定时任务 82H")
	@PostMapping("setSafeTaskConf")
	public RespWrapper<Boolean> setSafeTaskConf(@RequestBody SafeTaskConfHelp safe){
		if(safe == null) return RespWrapper.makeFailResp("配置数据不能为空");
		List<SafeTaskHelp> list = safe.getList();
		if(list.isEmpty()) return RespWrapper.makeFailResp("任务不能为空");
		String terminalNo = safe.getTerminalNo();
		if(StringUtil.isEmpty(terminalNo))return RespWrapper.makeFailResp("集中器编号不能为空");
		return protocolService.setSafeTaskConf(terminalNo,list);
	}
	@ApiOperation(value = "安全用电增加：删除定时任务 84H")
	@DeleteMapping("deleteSafeTask/{terminalNo}")
	public RespWrapper<Boolean> deleteSafeTask(@PathVariable("terminalNo")String terminalNo){
		return protocolService.deleteSafeTask(terminalNo,null);
	}
	@ApiOperation(value = "安全用电增加：查询节点实时数据 15H")
	@PostMapping("findAlwaysData")
	public RespWrapper<RepAlwaysData> findAlwaysData(@RequestBody AlwaysDataHelp help){
		if(StringUtil.isEmpty(help.getTerminalNo()))
			return RespWrapper.makeFailResp("设备编号不能为空");
		return protocolService.findAlwaysData(help);
	}
	@ApiOperation(value = ":安全用电增加：集中器继电器状态查询（节点开关\n" +
			"状态查询）72H")
	@GetMapping("findSafeTerminalStatus")
	public RespWrapper<BitHelp> findSafeTerminalStatus(String terminalNo){
		return protocolService.findSafeTerminalStatus(terminalNo);
	}


	@ApiOperation(value="获取集中器的软硬件信息")
	@GetMapping("/getHardwareInfo/{addrNo}")
	public RespWrapper<HardwareVersion> getHardwarInfo(@PathVariable("addrNo")String addrNo){
		if(StringUtil.isEmpty(addrNo))return RespWrapper.makeFailResp("获取集中器的软硬件信息时，集中器唯一标识不能为空");
		return protocolService.getHardwarInfo(addrNo);
	}
	@ApiOperation(value = "设置集中器 Timer 周期 E4H")
	@PostMapping("setTimerParams")
	public RespWrapper<Boolean> setTimerParams(@RequestBody TimerHelp help){
		if(StringUtil.isEmpty(help.getTerminalNo()))
			return RespWrapper.makeFailResp("设备编号不能为空");
		return protocolService.setTimerParams(help);
	}
	@ApiOperation(value = "查询集中器 Timer 配置 E3H")
	@GetMapping("findTimerParams/{terminalNo}")
	public RespWrapper<List<TimerQueryHelp>> findTimerParams(@PathVariable("terminalNo")String terminalNo){
		return protocolService.findTimerParams(terminalNo);
	}

	@Deprecated
	@ApiOperation(value="配置节点參数，新版本待更新")
	@PostMapping("/setDeviceParams")
	public RespWrapper<Boolean> setDeviceParams(@RequestBody EquipNodeParams dp, int controlPattern){
		OrderCodeType valueOf = OrderCodeType.valueOf(controlPattern);
		if( valueOf == null) return RespWrapper.makeFailResp("请检查控制方式【"+controlPattern+"】是否正确！");
		return protocolService.setDeviceParams(dp,valueOf);
	}
	@Deprecated
	@ApiOperation(value="查询节点配置旧版本，新版本待更新")
	@GetMapping("/getNodeConfParam/{terminalNo}/{nodeNo}/{plugNo}")
	public RespWrapper<EquipNodeParams> getNodeConfParam(@PathVariable("terminalNo")String terminalNo, @PathVariable("nodeNo")String nodeNo, @PathVariable("plugNo") int plugNo){
		if(StringUtil.isEmpty(nodeNo)) return RespWrapper.makeFailResp("查询节点配置时，节点编号不能为空");
		if(StringUtil.isEmpty(terminalNo)) return RespWrapper.makeFailResp("查询节点配置，集中器编号不能为空");
		return protocolService.getNodeConfParam(terminalNo,nodeNo, plugNo);
	}

	@ApiOperation("系统控制 F2H；参数：addrNo=集中器编号，mode类型（1：集中器重启动；2：集中器重启进入升级模式，3：读取集中器软件版本号）")
	@GetMapping("/systemController/{addrNo}/{mode}")
	public RespWrapper<String> systemController(@PathVariable("addrNo")String addrNo, @PathVariable("mode")int mode){
		SystemControllerMode valueOf = SystemControllerMode.valueOf(mode);
		if(valueOf == null)return RespWrapper.makeFailResp("请检查系统控制方式【"+mode+"】是否正确！");
		return protocolService.systemController(addrNo,valueOf);
	}
	@ApiOperation(value="查询节点状态；nodeNo节点编号，plugNo插座编号")
	@GetMapping("/getNodeFaultInfo/{nodeNo}/{plugNo}")
	public RespWrapper<NodeInfo> getNodeFaultInfo(@PathVariable("nodeNo") String nodeNo, @PathVariable("plugNo") int plugNo){
		if(StringUtil.isEmpty(nodeNo))return RespWrapper.makeFailResp("查询节点状态时，节点编号不能为空");
		return protocolService.getNodeFalutInfo(nodeNo, plugNo);
	}
	@ApiOperation(value="安全用电增加：查询节点阈值 14H")
	@PostMapping("findPlugNoParams")
	public RespWrapper<DeviceParamsHelp> findPlugNoParams(@RequestBody ThrParamHelp help){
		return protocolService.findPlugNoParams(help);
	}
	@ApiOperation(value = "安全用电增加：配置节点阈值 02H")
	@PostMapping("setTerminalParams")
	public RespWrapper<Boolean> setTerminalParams(@RequestBody ThrParamHelp help){
		return protocolService.setTerminalParams(help);
	}


	@ApiOperation(value="获取节点版本号")
	@GetMapping("/getNodeVersion/{nodeNo}")
	public RespWrapper<NodeVersion> getNodeVersion(@PathVariable("nodeNo") String nodeNo){
		if(StringUtil.isEmpty(nodeNo))return RespWrapper.makeFailResp("获取节点版本号时，节点编号不能为空");
		return protocolService.getNodeVersion(nodeNo);
	}
	@ApiOperation(value = "安全用电OTA升级接口")
	@PostMapping("safeOTAUpdate")
	public RespWrapper<String> fadSafeOTAUpdate(@RequestBody OtaUpHelp help){
		return protocolService.fadSafeOTAUpdate(help);
	}


	@ApiOperation(value = "设置集中器协议 集中器ip与端口")
	@Deprecated
	@GetMapping("/setupIpPort/{addrNo}/{ip}/{port}/{operationCode}")
	public RespWrapper<Boolean> setupIpPort(@PathVariable("addrNo") String addrNo, @PathVariable("ip") String ip, @PathVariable("port") String port, @PathVariable("operationCode") int operationCode){
		if(StringUtil.isEmpty(ip) || StringUtil.isEmpty(port) || StringUtil.isEmpty(addrNo)){
			return RespWrapper.makeFailResp("入参有误！请检查");
		}
		try {
			OperationCode valueOf = OperationCode.valueOf(operationCode);
			if(valueOf == null)return RespWrapper.makeFailResp("操作指令不在范围内！");
		} catch (Exception e) {
			return RespWrapper.makeFailResp("系统忙！");
		}
		return protocolService.setupIpPort(addrNo,ip,port,operationCode);
	}

	@ApiOperation(value="开始充电")
	@PostMapping("/startCharge")
	@Deprecated
	public RespWrapper<ChargeBase> startCharge(@RequestBody ChargeStart chargeStart){
		if(ChargeMode.valueOf(chargeStart.getMode()) == null){
			return RespWrapper.makeFailResp("请检查充电模式【"+chargeStart.getMode()+"】是否在固定范围！");
		}
		return protocolService.startCharge(chargeStart);
	}

	// 结束充电
	@Deprecated
	@ApiOperation(value="结束充电")
	@GetMapping("/endCharge/{nodeNo}/{orderNo}/{plugNo}")
	public RespWrapper<ChargeEnd> endCharge(@PathVariable("nodeNo") String nodeNo, @PathVariable("orderNo") String orderNo, @PathVariable("plugNo") int plugNo){
		if(StringUtil.isEmpty(nodeNo) || StringUtil.isEmpty(orderNo))return RespWrapper.makeFailResp("入参有误！请检查");
		return protocolService.endCharge(nodeNo,orderNo,plugNo,true);
	}

	@Deprecated
	@ApiOperation(value="获取集中器主/备ip与port { addrNo 集中器id,operationCode  操作指令}")
	@GetMapping("/getTerminalIpPort/{addrNo}/{operationCode}")
	public RespWrapper<Map<String, String>> getTerminalIpPort(@PathVariable("addrNo")String addrNo, @PathVariable("operationCode")int operationCode){
		if(StringUtil.isEmpty(addrNo))return RespWrapper.makeFailResp("查询集中器协议（端口与ip）时，集中器唯一标识不能为空");
		try {
			OperationCode valueOf = OperationCode.valueOf(operationCode);
			if(valueOf == null)return RespWrapper.makeFailResp("操作指令不在范围内！");
		} catch (Exception e) {
			return RespWrapper.makeFailResp("系统忙！");
		}
		return protocolService.getTerminalIpPort(addrNo,operationCode);
	}

	@ApiOperation(value="设置工作模式")
	@Deprecated //该方法在16路电单车移除
	@GetMapping("/setNodeWorkMode/{nodeNo}/{workMode}/{controlCode}")
	public RespWrapper<Boolean> setNodeWorkMode(@PathVariable("nodeNo") String nodeNo, @PathVariable("workMode") int workMode, @PathVariable("controlCode")int controlCode){
		if(StringUtil.isEmpty(nodeNo)){
			return RespWrapper.makeFailResp("请节点id是否为空【"+nodeNo+"】是否正确！");
		}
		if(OrderCodeType.valueOf(controlCode) == null){
			return RespWrapper.makeFailResp("请检查控制方式【"+controlCode+"】是否正确！");
		}
		return protocolService.setNodeWorkMode(nodeNo,workMode,controlCode);
	}

	@Deprecated
	@ApiOperation(value="设置RTC 对于服务器来说该指令无用")
	@GetMapping("/setTerminalRTC/{nodeNo}/{time}/{controlCode}")
	public RespWrapper<Boolean> setTerminalRTC(@PathVariable("nodeNo") String nodeNo, @PathVariable("time")Long time, @PathVariable("controlCode")int controlCode){
		OrderCodeType valueOf = OrderCodeType.valueOf(controlCode);
		if(valueOf == null)return RespWrapper.makeFailResp("请检查控制方式【"+controlCode+"】是否正确");
		if(StringUtil.isEmpty(nodeNo))return RespWrapper.makeFailResp("设置RTC时，节点编号不能为空");
		return protocolService.setTerminalRTC(nodeNo,time,valueOf);
	}


	@ApiOperation(value="获取节点充电数据时")
	@Deprecated
	@GetMapping("/getNodeChargeInfo/{nodeNo}/{plugNo}")
	public RespWrapper<NodeChargeData> getNodeChargeInfo(@PathVariable("nodeNo") String nodeNo, @PathVariable("plugNo") int plugNo){
		if(StringUtil.isEmpty(nodeNo))return RespWrapper.makeFailResp("获取节点充电数据时，节点编号不能为空");
		return protocolService.getNodeChargerInfo(nodeNo, plugNo);
	}




	@ApiOperation(value="获取节点充电记录")
	@Deprecated
	@GetMapping("/getNodeChargeRecord/{nodeNo}/{resNo}")
	public RespWrapper<NodeChargeRecord> getNodeChargeRecord(@PathVariable("nodeNo") String nodeNo, @PathVariable("resNo") int resNo){
		if(StringUtil.isEmpty(nodeNo))return RespWrapper.makeFailResp("获取节点充电记录时，节点编号不能为空");
		return protocolService.getNodeChargeRecord(nodeNo,resNo);
	}



	/**
	 * 设备OTA升级
	 * @param terminalOTA
	 * @return
	 */
	@ApiOperation(value = "OTA 升级，这个旧版本")
	@Deprecated
	@PostMapping("/fadTerminalUpdateByOTA")
	public RespWrapper<String> fadTerminalUpdateByOTA(@RequestBody TerminalOTA terminalOTA){
		if(terminalOTA.getTerminalList() == null) return RespWrapper.makeFailResp("ota升级时，集中器列表不能为空");
		return protocolService.fadTerminalUpdateByOTA(terminalOTA);
	}

	/**
	 * 智能卡
	 * @param terminalCard
	 * @return
	 */
	@ApiOperation(value = "查询智能卡")
	@Deprecated
	@PostMapping("/fadTerminalCard")
	public RespWrapper<Boolean> fadTerminalCard(@RequestBody TerminalCard terminalCard){
		if(StringUtil.isEmpty(terminalCard.getUniqueNo())) return RespWrapper.makeFailResp("智能卡响应节时，节点编号不能为空");
		if(terminalCard.getAnswerCard() == null) return RespWrapper.makeFailResp("智能卡响应时，响应码不能为空");
		return protocolService.fadTerminalCard(terminalCard);
	}


	// 智能卡的响应
	/*@GetMapping("/answerEquipCard")
	public RespWrapper<Boolean> answerEquipCard(String nodeNo,Integer answerCode,Integer plugNo,Integer balance){
		if(StringUtil.isEmpty(nodeNo)){
			return RespWrapper.makeFailResp("响应智能卡时，节点编号不能为空");
		}
		if(plugNo == null){
			return RespWrapper.makeFailResp("响应智能卡时，插座编号不能为空");
		}
		if(answerCode == null){
			return RespWrapper.makeFailResp("响应智能卡充电，响应码不能为空");
		}
		if(AnswerCard.valueOf(answerCode) == null){
			return RespWrapper.makeFailResp("响应智能卡充电，响应码不存在");
		}
		return protocolService.answerEquipCard(nodeNo,answerCode,plugNo,balance);
	}*/
}
