package com.comtika.safe.controller.site;



import com.comtika.safe.entity.site.help.SiteExcelHelp;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.site.CtkSite;
import com.comtika.safe.service.site.SiteService;

@RestController
@RequestMapping("/site")
public class SiteController extends BaseController{

	@Autowired
	private SiteService siteService;

	@ApiOperation(value = "站点列表", notes = "站点列表")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "merchantId", 	value = "商户id",		required = false, dataType = "Long",	paramType="query"),
        @ApiImplicitParam(name = "siteName", 	value = "站点名称", 		required = false, dataType = "String",	paramType="query"),
        @ApiImplicitParam(name = "siteType", 	value = "站点类型",		required = false, dataType = "Integer",	paramType="query"),
        @ApiImplicitParam(name = "siteStatus", 	value = "站点运营状态", 	required = false, dataType = "Integer",	paramType="query"),
        @ApiImplicitParam(name = "area", 		value = "地区 ", 		required = false, dataType = "String",	paramType="query"),
        @ApiImplicitParam(name = "createTime", 	value = "创建时间", 		required = false, dataType = "Long",	paramType="query")
    })
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public RespWrapper<Object> list() {
        return siteService.list(request,super.getPageQuery(true));
    }

	@ApiOperation(value = "新增站点", notes = "新增站点")
    @ApiImplicitParam(name = "ctkSite", value = "参数实体", dataType = "CtkSite")
    @RequestMapping(value = "/addSite", method = RequestMethod.POST)
    public RespWrapper<Object> addSite(@RequestBody CtkSite ctkSite) {
        return siteService.addSite(ctkSite,super.findUserCache(), request);
    }

    @ApiOperation(value ="新站点列表查询接口")
    @PostMapping("fadFindSiteList")
    public RespWrapper fadFindSiteList(@RequestBody SiteExcelHelp help){
	    return siteService.fadFindSiteList(help,findUserCache());
    }
    @ApiOperation(value = "站点列表导出")
    @PostMapping("siteListExcel")
    public RespWrapper siteListExcel(SiteExcelHelp help){
	    return siteService.siteListExcel(help,findUserCache(),response);
    }




	@ApiOperation(value = "修改站点", notes = "修改站点")
    @ApiImplicitParam(name = "ctkSite", value = "参数实体", dataType = "CtkSite")
    @RequestMapping(value = "/updateSite", method = RequestMethod.POST)
    public RespWrapper<Object> updateSite(@RequestBody CtkSite ctkSite) {
        return siteService.updateSite(ctkSite,super.findUserCache(), request);
    }

	@ApiOperation(value = "站点详情", notes = "站点详情")
    @ApiImplicitParam(name = "ctkSite", value = "参数实体", dataType = "CtkSite")
    @RequestMapping(value = "/querySiteDetail", method = RequestMethod.POST)
    public RespWrapper<Object> querySiteDetail(Long siteId) {
        return siteService.querySiteDetail(siteId, request);
    }


	@ApiOperation(value = "站点列表,查询下拉框", notes = "站点列表,查询下拉框")
    @RequestMapping(value = "/querySiteCollectionList", method = RequestMethod.GET)
    public RespWrapper<Object> querySiteCollectionList() {
        return siteService.querySiteCollectionList(super.findUserCache(), request);
    }
}
