package com.comtika.safe.controller.task;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.entity.task.help.*;
import com.comtika.safe.service.task.ITaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController
@RequestMapping("task")
@Api(description = "任务API")
public class TaskController extends BaseController {

    @Autowired
    private ITaskService iTaskService;

    @ApiOperation(value = "查询设备列表")
    @PostMapping("findTaskPageList")
    public RespWrapper<PageQuery<Map<String,Object>>> findTaskPageList(@RequestBody TaskQueryHelp help){
        return iTaskService.findTaskPageList(help,findUserCache());
    }
    @ApiOperation(value = "定时任务导出")
    @PostMapping("excelTaskList")
    public RespWrapper<Boolean> excelTaskList(TaskQueryHelp task){
        return iTaskService.excelTaskList(task,findUserCache(),response);
    }

    @ApiOperation(value = "添加定时任务")
    @PostMapping("addTaskHandle")
    public RespWrapper<Boolean> addTaskHandle(@RequestBody TaskHelp help){
        RespWrapper<Boolean> check = help.check();
        if(!check.isSuccess())return check;
        return iTaskService.addTaskHandle(help);
    }
    @ApiOperation(value = "修改定时任务")
    @PostMapping("updateTaskHandle")
    public RespWrapper<Boolean> updateTaskHandle(@RequestBody TaskHelp help){
        RespWrapper<Boolean> check = help.check();
        if(!check.isSuccess())return check;
        return iTaskService.updateTaskHandle(help);
    }
    @ApiOperation(value = "删除定时任务")
    @PostMapping("deleteTask/{id}")
    public RespWrapper<Boolean> deleteTask(@PathVariable("id")Integer id){
        return iTaskService.deleteTask(id);
    }
    @ApiOperation(value = "查询定时任务详情")
    @GetMapping("findTaskInfo/{id}")
    public RespWrapper<TaskHelp> findTaskInfo(@PathVariable("id")Integer id){
        return iTaskService.findTaskInfo(id);
    }
    @ApiOperation(value = "下发定时任务;参数为定时任务唯一编号id")
    @PostMapping("takeTask/{id}")
    public RespWrapper<Boolean> takeTask(@PathVariable("id")Integer id){
        return iTaskService.takeTask(id);
    }


    @ApiOperation(value = "定时任务模版查询")
    @PostMapping("findTaskTemplateList")
    public RespWrapper findTaskTemplateList(@RequestBody TaskTemplateQueryHelp help){
        return iTaskService.findTaskTemplateList(help,findUserCache());
    }
    @ApiOperation(value = "添加定时任务模版接口")
    @PostMapping("addTaskTemplate")
    public RespWrapper addTaskTemplate(@RequestBody TaskTemplateHelp help){
        RespWrapper<Boolean> check = help.check();
        if(!check.isSuccess()) return check;
        return iTaskService.addTaskTemplate(help,findUserCache());
    }
    @ApiOperation(value = "修改定时任务模版接口")
    @PostMapping("updateTaskTemplate")
    public RespWrapper updateTaskTemplate(@RequestBody TaskTemplateHelp help){
        RespWrapper<Boolean> check = help.check();
        if(!check.isSuccess()) return check;
        if(help.getId() == null) return RespWrapper.makeFailResp("模版id 不能为空");
        return iTaskService.updateTaskTemplate(help,findUserCache());
    }
    @ApiOperation(value = "删除定时任务模版接口")
    @DeleteMapping("deleteTaskTemplate/{id}")
    public RespWrapper deleteTaskTemplate(@PathVariable("id")Integer id){
        return iTaskService.deleteTaskTemplate(id,findUserCache());
    }
    @Deprecated
    @ApiOperation(value = "查询定时任务模版，配置模版前调用查询")
    @GetMapping("findTaskTemplate/{terminalNo}")
    public RespWrapper findTaskTemplate(@PathVariable("terminalNo")String terminalNo){
        return iTaskService.findTaskTemplate(terminalNo,findUserCache());
    }
    @ApiOperation(value = "根据模板名称查询模板，如果模板名称为空，查询当前用户的所有任务模板")
    @GetMapping("findTaskTemplateByName")
    public RespWrapper findTaskTemplateByName(String name){
        return iTaskService.findTaskTemplateByName(name,findUserCache());
    }

    @ApiOperation(value = "根据定时任务模版配置定时任务")
    @PostMapping("configTaskTemplate")
    public RespWrapper configTaskTemplate(@RequestBody AddTaskByTemplateHelp help){
        //   private List<Integer> bitList = new ArrayList<>();//回路数 [1,2,3]
        return iTaskService.configTaskTemplate(help,findUserCache());
    }
    @ApiOperation(value = "判断用户定时模版是否超标；code=0，可以添加；code=2超标，不能添加；code=-1 提示msg")
    @GetMapping("checkUserTemplateTask")
    public RespWrapper<Boolean> checkUserTemplateTask(){
        return iTaskService.checkUserTemplateTask(findUserCache());
    }



}
