package com.comtika.safe.controller.warn;

import com.comtika.safe.common.action.BaseController;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.entity.lucene.LuceneReturnEntity;
import com.comtika.safe.entity.report.AlwayQueryHelp;
import com.comtika.safe.entity.report.NewestQueryHelp;
import com.comtika.safe.entity.report.WarnQueryHelp;
import com.comtika.safe.entity.sys.SysExcelField;
import com.comtika.safe.entity.warn.help.WarnHelp;
import com.comtika.safe.service.warn.IWarnService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 16:03 2020/8/24
 * @DESC:
 **/
@RestController
@Api(description = "安全用电一些接口：告警")
@RequestMapping("warn")
public class WarnController extends BaseController{
    @Autowired
    private IWarnService iWarntService;
    // 这里查询数据
    @ApiOperation(value = "查询设备实时上报数据")
    @PostMapping("findReportDataList")
    public RespWrapper<LuceneReturnEntity> findReportDataList(@RequestBody AlwayQueryHelp help){
        if(StringUtil.isEmpty(help.getTerminalNo()))
            return RespWrapper.makeFailResp("设备编号不能为空");
        return iWarntService.findReportDataList(help);
    }
    @ApiOperation(value = "导出设备实时上报数据")
    @PostMapping("excelReportDataList")
    public RespWrapper<Boolean> excelReportDataList(AlwayQueryHelp help){
        return iWarntService.excelReportDataList(help,response);
    }

    @ApiOperation(value = "数据统计")
    @PostMapping("findNewestDataList")
    public RespWrapper<PageQuery<Map<String,Object>>> findNewestDataList(@RequestBody NewestQueryHelp help){
        return iWarntService.findNewestDataList(help,findUserCache());
    }

    @ApiOperation(value = "告警列表查询")
    @PostMapping("findWarnPageList")
    public RespWrapper<PageQuery<Map<String,Object>>> findWarnPageList(@RequestBody WarnQueryHelp help){
        return iWarntService.findWarnPageList(help,findUserCache());
    }
    @ApiOperation(value = "处理告警标记位")
    @PostMapping("warnStatusHandle/{warnId}/{status}")
    public RespWrapper<Boolean> warnStatusHandle(@PathVariable("warnId")Integer warnId,@PathVariable("status")Integer status){
        return iWarntService.warnStatusHandle(findUserCache(),warnId,status);
    }

    @ApiOperation(value = "查询站点告警列表排名")
    @GetMapping("findSiteWarnRanking")
    public RespWrapper<Collection<WarnHelp>> findSiteWarnRanking(){
        return iWarntService.findSiteWarnRanking(findUserCache());
    }
    @ApiOperation(value = "获取月份告警列表")
    @GetMapping("findMonthWarnRanking")
    public RespWrapper<Map<String,Object>> findMonthWarnRanking(){
        return iWarntService.findMonthWarnRanking(findUserCache());
    }

    @ApiOperation(value = "获取告警数首页统计")
    @GetMapping("findWarnHomeTotal")
    public RespWrapper<Map<String,Object>> findWarnHomeTotal(){
        return iWarntService.findWarnHomeTotal(findUserCache());
    }

    @ApiOperation("获取对应的导出字段")
    @GetMapping("findExcelField/{id}")
    public RespWrapper<List<SysExcelField>> findExcelField(@PathVariable("id")Integer id){
        return iWarntService.findExcelField(id);
    }
    @ApiOperation(value = "告警导出")
    @PostMapping("warnExcelHandle")
    public RespWrapper<Boolean> warnExcelHandle(WarnQueryHelp help){
        List<Integer> excelList = help.getExcelList();
        if(excelList.isEmpty()) return RespWrapper.makeFailResp("导出字段不能为空");
        return iWarntService.warnExcelHandle(help,findUserCache(),response);
    }

    @ApiOperation(value = "实时数据导出")
    @PostMapping("newestDataExcel")
    public RespWrapper<Boolean> newestDataExcel(NewestQueryHelp help){
        return iWarntService.newestDataExcel(help,findUserCache(),response);
    }
    @GetMapping("findWarnInfo/{id}")
    public RespWrapper findWarnInfo(@PathVariable("id")Integer id){
        return iWarntService.findWarnInfo(id);
    }

    // /warn/findWarnInfo/{id}
}
