package com.comtika.safe.core;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import lombok.extern.log4j.Log4j2;

/**
 * 版本校验规则
 */
@Log4j2
public class VersionUtil {
    public static void main(String[] args){
        System.out.println(checkVersion("ESC-04-SP Ⅰ"));
    }
    /**
     * ESC-04-SP I -Host-20060602.bin
     */
    public static boolean checkVersion(String fileName){
        if(StringUtil.isEmpty(fileName))return false;
        String[] s = fileName.split(" ");
        int size = s.length;
        // 如果size = 2 则 说明不是升级包，是产品型号定义
        if(size == 2){
            return checkVersionRome(s[1]) && checkDeviceVersion(s[0]);
        }else if(size == 3){
            return checkVersionRome(s[1]) && checkDeviceVersion(s[0]) && checkOTASuffix(s[2]);
        }
        return false;
    }
    private static final String[] VERSION_TYPE = new String[]{
         "Host","Unit","PLCM","PLCS"
    };
    /**
     * 校验这个数据-Host-20060602.bin
     * @param version
     * @return
     */
    private static boolean checkOTASuffix(String version) {
        if(StringUtil.isEmpty(version))return false;
        if(version.startsWith("-")){
            version = version.substring(1);
        }
        String[] split = version.split("-");
        if(split.length == 2){
            return checkOTAType(split[0]) && checkFileName(split[1]);
        }
        return false;
    }
    // 20060602.bin
    private static boolean checkFileName(String s) {
        if(s.endsWith(".bin")){
            try {
                String substring = s.substring(0, s.indexOf('.'));
                if(substring.length() == 8){
                    Integer.valueOf(substring);
                    return true;
                }
            }catch (Exception e){
            }
        }
        return false;
    }
    // Host
    private static boolean checkOTAType(String type){
        for(String tp:VERSION_TYPE){
            if(tp.equals(type))return true;
        }
        return false;
    }

    private static boolean checkDeviceVersion(String version) {
        if(StringUtil.isEmpty(version))return false;
        try {
            String[] split = version.split("-");
            if(split.length == 3){
                if("ESC".equals(split[0])){
                    Integer.valueOf(split[1]);
                    String s = split[2];
                    return "SP".equals(s) || "TP".equals(s);
                }
            }
        }catch (Exception e){
            log.error("校验版本号错误：{}",e.getMessage());
        }
        return false;
    }

    // 罗马字符版本号
    private static final String[] ROME_STR = new String[]{
       "Ⅰ","Ⅱ","Ⅲ","Ⅳ","Ⅴ","Ⅵ","Ⅶ","Ⅷ","Ⅸ","Ⅹ","Ⅺ","Ⅻ","XIII","XIV","XV","XVI","XVII","XVIII","XIX","XX"
    };
    private static boolean checkVersionRome(String rome){
        if(StringUtil.isEmpty(rome))return false;
        for(String str:ROME_STR){
            if (str.equals(rome)) {
                return true;
            }
        }
        return false;
    }

    // ========= jpg、jpeg、png
    private static String[] MERCHANT_IMGAE_SUFFIX = new String[]{".jpg",".jpeg",".png"};
    public static RespWrapper<String> findSuffix(String fileName){
        for(String str:MERCHANT_IMGAE_SUFFIX){
            if(fileName.endsWith(str))
                return RespWrapper.makeSuccessResp(str);
        }
        return RespWrapper.makeFailResp("不支持该后缀");
    }

}
