package com.comtika.safe.core.aop;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.CodeSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.enums.api.ApiRequestURI;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.LuceneUtils;

import common.Logger;

@Aspect
@Order(5)
@Component
public class ControllerLogAspect {

	//private Logger logger = Logger.getLogger(this.getClass());

	@Pointcut("execution(public * com.comtika.safe.controller..*.*(..))")
	public void controllerLog() {
	
	}

	@Before("controllerLog()")
	public void doBefore(JoinPoint joinPoint) {
		String data = "";
		try {
			// 接收到请求，记录请求内容
			ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
			HttpServletRequest request = attributes.getRequest();
			//获取session内的用户信息
			UserCoreCache userCoreCache = (UserCoreCache) request.getSession().getAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN);
			//获取请求接口地址
			String requestURI = request.getRequestURI();
			//判断该请求接口地址是否需要写入日志
			ApiRequestURI apiRequestURI = ApiRequestURI.getApiRequestByUri(requestURI);
			// 只记录post方法
			if (null != apiRequestURI) {
				// 记录下请求内容
				if (joinPoint.getArgs().length > 0) {
					//获取请求参数名称
					String[] paramNames = ((CodeSignature) joinPoint.getSignature()).getParameterNames();
					for (int i = 0 ; i <joinPoint.getArgs().length; i++ ) {
						Object o = joinPoint.getArgs()[i];
						if (o instanceof HttpServletRequest || o instanceof HttpServletResponse) {
							continue;
						}
						String numT = (i == joinPoint.getArgs().length - 1) ? "":","; 
						data = data + paramNames[i] + ":" + o + numT;
						//data = data + JSON.toJSONString(o);
					}
					
					LuceneUtils.requestParameter(userCoreCache,System.currentTimeMillis(),apiRequestURI,data);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@AfterReturning(returning = "ret", pointcut = "controllerLog()")
	public void doAfterReturning(Object ret){
		try {
			// 处理完请求，返回内容
			//logger.info("返回 : " + JSON.toJSONString(ret));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

}
