package com.comtika.safe.core.lucene;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;

import java.io.IOException;
import java.util.List;

/**
 * 相当于mysql的in 操作
 */
public class IncludeQuery extends Query {

    private String fieldName;
    private List<?> list;
    BooleanQuery.Builder builder = new BooleanQuery.Builder();


    public IncludeQuery(String fieldName, List<?> list) {
        this.list = list;
        this.fieldName = fieldName;
        for (int i = list.size() - 1; i >= 0; i--) {
            Term t = new Term(fieldName, list.get(i) + "");
            // boolQuery.add();
            builder.add(new TermQuery(t), BooleanClause.Occur.SHOULD);
        }
    }


    @Override
    public Query rewrite(IndexReader reader) throws IOException {
        return builder.build();
    }

    public Query findQuery(){
        return builder.build();
    }


    public String toString(String field) {
        StringBuffer buffer = new StringBuffer();
        buffer.append(fieldName);
        buffer.append(":(");
        for (int i = list.size() - 1; i >= 0; i--) {
            buffer.append(list.get(i));
            if (i != 0) {
                buffer.append(",");
            }
        }
        buffer.append(")");
        return buffer.toString();
    }

    @Override
    public boolean equals(Object o) {
        return this.sameClassAs(o) && ((IncludeQuery)o).findQuery().equals(findQuery());
    }

    @Override
    public int hashCode() {
        return findQuery().hashCode();
    }
}
