package com.comtika.safe.core.lucene;

import java.nio.file.Paths;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;

public class IndexUse {
	
	/**
     * 通过关键字在索引目录中查询
     * @param indexDir    索引文件所在目录
     * @param q    关键字
     */
    public static void search(String indexDir, String q) throws Exception{
    	IndexReader indexReader = null;
    	try {
    		FSDirectory indexDirectory = FSDirectory.open(Paths.get(indexDir));
//          注意:索引输入流不是new出来的，是通过目录读取工具类打开的
          indexReader = DirectoryReader.open(indexDirectory);
//          获取索引搜索对象
          IndexSearcher indexSearcher = new IndexSearcher(indexReader);
          Analyzer analyzer = new StandardAnalyzer();
          QueryParser queryParser = new QueryParser("loginName", analyzer);
//          获取符合关键字的查询对象
          Query query = queryParser.parse(q);
          
          long start=System.currentTimeMillis();
//          获取关键字出现的前十次
          TopDocs topDocs = indexSearcher.search(query , 10);
          long end=System.currentTimeMillis();
          System.out.println("匹配 "+q+" ，总共花费"+(end-start)+"毫秒"+"查询到"+topDocs.totalHits+"个记录");
          
          for (ScoreDoc scoreDoc : topDocs.scoreDocs) {
        	  System.out.println(scoreDoc);
              int docID = scoreDoc.doc;
//              索引搜索对象通过文档下标获取文档
              Document doc = indexSearcher.doc(docID);
              System.out.println(doc);
              System.out.println(doc.get("loginName") + "," + 
              doc.get("loginTime") + ","+ doc.get("orderName") + 
              "," + doc.get("success") + ","+ doc.get("ip"));
              //System.out.println("通过索引文件："+doc.get("fullPath")+"拿数据");
          }
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			if( null != indexReader){
				System.out.println(111);
				indexReader.close();
				System.out.println(222);
			}
		}
    }
	
    public static void main(String[] args) {
    	String indexDir = "D:/ProjectDevelopment/lucene/LOGIN_LOG";;
        String q = "admin";
        try {
            IndexUse.search(indexDir, q);
        } catch (Exception e) {
            e.printStackTrace();
        }
	}
	
}
