package com.comtika.safe.core.lucene;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.cn.smart.SmartChineseAnalyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.Date;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 16:02 2020/8/6
 * @DESC:
 **/
public class LucenceWrite {
    /*索引路径*/
    static String indexDir = LuceneUtils.indexDir;

    //中文分词器
    static Analyzer analyzer = new SmartChineseAnalyzer();

    /**
     * StringField 不分词 要索引 要存储(可选)
     * TextField 要分词 要索引 要存储（可选）
     * StoredField 不分词 不索引 要存储（可选）
     * 其他的（Long/Float）要分词 要索引 要存储（可选）
     *
     * 是否分词(tokenized)
     * 是：作分词处理，即将Field值进行分词，分词的目的是为了索引
     * 否：不作分词处理
     *
     * 是否索引(indexed)
     * 是：进行索引。将Field分词后的词或整个Field值进行索引，索引的目的是为了搜索
     * 否：不索引。该域的内容无法搜索到
     *
     * 是否存储(stored)
     * 是：将Field值存储在文档中，存储在文档中的Field才可以从Document中获取
     * 否：不存储的Field无法通过Document获取
     * @throws Exception
     */

    // 保存索引  这里是将数据的数据保存到
    public static void write()throws Exception {
        /*索引存放的位置*/
        Directory directory = null;
        IndexWriterConfig config = null;
        IndexWriter writer = null;
        try {
            directory = FSDirectory.open(Paths.get(indexDir));//磁盘目录,RAMDirectory内存
            config = new IndexWriterConfig(analyzer);
            writer = new IndexWriter(directory,config);

            System.out.println(writer.deleteAll());
                System.out.println("删除所有");
                for(int i=0;i<10;i++){
                //创建Document
                Document doc = new Document();
                //创建Field
                Field userid = new Field("userId",i+"", StringField.TYPE_STORED);//是否存在
                // StringField field = new Field("userId",i+"");
                Field username = new Field("nickName","张三丰-"+i,StringField.TYPE_STORED);
                // 不分词 不存储
                Field truename1 = new StoredField("createTime",new Date().getTime()+"");
                Field truename = new NumericDocValuesField("createTime",new Date().getTime());
                //Field truename1 = new LongPoint("date",new Date().getTime());

                //填充Document
                doc.add(userid);
                doc.add(username);
                doc.add(truename1);
                doc.add(truename);
                //创建索引库
                writer.addDocument(doc);
                /*提交事务*/
                writer.commit();
            }

        }catch (IOException e){
            e.printStackTrace();
        }finally {
            /*关闭流*/
            try {
                if(writer != null)
                    writer.close();
                System.out.println("writer流已关闭。。。");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    public static void main(String[] args)throws Exception {
        write();
    }
}
