package com.comtika.safe.core.lucene;

import lombok.extern.log4j.Log4j2;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.NumericDocValuesField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexableField;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.*;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.comtika.safe.entity.lucene.LuceneReturnEntity;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.*;
import java.util.Map.Entry;


/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 16:10 2020/8/6
 * @DESC: lucene 查询操作
 **/
@Log4j2
public class LuceneQueryUtils {
    /**
     * 生产排序 Sort
     * @param field 排序字段
     * @param type  排序类型
     * @param desc  是否倒序
     * @return
     */
    public static Sort createSort(String field, SortField.Type type, boolean desc){
        SortField sortField = new SortField(field,type,desc);
        return new Sort(sortField);
    }

    /**
     * 全like
     * @param field
     * @param value
     * @return
     */
    public static Query like(String field, String value){
        return new WildcardQuery(new Term(field,"*"+value+"*"));
    }
    public static Query leftLike(String field, String value){
        return new WildcardQuery(new Term(field,"*"+value));
    }
    public static Query rightLike(String field, String value){
        return new WildcardQuery(new Term(field,value+"*"));
    }

    public static Query timeRange(String field,long startTime,long endTime){
        return NumericDocValuesField.newSlowRangeQuery(field, startTime, endTime);
    }

    /**
     * 这个是全相同操作 一般使用 StringField 字段
     * 像查询姓名、站点等
     * @param field
     * @param value
     * @return
     */
    public static Query equalsField(String field, String value){
        return new TermQuery(new Term(field,value));
    }

    /**
     * sql 的in 操作
     * @param field
     * @param list
     * @return
     */
    public static Query in(String field, List<?> list){
        return new IncludeQuery(field,list);
    }

    //统计的变量
  /*  private TermsBuilder totalFeild(){
        AbstractAggregationBuilder settleTotalRefundFee = AggregationBuilders.sum("toatlRefundAmount").field("refundAmount");//支付
        AbstractAggregationBuilder settleTotalFee = AggregationBuilders.sum("totalFeeAmount").field("settleAmount");//退款
        TermsBuilder aggregation = AggregationBuilders.terms("payType").field("payType");
        aggregation.subAggregation(settleTotalFee).subAggregation(settleTotalRefundFee);
        return aggregation;
    }*/


    /**
     * 查询索引
     * @param field
     * @param value
     * @return
     */
    public static List<Map<String,Object>> read(String field, String value){
        List<Map<String,Object>> list = new ArrayList<>();
        try {
            Directory directory = FSDirectory.open(Paths.get(LuceneUtils.indexDir));
            DirectoryReader reader = DirectoryReader.open(directory);
            IndexSearcher searcher = new IndexSearcher(reader);
//             QueryParser parser = new QueryParser(field,analyzer);
            Query query = getBooleanQuery();

            //Sort sort = new Sort();
            //sort.setSort(new SortField("createTime", SortField.Type.SCORE, false));
            // Sort sort=new Sort(new SortField[] {NumericDocValuesField.FIELD_SCORE,new SortField("title",SortField.Type.STRING,true)});
            // 这个是按照实际排序 排序设置
            System.out.println(query.toString());
            // AggregatorFactory
            //LongTermsAggregator
            // GlobalOrdinalsStringTermsAggregator
            //SearchSourceBuilder


            TopDocs topDocs = searcher.search(query,5,createSort("createTime",SortField.Type.LONG,true));


            Long count = topDocs.totalHits;
            ScoreDoc[] scoreDocs = topDocs.scoreDocs;
            // TotalHits totalHits = topDocs.totalHits;
            System.out.println("总共查到"+count+"个文档");
            for (ScoreDoc doc : topDocs.scoreDocs){
                Document document = searcher.doc(doc.doc);
                System.out.println(document);
                System.out.println(document.get(field)+"评分是"+doc.score);
                Map<String,Object> map = new HashMap<>();
                map.put("userId",document.get(field));
                list.add(map);
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return list;
    }
    public static void main(String[] args){
        //System.out.println(read("userId", "1"));
    	String indexDir = "D:/ProjectDevelopment/lucene/LOGIN_LOG";
    	Map<String,Object> map = new HashMap<String, Object>();
    	//map.put("orderName", "后");
    	//map.put("loginName", "admin");
    	Sort sort =  createSort("loginTime", SortField.Type.LONG, true);
    	//Query a = timeRange("loginTime", 1598405592361L, 1598405655585L);
    	LuceneReturnEntity in = queryLucene(indexDir, 1, 10, map, sort,null);
    	System.out.println(in.getTotalRow());
    	List<Map<String,Object>> list = in.getList();
    	for (Map<String, Object> map2 : list) {
			System.out.println(map2);
		}

    }

    /**
     * 通配符搜索,?表示一个字符，*表示0或多个
     * 作用和数据库模糊搜索的like相似
     * @param field
     * @param value
     * @return
     */
    @SuppressWarnings("unused")
	private Query getWildcardQuery(String field, String value){
        String keyword = "*"+value+"*";
        Term term = new Term(field,keyword);
        WildcardQuery query = new WildcardQuery(term);
        return query;
    }

    /**
     * 模糊搜索
     * 只能模糊+2 个字段
     * @param field
     * @param value
     * @return
     */
    @SuppressWarnings("unused")
	private Query getFuzzyQuery(String field, String value){
        Term term = new Term(field,value);
        FuzzyQuery query = new FuzzyQuery(term);
        return query;
    }

    /**
     * 精确搜索
     * @param field
     * @param value
     * @return
     */
    @SuppressWarnings("unused")
	private Query getTermQuery(String field, String value){
        Term term = new Term(field,value);
        TermQuery query = new TermQuery(term);
        return query;
    }

    /**
     * 多条件查询
     * @return
     */
    @SuppressWarnings("unused")
	private static Query getBooleanQuery(){
        String field2 = "nickName";
        String value2 = "张三丰-2";
        Query query2 = new FuzzyQuery(new Term(field2,value2));
        String keyword = "张丰*";
        Term term = new Term(field2,keyword);
        WildcardQuery query = new WildcardQuery(term);
        BooleanQuery.Builder builder = new BooleanQuery.Builder();

        // Query query = NumericDocValuesField.newSlowRangeQuery("createTime", 1496704620364L, 1698704640368L);
        //SHOULD+SHOULD，表示“or”
//        builder.add(query1,BooleanClause.Occur.SHOULD);
//        builder.add(query2,BooleanClause.Occur.SHOULD);
        //MUST+MUST，表示“and”
//        builder.add(query1,BooleanClause.Occur.MUST);
//        builder.add(query2,BooleanClause.Occur.MUST);
        //MUST+MUST_NOT，表示MUST查询的结果不包含MUST_NOT查询的结果
//        builder.add(query1,BooleanClause.Occur.MUST);
//        builder.add(query2,BooleanClause.Occur.MUST_NOT);
        //FILTER,通过过滤将评分改为0.0
//        builder.add(query1,BooleanClause.Occur.FILTER);
       // builder.add(query1,BooleanClause.Occur.MUST);
        // IncludeQuery userId = new IncludeQuery("userId", Arrays.asList(1, 2));
        //System.out.println(userId.toString());
        //builder.add(userId,BooleanClause.Occur.MUST);
        builder.add(query2,BooleanClause.Occur.MUST);
        BooleanQuery booleanQuery = builder.build();
        return booleanQuery;
    }

    /**
     *
     * @param luceneType		索引位置
     * @param pageNumber	页数
     * @param pageSize		页容量
     * @param sort			排序字段
     * @return
     *
     */
    public static LuceneReturnEntity fadQueryLucene(LuceneUtils.LuceneType luceneType ,
                                                 Integer pageNumber,Integer pageSize,Query query,Sort sort){
        List<Map<String,Object>> list = new ArrayList<>();
        // List<Map<String,Object>> dataList = new ArrayList<>();
        LuceneReturnEntity in = new LuceneReturnEntity();
        try {
            IndexSearcher searcher = LuceneUtils.findSearcher(luceneType);
            TopDocs topDocs = null;
            if(sort != null){
                //排序查询
                topDocs = searcher.search(query , pageNumber * pageSize ,sort);
            }else {
                //不排序查询
                topDocs = searcher.search(query , pageNumber * pageSize);
            }
            Long totalRow = topDocs.totalHits;
            // log.error("获取数据为：{}",topDocs.totalHits);
            ScoreDoc[] scoreDocs = topDocs.scoreDocs;
            int length = scoreDocs.length;
            for(int i=(pageNumber-1)*pageSize;i<length;i++) {
                Map<String,Object> data = new HashMap<String, Object>();
                int docID = scoreDocs[i].doc;
                //索引搜索对象通过文档下标获取文档
                Document doc = searcher.doc(docID);
                //获取所有参数名
                List<IndexableField> listField = doc.getFields();
                for (IndexableField indexableField : listField) {
                    data.put(indexableField.name(), doc.get(indexableField.name()));
                }
                list.add(data);
            }
            //将list集合进行分页
          /*  if( list != null && !list.isEmpty() && list.size() > 0){
                int startIndex = ( pageNumber - 1) * pageSize;
                int endIndex = startIndex + pageSize;
                if ( endIndex >= list.size() ) {
                    endIndex = list.size();
                }
                dataList = list.subList(startIndex, endIndex);
            }*/
            in.setList(list);
            in.setTotalRow(totalRow);
        } catch (Exception e) {
            e.printStackTrace();
            in.setList(new ArrayList<>());
            in.setTotalRow(0L);
            // return in;
        }
        return in;
    }


    /**
     *
     * @param indexDir		索引位置
     * @param pageNumber	页数
     * @param pageSize		页容量
     * @param map			参数集合
     * @param sort			排序字段
     * @param timeRange		范围查询条件
     * @return
     *
     */
    public static LuceneReturnEntity queryLucene(String indexDir ,
    		Integer pageNumber,Integer pageSize,Map<String,Object> map,Sort sort,Query timeRange){
    	IndexReader indexReader = null;
    	List<Map<String,Object>> list = new ArrayList<Map<String,Object>>();
    	// List<Map<String,Object>> dataList = new ArrayList<Map<String,Object>>();
    	LuceneReturnEntity in = new LuceneReturnEntity();
    	try {
    		FSDirectory indexDirectory = FSDirectory.open(Paths.get(indexDir));
    		//注意:索引输入流不是new出来的，是通过目录读取工具类打开的
    		indexReader = DirectoryReader.open(indexDirectory);
    		//获取索引搜索对象
    		IndexSearcher indexSearcher = new IndexSearcher(indexReader);

    		//参数赋值
    		BooleanQuery.Builder builder = new BooleanQuery.Builder();
    		if( null != map && map.size() > 0){
    			Query query = null;
    			for( Entry<String, Object> entry : map.entrySet() ){
    				//所有查询，只需要右边like
    				query = rightLike(entry.getKey(),entry.getValue() + "");
    				//query = new FuzzyQuery(new Term(entry.getKey(),entry.getValue() + ""));
    				builder.add(query,BooleanClause.Occur.MUST);
    			}
    		}
    		if( null != timeRange){
    			builder.add(timeRange,BooleanClause.Occur.MUST);
    		}
    		//查询
    		Query query = builder.build();
    		TopDocs topDocs = null;
    		if(sort != null){
    			//排序查询
    			topDocs = indexSearcher.search(query , pageNumber * pageSize ,sort);
    		}else {
    			//不排序查询
            	topDocs = indexSearcher.search(query , pageNumber * pageSize);
            }
    		Long totalRow = topDocs.totalHits;
    		// Map<String,Object> data = null;
    		/*for (ScoreDoc scoreDoc : topDocs.scoreDocs) {
    			data = new HashMap<String, Object>();
                int docID = scoreDoc.doc;
                //索引搜索对象通过文档下标获取文档
                Document doc = indexSearcher.doc(docID);
                //获取所有参数名
                List<IndexableField> listField = doc.getFields();
                for (IndexableField indexableField : listField) {
                	//根据参数名获取参数值
                	data.put(indexableField.name(), doc.get(indexableField.name()));
				}
                list.add(data);
            }
    		//将list集合进行分页
    		if( list != null && !list.isEmpty() && list.size() > 0){

    			//如果当前页数大于等于总记录数对页容量取余+1，则说明已经超出了分页范围
    			*//*if(pageNumber > (int)(list.size() / pageSize) + 1){
    				return in;
    			}*//*
    			int startIndex = ( pageNumber - 1) * pageSize;
    			int endIndex = startIndex + pageSize;
    			if ( endIndex >= list.size() ) {
    				endIndex = list.size();
    			}
    			dataList = list.subList(startIndex, endIndex);
    		}*/
            ScoreDoc[] scoreDocs = topDocs.scoreDocs;
            int length = scoreDocs.length;
            for(int i=(pageNumber-1)*pageSize;i<length;i++) {
                Map<String,Object> data = new HashMap<String, Object>();
                int docID = scoreDocs[i].doc;
                //索引搜索对象通过文档下标获取文档
                Document doc = indexSearcher.doc(docID);
                //获取所有参数名
                List<IndexableField> listField = doc.getFields();
                for (IndexableField indexableField : listField) {
                    data.put(indexableField.name(), doc.get(indexableField.name()));
                }
                list.add(data);
            }
    		in.setList(list);
    		in.setTotalRow(totalRow);
		} catch (Exception e) {
			e.printStackTrace();
			in.setList(new ArrayList<Map<String,Object>>());
			in.setTotalRow(0L);
			return in;
		}
    	return in;
    }



}
