package com.comtika.safe.core.lucene;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.api.ApiRequestURI;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.pool.WriterPool;
import com.comtika.safe.entity.log.LoginLog;
import com.comtika.safe.entity.safe.RepDataRecord;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.cn.smart.SmartChineseAnalyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 14:24 2020/8/21
 * @DESC: lucene 公共类
 **/
public class LuceneUtils {
    public static String indexDir = "/data/lucene/";
    //中文分词器
    static Analyzer analyzer = new SmartChineseAnalyzer();
    static {
        String osName = System.getProperties().getProperty("os.name");
        if(StringUtil.isNotEmpty(osName) && osName.toUpperCase().startsWith("WINDOWS")){
            // 这里是win时 保存的路径
            indexDir = "D:/ProjectDevelopment/lucene/";
        }
    }
    public  enum LuceneType{
        REP_DATA,//
        LOGIN_LOG,//登录日志
        VISIT_LOG,//访问日志
        ERROR_LOG;//异常日志
    }
    /**
     * StringField 不分词 要索引 要存储(可选)
     * TextField 要分词 要索引 要存储（可选）
     * StoredField 不分词 不索引 要存储（可选）
     * 其他的（Long/Float）要分词 要索引 要存储（可选）
     * 是否分词(tokenized)
     * 是：作分词处理，即将Field值进行分词，分词的目的是为了索引
     * 否：不作分词处理
     *
     * 是否索引(indexed)
     * 是：进行索引。将Field分词后的词或整个Field值进行索引，索引的目的是为了搜索
     * 否：不索引。该域的内容无法搜索到
     *
     * 是否存储(stored)
     * 是：将Field值存储在文档中，存储在文档中的Field才可以从Document中获取
     * 否：不存储的Field无法通过Document获取
     * @throws Exception
     */
    public static void deleteAllRepRecord(LuceneType type){
        IndexWriter writer = null;
        try {
            writer = findWriter(type);
            writer.deleteAll();
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            try {
                if(writer != null)
                    writer.close();
            }catch (Exception e){

            }
        }
    }
    public static String findIndexKey(LuceneType type){
        return indexDir + type.name();
    }

    public static IndexWriter findWriter(LuceneType type)throws Exception{
        Directory directory = FSDirectory.open(Paths.get(indexDir + type.name()));//磁盘目录,RAMDirectory内存
        IndexWriterConfig config = new IndexWriterConfig(analyzer);
        return new IndexWriter(directory,config);
    }
    public static IndexSearcher findSearcher(LuceneType type)throws Exception{
        FSDirectory indexDirectory = FSDirectory.open(Paths.get(indexDir+type.name()));
        //注意:索引输入流不是new出来的，是通过目录读取工具类打开的
        //获取索引搜索对象
       return new IndexSearcher(DirectoryReader.open(indexDirectory));
    }

    // 这里写记录
    public static RespWrapper<Boolean> writeRepRecord(RepDataRecord res){
        /*索引存放的位置*/
        WriterPool witePool = WriterPool.findWitePool(LuceneType.REP_DATA);
        IndexWriter writer = witePool.getConnection();
        try {
            // writer = findWriter(LuceneType.REP_DATA);
            //创建Document
            Document doc = new Document();
            //创建Field
            Field id = new Field("id",res.getId()+"", StringField.TYPE_STORED);//是否存在
            doc.add(id);
            Field nodeNo = new Field("nodeNo",res.getNodeNo(),StringField.TYPE_STORED);
            doc.add(nodeNo);
            Field deviceId = new Field("deviceId",res.getDeviceId()+"",StringField.TYPE_STORED);
            doc.add(deviceId);
            Field siteId = new Field("siteId",res.getSiteId()+"",StringField.TYPE_STORED);
            doc.add(siteId);
            Field siteName = new Field("siteName",res.getSiteName(),StringField.TYPE_STORED);
            doc.add(siteName);
            Field merchantId = new Field("merchantId",res.getMerchantId()+"",StringField.TYPE_STORED);
            doc.add(merchantId);
            Field merchantName = new Field("merchantName",res.getMerchantName(),StringField.TYPE_STORED);
            doc.add(merchantName);
            Field subT = new Field("subT",res.getSubT()+"",StringField.TYPE_STORED);
            doc.add(subT);
            Field subV = new Field("subV",res.getSubV()+"",StringField.TYPE_STORED);
            doc.add(subV);
            Field subI = new Field("subI",res.getSubI()+"",StringField.TYPE_STORED);
            doc.add(subI);
            Field subP = new Field("subP",res.getSubP()+"",StringField.TYPE_STORED);
            doc.add(subP);
            Field subE = new Field("subE",res.getSubE()+"",StringField.TYPE_STORED);
            doc.add(subE);
            Field subLeakI = new Field("subLeakI",res.getSubLeakI()+"",StringField.TYPE_STORED);
            doc.add(subLeakI);
            Field mainT = new Field("mainT",res.getMainT()+"",StringField.TYPE_STORED);
            doc.add(mainT);
            Field plugState = new Field("plugState",res.getPlugState()+"",StringField.TYPE_STORED);
            doc.add(plugState);
            Field plugNo = new Field("plugNo",res.getPlugNo()+"",StringField.TYPE_STORED);
            doc.add(plugNo);
            Field typeId = new Field("typeId",res.getTypeId()+"",StringField.TYPE_STORED);
            doc.add(typeId);
            Field typeName = new Field("typeName",res.getTypeName(),StringField.TYPE_STORED);
            doc.add(typeName);
            Field serialNo = new Field("serialNo",res.getSerialNo(),StringField.TYPE_STORED);
            doc.add(serialNo);

            Field terminalNo = new Field("terminalNo",res.getTerminalNo(),StringField.TYPE_STORED);
            doc.add(terminalNo);

            // 直接显示时间
            Field showTime = new StoredField("showTime", DateTimeUtils.formatDate(new Date(res.getCreateTime()),DateTimeUtils.FORMAT));
            doc.add(showTime);
            // 时间查询帮助类
            Field createTime = new NumericDocValuesField("createTime",res.getCreateTime());
            doc.add(createTime);
            /**
             *  private String nodeNo;
             *     private String terminalNo;
             *     private Integer deviceId;
             *     private Integer siteId;
             *     private String siteName;
             *     private Integer merchantId;
             *     private String merchantName;
             *     private Integer subT;
             *     private Integer subV;
             *     private Integer subI;
             *     private Integer subP;
             *     private Integer subE;
             *     private Integer subLeakI;
             *     private Integer mainT;
             *     private Integer plugState;
             *     private Integer plugNo;
             */
            //创建索引库
            writer.addDocument(doc);
            /*提交事务*/
            writer.commit();
        }catch (Exception e){
            e.printStackTrace();
            return RespWrapper.makeFailResp("新增数据到lucene 错误");
        }finally {
            witePool.returnConnection(writer);
            /*关闭流*/
            //try {
            //    if(writer != null)
            //        writer.close();
            //    System.out.println("writer流已关闭。。。");
            //} catch (IOException e) {
            //    e.printStackTrace();
            //}
        }
        return RespWrapper.makeSuccessResp(true);
    }

	public static RespWrapper<Object> insertLoginLog(LoginLog loginLog) {
		/*索引存放的位置*/
        WriterPool witePool = WriterPool.findWitePool(LuceneType.LOGIN_LOG);
        IndexWriter writer = witePool.getConnection();
		try {
			// writer = findWriter(LuceneType.LOGIN_LOG);
			//创建Document
            Document doc = new Document();
            //创建Field
            Field loginName = new Field("loginName",loginLog.getLoginName(), StringField.TYPE_STORED);//
            doc.add(loginName);

            Field loginTime1 = new Field("loginTime",loginLog.getLoginTime()+"", StringField.TYPE_STORED);
            doc.add(loginTime1);

            Field loginTime = new NumericDocValuesField("loginTime",loginLog.getLoginTime());
            doc.add(loginTime);

            Field orderName = new Field("orderName",loginLog.getOrderName(), StringField.TYPE_STORED);//
            doc.add(orderName);

            Field success = new Field("success",loginLog.getSuccess() + "", StringField.TYPE_STORED);//
            doc.add(success);

            Field ip = new Field("ip",loginLog.getIp(), StringField.TYPE_STORED);//
            doc.add(ip);
            //创建索引库
            writer.addDocument(doc);
            /*提交事务*/
            writer.commit();
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("新增登录日志失败，请联系管理员!");
		}finally {
            /*关闭流*/
            witePool.returnConnection(writer);
            //try {
            //    if(writer != null){
            //    	System.out.println("writer开始关闭。。。");
            //    	writer.close();
            //    }
            //    System.out.println("writer流已关闭。。。");
            //} catch (IOException e) {
            //    e.printStackTrace();
            //}
        }
		return RespWrapper.makeSuccessResp("新增成功！");
	}

	public static RespWrapper<Object> requestParameter(UserCoreCache userCoreCache,
			Long start, ApiRequestURI apiRequestURI, String data) {
		/*索引存放的位置*/
        WriterPool witePool = WriterPool.findWitePool(LuceneType.VISIT_LOG);
        IndexWriter writer = witePool.getConnection();
		try {
			// writer = findWriter(LuceneType.VISIT_LOG);
			//创建Document
            Document doc = new Document();
            //创建Field
            Field loginName = new Field("loginName",userCoreCache.getLoginName(), StringField.TYPE_STORED);//
            doc.add(loginName);

            Field name = new Field("name",userCoreCache.getName(), StringField.TYPE_STORED);
            doc.add(name);

            Field visitTime = new NumericDocValuesField("visitTime",start);
            doc.add(visitTime);
            Field visitTime1 = new Field("visitTime",start + "", StringField.TYPE_STORED);
            doc.add(visitTime1);

            Field visitData = new Field("visitData",data, StringField.TYPE_STORED);//
            doc.add(visitData);

            Field visitUri = new Field("visitUri",apiRequestURI.getUri(), StringField.TYPE_STORED);//
            doc.add(visitUri);

            Field visitType = new Field("visitType",apiRequestURI.getType(), StringField.TYPE_STORED);//
            doc.add(visitType);

            Field visitMsg = new Field("visitMsg",apiRequestURI.getMsg(), StringField.TYPE_STORED);//
            doc.add(visitMsg);

            //创建索引库
            writer.addDocument(doc);
            /*提交事务*/
            writer.commit();
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("新增访问日志失败，请联系管理员!");
		}finally {
            /*关闭流*/
            witePool.returnConnection(writer);
            //try {
            //    if(writer != null){
            //    	System.out.println("writer开始关闭。。。");
            //    	writer.close();
            //    }
            //    System.out.println("writer流已关闭。。。");
            //} catch (IOException e) {
            //    e.printStackTrace();
            //}
        }
		return RespWrapper.makeSuccessResp("新增成功！");
	}

	/**
	 * 异常日志新增
	 * @param apiErrorType	接口类型
	 * @param uri			接口uri
	 * @param interfaceName	接口名称
	 * @param errorMsg		接口异常日志信息
	 * @return
	 */
	public static boolean apiErrorLucene(String errorType,HttpServletRequest request,String interfaceName,String errorMsg){
		/*索引存放的位置*/
        WriterPool witePool = WriterPool.findWitePool(LuceneType.ERROR_LOG);
        IndexWriter writer = witePool.getConnection();
        Long start = System.currentTimeMillis();//获取当前时间戳
		try {
			UserCoreCache userCoreCache = (UserCoreCache) request.getSession().getAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN);
			if( null == userCoreCache ){
				return false;
			}
			String loginName = userCoreCache.getLoginName();//获取登录名
			String name = userCoreCache.getName();//获取用户名称
			String uri = request.getRequestURI();//获取当前请求的
			// writer = findWriter(LuceneType.ERROR_LOG);
			//创建Document
            Document doc = new Document();
            //创建Field
            //接口异常时间,并且设置该时间可排序
            doc.add(new NumericDocValuesField("errorTime",start));
            doc.add(new Field("errorTime",start + "", StringField.TYPE_STORED));
            //登录名
            doc.add(new Field("loginName",loginName, StringField.TYPE_STORED));
            //登录名
            doc.add(new Field("name",name, StringField.TYPE_STORED));
            //接口uri
            doc.add(new Field("uri",uri, StringField.TYPE_STORED));
            //接口名称
            doc.add(new Field("interfaceName",interfaceName, StringField.TYPE_STORED));
            //接口模块
            doc.add(new Field("errorType",errorType, StringField.TYPE_STORED));
            //接口异常日志信息
            doc.add(new Field("errorMsg",errorMsg, StringField.TYPE_STORED));
            //创建索引库
            writer.addDocument(doc);
            /*提交事务*/
            writer.commit();
            return true;
		} catch (Exception e) {
			e.printStackTrace();
		}finally {
            /*关闭流*/
            witePool.returnConnection(writer);
            // try {
            //     if(writer != null){
            //     	writer.close();
            //     }
            // } catch (IOException e) {
            //     e.printStackTrace();
            // }
        }
		return false;
	}

}
