package com.comtika.safe.core.message;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import lombok.extern.log4j.Log4j2;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.PostMethod;

import java.util.Date;
import java.util.regex.Pattern;

@Log4j2
public class MessageUtils {
    private static final Pattern PATTERN_MOBILE = Pattern.compile("^1[3-9]\\d{9}$");
    private static final String SEND_URL = "http://106.ihuyi.cn/webservice/sms.php?method=Submit";
    private static final String USER_NAME = "cf_grt";
    private static final String PASSWORD = "grtsms2016";
    private static final String ONE_CONTENT = "您的验证码是：%s。请不要把验证码泄露给其他人。如非本人操作，可不用理会！";
    private static final String THREE_CONTENT = "请注意，充电订单异常关闭！订单编号：%s,流水号：%s,站点名称：%s,结束原因：%s,请及时分析关闭原因！";

    private static final String SAFE_CONTENT = "安全用电：[%s]站点的设备%s于%s发生%s的啦！请及时处理，防止造成严重事故！";

    // 发生验证码接口
    public static RespWrapper<Boolean> sendCode(String mobile,String code){
        String content = String.format(ONE_CONTENT,code);
        return send(mobile,content);
    }

    private static RespWrapper<Boolean> send(String mobile, String content){
        // 这里校验手机号格式是否满足要求
        if (mobile == null || !ProjectConstant.PhonePattern.PHONE_NO.matcher(mobile).matches())
            return RespWrapper.makeFailResp("手机号格式不对");
        try {
            HttpClient client = new HttpClient();
            PostMethod method = new PostMethod(SEND_URL);
            client.getParams().setContentCharset("UTF-8");
            method.setRequestHeader("ContentType", "application/x-www-form-urlencoded;charset=UTF-8");
            NameValuePair[] data = new NameValuePair[]{new NameValuePair("account", USER_NAME), new NameValuePair("password", PASSWORD), new NameValuePair("mobile", mobile), new NameValuePair("content", content)};
            method.setRequestBody(data);
            client.executeMethod(method);
            String SubmitResult = method.getResponseBodyAsString();
            Document doc = DocumentHelper.parseText(SubmitResult);
            Element root = doc.getRootElement();
            if ("2".equals(root.elementText("code"))) {
                return RespWrapper.makeSuccessResp(true);
            }
            return RespWrapper.makeFailResp(root.elementText("msg"));
        }catch (Exception e){
            log.error("发送短信消息异常：{}",e.getMessage());
            return RespWrapper.makeFailResp("发送短信消息异常："+e.getMessage());
        }

    }
    // RespWrapper<Boolean> booleanRespWrapper = sendSafeMessage("18588458430", "测试站点", "016666666680", System.currentTimeMillis(), "第4回路过流告警的");
    public static RespWrapper<Boolean> sendSafeMessage(String mobile,String siteName,String terminalNo,Long time,String content){
        // 发生告警
        if(mobile == null || !PATTERN_MOBILE.matcher(mobile).matches())
            return RespWrapper.makeFailResp("请检查手机号");
        String format = String.format(SAFE_CONTENT, siteName, terminalNo, DateTimeUtils.formatDate(new Date(time), DateTimeUtils.FORMAT), content);
        return send(mobile,format);
    }
    public static void main(String[] args)throws Exception{

        // RespWrapper<Boolean> booleanRespWrapper = sendSafeMessage("18588458430", "测试站点", "016666666680", System.currentTimeMillis(), "第4回路过流告警");
        //  System.out.println(booleanRespWrapper.getMsg());
        //sendCode("18588458430","123456");
    }
}
