package com.comtika.safe.core.protocol.cache;

import com.comtika.safe.common.utils.StringUtil;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @ClassName: LocalProtocolCache
 * @Description:协议层本地缓存 使用ConcurrentHashMap缓存 这里有一个问题
 *                      调试时sokcet不知道什么时候就不断的增加啦！现在改为--->socket是否相等措施
 * @author zhaoLuYang
 * @date2018年6月3日 下午9:30:43
 * @version V1.0
 *
 * 本地缓存增加黑名单操作  2018.12.26
 */
public class LocalProtocolCache {
	/** 集中器唯一编号与通讯唯一Channel通道 */
	public static Map<String, Channel> TERMINAL_CHANNEL = new ConcurrentHashMap<String, Channel>();
	public static Map<String, Long> DROP_LINE_MAP = new ConcurrentHashMap<>();

	/**为了减少redis压力，白名单与黑名单配合 */
	//public static Set<String> TERMINAL_WHITE_LIST = new HashSet<>();
	public static ConcurrentHashMap<String,Boolean> TERMINAL_WHITE_LIST = new ConcurrentHashMap<>();

	/**集中器心跳时间（用于判断集中器是否在线） HEARTBEAT_MAP*/
	public static Map<String, Long> HEARTBEAT_MAP = new ConcurrentHashMap<String, Long>();

	/**简介充电数据（用来判断用户1个小时断电续充功能）*/
	public static Map<String, Long> SIMPLE_CHARGE_DATA = new ConcurrentHashMap<String, Long>();

	/**在线列表*/
	public static ConcurrentHashMap<String,Boolean> TERMINAL_ONLINE_LIST = new ConcurrentHashMap<>();

	public static void setHeartbeat(String terminalNo){
		HEARTBEAT_MAP.put(terminalNo, System.currentTimeMillis());
	}
	/**心跳过期*/
	public static void removeHearbeatValue(String addrNo){
		TERMINAL_ONLINE_LIST.remove(addrNo);
		HEARTBEAT_MAP.remove(addrNo);
	}

	/** @Description: 清空所有缓存的键值对 */
	public static void cleanAllCache() {
		TERMINAL_CHANNEL.forEach((k, v)->{
			try {
				if(v != null) v.close();
			}catch (Exception e){}
		});
		TERMINAL_CHANNEL.clear();
	}

	/**
	 * Netty
	 * @param addrNo
	 */
	public static void closedChannelCache(String addrNo) {
		Channel channel = null;
		try {
			if (StringUtil.isNotEmpty(addrNo)) {
				channel = TERMINAL_CHANNEL.get(addrNo);
				TERMINAL_CHANNEL.remove(addrNo);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (channel != null) {
				// 一拖四 从设备同一个socket 所有1拖4 不关闭socket
				if(!addrNo.startsWith(ONE_TO_FOUR))
					channel.close();

			}
		}
	}
	private static final String ONE_TO_FOUR = "0014";


	/**
	 * 根据缓存名称获取IoSession
	 */
	public static Channel findChannelByTerminalNo(String termianNo) {
		return TERMINAL_CHANNEL.get(termianNo);
	}

	/**
	 * 判断channel是否最新	true=最新 false= 旧的
	 * 		如果是旧的更新最新的channel
	 */
	public static synchronized void judgeTerminalCache(String addrNo, ChannelHandlerContext ctx) {
		try{
			Channel channel = ctx.channel();
			Channel oldChannel = TERMINAL_CHANNEL.get(addrNo);
			if(oldChannel != null ){
				if(channel.equals(oldChannel))//相等 直接返回
					return;
				else
					oldChannel.close();//關閉舊的
			}
			TERMINAL_CHANNEL.put(addrNo, channel);
		}catch (Exception e){
			e.printStackTrace();
		}
	}
}
