package com.comtika.safe.core.protocol.cache;

import com.comtika.safe.common.cache.DefaultGuavaCache;
import com.comtika.safe.common.cache.GoogleGuavaConfig;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.encrypt.HexUtil;
import com.comtika.safe.core.protocol.entity.netty.OrderEntity;
import com.comtika.safe.core.protocol.entity.safe.TerminalSafeInfo;
import com.comtika.safe.core.protocol.eunms.netty.OperationCode;
import com.comtika.safe.entity.safe.TerminalEleTotal;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 使用本地缓存 保存心电通信
 */
public class SafeCache {
	private static DefaultGuavaCache SIGNAL_CACHE;//信号量缓存
	private static DefaultGuavaCache DATA_CACHE;// 数据缓存 数据返回

	public static DefaultGuavaCache FIVE_CREATE_CACHE;
	public static DefaultGuavaCache ONE_HOUR_CACHE;
	// private static final int DELAY = 60;
	private static final int MAX_SIZE = 8999;
	public static DefaultGuavaCache FIFTEEN_ACCESS_CACHE;
	static {
		initSignalCache();
		initDataAccess();
		initFiveCreateCache();
		initOneHourCache();
		initFifteenAccessCache();
	}
	// 15分钟
	private static void initFifteenAccessCache() {
		GoogleGuavaConfig config = new GoogleGuavaConfig();
		config.setCreateExpire(false);
		config.setExpireTime(15);
		config.setInitialCapacity(1);
		config.setUnit(TimeUnit.MINUTES);
		FIFTEEN_ACCESS_CACHE = new DefaultGuavaCache(config);
	}

	private static void initOneHourCache() {
		GoogleGuavaConfig config = new GoogleGuavaConfig();
		config.setCreateExpire(false);
		config.setExpireTime(1);
		config.setUnit(TimeUnit.HOURS);
		config.setMaximumSize(1024);
		config.setInitialCapacity(1);
		ONE_HOUR_CACHE = new DefaultGuavaCache(config);
	}
	// 5分钟失效
	private static void initFiveCreateCache(){
		GoogleGuavaConfig config = new GoogleGuavaConfig();
		config.setCreateExpire(true);
		config.setExpireTime(5);
		config.setInitialCapacity(1);
		config.setMaximumSize(128);
		config.setUnit(TimeUnit.MINUTES);
		FIVE_CREATE_CACHE = new DefaultGuavaCache(config);
	}
	// 主要使用在心电跟踪方法
	private static void initDataAccess(){
		GoogleGuavaConfig config1 = new GoogleGuavaConfig();
		config1.setCreateExpire(false);
		config1.setMaximumSize(64);
		config1.setInitialCapacity(1);
		config1.setExpireTime(5);
		config1.setUnit(TimeUnit.MINUTES);
		DATA_CACHE = new DefaultGuavaCache(config1);
	}
	// 1分钟失效
	private static void initSignalCache(){
		GoogleGuavaConfig config = new GoogleGuavaConfig();
		config.setCreateExpire(true);
		config.setExpireTime(1);
		config.setUnit(TimeUnit.MINUTES);
		config.setInitialCapacity(1);
		SIGNAL_CACHE = new DefaultGuavaCache(config);
	}

	private static final String ELE_TOTAL_KEY = "%s-%s";
	public static TerminalEleTotal findTerminalEleTotal(int id, long time){
		String key = String.format(ELE_TOTAL_KEY, id, time);
		return (TerminalEleTotal) FIFTEEN_ACCESS_CACHE.getIfPresent(key);
	}
	public static void addTerminalEleTotal(int id,long time,TerminalEleTotal ele){
		FIFTEEN_ACCESS_CACHE.set(String.format(ELE_TOTAL_KEY, id, time),ele);
	}

	private static final String PHONE_CODE_KEY = "$phone-code-";
	public static void addPhoneNoCode(String phoneNo,String code){
		FIVE_CREATE_CACHE.set(PHONE_CODE_KEY+phoneNo,code);
	}
	public static String findPhoneNoCode(String phoneNo){
		String key = PHONE_CODE_KEY + phoneNo;
		Object present = FIVE_CREATE_CACHE.getIfPresent(key);
		if(present != null) FIVE_CREATE_CACHE.remove(key);
		return present.toString();
	}


	private static final String TERMINAL_UNIQUE = "safe_base_%s";
	public static TerminalSafeInfo findTerminalBaseInfo(String terminalNo){
		return (TerminalSafeInfo) DATA_CACHE.getIfPresent(String.format(TERMINAL_UNIQUE,terminalNo));
	}

	public static void removeTerminalBaseInfo(String terminalNo){
		if(StringUtil.isNotEmpty(terminalNo))
			DATA_CACHE.remove(String.format(TERMINAL_UNIQUE,terminalNo));
	}
	private static final String KEY_MISS = "no_add_warn_";
	public static boolean isEmptyOrAdd(String terminalNo,boolean isAdd){
		String key = KEY_MISS+terminalNo;
		if(isAdd){
			SIGNAL_CACHE.set(key,true);
			return true;
		}
		return SIGNAL_CACHE.getIfPresent(key) != null;
	}

	public static void addTerminalBaseInfo(String terminalNo,TerminalSafeInfo base){
		DATA_CACHE.set(String.format(TERMINAL_UNIQUE,terminalNo),base);
	}


	private static final String SIGNAL_PREFIX = "signal_prefix_";
	private static final String DATA_PREFIX = "data_prefix_";
	//private static final String SEND_DATA = "发送数据【%s】,时间：%s";
	//private static final String ACCEPT_DATA = "接受数据【%s】,时间：%s";
	private static final String SEND_DATA_FAD = "时间：%s；方向：%s；MAC:%s；控制码：%s；命令码：%s；原始帧：%s；数据：%s；概要:%s。";


	private static String  findDataKey(String terminalNo){
		return DATA_PREFIX+terminalNo;
	}
	private static String findSignKey(String terminalNo){
		return SIGNAL_PREFIX+terminalNo;
	}
    public static void set(OrderEntity oe, boolean isSend){
	    if(oe != null){
	        String terminalNo = oe.getId();
            if(checkSignal(terminalNo)){
                String value = formatData(oe,isSend);
				saveDataToCache(terminalNo,value);
            }
        }
    }
    public static Byte[] findOTAData(String key){
		return (Byte[]) DATA_CACHE.getIfPresent(key);
	}
	public static void saveOTAData(String key,Byte[] bytes){
		if(bytes != null)
			DATA_CACHE.set(key,bytes);
	}
	public static void removeOTAData(String key) {
		DATA_CACHE.remove(key);
	}

    public static void set(OrderEntity oe,byte[] allData){
	    if(oe != null){
	        String terminalNo = oe.getId();
            if(checkSignal(terminalNo)){
                String value = formatData(oe,true, HexUtil.bytes2HexStr(allData));
				saveDataToCache(terminalNo,value);
            }
        }
    }

    private static void saveDataToCache(String terminalNo,String value){
		String key = findDataKey(terminalNo);// 数据key前缀
		Object obj = DATA_CACHE.getIfPresent(key);
		if(obj != null){
			if(obj instanceof  List){
				List<String> lt = (List<String>)obj;
				if(lt.size() > MAX_SIZE)lt.remove(0);
				lt.add(value);
				return;
			}
		}
		List<String> list = new ArrayList<>();
		list.add(value);
		DATA_CACHE.set(key, list);
	}
    private static String formatData(OrderEntity oe,boolean isSend){
		return formatData(oe,isSend,HexUtil.bytes2HexStr(oe.getAllData()));
    }
    private static final String SEND_MAG = "发送";
    private static final String REQ_MAG = "接收";
    private static final String SIXTEEN_TAG = "H";
    private static String formatData(OrderEntity oe,boolean isSend,String allData){
		String dateTime = DateTimeUtils.formatDate(DateTimeUtils.FORMAT_NETTY);
		String direction = isSend?SEND_MAG:REQ_MAG;
		String data = HexUtil.bytes2HexStr(oe.getData());
		String controllerCode = HexUtil.byte2HexStr((byte) oe.getOrderCode());//控制码
		String commCode = HexUtil.byte2HexStr((byte) oe.getOperationCode());//命令码
		String remark = OperationCode.valueOfName(oe.getOperationCode());
		return String.format(SEND_DATA_FAD,dateTime,direction,oe.getId(),controllerCode+SIXTEEN_TAG,commCode+SIXTEEN_TAG,allData,data,remark);
    }
	/**
	 * 添加缓存
	 */
	public static void set(String terminalNo,String value){
		if(checkSignal(terminalNo)){
			String key = findDataKey(terminalNo);// 数据key前缀
			Object obj = DATA_CACHE.getIfPresent(key);
			if(obj != null){
				if(obj instanceof  List){
					List<String> lt = (List<String>)obj;
					lt.add(value);
					return;
				}
			}
			List<String> list = new ArrayList<>();
			list.add(value);
			DATA_CACHE.set(key, list);
		}
	}
	// 校验是否在信号量中 如果在说明需要增加数据
	private static boolean checkSignal(String terminalNo){
		return SIGNAL_CACHE.getIfPresent(findSignKey(terminalNo)) != null;
	}
    private static final List<String> EMPTY = new ArrayList<>(0);

	/**
	 * 获取缓存值
	 */
	public static List<String>  get(String terminalNo){
		SIGNAL_CACHE.set(findSignKey(terminalNo), true);// 这里增加信号量
		// 这里获取数据当前数据
		Object obj = DATA_CACHE.getIfPresent(findDataKey(terminalNo));
		if(obj == null)return EMPTY;
		return (List<String>) obj;
	}



	public static void removeProtocolDataList(String key) {
		DATA_CACHE.remove(findDataKey(key));
	}
	public static void main(String[] args){
		byte[] bytes = new byte[]{104, 0, 16, 0, 80, 2, -128, 104, -128, 2, -15, 1, 38, 22};
		String str = HexUtil.bytes2HexStr(bytes);
		System.out.println(str);
	}


}
