package com.comtika.safe.core.protocol.entity;

import com.comtika.safe.core.protocol.eunms.OnOffType;
import lombok.Data;

import java.util.Objects;

/**
 * @desc	:	故障是否自动上报
 * @author	:	actorLucky[494977551@qq.com]
 * @date	:	2018年5月21日 下午3:39:47
 */
@Data
public class FaultEnableReport{

	public FaultEnableReport() {
		super();
	}
	public FaultEnableReport(byte[] by){
		Objects.requireNonNull(by);
		int size = by.length;
		if(size != 16){
			throw new RuntimeException("创建该对象必须传大小为16字节数组");
		}
		this.tooV = OnOffType.valueOf(by[0]);
		this.oweV = OnOffType.valueOf(by[1]);
		this.tooI = OnOffType.valueOf(by[2]);
		this.tooLoad = OnOffType.valueOf(by[3]);
		this.tooHot = OnOffType.valueOf(by[4]);
		this.loadConFail = OnOffType.valueOf(by[5]);
		this.protectI = OnOffType.valueOf(by[6]);
		this.groundFail = OnOffType.valueOf(by[7]);
		this.chargeTimeout = OnOffType.valueOf(by[8]);
	}
	//一件设置默认值
	public FaultEnableReport(boolean defaultValue){
		int dv = OnOffType.OFF.getId();
		if(defaultValue){
			dv = OnOffType.ON.getId();
		}
		this.tooV = OnOffType.valueOf(dv);
		this.oweV = OnOffType.valueOf(dv);
		this.tooI = OnOffType.valueOf(dv);
		this.tooLoad = OnOffType.valueOf(dv);
		this.tooHot = OnOffType.valueOf(dv);
		this.loadConFail = OnOffType.valueOf(dv);
		this.protectI = OnOffType.valueOf(dv);
		this.groundFail = OnOffType.valueOf(dv);
		this.chargeTimeout = OnOffType.valueOf(dv);
	}
	/**
	 * 故障字：Bit0：过压；Bit1：欠压；Bit2：过流；
		Bit3：过载；Bit4：过温；
		Bit5：负载连接失败(请先确保连接负载，再下订单)
		Bit6：漏电流保护；Bit7：接地故障；
		Bit8：充电超时；
	 */
	private OnOffType tooV;//过压
	private OnOffType oweV;//欠压
	private OnOffType tooI;//过流
	private OnOffType tooLoad;//过载
	private OnOffType tooHot;//过温
	private OnOffType loadConFail;//负载连接失败(请先确保连接负载，再下订单)
	private OnOffType protectI;//漏电流保护
	private OnOffType groundFail;//接地故障
	private OnOffType chargeTimeout;//充电超时

	@Override
	public String toString() {
		/**
		 *  故障字：Bit0：过压；Bit1：欠压；Bit2：过流；
		Bit3：过载；Bit4：过温；
		Bit5：负载连接失败(请先确保连接负载，再下订单)
		Bit6：漏电流保护；Bit7：接地故障；
		Bit8：充电超时；
		 */
		return "FaultEnableReport [tooV(过压)=" + tooV.toString() + ", oweV(欠压)=" + oweV.toString() + ", tooI(过流)="
				+ tooI.toString() + ", tooLoad(过载)=" + tooLoad.toString()+ ", tooHot(过温)=" + tooHot.toString()
				+ ", loadConFail(负载连接失败(请先确保连接负载，再下订单))=" + loadConFail.toString() + ", protectI(漏电流保护)=" + protectI.toString()
				+ ", groundFail(接地故障)=" + groundFail.toString() + ", chargeTimeout(充电超时)="
				+ chargeTimeout.toString() + "]";
	}


}
