package com.comtika.safe.core.protocol.entity.device;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.protocol.ComtikaExtra;
import com.comtika.safe.core.protocol.eunms.device.EquipmentCode;
import lombok.Data;

/**
 *
 * @ClassName: EquipNodeParams
 * @Description: 16路新版本设备节点参数/查询节点參数
 *  * @author actor_miss
 * @date2018年9月7日 下午2:35:09
 * @version V1.2.1
 */
@Data
public class EquipNodeParams extends ComtikaExtra {

	public EquipNodeParams(){
		this.terminalCode = EquipmentCode.GPRS_16;
	}
	private EquipmentCode terminalCode;
	private boolean lookup;//是否查询集中器參数
	private boolean single;//是否单点配置 只有在配置时，有用

	private Boolean fullParams;//所有參数 但这个參数为true时，配置所有集中器下的參数

	/**
	 * 11	6Bs	NodeID	节点ID号
17	2Bs	Current_Below	停充结算下限阈值，单位1mA（0~1000）
19	1B	Temp_Above	温度保护上限阈值，单位1℃（60~120）
20	2Bs	Current_Above	电流保护上限阈值，单位mA（0~5000）
22	2Bs	Power_Above	功率保护上限阈值，单位0.1W(0~11000)
24	1B
25	1B
26	1B
27	1B
28	1B
29	2Bs	Voltage_Below	电压保护下限阈值，单位0.1V（800 ~1200）
31	2Bs	Voltage_Above	电压保护上限阈值，单位0.1V（2200~2900）
33	1B 	Plug_Nb 	插座编号,从0开始
34	4Bs	Signal_ID 	消息id号。接收和发送保持一致。
	 */
	/**集中器编号*/
	private String terminalNo;

	/**节点编号*/
	private String nodeNo;
	/**插座编号*/
	private Integer plugNo;//0-7

	/**最小电流*/
	private Integer minI;//0~1000
	/**最大电流*/
	private Integer maxI;//0~5000
	/**最大温度*/
	private Integer maxT;//60~120
	/**最大功能*/
	private Integer maxP;//0~11000
	/**最大电压*/
	private Integer maxV;//2200~2900
	/**最小电压*/
	private Integer minV;//800 ~2200

	public RespWrapper<Boolean> chechParams(){
		if(StringUtil.isEmpty(nodeNo)) return RespWrapper.makeFailResp("节点编号不能为空");
		if(plugNo == null || plugNo > 7 || plugNo < 0)  return RespWrapper.makeFailResp("请检查插座编号");
		if(minI == null || minI < 0 || minI > 1000)  return RespWrapper.makeFailResp("请检查停充结算下限阈值");
		if(maxI == null || maxI < 0 || maxI > 5000 ) return RespWrapper.makeFailResp("请检查电流保护上限阈值");
		if(maxT == null || maxT < 60 || maxT > 120 ) return RespWrapper.makeFailResp("请检查温度保护上限阈值");
		if(maxP == null || maxP < 0 || maxP > 11000 ) return RespWrapper.makeFailResp("请检查功率保护上限阈值");
		if(maxV == null || maxV < 2200 || maxV > 2900 ) return RespWrapper.makeFailResp("请检查电压保护上限阈值");
		if(minV == null || minV < 800 || minV > 2200 ) return RespWrapper.makeFailResp("请检查电压保护下限阈值");

		return RespWrapper.makeSuccessResp(true);
	}
}
