package com.comtika.safe.core.protocol.entity.safe;

import com.comtika.safe.core.protocol.utils.BaseTypeConvert;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;


/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 16:05 2020/8/21
 * @DESC:
 **/
@Data
public class WarnErrHelp {
    private boolean preUpV;//过压预警0
    private boolean preLowV;// 欠压预警1
    private boolean preUpI;// 过流预警2
    private boolean preUpP;// 过功预警3
    private boolean preUpT;// 过温预警4
    private boolean preLeakI;// 漏电预警5
    private boolean lv6;//预留6
    private boolean preMainUpT;//主机过温预警 （节点 ID 为集中器时有效）7
    private boolean warnUpV;//过压报警8
    private boolean warnLowV;//欠压报警9
    private boolean warnUpI;//过流报警10
    private boolean warnUpP;//过功报警11
    private boolean warnUpT;//过温报警12
    private boolean warnLeakI;//漏电报警13
    private boolean warnShort;//短路报警14
    private boolean warnMainUpT;//主机过温报警15
    private boolean warnSmoke;// 烟雾报警 16

    @Override
    public String toString(){
        StringBuffer sb = new StringBuffer();
        if(preUpV)sb.append("过压预警,");
        if(preLowV) sb.append("欠压预警,");
        if(preUpI) sb.append("过流预警,");
        if(preUpP) sb.append("过功预警,");
        if(preUpT) sb.append("过温预警,");
        if(preMainUpT) sb.append("主机过温预警,");
        if(warnUpV) sb.append("过压报警,");
        if(warnLowV) sb.append("欠压报警,");
        if(warnUpI) sb.append("过流报警,");
        if(warnUpP) sb.append("过功报警,");
        if(warnUpT) sb.append("过温报警,");
        if(warnLeakI) sb.append("漏电报警,");
        if(warnShort) sb.append("短路报警,");
        if(warnMainUpT) sb.append("主机过温报警,");
        if(warnSmoke) sb.append("烟雾报警,");
        String s = sb.toString();
        if(s.length() == 0) return "暂没有预警与报警";
        return s.substring(0,s.length()-1);
    }
    public boolean isPreWarn(){
        if(preUpV || preLowV || preUpI || preUpP || preUpT || preMainUpT) return true;
        return false;
    }
    public boolean fadIsPreWarn(int type){
        switch (type){
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
            case 16:
                return true;
        }
        return false;
    }


    public Map<Integer,String> findError(){
        Map<Integer,String> res = new HashMap<>();
        if(preUpV)res.put(1,"过压预警");
        if(preLowV) res.put(2,"欠压预警");
        if(preUpI) res.put(3,"过流预警");
        if(preUpP) res.put(4,"过功预警");
        if(preUpT) res.put(5,"过温预警");
        if(preMainUpT) res.put(6,"主机过温预警");
        if(warnUpV) res.put(7,"过压报警");
        if(warnLowV) res.put(8,"欠压报警");
        if(warnUpI) res.put(9,"过流报警");
        if(warnUpP) res.put(10,"过功报警");
        if(warnUpT) res.put(11,"过温报警");
        if(warnLeakI) res.put(12,"漏电报警");
        if(warnShort) res.put(13,"短路报警");
        if(warnMainUpT) res.put(14,"主机过温报警");
        if(warnSmoke) res.put(15,"烟雾报警");
        if(preLeakI) res.put(16,"漏电预警");
        return res;
    }


    public static WarnErrHelp createWarnHelp(byte[] bytes,String nodeNo) throws Exception {
        WarnErrHelp help = new WarnErrHelp();
        byte[] lv1 = BaseTypeConvert.byte2ExpectBytes(bytes[3], 8);
        byte[] lv2 = BaseTypeConvert.byte2ExpectBytes(bytes[2], 8);
        // 这里时主机报警
        if(nodeNo.endsWith("80")){
            // 主板
            help.preMainUpT = convert(lv1[7]);
            help.warnMainUpT = convert(lv2[7]);
            byte[] lv3 = BaseTypeConvert.byte2ExpectBytes(bytes[1], 8);
            help.warnSmoke = convert(lv3[0]);
        }else {
            // 下面这里时子板报警
            help.preUpV = convert(lv1[0]);
            help.preLowV = convert(lv1[1]);
            help.preUpI = convert(lv1[2]);
            help.preUpP = convert(lv1[3]);
            help.preUpT = convert(lv1[4]);
            help.preLeakI = convert(lv1[5]);
            help.warnUpV = convert(lv2[0]);
            help.warnLowV = convert(lv2[1]);
            help.warnUpI = convert(lv2[2]);
            help.warnUpP = convert(lv2[3]);
            help.warnUpT = convert(lv2[4]);
            help.warnLeakI = convert(lv2[5]);
            help.warnShort = convert(lv2[6]);
        }
        return help;
    }

    private static boolean convert(int t) {
        return t == 1 ? true : false;
    }


}
