package com.comtika.safe.core.protocol.netty;

import com.comtika.safe.core.protocol.netty.config.NettyChannelInitializer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/***
 * netty 服务端
 */
@Component
@Log4j2
public class NettyServer implements Runnable{

    private static EventLoopGroup boosGrop = null;
    private static EventLoopGroup workGrop = null;

    @Autowired
    private NettyChannelInitializer nettyChannelInitializer;

    private static ExecutorService handle = Executors.newSingleThreadExecutor((t)->new Thread(t,"netty 线程池"));

    public void start(){
        handle.execute(this);
    }

    public void stop(){
        closeLoopGroup();
        handle.shutdown();
    }
    private void closeLoopGroup(){
        if(workGrop != null){
            workGrop.shutdownGracefully();
        }
        if(boosGrop != null){
            boosGrop.shutdownGracefully();
        }
    }
    /**开始*/
    @Override
    public void run(){
        boosGrop = new NioEventLoopGroup(1);
        workGrop = new NioEventLoopGroup();
        try {
            ServerBootstrap b = new ServerBootstrap();//(2)
            b.group(boosGrop,workGrop)
                    .channel(NioServerSocketChannel.class)//(3)
                    .childHandler(nettyChannelInitializer)
                    .option(ChannelOption.SO_BACKLOG, 2048)// (5)
                    .option(ChannelOption.SO_REUSEADDR,true)
                    .childOption(ChannelOption.SO_KEEPALIVE, true); // (6)
            // 绑定端口，开始接收进来的连接
            ChannelFuture f = b.bind(9091).sync(); // (7)
            log.info("服务器启动完成，监听端口： " + 9091 );
            // 等待服务端监听端口关闭
            f.channel().closeFuture().sync();
        }catch (Exception e){
           // e.printStackTrace();
            log.error("netty 启动失败，原因:{}",e.getMessage());
            System.exit(1);
        }finally {
            closeLoopGroup();
        }
    }


}
