package com.comtika.safe.core.protocol.netty;

import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.protocol.entity.netty.OrderEntity;
import com.comtika.safe.core.protocol.ProtocolConstant;
import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.cache.SafeCache;
import com.comtika.safe.core.protocol.eunms.netty.OperationCode;
import com.comtika.safe.entity.equip.terminal.CtkEquipTerminal;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;


//SimpleChannelInboundHandler   ChannelInboundHandlerAdapter
@Component
@ChannelHandler.Sharable
@Log4j2
public class NettyServerHandle extends ChannelInboundHandlerAdapter  {//ChannelInboundHandlerAdapter  //ChannelHandlerAdapter


    @Autowired
    private ProtocolHandle protocolHandle;

    private boolean checkTerminNoRight(String terminalNo){
        if(StringUtil.isEmpty(terminalNo)) return false;
        if(terminalNo.endsWith(ProtocolConstant.TERMINAL_END_NO)){
            return true;
        }
        return false;
    }
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        try{
            OrderEntity oe = (OrderEntity)msg;
            SafeCache.set(oe,false);//心电跟踪
            // System.out.println("获取的协议数据：["+oe.toJson()+"]时间："+ DateTimeUtils.formatDate(DateTimeUtils.FORMAT));
            //TODO 增加校验规则 判断集中器是否一80为结尾
            if(!checkTerminNoRight(oe.getId())){
                log.error("订单编号非法,订单编号为：{},实体数据为：{}，上报订单原始数据为：",oe.getId(),oe,Arrays.toString(oe.getAllData()));
                ctx.close();
                return;
            }
            if (protocolHandle.chechTerminalWhiteList(ctx,oe.getId())) {////判断是否在黑白名单中
                return;
            }
            /**更新channel*/
            LocalProtocolCache.judgeTerminalCache(oe.getId(), ctx);
            OperationCode operationCode = OperationCode.valueOf(oe.getOperationCode());
            if(operationCode == null){
                // 没有指明的协议
                protocolHandle.otherProtocolHandle(oe);
                // ctx.close();
                // LOGGER.error(LogsFactory.build(SystemLogsType.ADD_LOG,"没有该协议的指令码","上报实体："+oe.toJson()));
                return;
            }
            LocalProtocolCache.setHeartbeat(oe.getId());// 更新心跳
            /**集中器在线列表，如果在列表中（只有在用户登入后添加列表，集中器掉线需要将该值移除）*/
            if(LocalProtocolCache.TERMINAL_ONLINE_LIST.containsKey(oe.getId())){
                if(operationCode == OperationCode.LOGIN){
                    protocolHandle.login(oe,ctx);
                    return;
                }
            }else{
                /** 查询集中器是否配置过*/
                CtkEquipTerminal ctkt = protocolHandle.findTerminal(ctx,oe.getId());
                LocalProtocolCache.TERMINAL_ONLINE_LIST.put(oe.getId(),true);//添加到在线列表中
                LocalProtocolCache.TERMINAL_WHITE_LIST.put(ctkt.getTerminalNo(),true);//加入白名单
                boolean tmpBl = false;
                if(operationCode == OperationCode.LOGIN){//如果是登入 直接响应登入
                    protocolHandle.login(oe,ctx);
                    protocolHandle.removeTerminalBlacklist(oe.getId());
                    tmpBl = true;
                }
                /*** 模拟登入*/
                protocolHandle.simulationTerminalLogin(ctkt);
                if (tmpBl) return;
            }
            switch (operationCode) {
                case SKIP:
                    protocolHandle.heartbeatSkip(oe, ctx);
                    break;
                case SAFE_DATA_UP:
                    protocolHandle.handleTerminalRepData(oe,ctx);
                    break;
                //case UPDATE_IP_PORT:
                //case SET_BACKUP_IP_PORT:
                case SET_HOST_IPPORT:
                    protocolHandle.updateTerminalProtocol(oe);
                    break;
                //case GET_BACKUP_IP_PORT:
                //case GET_IP_PORT:
                case GET_HOST_IPPORT:
                    protocolHandle.findTerminalProtocol(oe);
                    break;
                case GET_DETAIL_HD:
                    protocolHandle.findHardwareVersion(oe);
                    break;
                case SET_NODE_PARAM:
                    protocolHandle.setNodeParams(oe);
                    break;
                case SYS_CONTROL:
                    protocolHandle.systemController(oe);
                    break;
                // case SET_NODE_PATTERN:
                case SET_NODE_RTC:
                    protocolHandle.setNodeRTCOrWord(oe);
                    break;
                case GET_NODE_STATUS:
                    protocolHandle.findNodeStatus(oe);
                    break;
                case GET_NODE_CDCS:
                    protocolHandle.findNodeChargerData(oe);
                    break;
                case GET_NODE_VERSION:
                    protocolHandle.findNodeVersion(oe);
                    break;
                case GET_NODE_PARAM:
                    protocolHandle.findNodeParams(oe);
                    break;
                // case GET_NODE_RECORD:
                //     protocolHandle.findNodeChargerRecord(oe);
                //     break;
                case CHARGE_START:
                    protocolHandle.prepareChargerStart(oe);
                    break;
                case CHARGE_END:
                    protocolHandle.prepareChargerEnd(oe);
                    break;
                case NODE_EXCEPTION:
                    protocolHandle.activeEndCharger(oe);
                    break;
                case NODE_WARN_DATA:
                    protocolHandle.terminalWarnData(oe, ctx);
                    break;
                case UP_DATA:
                    protocolHandle.terminalUpData(oe);
                    break;
                case AUTO_CARD:
                    protocolHandle.terminalAutoCard(oe);
                    break;
                case OTA_ASK_OK:
                case OTA_NOTICE_GO:
                case OTA_READ_STORGE:
                case OTA_SEND_END:
                case OTA_SEND_RUN:
                case OTA_UPDATE_START:
                    protocolHandle.terminalLongOrder(oe);
                    break;
                case TERMINAL_POWER_UP:
                    protocolHandle.terminalPowerUpEvent(oe,ctx);
                    break;
                case UPDATE_TERMINAL_MAC:
                case FIND_TERMINAL_MAC:
                    protocolHandle.terminalMACHandle(oe);
                    break;
                case UPDATE_TERMINAL_TIME:
                case FIND_TERMINAL_TIME:
                    protocolHandle.updateTerminalTime(oe,operationCode);
                    break;
                case FIND_ELE_TOTAL:
                    protocolHandle.findEleTotalStatus(oe);
                    break;
                case PROTO_ICCID:// 集中器主动上报iccid
                case PROTO_ICCID_UP: // 查询集中器iccid
                    protocolHandle.handleTerminalIccId(oe,operationCode,ctx);
                    break;
                case PROTO_IMEI://集中器主动上报imei
                case PROTO_IMEI_UP:// 查询集中器imei
                    protocolHandle.handleTerminalImei(oe,operationCode,ctx);
                    break;
                //白名单
                case FIND_MATER_NODE_WHITE_LIST:
                case SET_MATER_NODE_WHITE_LIST:
                case FIND_TERMINAL_WHITE_LIST:
                case FIND_TERMINAL_ONLINE_WHITE_LIST:
                    protocolHandle.terminalWhileList(oe,operationCode);
                    break;
                case SAFE_WARN_UP:
                    // 这里直接响应数据
                    protocolHandle.handelWarnData(oe,ctx,operationCode);
                    break;
                case SAFE_THR_FIND:
                    protocolHandle.handleFindTerminalParams(oe,ctx);
                    break;
                case SAFE_CONFIG_THR:
                    protocolHandle.handleSetTerminalParams(oe);
                    break;
                case SAFE_ELE_DELETE:
                case SAFE_SWITCH_OFF:
                case SAFE_SWITCH_ON:
                case SAFE_TASK_ADD:
                case SAFE_TASK_DELETE:
                    protocolHandle.handleBooleanResult(oe);
                    break;
                case SAFE_DATA_FIND:
                    protocolHandle.handleFindReportData(oe);
                    break;
                case SAFE_TASK_FIND:
                    protocolHandle.handleFindSafeTask(oe);
                    break;
                case SAFE_SWITCH_FIND:
                    protocolHandle.handleFindTerminalStatus(oe);
                    break;
                case HANDLE_AUX_STATUS:
                    protocolHandle.handleAuxStatus(oe);
                    break;
                case FIND_SERVER_TIME://设备查询服务器时间
                    protocolHandle.findServerTimeHandle(oe,ctx);
                    break;
                case CTJ_FIND_TIME_CONF:
                case CTJ_CONF_TIME_HANDLE:// timer 查询与配置
                    protocolHandle.handleTimerData(oe,operationCode);
                    break;
                default:
                    // RespResult.buidException("没有对应的指令处理码，上报数据为：" + oe.toJson());
                    protocolHandle.otherProtocolHandle(oe);
                    break;
            }
        } catch (Exception e) {
            log.error("在协议层错误:{}", e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        ctx.flush();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        if(!ctx.channel().isActive()){
            cause.printStackTrace();
            ctx.close();
        }else {
            //super.exceptionCaught(ctx,cause);
            System.out.println("异常信息："+cause.getMessage());
        }
    }

}
