package com.comtika.safe.core.protocol.netty;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.exception.RespException;
import com.comtika.safe.common.response.RespResult;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.encrypt.HexUtil;
import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.cache.ProtocolRedisCache;
import com.comtika.safe.core.protocol.entity.AuxHandleHelp;
import com.comtika.safe.core.protocol.entity.DeviceParamsHelp;
import com.comtika.safe.core.protocol.entity.WattMeterHelp;
import com.comtika.safe.core.protocol.entity.charge.ChargeBase;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.device.EquipNodeParams;
import com.comtika.safe.core.protocol.entity.netty.OrderEntity;
import com.comtika.safe.core.protocol.entity.netty.ResProtocol;
import com.comtika.safe.core.protocol.entity.node.*;
import com.comtika.safe.core.protocol.entity.safe.BitHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskHelp;
import com.comtika.safe.core.protocol.entity.safe.WarnDataHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerQueryHelp;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.eunms.AckCode;
import com.comtika.safe.core.protocol.eunms.device.EquipmentCode;
import com.comtika.safe.core.protocol.eunms.netty.OperationCode;
import com.comtika.safe.core.protocol.eunms.netty.OrderCodeType;
import com.comtika.safe.core.protocol.repData.TerminalRepDataHandle;
import com.comtika.safe.core.protocol.responseUtil.DefaultFuture;
import com.comtika.safe.core.protocol.utils.BaseTypeConvert;
import com.comtika.safe.core.protocol.utils.OrderDataConvertUtil;
import com.comtika.safe.core.protocol.utils.ProtocolIDUtil;
import com.comtika.safe.core.protocol.utils.SafeDataConvertUtil;
import com.comtika.safe.dao.equip.terminal.EquipTerminalDao;
import com.comtika.safe.entity.equip.terminal.CtkEquipTerminal;
import com.comtika.safe.entity.report.RepAlwaysData;
import com.comtika.safe.entity.safe.RepDataRecord;
import com.comtika.safe.service.safe.IRepService;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author actor_miss[494977551@qq.com]
 * @date 2019/1/12 17:32
 */
@Component
@Log4j2
public class ProtocolHandle {
//    @Autowired
//    private AmqpTemplate amqpTemplate;
    @Autowired
    private TerminalRepDataHandle terminalRepDataHandle;

    @Autowired
    private ProtocolRedisCache protocolRedisCache;

    @Autowired
    private EquipTerminalDao terminalDao;
   /* @Autowired
    private ProtocolService protocolService;*/
    /**登入*/
    public void login(OrderEntity oe, ChannelHandlerContext ctx) {
        OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
        oe.setOrderCode(valueOf.getResCode());
        oe.setData(new byte[]{(byte) AckCode.SUCCESS.getId()});
        Channel channel = ctx.channel();
        channel.writeAndFlush(oe);
        // 这里增加同步时间接口
        // ProtocolService
        // 登陆设置时间
        // synchronizationDeviceTime(oe.getId(),channel);
    }
    public void synchronizationDeviceTime(String terminalNo,Channel channel){
        OrderEntity orderEntity = new OrderEntity(terminalNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.UPDATE_TERMINAL_TIME.getCode(), BaseTypeConvert.createTerminalTime(System.currentTimeMillis()));
        // protocolService.fadSetDeviceTime(oe.getId(),)
        channel.writeAndFlush(orderEntity);
    }
    // 设备请求服务器时间
    public void findServerTimeHandle(OrderEntity oe, ChannelHandlerContext ctx) {
       // OrderEntity orderEntity = new OrderEntity(terminalNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.UPDATE_TERMINAL_TIME.getCode(), BaseTypeConvert.createTerminalTime(System.currentTimeMillis()));
        // protocolService.fadSetDeviceTime(oe.getId(),)
        oe.setOrderCode(OrderCodeType.CLIENT_PARAM_SET.getResCode());
        oe.setData(BaseTypeConvert.createTerminalTime(System.currentTimeMillis()));
        ctx.channel().writeAndFlush(oe);
    }
    /**心跳*/
    public void heartbeatSkip(OrderEntity oe,ChannelHandlerContext ctx) {
        OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
        // LocalProtocolCache.setHeartbeat(oe.getId());
        oe.setOrderCode(valueOf.getResCode());
        oe.setData(new byte[]{(byte) AckCode.SUCCESS.getId()});
        ctx.channel().writeAndFlush(oe);
    }
    /**内部发送数据类  这里防止有redis中的sgin不断增加*/
    private void signSendData(String sign,Object o){
        DefaultFuture.received(sign,o);//使用自己封装的异步回调
    }
    /**更新集中器协议*/
    public void updateTerminalProtocol(OrderEntity oe) {
        ResProtocol res = new ResProtocol(AckCode.valueOf(oe.getData()[0]),oe.getId()+oe.getOperationCode(),null);
        signSendData(oe.getId()+oe.getOperationCode(),res);
    }
    /**F9 设置ip 查询集中器远程更新IP和端口 FDH*/
    public void findTerminalProtocol(OrderEntity oe) {
        String sign = oe.getId()+oe.getOperationCode();
        ResProtocol<Map<String,String>> res = new ResProtocol<>();
        do{
            if(oe.getData().length == 0){//如果数据为空 则返回
                res.setSign(sign);
                Map<String, String> mp = new HashMap<>();
                mp.put("resMsg", "你还没有配置:【"+ OperationCode.valueOf(oe.getOperationCode()).getName()+"】的ip与端口");
                res.setData(mp);
                break;
            }
            Map<String, String> data = RespResult.resDataChange(OrderDataConvertUtil.getIpMapByByte(oe.getData()));
            res.setData(data);
            res.setSign(sign);
        }while (false);
        signSendData(sign,res);
    }
    //查询集中器软硬件信息 4AH 这里应该更新到数据库  并且给一个接口更新这边的信息    平时数据从数据库查询
    public void findHardwareVersion(OrderEntity oe) {
        HardwareVersion hardwareStatus =   RespResult.resDataChange(OrderDataConvertUtil.getHardwareStatus(oe.getData()));
        ResProtocol<HardwareVersion> hv = new ResProtocol<>();
        hv.setData(hardwareStatus);
        hv.setAckCode(AckCode.SUCCESS);
        String sign = oe.getId()+oe.getOperationCode();
        hv.setSign(sign);
        signSendData(sign,hv);
    }
    /**	设置节点消息的我们使用签名当key   配置节点参数  01H*/
    public void setNodeParams(OrderEntity oe) {
        byte[] data = oe.getData();
        if(data.length == 6){
            ResProtocol<Integer> res = new ResProtocol<>();
            res.setAckCode(AckCode.valueOf(data[0]));
            res.setData((int)data[1]);
            String sign = ProtocolIDUtil.decodeEvenProtocolID(new byte[]{data[2],data[3],data[4],data[5]});
            res.setSign(sign);
            signSendData(sign,res);
            return;
        }
        throw new RespException("集中器配置參数响应有误");
    }

    /** 判断数据是否是   字节为4/1  系统控制*/
    public void systemController(OrderEntity oe) {
        byte[] data = oe.getData();
        String sign = oe.getId()+oe.getOperationCode();
        ResProtocol<String> res = new ResProtocol<>();
        int size = data.length;
        res.setSign(sign);
        if(size == 4 && data[0] == 0x73){//读取集中器软件版本号
            String str = ProtocolIDUtil.decodeEvenProtocolID(new byte[]{data[1],data[2],data[3]});
            if(str != null){
                res.setAckCode(AckCode.SUCCESS);
                res.setData(str);
            }else res.setData("没有找到对应的版本！");
        }else if(size == 1){
            AckCode ac = AckCode.valueOf(data[0]);
            if(ac != null) res.setAckCode(ac);
            else res.setData("没有对应的响应码！");
        }else res.setData("未知原因！");
        signSendData(sign,res);
    }
    //设置节点工作模式 02H 设置节点RTC  03H
    public void setNodeRTCOrWord(OrderEntity oe) {
        byte[] bs = oe.getData();
        if(bs.length == 5){
            ResProtocol res = new ResProtocol();
            String sign = ProtocolIDUtil.decodeEvenProtocolID(new byte[]{bs[1],bs[2],bs[3],bs[4]});
            res.setAckCode(AckCode.valueOf(bs[0]));
            res.setSign(sign);
            signSendData(sign,res);
            return;
        }
        throw new RespException("配置节点工作模式或配置节点RTC有误！");
    }
    /**查询节点状态*/
    public void findNodeStatus(OrderEntity oe) {
        NodeInfo data = RespResult.resDataChange(OrderDataConvertUtil.getNodeInfo(oe.getData(), false));
        ResProtocol<NodeInfo> res = new ResProtocol<>();
        String sign = data.getExtraString(ProjectConstant.SIGN_ID);
        res.setData(data);
        res.setAckCode(data.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**查询节点充电数据*/
    public void findNodeChargerData(OrderEntity oe) {
        NodeChargeData data = RespResult.resDataChange(OrderDataConvertUtil.getNodeChargeData(oe.getData(), true));
        ResProtocol<NodeChargeData> res = new ResProtocol<>();
        String sign = data.getExtraString(ProjectConstant.SIGN_ID);
        res.setData(data);
        res.setSign(sign);
        res.setAckCode(data.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        signSendData(sign,res);
    }
    /**查询节点版本*/
    public void findNodeVersion(OrderEntity oe) {
        NodeVersion data = RespResult.resDataChange(OrderDataConvertUtil.getNodeVersion(oe.getData()));
        ResProtocol<NodeVersion> res = new ResProtocol<>();
        String sign = data.getExtraString(ProjectConstant.SIGN_ID);
        res.setAckCode(data.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setData(data);
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**查询节点配置參数*/
    public void findNodeParams(OrderEntity oe) {
        EquipNodeParams sb = RespResult.resDataChange(OrderDataConvertUtil.findNodeParam(oe.getData(), false));
        ResProtocol<EquipNodeParams> res = new ResProtocol<>();
        sb.setTerminalNo(oe.getId());
        String sign = sb.getExtraString(ProjectConstant.SIGN_ID);
        res.setAckCode(sb.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setData(sb);
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**查询节点充电记录*/
    public void findNodeChargerRecord(OrderEntity oe) {
        NodeChargeRecord sb = RespResult.resDataChange(OrderDataConvertUtil.getNodeChargeRecord(oe.getData()));
        ResProtocol<NodeChargeRecord> res = new ResProtocol<>();
        String sign = sb.getExtraString(ProjectConstant.SIGN_ID);
        res.setData(sb);
        res.setAckCode(sb.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**充电开始*/
    public void prepareChargerStart(OrderEntity oe) {
        ChargeBase sb = RespResult.resDataChange(OrderDataConvertUtil.orderChargeStart(oe.getData(), false));
        ResProtocol<ChargeBase> res = new ResProtocol<>();
        String sign = sb.getExtraString(ProjectConstant.SIGN_ID);
        res.setData(sb);
        res.setAckCode(sb.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**结束充电*/
    public void prepareChargerEnd(OrderEntity oe) {
        ChargeEnd sb = RespResult.resDataChange(OrderDataConvertUtil.orderChargeEnd(oe.getData()));
        ResProtocol<ChargeEnd> res = new ResProtocol<>();
        String sign = sb.getExtraString(ProjectConstant.SIGN_ID);
        res.setData(sb);
        res.setAckCode(sb.getExtra(ProjectConstant.ACKCODE, AckCode.class));
        res.setSign(sign);
        signSendData(sign,res);
    }
    /**硬件主动结束充电*/
    public void activeEndCharger(OrderEntity oe) {
        NodeReportData data = RespResult.resDataChange(OrderDataConvertUtil.nodeFaultSendMsg(oe.getData(), false));
        terminalRepDataHandle.terminalRepDataEX(data);
    }
    /**处理告警数据*/
    public void terminalWarnData(OrderEntity oe, ChannelHandlerContext ctx) {
        byte[] by = new byte[8];
        byte[] req = oe.getData();
        System.arraycopy(req, 0, by, 0, 6);
        by[6] = 0;
        by[7] = req[6];
        RespWrapper<NodeWarnData> warnData = OrderDataConvertUtil.convertWarnData(req,false);
        if(warnData.isSuccess()){
            NodeWarnData data = warnData.getData();
            if( data.getList() == null ){
                oe.setData(by);
                ctx.writeAndFlush(oe);
                throw new RespException("告警标识符为空");
            }
            data.setAddrNo(oe.getId());
            data.setEquipmentCode(EquipmentCode.GPRS_16);
            // todo mq 告警后面处理
            //amqpTemplate.convertAndSend(ProjectConstant.MSG_PRO_KEY, ProtocolConstant.PROTOBUF_SERIALIZE.serialize(data));
            by[6] = 1;
            oe.setData(by);
            ctx.channel().writeAndFlush(oe);
            return;
        }
        oe.setData(by);
        ctx.channel().writeAndFlush(oe);
        throw new RespException(warnData.getMsg());
    }
    /**处理上报数据*/
    public void terminalUpData(OrderEntity oe) {
        NodeChargeData data = RespResult.resDataChange(OrderDataConvertUtil.getNodeChargeData(oe.getData(), false));
        terminalRepDataHandle.terminalRepChargeData(data);
    }
    /**智能卡*/
    public void terminalAutoCard(OrderEntity oe) {
        /*byte[] data = oe.getData();
        int size = data.length;
        if(size != 13) throw new RuntimeException("刷卡主动上报数据有误！");
        SmartCard sc = new SmartCard();
        byte[] bt = new byte[6];
        System.arraycopy(data, 0, bt, 0, 6);
        sc.setNodeNo(ProtocolIDUtil.decodeEvenProtocolID(bt));
        sc.setSocketNo(data[6]);
        System.arraycopy(data, 7, bt, 0, 6);
        sc.setCardId(ProtocolIDUtil.decodeEvenProtocolID(bt));
        sc.setAddrNo(oe.getId());
        *//** 智能卡*//*
        amqpTemplate.convertAndSend(ProjectConstant.SMART_CARD_PRO_SERVER, ProtocolConstant.PROTOBUF_SERIALIZE.serialize(sc));
*/    }
    /**超长指令*/
    public void terminalLongOrder(OrderEntity oe) {
        ResProtocol<Integer> res = new ResProtocol<>();
        if(oe.getOrderCode() == OrderCodeType.CON_SUPER_LONG.getResCode()){
            if(oe.getOperationCode() == OperationCode.OTA_READ_STORGE.getCode()){
                String decodeEvenProtocolID = ProtocolIDUtil.decodeEvenProtocolID(oe.getData());
                res.setData(Integer.parseInt(decodeEvenProtocolID, 16));
            }
            res.setAckCode(AckCode.SUCCESS);
        }else{
            res.setAckCode(AckCode.REQ_FAIL);
        }
        signSendData(oe.getId()+oe.getOperationCode(),res);
    }
    /**模拟登入*/
    public void simulationTerminalLogin(CtkEquipTerminal ctkt) {
        terminalRepDataHandle.simulationTerminalLogin(ctkt);//模拟登入
    }



    /**判断集中器编号是否在黑白名单中*/
    public boolean chechTerminalWhiteList(ChannelHandlerContext ctx,String terminalNo){
        /**判断是否在白名单中*/
        if(LocalProtocolCache.TERMINAL_WHITE_LIST.containsKey(terminalNo)) return false;
        /**判断是否在黑名单中*/
        if(protocolRedisCache.checkTerminalBlacklist(terminalNo)){//黑名单
            ctx.close();
            //throw new RespException(String.format("集中器编号%s，在黑名单中",terminalNo));
            System.out.println(String.format("集中器编号%s，在黑名单中",terminalNo));
            return true;
        }
        return false;
    }

    /**查询集中器*/
    public CtkEquipTerminal findTerminal(ChannelHandlerContext ctx,String terminalNo){
        CtkEquipTerminal ctkt = new CtkEquipTerminal();
        ctkt.setTerminalNo(terminalNo);
        ctkt = terminalDao.templateOne(ctkt);
        /**查询集中器失败*/
        if ( ctkt == null ) {
            protocolRedisCache.addTerminalBlacklist(terminalNo);
            ctx.close();
            throw new RespException(String.format("查询集中器编号%s，在黑名单中",terminalNo));
        }
        return ctkt;
    }
    /**清空redis的黑名单*/
    public void removeTerminalBlacklist(String id) {
        protocolRedisCache.removeTerminalBlacklist(id);
    }
    /**集中器上电事件*/
    public void terminalPowerUpEvent(OrderEntity oe,ChannelHandlerContext ctx) {
        try {
            //if(PatternType.TEST == ProjectPattern.TEST_PATTERN) System.out.println("集中器上电事件");
            log.error("集中器上电事件，集中器：{}",oe.getId());
            oe.setOrderCode(0x80);
            ctx.channel().writeAndFlush(oe);
            terminalRepDataHandle.terminalPowerUpEvent(oe.getId());
        }catch (Exception e){
            e.printStackTrace();
        }
    }
    // mac 处理方法
    public void terminalMACHandle(OrderEntity oe) {
        ResProtocol<Boolean> res = new ResProtocol<>();
        String sign = oe.getId()+oe.getOperationCode();
        res.setSign(sign);
        res.setData(true);
        signSendData(sign,res);
    }
    // 更新时间
    public void updateTerminalTime(OrderEntity oe, OperationCode operationCode) {
        ResProtocol<String> res = new ResProtocol<>();
        String sign = oe.getId()+oe.getOperationCode();
        res.setSign(sign);
        if(operationCode == OperationCode.FIND_TERMINAL_TIME){
            res.setData(handleTime(oe.getData()));
        }else {
            res.setData(HexUtil.bytes2HexStr(oe.getData()));
        }
        signSendData(sign,res);
    }
    private static String handleTime(byte[] data){
        if(data.length == 12){
            StringBuilder sb = new StringBuilder();
            sb.append(BaseTypeConvert.bytes2IntBE(new byte[]{data[0],data[1]}));
            sb.append("-");
            sb.append(BaseTypeConvert.byte2Int(data[2]));
            sb.append("-");
            sb.append(BaseTypeConvert.byte2Int(data[3]));
            sb.append(" ");
            sb.append(BaseTypeConvert.byte2Int(data[5]));
            sb.append("-");
            sb.append(BaseTypeConvert.byte2Int(data[6]));
            sb.append("-");
            sb.append(BaseTypeConvert.byte2Int(data[7]));
            sb.append(" 星期");
            sb.append(BaseTypeConvert.byte2Int(data[4]));
            sb.append(" 运行时间为:");
            sb.append(BaseTypeConvert.bytes2IntBE(new byte[]{data[8],data[9],data[10],data[11]}));
            sb.append("s;原始数据为:"+HexUtil.bytes2HexStr(data));
            return sb.toString();
        }
        return "原始数据为:"+HexUtil.bytes2HexStr(data);
    }
    //19年 10月 11日 05星期    03 12 36 EE00001003
    //19101105  031236EE000010F9
    // 07 E2 0C 0C  03 0C 0D   00 00 00 00 30
    // 07 E2 0C 0C  03  0C 0D 00 00 00 00 30
    public static void main(String[] args){
        byte[] bt = new byte[]{7, -29, 10, 13, 7, 15, 38, 56,0,0,0,0x30};
        System.out.println(handleTime(bt));
    }

    public void otherProtocolHandle(OrderEntity oe) {
        ResProtocol res = new ResProtocol<>();
        String sign = oe.getId()+oe.getOperationCode();
        res.setSign(sign);
        res.setData(HexUtil.bytes2HexStr(oe.getData()));
        signSendData(sign,res);
    }
    // 电能查询方法
    public void findEleTotalStatus(OrderEntity oe) {
        ResProtocol<WattMeterHelp> res = new ResProtocol<>();
        String sign = oe.getId()+oe.getOperationCode();
        res.setSign(sign);
        String orgData = HexUtil.bytes2HexStr(oe.getData());
        RespWrapper<WattMeterHelp> data = OrderDataConvertUtil.findEleTotalStatus(oe.getData());
        if(data.isSuccess()){
            WattMeterHelp data1 = data.getData();
            data1.setOrgData(orgData);
            res.setData(data1);
        }else {
            WattMeterHelp help = new WattMeterHelp();
            help.setOrgData(orgData);
            help.setMsg(data.getMsg());
            res.setData(help);
        }
        signSendData(sign,res);
    }
    // 处理设备ICC_ID
    public void handleTerminalIccId(OrderEntity oe, OperationCode operationCode,ChannelHandlerContext ctx) {
        //String iccId = new String(oe.getData());
        String iccId = new String(oe.getData());
        // 更新数据库
        terminalDao.updateTerminalIccId(iccId,oe.getId());
        // 更新成功
        if(operationCode == OperationCode.PROTO_ICCID){
            // 效应上报数据
            OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
            oe.setOrderCode(valueOf.getResCode());
            oe.setData(new byte[0]);
            ctx.channel().writeAndFlush(oe);
        }else{
            String sign = oe.getId()+oe.getOperationCode();
            _doSing(sign,iccId);
        }
    }
    // 处理设备的IMEI
    public void handleTerminalImei(OrderEntity oe, OperationCode operationCode,ChannelHandlerContext ctx) {
        //String imei = new String(oe.getData());
        String imei = new String(oe.getData());
        // 更新数据库
        terminalDao.updateTerminalImei(imei,oe.getId());
        if(operationCode == OperationCode.PROTO_IMEI){
            // 效应上报数据
            OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
            oe.setOrderCode(valueOf.getResCode());
            oe.setData(new byte[0]);
            ctx.channel().writeAndFlush(oe);
        }else{
            String sign = oe.getId()+oe.getOperationCode();
            _doSing(sign,imei);
        }
    }
    private void _doSing(String sign,String key){
        ResProtocol<String> res = new ResProtocol<>();
        res.setAckCode(AckCode.SUCCESS);
        res.setSign(sign);
        res.setData(key);
        signSendData(sign,res);
    }
    // 白名单
    public void terminalWhileList(OrderEntity oe, OperationCode operationCode) {
        String sign = oe.getId()+oe.getOperationCode();
        if(operationCode == OperationCode.FIND_MATER_NODE_WHITE_LIST){
            boolean fla = false;
            if (oe.getData().length == 1 && oe.getData()[0] == 1) {
                fla = true;
            }else log.error("超长指令接受错误:{}",oe);
            signSendData(sign,fla);
        }else if(OperationCode.SET_MATER_NODE_WHITE_LIST == operationCode){
            boolean fla = false;
            if (oe.getData().length == 1 && oe.getData()[0] == 1) {
                fla = true;
            }else log.error("超长指令接受错误:{}",oe);
            signSendData(sign,fla);
        }else if(operationCode == OperationCode.FIND_TERMINAL_WHITE_LIST || operationCode == OperationCode.FIND_TERMINAL_ONLINE_WHITE_LIST){
            // 这里查询已经配置的白名单
            List<String> whiteList = OrderDataConvertUtil.bytesToWhiteList(oe.getData());
            signSendData(sign,whiteList);
        }
    }
    // 直接响应数据给设备端
    public void handelWarnData(OrderEntity oe, ChannelHandlerContext ctx, OperationCode operationCode) {
        byte[] data = oe.getData();
        if (data.length != 25) {
            log.error("告警数据格式错误");
            return;
        }
        try {
            WarnDataHelp dataHelp = SafeDataConvertUtil.reqWarnDataConvert(data);
            dataHelp.setTerminalNo(oe.getId());
            iRepService.handleWarnData(dataHelp);
        }catch (Exception e){
            e.printStackTrace();
            log.error("设备告警数据解析失败");
        }
        byte[] to = new byte[8];
        System.arraycopy(data, 0, to, 0, 6);
        to[6] = 1;
        to[7] = data[6];
        oe.setData(to);
        OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
        oe.setOrderCode(valueOf.getResCode());
        ctx.channel().writeAndFlush(oe);
    }
    // 这里处理安全用电数据查询
    public void handleFindTerminalParams(OrderEntity oe, ChannelHandlerContext ctx) {
        byte[] data = oe.getData();
        RespWrapper<DeviceParamsHelp> params = SafeDataConvertUtil.findTerminalParams(data);
        byte[] bytes = new byte[4];
        // 42 最后一位 --> 41     37 38 39 40
        System.arraycopy(data, 37, bytes, 0, 4);
        String sign = ProtocolIDUtil.decodeEvenProtocolID(bytes);
        ResProtocol<DeviceParamsHelp> res = new ResProtocol<>();
        // String sign = oe.getId()+oe.getOperationCode();
        res.setAckCode(AckCode.SUCCESS);
        res.setData(params.getData());
        res.setSign(sign);
        signSendData(sign,res);
    }

    public void handleSetTerminalParams(OrderEntity oe) {
        byte[] data = oe.getData();
        if (data.length == 12) {
            byte[] bytes = new byte[4];
            // 8 9 10 11
            System.arraycopy(data, 8, bytes, 0, 4);
            String sign = ProtocolIDUtil.decodeEvenProtocolID(bytes);
            //log.error("设置设备阈值响应签名:{}",sign);
            ResProtocol<Boolean> res = new ResProtocol<>();
            res.setAckCode(AckCode.SUCCESS);
            res.setData(true);
            res.setSign(sign);
            signSendData(sign,res);
        }else  log.error("配置参数字节数错误");
    }

    // 清除电能，继电器开关
    public void handleBooleanResult(OrderEntity oe){
        String sign = oe.getId()+oe.getOperationCode();
        ResProtocol<Boolean> res = new ResProtocol<>();
        res.setData(true);
        res.setSign(sign);
        res.setAckCode(AckCode.SUCCESS);
        signSendData(sign,res);
    }

    // 查询任务数据
    public void handleFindSafeTask(OrderEntity oe) {
        String sign = oe.getId()+oe.getOperationCode();
        RespWrapper<List<SafeTaskHelp>> list = SafeDataConvertUtil.handleSafeFindTask(oe.getData());
        if(!list.isSuccess()){
            log.error("查询设备任务解析数据错误：{}",list.getMsg());
        }else {
            ResProtocol<List<SafeTaskHelp>> res = new ResProtocol<>();
            res.setData(list.getData());
            res.setSign(sign);
            res.setAckCode(AckCode.SUCCESS);
            signSendData(sign,res);
        }
    }

    public void handleFindTerminalStatus(OrderEntity oe) {
        String sign = oe.getId()+oe.getOperationCode();
        byte[] data = oe.getData();
        if(data.length != 3){
            log.error("集中器继电器状态查询 响应数组长度错误");
            return;
        }
        try {
            BitHelp help = BitHelp.bytesToBit(new byte[]{data[0], data[1]});
            AckCode ackCode = AckCode.ACK_MAP.get(data[2]);
            ResProtocol<BitHelp> res = new ResProtocol<>();
            res.setData(help);
            res.setSign(sign);
            res.setAckCode(ackCode);
            signSendData(sign,res);
        }catch (Exception e){
            e.printStackTrace();
            log.error("集中器继电器状态查询 解析数值错误");
        }

    }
    @Autowired
    private IRepService iRepService;

    public void handleTerminalRepData(OrderEntity oe, ChannelHandlerContext ctx) {
        byte[] data = oe.getData();
        if (data.length != 27) {
            log.error("实时上报数据长度不是27个字节，请检查");
            return;
        }
        byte[] to = new byte[8];
        System.arraycopy(data, 0, to, 0, 6);
        to[7] = data[26];
        RepDataRecord res = SafeDataConvertUtil.repDataConvert(data);
        res.setTerminalNo(oe.getId());
        // 这里需要处理一些数据
        try {
            iRepService.terminalRepData(res);
        }catch (Exception e){
            log.error("新增上报数据错误");
        }
        to[6] = 1;
        oe.setData(to);
        OrderCodeType valueOf = OrderCodeType.valueOf(oe.getOrderCode());
        oe.setOrderCode(valueOf.getResCode());
        ctx.channel().writeAndFlush(oe);
    }

    public void handleFindReportData(OrderEntity oe) {
        byte[] data = oe.getData();
        if(data.length != 36){
            log.error("查询实时数据长度错误，不是26个字节");
            return;
        }
        try {
            RepAlwaysData rep = SafeDataConvertUtil.handleFindReportData(data);
            //     31 32 33 34
            byte[] bytes = new byte[4];
            System.arraycopy(data, 31, bytes, 0, 4);
            String sign = ProtocolIDUtil.decodeEvenProtocolID(bytes);
            ResProtocol<RepAlwaysData> res = new ResProtocol<>();
            // String sign = oe.getId()+oe.getOperationCode();
            res.setAckCode(AckCode.SUCCESS);
            res.setData(rep);
            res.setSign(sign);
            signSendData(sign,res);
        }catch (Exception e){
            log.error("查询实时数据长度错误，异常：{}",e.getMessage());
        }

    }
    public void handleAuxStatus(OrderEntity oe) {
        byte[] data = oe.getData();
        if (data.length != 3) {
            log.error("集中器辅助继电器控制（开/ 关/ 查询 状态 ）74H,长度错误：{}",oe);
            return;
        }
        byte datum = data[0];
        String sign = oe.getId()+oe.getOperationCode()+datum;
        // 处理数据
        AuxHandleHelp handleHelp = new AuxHandleHelp();
        handleHelp.setStatus((int)datum);
        handleHelp.setTerminalNo(oe.getId());
        if(datum == 2){
            handleHelp.byteToBit(data[1]);
        }
        ResProtocol<AuxHandleHelp> res = new ResProtocol<>();
        res.setData(handleHelp);
        res.setAckCode(AckCode.SUCCESS);
        res.setSign(sign);
        signSendData(sign,res);
    }


    public void handleTimerData(OrderEntity oe, OperationCode operationCode) {
        byte[] data = oe.getData();
        String sign = oe.getId()+oe.getOperationCode();
        if(operationCode == OperationCode.CTJ_FIND_TIME_CONF){
            if(data.length == 18){
                ResProtocol<List<TimerQueryHelp>> res = new ResProtocol<>();
                // 处理数据
                res.setSign(sign);
                res.setAckCode(AckCode.valueOf(data[0]));
                List<TimerQueryHelp> list = new ArrayList<>(16);
                // 这里处理数据
                SafeDataConvertUtil.doHandleTimerData(list,data);
                res.setData(list);
                signSendData(sign,res);
            }else log.error("查询集中器 Timer 配置 E3H 格式错误");

        }else {
            if(data.length == 1){
                ResProtocol<Boolean> res = new ResProtocol<>();
                res.setData(true);
                res.setSign(sign);
                AckCode ackCode = AckCode.valueOf(data[0]);
                res.setAckCode(ackCode);
                signSendData(sign,res);
            }else log.error("设置集中器 Timer 周期 响应E4H 格式错误");

        }

    }
}
