package com.comtika.safe.core.protocol.repData;


import com.comtika.safe.common.enums.equip.terminal.TerminalStatusEnum;
import com.comtika.safe.common.response.RespResult;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.protocol.*;
import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.cache.ProtocolRedisCache;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.entity.node.NodeChargeData;
import com.comtika.safe.core.protocol.entity.node.NodeReportData;
import com.comtika.safe.core.protocol.eunms.charge.ChargeEndType;
import com.comtika.safe.core.protocol.eunms.charge.ChargeResCode;
import com.comtika.safe.core.protocol.eunms.version.OTAStatus;
import com.comtika.safe.core.protocol.utils.FaultDiagnosis;
import com.comtika.safe.dao.equip.terminal.EquipTerminalDao;
import com.comtika.safe.entity.equip.terminal.CtkEquipTerminal;
import com.comtika.safe.service.protocol.ProtocolService;
import lombok.extern.log4j.Log4j2;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.*;


/**
 * @author actor_miss[494977551@qq.com]
 * @date 2018/12/26 16:58
 * 这里是处理硬件上报数据
 */
@Component
@Log4j2
public class TerminalRepDataHandle {
    //订单编号阈值 校验 不在该阈值的订单不做任何处理
    private final static long ORDER_START_THRESHOLD = 1515481860000000L;//20180101
    private final static long ORDER_END_THRESHOLD = 2145888000000000L;//20380101
    private final static long TWO_DAY = 2* DateTimeUtils.ONE_DAY;//2天时间 （查询2天内在充电的订单编号）

    private static final Logger LOGGER = LoggerFactory.getLogger(TerminalRepDataHandle.class);
    @Autowired
    private ProtocolService protocolService;

    @Autowired
    private EquipTerminalDao terminalDao;//集中器

    /*@Autowired
    private HsVesionDao hsVesionDao;*/

    @Autowired
    private ProtocolRedisCache protocolRedisCache;

//    @Autowired
//    private RabbitMQSend rabbitMQSend;
    /**初始化将充电中的订单查询出来*/
    public void init(){
        // 这里初始化设备都设置为离线模式
        initTerminalStatus();
        protocolRedisCache.start();
       // updateChargingList(true);
    }
    // 默认把在线的设备编号放置到掉线检查中
    private void initTerminalStatus(){
        List<String> strings = terminalDao.initTerminalStatus();
        final long startTime = System.currentTimeMillis();
        strings.forEach(s->{
            // 数据库的状态为在线，设备不在线，这时就无法更新状态了；原因是这个原因导致数据库状态与设备状态不一致的
            LocalProtocolCache.HEARTBEAT_MAP.put(s,startTime);//加入到心跳方法中
            // LocalProtocolCache.DROP_LINE_MAP.put(s,startTime); ---> 去掉加入到掉线方法
        });
    }

    public void stop(){
        protocolRedisCache.stop();
    }
    /**更新充电记录**/
    public void updateChargingList(boolean isUpdateSimpleChargeData){
        try {
            final long startTime = System.currentTimeMillis();
            Map<String,Object> map = new HashMap<>();
            map.put("time",System.currentTimeMillis() - TWO_DAY);
            // 查询充电列表
            // todo 充电列表数据
            /*List<String> templateList = terminalDao.findChargingList(map);
            templateList.forEach(terminalNo->{
                protocolRedisCache.addChargeOrderNo(terminalNo);
                if(isUpdateSimpleChargeData){
                    if(!LocalProtocolCache.SIMPLE_CHARGE_DATA.containsKey(terminalNo)){
                        LocalProtocolCache.SIMPLE_CHARGE_DATA.put(terminalNo,startTime);
                    }
                }
            });*/
        }catch (Exception e){
            addErrorLog("%s,时间：%s","自动更新充电列表异常",e);
        }
    }

    /**处理硬件结束数据*/
    public void terminalRepDataEX(NodeReportData data){
        //System.out.println("上报结束数据："+data.toJson());
        if(!protocolRedisCache.checkOrderIsHandle(data.getOrderNo())){
            _terminalRepDataEX(data);
        }
    }
    @Async
    public void _terminalRepDataEX(NodeReportData data){
        try {
            //充电中订单
            if(protocolRedisCache.checkChargeOrderNo(data.getOrderNo())){//订单在充电中
                sendMsgToServer(data,false);
                protocolRedisCache.removeChargeOrderNo(data.getOrderNo());
            }else {//订单不在充电中
                sendMsgToServer(data,true);
            }
            //这里需要响应 设备接受数据成功
            protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.DEL,data.getOrderNo());
            protocolRedisCache.removeUserChargeData(data.getOrderNo());
            RespResult.resDataChange(protocolService.respActiveEndOper(data.getNodeId(), 1, data.getPlugNo()));
        }catch (Exception e){
            addErrorLog("处理上报数据异常，上报数据：%s,时间：%s",data,e);
        }
    }

    /***
     * 发送消息到服务层
     * @param nd
     * @param isSonSheet
     * @throws Exception
     */
    private void sendMsgToServer(NodeReportData nd, boolean isSonSheet) throws Exception {
        /** 节点异常上报数据分如下情况： 这些情况需要结束充电订单 1、节点充电满后自停 2、节点故障后停止充电 这里需要调用结束订单的接口*/
        ChargeEnd chargeEnd = new ChargeEnd();
        if(isSonSheet) chargeEnd.setSonSheet(true);
        chargeEnd.setChargeTime(nd.getChargeTime());
        chargeEnd.setOrderNo(nd.getOrderNo());
        chargeEnd.setEleEP(nd.getEleEP());
        chargeEnd.setEleEPLast(nd.getEleEPLast());
        chargeEnd.setPlugNo(nd.getPlugNo());
        ChargeEndType endType = nd.getEndType();
        //默认设置为数据上报
        chargeEnd.setResCode(ChargeResCode.ACTIVE_UP_DATA);
        chargeEnd.setReason(FaultDiagnosis.faultDiagnosis(nd.getFault()));
        if(endType != null){
            if(endType == ChargeEndType.PULL_OUT){//插座拔出
                chargeEnd.setResCode(ChargeResCode.SOCKET_PULL_OUT);
            }else if(endType == ChargeEndType.FINISH_FULL || endType == ChargeEndType.FINISH){//设备已充满
                chargeEnd.setResCode(ChargeResCode.FINISH_FULL);
            }
        }
        _sendMsgToServer(chargeEnd);
    }
    private void _sendMsgToServer(ChargeEnd data){
        if(data != null){
            //rabbitMQSend.sendMsg(data);
        }

    }

    /**上报充电数据处理...*/
    public void terminalRepChargeData(NodeChargeData data){
        data.setUpdateTime(System.currentTimeMillis());
        //System.out.println("上报数据："+data.toJson());
        if(!checkOrderNoRight(data)) return;
        // 还在充电 继续
        if (protocolRedisCache.checkChargeOrderNo(data.getOrderNo())) {
            /**这里继续充电*/
            protocolRedisCache.updateUserChargeData(data.getOrderNo(),data);
            protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.UPDATE,data.getOrderNo());
        } else {
            _terminalRepChargeData(data);
        }
    }
    //校验硬件上报数据的订单编号是否正确
    private boolean checkOrderNoRight(NodeChargeData data){
        long orderLong = StringUtil.convertLong(data.getOrderNo(), Long.MAX_VALUE);
        if(orderLong < ORDER_START_THRESHOLD || orderLong > ORDER_END_THRESHOLD){
            //System.out.println("注意，上报充电数据订单不和规则；上报数据为："+data.toJson());
          /*  if(!protocolRedisCache.checekEXOrderList(data.getOrderNo(),data.getPlugNo())){
                log.error("特别注意，上报充电数据,订单编号异常:{}",data);
            }*/
            return false;
        }
        return true;
    }

    //结束订单使用异步
    @Async
    public void _terminalRepChargeData(NodeChargeData data){
        try {
            /**如果不在缓存中 说明已经结束了 直接使用 结束后会上报主动结束參数*/
            ChargeEnd chargeEnd = RespResult.resDataChange(protocolService.endCharge(data.getNodeId(), data.getOrderNo(), data.getPlugNo(),false));
            /**这里说明已经失败了，需要拿现在的上报的数据存子单*/
            if(chargeEnd.getChargeTime() == 0 && data.getChargeTime() > 0){
                chargeEnd.setEleEP(data.getEleEP());
                chargeEnd.setEleEPLast(data.getEleEPLast());
                chargeEnd.setChargeTime(data.getChargeTime());
            }
            chargeEnd.setSonSheet(true);
            _sendMsgToServer(chargeEnd);
        }catch (Exception e){
           // addErrorLog("处理硬件主动上报充电数据异常,上报数据:%s,时间:%s",data.toJson(),e);
            log.error("处理硬件主动上报充电数据异常,上报数据:{},异常:{}",data,e.getMessage());
        }
    }
    /**集中器登入**/
    public void simulationTerminalLogin(CtkEquipTerminal terminal){
        String terminalNo = terminal.getTerminalNo();
        LocalProtocolCache.DROP_LINE_MAP.remove(terminalNo);// 这里去掉掉线数据
        try {
            CtkEquipTerminal upTerminal = new CtkEquipTerminal();
            upTerminal.setTerminalId(terminal.getTerminalId());
            boolean bl = false;
            /***这里设计是否要更新2次数据库---> 1 修改登入状态 2 ota升级修改状态 （由于....）*/
            /**集中器登入*/
            if(terminal.getStatus() == null || terminal.getStatus() != TerminalStatusEnum.ON_LINE.getId()){
                // long loginTime = System.currentTimeMillis();
                bl = true;
                upTerminal.setStatus(TerminalStatusEnum.ON_LINE.getId());
                // upTerminal.setLoginTime(loginTime);
                // upTerminal.setMerchantId(terminal.getMerchantId());
                //terminalDao.updateTemplateById(upTerminal);
            }
            /**ota版本查询*/
            if(terminal.getOtaStatus() != null && terminal.getOtaStatus() == OTAStatus.OTA_RUNING.getId()){
                upTerminal.setOtaVersion(terminal.getOtaVersion());
                upTerminal.setOtaStatus(OTAStatus.OTA_SUCCESS.getId());
                upTerminal.setRemark("升级成功");
                bl = true;
                //_simulationTerminalLogin(terminal);
            }
            if(bl)  terminalDao.updateTemplateById(upTerminal);
        }catch (Exception e){
            addErrorLog("集中器登入异常，集中器编号为%s,时间为%s",terminalNo,e);
        }
    }
    //@Async  有点问题 集中器登入后无法访问版本信息==》当前版本判断ota升级成功的依据为：设备ota校验升级包正确+通知升级成功+集中器登入成功
 /*   @Deprecated
    private void _simulationTerminalLogin(CtkEquipTerminal terminal){
        boolean isOTAOk = true;
        CtkEquipTerminal upTerminal = null;
        try {
            System.out.println("OTA查询实体："+terminal.toJson());
            boolean bl = true;
            StringBuffer sb = new StringBuffer();
            upTerminal = new CtkEquipTerminal();
            upTerminal.setId(terminal.getId());
            do {
                HardwareVersion hv = RespResult.resDataChange(retryOper(terminal.getTerminalNo()));
                isOTAOk = false;
                CtkHsVersion hs = hsVesionDao.single(terminal.getOtaNewVersion());
                if(hs == null){
                    sb.append("根据升级版本：【"+terminal.getOtaNewVersion()+"】，查询版本信息失败");
                    break;
                }
                if(!hv.getSVersion().equalsIgnoreCase(hs.getHsVersionName())){
                    sb.append("升级版本号【"+hs.getHsVersionName()+"】,与查询硬件版本【"+hv.getSVersion()+"】不相同");
                    break;
                }
                upTerminal.setOtaStatus(OTAStatus.OTA_SUCCESS.getId());
                upTerminal.setOtaNowVersion(terminal.getOtaNewVersion());
                upTerminal.setOtaResRemark("升级成功");
                bl = false;
            }while (false);
            if(bl){
                upTerminal.setOtaStatus(OTAStatus.OTA_FALL.getId());
                upTerminal.setOtaResRemark(sb.toString());
            }
            terminalDao.updateTemplateById(upTerminal);
            *//***清除OTA缓存的值*//*
            protocolRedisCache.removeOTAPoint(terminal.getTerminalNo());
        }catch (Exception e){
            addErrorLog("集中器登入异常，集中器编号为%s,时间为%s",terminal.getTerminalNo(),e);
        }finally {
            if(isOTAOk){//这里说明OTA异常
                //System.out.println("更新实体："+upTerminal.toJson());
                upTerminal.setOtaStatus(OTAStatus.OTA_FALL.getId());
                upTerminal.setOtaResRemark("查询集中器版本失败");
                terminalDao.updateTemplateById(upTerminal);
            }
        }
    }
*/


    /**TODO 这里需要修改协议*/
    private RespWrapper<HardwareVersion> retryOper(String terminalNo)throws Exception {
        for (int i = 0; i < ProtocolConstant.RETRY_NUM; i++) {
            RespWrapper<HardwareVersion> hardwarInfo = protocolService.getHardwarInfo(terminalNo);
            if(hardwarInfo.isSuccess()) return hardwarInfo;
            //Thread.sleep(500);
        }
        return RespWrapper.makeFailResp("查询OTA版本重试【"+ProtocolConstant.RETRY_NUM+"】次失败");
    }

    /**异常*/
    private void addErrorLog(String msg,Object data,Exception e){
        log.error("一次消息：{},数据：{}，异常：{}",msg,data,e);
       /* if(PatternType.TEST == ProjectPattern.TEST_PATTERN){
            System.out.println("注意："+String.format(msg,data,DateTimeUtils.formatDate(new Date(),DateTimeUtils.FORMAT))+"，错误信息："+e.getMessage());
        }else{
            //log.error(LogsFactory.build(SystemLogsType.ADDRNO_LOG,String.format(msg,data,DateTimeUtils.formatDate(new Date(),DateTimeUtils.FORMAT)),e.getMessage()));
        }*/
    }
    /**集中器上电事件*/
    public void terminalPowerUpEvent(String terminalNo) {
        if(!protocolRedisCache.chechTerminalPowerUpHandle(terminalNo)){
            // 集中器上电事件
           // _terminalPowerUpEvent(terminalNo);
        }
    }
    @Async
    public void _terminalPowerUpEvent(String termianlNo){
        //查询集中器下的所有订单 并结算所有在充电的订单
        /**根据集中器编号查询正在充电中的订单*/
        List<String> orderList = terminalDao.findChargingListByTerminalNo(termianlNo);
        if(orderList == null || orderList.size() < 1)return;
        List<String> keys = new ArrayList<>();
        orderList.forEach(orderNo->{
            try {
                ChargeEnd chargeEnd = new ChargeEnd();
                chargeEnd.setOrderNo(orderNo);
                chargeEnd.setReason(ChargeResCode.POWER_UP.getName());
                chargeEnd.setResCode(ChargeResCode.POWER_UP);
                protocolRedisCache.handleUserChargeData(chargeEnd,orderNo);
                protocolRedisCache.removeChargeOrderNo(orderNo);
                chargeEnd.setSonSheet(false);
                //这里发送消息到服务层
                _sendMsgToServer(chargeEnd);
                keys.add(orderNo);
            }catch (Exception e){
                e.printStackTrace();
            }
        });
        if(keys.size() > 0){
            keys.forEach(o->{
                protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.DEL,o);
            });
        }
    }
}
