package com.comtika.safe.core.protocol.responseUtil;

import com.comtika.safe.common.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author actor
 * @date 2018/12/11 15:52
 * 为了解决软件与硬件通讯使用 异步通讯
 */
public class DefaultFuture  implements ResponseFuture {

    private static final Logger logger = LoggerFactory.getLogger(DefaultFuture.class);
    public static final Map<String, DefaultFuture> FUTURES = new ConcurrentHashMap();
    private final int timeout;//默认超时时间
    private final Lock lock = new ReentrantLock();
    private final Condition done;
    //private final long start;//维护超时时间
    private volatile Object response;
    private final String sign;
    public DefaultFuture(String sign,int time, TimeUnit uint){
        this.done = this.lock.newCondition();
        this.timeout = (int)TimeUnit.MILLISECONDS.convert(time,uint);
       // this.start = System.currentTimeMillis();
        this.sign = sign;
        FUTURES.put(sign,this);
    }
    public DefaultFuture(String sign){
        this(sign,5,TimeUnit.SECONDS);//默认为5秒
    }

    @Override
    public Object get() throws Exception {//默认时长
        return this.get(this.timeout,TimeUnit.MILLISECONDS);
    }

    @Override
    public Object get(int timeout,TimeUnit unit) throws Exception {
        timeout =(int) TimeUnit.MILLISECONDS.convert(timeout,unit);
        if (timeout <= 0) {
            timeout = 1000;
        }
        if (!this.isDone()) {
            long start = System.currentTimeMillis();
            this.lock.lock();
            try {
                while(!this.isDone()) {
                    this.done.await((long)timeout, TimeUnit.MILLISECONDS);
                    if (this.isDone() || System.currentTimeMillis() - start > (long)timeout) {
                        break;
                    }
                }
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            } finally {
                this.lock.unlock();
                if(FUTURES.containsKey(this.sign)){
                    FUTURES.remove(this.sign);//如果存在 直接移除
                }
            }
            if (!this.isDone()) {
                return null;
            }
        }
        return this.response;
    }

    @Override
    public boolean isDone() {
        return response != null;
    }

    public static void received(String sign,Object response) {
        if(StringUtil.isEmpty(sign)) return;
        DefaultFuture future = (DefaultFuture)FUTURES.remove(sign);
        if (future != null) {
            future.doReceived(response);
        } else {
            //这里说明响应超时
            //logger.warn("获取当前实例失败" + (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS")).format(new Date()) + ", 实例： " + response);
        }
    }
    private void doReceived(Object res) {
        this.lock.lock();
        try {
            this.response = res;
            if (this.done != null) {
                this.done.signal();
            }
        } finally {
            this.lock.unlock();
        }
    }

}
