package com.comtika.safe.core.protocol.timer;

import com.comtika.safe.common.enums.equip.terminal.TerminalStatusEnum;
import com.comtika.safe.core.protocol.ProtocolConstant;
import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.cache.ProtocolRedisCache;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.eunms.charge.ChargeResCode;
import com.comtika.safe.core.protocol.repData.TerminalRepDataHandle;
import com.comtika.safe.dao.equip.terminal.EquipTerminalDao;
import com.comtika.safe.service.safe.IRepService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @desc    :	此类测试时使用 因为只有一台集中器 难以维持缓存正确性
 * @author    :	actorLucky[494977551@qq.com]
 * @date    :	2018年5月24日 下午3:41:08
 * 检测集中器心跳
 */
@Component
public class CheckTerminalHeartbeat {
    private static final AtomicInteger ATOMIC_INTEGER_TIMEOUT = new AtomicInteger((ProtocolConstant.DEFAULT_HANDLE_EX_ORDER >> 1));
    private static final AtomicInteger ATOMIC_UPDATE_ORDER = new AtomicInteger();
    private long chceckRate;
    private long delayTimeout;

    public void setChceckRate(long chceckRate) {
        this.chceckRate = chceckRate;
    }

    public void setDelayTimeout(long delayTimeout) {
        this.delayTimeout = delayTimeout * 1000;
    }

    private static final ScheduledThreadPoolExecutor TIMER_CHECK_TERMINAL = new ScheduledThreadPoolExecutor(2, (r) -> new Thread(r, "Thread name:协议层，处理心跳与异常挂单线程"));
//    @Autowired
//    private RabbitMQSend rabbitMQSend;

    @Autowired
    private TerminalRepDataHandle terminalRepDataHandle;

    @Autowired
    private ProtocolRedisCache protocolRedisCache;
    @Autowired
    private EquipTerminalDao terminalDao;
    @Autowired
    private IRepService iRepService;
	/*@Autowired
	private MessageFeginService messageFeginService;*/

	private final int checkRate = 30;//30s 心跳检查

    public void start() {
        // 增加一个测试
        //LocalProtocolCache.HEARTBEAT_MAP.put("123456789",System.currentTimeMillis());
        System.out.println("检查心跳及异常订单处理开始....");
        /**检查一些过时的心跳，如果过时发mq到指令控制层*/
        try {
            TIMER_CHECK_TERMINAL.scheduleAtFixedRate(() -> {
                _timerHandleCore();
            }, checkRate, checkRate, TimeUnit.SECONDS);
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("检查心跳及异常订单处理，启动成功");
        // 下面是处理设备掉线的情况 1分种检查一次，这个是干什么的
        TIMER_CHECK_TERMINAL.scheduleAtFixedRate(() -> {
            // System.out.println("检查设备掉线。。。。。");
            doDropLineHandle();
        }, 1, 1, TimeUnit.MINUTES);
    }

    private static final long DROP_LINE_TIME_OUT = 30 * 1000 * 60L;//30分钟，发生掉线通过该

    // 这里处理掉线的数据
    private void doDropLineHandle() {
        long startTime = System.currentTimeMillis();
        List<String> list = new ArrayList<>(32);
        LocalProtocolCache.DROP_LINE_MAP.forEach((k, v) -> {
            if (startTime - v > DROP_LINE_TIME_OUT) {
                list.add(k);
            }
        });
        list.forEach(s -> {
            LocalProtocolCache.DROP_LINE_MAP.remove(s);
            // todo 设备下线处理数据
            //rabbitMQSend.sendWarnMessage(s);
            //messageFeginService.fadSendWarnMessageToManage(s);
            // 设备掉线了，这里需要通知数据
            iRepService.sendOffLineTerminalMessage(s);
        });
    }

    public void stop() {
        TIMER_CHECK_TERMINAL.shutdown();
    }

    /**
     * 处理类
     */
    private void _timerHandleCore() {
        long startTime = System.currentTimeMillis();
        _handleHeartbeat(startTime);// 处理掉线设备
        // 超过定时时间，执行该类
        //todo 一个小时处理断网、断电处理正常的订单
        // if (ATOMIC_INTEGER_TIMEOUT.incrementAndGet() >= ProtocolConstant.DEFAULT_HANDLE_EX_ORDER) {//
        //     ATOMIC_INTEGER_TIMEOUT.set(0);
        //     final int terminalTimeout = protocolRedisCache.findTerminalShowdownTimeoutMillisecond();
        //     /**handle showdown terminal*/
        //     _handleShowdownTerminal(startTime, terminalTimeout);
        // }
        // todo 定时的更新充电数据列表，防止数据库数据与正式的数据有点冲突
        //if (ATOMIC_UPDATE_ORDER.incrementAndGet() >= ProtocolConstant.DEFAULT_UPDATE_NUM) {
        //    ATOMIC_UPDATE_ORDER.set(0);
        //    //System.out.println("更新充电列表"); 更新充电列表 防止挂单
        //    terminalRepDataHandle.updateChargingList(false);
        //}
    }
    // 下面是处理断网断电订单

    /**
     * handle showdown terminal
     */
    private void _handleShowdownTerminal(final long startTime, final int terminalTimeout) {
        try {
            Map<String, Long> map = LocalProtocolCache.SIMPLE_CHARGE_DATA;
            List<String> keys = new ArrayList<>();
            map.forEach((k, v) -> {
                try {
                    if (startTime - v > terminalTimeout) {
                        keys.add(k);
                        ChargeEnd chargeEnd = new ChargeEnd();
                        chargeEnd.setOrderNo(k);
                        chargeEnd.setReason(ChargeResCode.NETWORK_TIMEOUT.getName());
                        chargeEnd.setResCode(ChargeResCode.NETWORK_TIMEOUT);
                        protocolRedisCache.handleUserChargeData(chargeEnd, k);
                        if (protocolRedisCache.checkChargeOrderNo(k)) {
                            protocolRedisCache.removeChargeOrderNo(k);
                            chargeEnd.setSonSheet(false);
                        } else chargeEnd.setSonSheet(true);
                        //这里发送消息到服务层
                        // todo 结束订单，这里处理下
                        // rabbitMQSend.sendMsg(chargeEnd);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            if (keys.size() > 0) {
                keys.forEach(k -> {
                    // rabbitMQSend.sendWarnMessage(k);// 发送告警消息 这里发送消息
                    protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.DEL, k);
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    // 检查心跳方法

    /**
     * handle heartbeat
     */
    private void _handleHeartbeat(long startTime) {
        try {
            List<String> removeList = new ArrayList<>();
            LocalProtocolCache.HEARTBEAT_MAP.forEach((k, v) -> {
                if (startTime - v > delayTimeout) {
                    terminalDao.updateTerminalStatusByTerminalNo(TerminalStatusEnum.IN_MAINTEN.getId(), k);//更新设备状态
                    /**强制设备掉线、集中器在线*/
                    //protocolRedisCache.forceTerminalDownNow(k);
                    removeList.add(k);
                }
            });
            removeList.forEach(k -> {
                LocalProtocolCache.removeHearbeatValue(k);//移除心跳
                LocalProtocolCache.closedChannelCache(k);//关闭channel 通道
                // 这里说明已经掉线的数据
                LocalProtocolCache.DROP_LINE_MAP.put(k, startTime);// 掉线的列表
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
