package com.comtika.safe.core.protocol.utils;

import com.comtika.safe.common.utils.IDUtil;
import com.comtika.safe.core.protocol.eunms.HardwareVersion;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.ThreadLocalRandom;

/**
 *
 * 只有在几个字节组成一组数据时才会用到大小端模式，对应一个字节中没有大小端
 *
 * 所谓的大端模式：是指数据的高位，保存在内存的低地址中，而数据的低位，保存在内存的高地址中，
 * 这样的存储模式有点儿类似于把数据当作字符串顺序处理：地址由小向大增加，而数据从高位往低位放；
 *
 * 小端模式 　　所谓的小端模式，是指数据的高位保存在内存的高地址中，而数 据的低位保存在内存的低地址中，
 * 这种存储模式将地址的高低和数据位权有效地结合起来，高地址部分权值高，低地址部分权值低，和我们的逻辑方法一致
 *
 *
 * 对于16路使用的是小端模式
 *
 * @desc : 基本数据类型转换
 * @author : actorLucky[494977551@qq.com]
 * @date : 2018年5月19日 下午5:46:03
 */
@SuppressWarnings("unused")
public class BaseTypeConvert {
	private static final ThreadLocalRandom RNNDOM = ThreadLocalRandom.current();
	private static final char[] DIGITIS = new char[] { '0', '1', '2', '3', '4',
			'5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
	public static final byte[] EMPTY_BYTES = new byte[0];
	private static int TWO_BT = 0xFF;
	private static int ONE_BT = 0xF;
	private static int MAX_SIZE = 8;
	private static int MIN_SIZE = 1;
	private static 	final String DEFAULT_STRING = "F";

	// TODO
	public static void main(String[] args) throws Exception {
		/** 下面测试指定生成数据大小的数组 */
		/*
		 * byte[] byte2ExpectBytes = byte2ExpectBytes((byte)25,2);
		 * System.out.println(Arrays.toString(byte2ExpectBytes)); byte
		 * expectSizeBytes2Bte = expectSizeBytes2Bte(byte2ExpectBytes);
		 * System.out.println(expectSizeBytes2Bte);
		 */
		/** 小端模式测试 */
		/*byte[] int2BytesLE = int2BytesBE(1, 8);
		System.out.println(Arrays.toString(int2BytesLE));*/
		//System.out.println(Arrays.toString(createDateFormat()));\
		/*for(int i=0;i<3;i++){
			System.out.println(positionFF(i));
		}*/
        //byte[] terminalTime = createTerminalTime(System.currentTimeMillis());
        //System.out.println(Arrays.toString(terminalTime));
        //System.out.println(bytes2HexStr(terminalTime));

	}
	private static int positionFF(int position){
		int res = Integer.MIN_VALUE;
		StringBuffer str = new StringBuffer();
		for(int i=0;i<position;i++){
			str.append(DEFAULT_STRING);
		}
		try {
			res = Integer.parseInt(str.toString(),16);
		}catch (Exception e){}
		return res;
	}



	/**
	 * 下面使用合并 4字节数组、8个字节数组 与一个字节相互转换的通用方法
	 */
	public static byte[] byte2ExpectBytes(byte bt, int expectSize) {
		if (expectSize % 2 != 0) {
			throw new RuntimeException(String.format(
					"expectSize must Even numbers,however %s not Even",
					expectSize));
		}
		if (expectSize > MAX_SIZE) {
			throw new RuntimeException("byte split must less then " + MAX_SIZE);
		}
		if (expectSize < MIN_SIZE) {
			throw new IllegalArgumentException(expectSize + " < " + MIN_SIZE);
		}
		byte[] res = new byte[expectSize];
		int offValue = MAX_SIZE / expectSize;
		byte offByte = offSize(offValue);
		for (int i = 0; i < expectSize; i++) {
			res[i] = (byte) ((bt >> i * offValue) & offByte);
		}
		return res;
	}

	private static byte offSize(int size) {
		byte re = 0;
		for (int i = 0; i < size; i++) {
			re |= (ONE_BT & 0x1) << i;
		}
		return re;
	}

	/** 将几个byte数组合并成一个byte数组值 */
	public static byte expectSizeBytes2Bte(byte[] bt) {
		int size = bt.length;
		if (size % 2 != 0) {
			throw new RuntimeException(
					String.format("byte[] length must Even numbers,however byte[] %s not Even",size));
		}
		if (size > MAX_SIZE) {
			throw new RuntimeException(String.format(
					"byte[] must less then %s size", MAX_SIZE));
		}
		int offValue = MAX_SIZE / size;
		byte offSize = offSize(offValue);
		byte res = 0;
		for (int i = 0; i < size; i++) {
			res |= (bt[i] & offSize) << i * offValue;
		}
		return res;
	}

	// 生成自动4位字节数组的唯一id start
	/**
	 * 生成签名
	 *
	 * @变更记录 2018年6月3日 上午8:43:54 zhaoLuYang create
	 */
	public static String createFadsSign() {
		return 10000 + RNNDOM.nextInt(89999) + IDUtil.getSign();
	}

	/**
	 * 将单个字节转成Hex String
	 *
	 * @param b
	 *            字节
	 *
	 * @return String Hex String
	 */
	public static String byte2HexStr(byte b) {
		char[] cr = new char[2];
		cr[1] = DIGITIS[b & ONE_BT];
		cr[0] = DIGITIS[(b >>> 4) & ONE_BT];
		return new String(cr);
	}

	public static int byteToInt(byte bt){
		if(bt < 0)
			return 256+bt;
		return bt;
	}
	/**
	 * 将字节数组转成Hex String 大端模式
	 *
	 * @param bytes
	 * @return String
	 */
	public static String bytes2HexStr(byte[] bytes) {
		if (bytes == null || bytes.length == 0) {
			return null;
		}
		return bytes2HexStr(bytes, 0, bytes.length);
	}

	/**
	 * 将字节数组转成 Hex String
	 *
	 * @param bytes
	 * @param offset
	 * @param length
	 * @return
	 */
	public static String bytes2HexStr(byte[] bytes, int offset, int length) {
		if (bytes == null || bytes.length == 0) {
			return null;
		}
		if (offset < 0) {
			throw new IllegalArgumentException("offset(" + offset + ")");
		}
		if (offset + length > bytes.length) {
			throw new IllegalArgumentException("offset + length(" + offset
					+ length + ") > bytes.length(" + bytes.length + ")");
		}

		char[] buf = new char[2 * length];
		for (int i = 0; i < length; i++) {
			byte b = bytes[i + offset];
			buf[2 * i + 1] = DIGITIS[b & ONE_BT];
			b = (byte) (b >>> 4);
			buf[2 * i + 0] = DIGITIS[b & ONE_BT];
		}
		return new String(buf);
	}

	/**
	 * 将单个hex Str转换成字节
	 *
	 * @param str
	 * @return byte
	 */
	public static byte hexStr2Byte(String str) {
		if (str != null && str.length() == 1) {
			return char2Byte(str.charAt(0));
		} else {
			return 0;
		}
	}

	/**
	 * 字符到字节
	 *
	 * @param ch
	 * @return byte
	 */
	public static byte char2Byte(char ch) {
		if (ch >= '0' && ch <= '9') {
			return (byte) (ch - '0');
		} else if (ch >= 'a' && ch <= 'f') {
			return (byte) (ch - 'a' + 10);
		} else if (ch >= 'A' && ch <= 'F') {
			return (byte) (ch - 'A' + 10);
		} else {
			return 0;
		}
	}

	public static int hexToInt(String str){
		//if(str.length() != 2) return 0;
		return Integer.valueOf(str,16);
	}

	/**
	 * hex String 转字节数组
	 *
	 * @param str
	 * @return
	 */
	public static byte[] hexStr2Bytes(String str) {
		if (str == null || str.equals("")) {
			return EMPTY_BYTES;
		}
		byte[] bytes = new byte[str.length() / 2];
		for (int i = 0; i < bytes.length; i++) {
			char high = str.charAt(i * 2);
			char low = str.charAt(i * 2 + 1);
			bytes[i] = (byte) (char2Byte(high) * 16 + char2Byte(low));
		}
		return bytes;
	}

	/**
	 * 将int 转 bit个字节数组 小端模式
	 */
	public static byte[] int2BytesLE(int value, int bit) {
		if (bit < 1) {
			return null;
		}
		byte[] src = new byte[bit];
		for (int i = bit - 1; i >= 0; i--) {
			src[i] = (byte) ((value >>> i * 8) & TWO_BT);
		}
		return src;
	}

	/**
	 * 将字节数组 转int 小端模式
	 */
	public static int bytes2IntLE(byte[] bArr) {
		int size = bArr.length;
		if (size < 1) {
			return 0;
		}
		int res = 0;
		for (int i = 0; i < size; i++) {
			res |= ((bArr[i] & TWO_BT) << i * 8);
		}
		return res;
	}


	/**
	 * 将int 转bit位字节数组 大端模式
	 */
	public static byte[] int2BytesBE(int value, int bit) {
		if (bit < 1) {
			return null;
		}
		byte[] src = new byte[bit];
		for (int i = 0; i < bit; i++) {
			src[i] = (byte) ((value >>> (bit - i - 1) * 8) & TWO_BT);
		}
		return src;
	}

	/**
	 * 将字节数组 转int 大端模式
	 *
	 * @param bArr
	 * @return
	 */
	public static int bytes2IntBE(byte[] bArr) {
		int size = bArr.length;
		if (size < 1) {
			return -1;
		}
		int res = 0;
		for (int i = 0; i < size; i++) {
			res |= ((bArr[i] & TWO_BT) << (size - 1 - i) * 8);
		}
		return res;
	}
	public static int byte2Int(byte bt){
		return bt & TWO_BT;
	}
	public static byte int2byte(int value){
	    return (byte) (value & TWO_BT);
    }

	/**
	 * 通过一个字节获取温度
	 */
	public static String byte2T(byte by) {
		return byte2T(by, true);
	}


	/**
	 * 通过一个字节获取温度
	 */

	public static String byte2T(byte by, boolean isString) {
		int t = 0;
		if ((by >> 7 & 0x1) == 1) {
			byte r = (byte) (by & ONE_BT);
			t = -r;
		}
		t = by;
		if (isString) {
			return t + "°C";
		}
		return t + "";
	}

	/**
	 * 生成当前时间 星期-年-月-日-时-分-秒 ；年=实际年-2000;十六进制格式
	 */
	public static byte[] createDateFormat() {
		LocalDateTime now = LocalDateTime.now();
		byte lv0 = (byte) now.getDayOfWeek().getValue();
		byte lv1 = (byte) Integer.parseInt((now.getYear() + "").substring(2),
				16);
		byte lv2 = (byte) Integer.parseInt(now.getMonthValue() + "", 16);
		byte lv3 = (byte) Integer.parseInt(now.getDayOfMonth() + "", 16);
		byte lv4 = (byte) Integer.parseInt(now.getHour() + "", 16);
		byte lv5 = (byte) Integer.parseInt(now.getMinute() + "", 16);
		byte lv6 = (byte) Integer.parseInt(now.getSecond() + "", 16);
		byte[] res = new byte[] { lv0, lv1, lv2, lv3, lv4, lv5, lv6 };
		return res;
	}
	// 年-月-日-星期-时-分-秒
	public static byte[] createTerminalTime(Long time){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date(time));
		byte lv0 = int2byte((calendar.get(Calendar.DAY_OF_WEEK) - 1) == 0 ? 7 : (calendar
						.get(Calendar.DAY_OF_WEEK) - 1)) ;
		int year = calendar.get(Calendar.YEAR);
        byte[] bytes = int2BytesBE(year, 2);
        byte lv2 = int2byte(calendar.get(Calendar.MONTH) + 1);
		byte lv3 = int2byte(calendar.get(Calendar.DAY_OF_MONTH));
		byte lv4 = int2byte(calendar.get(Calendar.HOUR_OF_DAY));
		byte lv5 = int2byte(calendar.get(Calendar.MINUTE));
		byte lv6 = int2byte(calendar.get(Calendar.SECOND));
		byte[] res = new byte[]{bytes[0],bytes[1],lv2,lv3,lv0,lv4,lv5,lv6};
		return res;
	}

	/**
	 *
	 * @Description: 生成指定时间的日期 星期-年-月-日-时-分-秒 ；年=实际年-2000;十六进制格式
	 */
	public static byte[] createDateFormat(long time) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		byte[] res = new byte[7];
		res[0] = (byte) Integer.parseInt(
				((calendar.get(Calendar.DAY_OF_WEEK) - 1) == 0 ? 7 : (calendar
						.get(Calendar.DAY_OF_WEEK) - 1)) + "", 16);
		res[1] = (byte) Integer.parseInt(
				String.valueOf(calendar.get(Calendar.YEAR)).substring(2), 16);
		res[2] = (byte) Integer.parseInt(calendar.get(Calendar.MONTH) + 1 + "",
				16);
		res[3] = (byte) Integer.parseInt(calendar.get(Calendar.DAY_OF_MONTH)
				+ "", 16);
		res[4] = (byte) Integer.parseInt(calendar.get(Calendar.HOUR_OF_DAY)
				+ "", 16);
		res[5] = (byte) Integer
				.parseInt(calendar.get(Calendar.MINUTE) + "", 16);
		res[6] = (byte) Integer
				.parseInt(calendar.get(Calendar.SECOND) + "", 16);
		return res;
	}

	// 获取时间date 从终端获取时间 start
	public static String getDateByBytes(byte[] by) {
		int size = by.length;
		if (size != 6) {
			throw new RuntimeException("获取时间失败！");
		}
		System.out.println(Arrays.toString(by));
		StringBuffer sb = new StringBuffer("20");
		sb.append((by[0] < 0 ? by[0] + 256 : by[0]) + "-");
		sb.append((by[1] < 0 ? by[1] + 256 : by[1]) + "-");
		sb.append((by[2] < 0 ? by[2] + 256 : by[2]) + " ");
		sb.append((by[3] < 0 ? by[3] + 256 : by[3]) + ":");
		sb.append((by[4] < 0 ? by[4] + 256 : by[4]) + ":");
		sb.append((by[5] < 0 ? by[5] + 256 : by[5]));
		return sb.toString();
	}

	/***
	 * 解析硬件上报版本号 isHardware 是否是硬件版本 true=硬件版本 false=软件版本
	 * */
	public static String getNodeVersion(byte[] by, boolean isHardware) {
		if (by == null || by.length != 4) {
			return null;
		}
		StringBuffer sb = new StringBuffer("20");
		sb.append(Integer.toHexString(by[0] < 0 ? by[0] + 256 : by[0]) + "年-");
		sb.append(Integer.toHexString(by[1] < 0 ? by[1] + 256 : by[1]) + "月-");
		sb.append(Integer.toHexString(by[2] < 0 ? by[2] + 256 : by[2]) + "日-");
		if (isHardware) {
			HardwareVersion hv = HardwareVersion.valueOf(by[3]);
			sb.append(hv==null?"无":(hv.getName() + "版"));
		} else {
			sb.append(Integer.toHexString(by[3] < 0 ? by[3] + 256 : by[3])
					+ "版");
		}
		return sb.toString();
	}
}
