package com.comtika.safe.core.protocol.utils;

import lombok.extern.log4j.Log4j2;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;

/**
 * IO 通用方法
 * 2018.12.01
 */
@Log4j2
public class IOUtils {

	private static final int BUFFER_SIZE = 8192;

	private IOUtils() {
	}

	/**
	 * BufferedInputStream、BufferedOutputStream（缓存字节流）使用方式和字节流差不多，但是效率更高（推荐使用）
	 *
	 * @throws IOException
	 */
	public static Byte[] bufferedIo(String fileDri) throws IOException {
		//对文件
		BufferedInputStream in = new BufferedInputStream(new FileInputStream(fileDri));
		List<Byte> list = new ArrayList<Byte>();
		int n = -1;
		while ((n = in.read()) != -1) {
			list.add((byte) n);
		}
		int size = list.size();
		Byte[] by = new Byte[size];
		for (int i = 0; i < size; i++) {
			by[i] = list.get(i);
		}
		in.close();
		return by;
	}
	/**从输入到输出*/
	public static long write(InputStream is, OutputStream os) throws IOException {
		return write((InputStream) is, (OutputStream) os, BUFFER_SIZE);
	}

	public static long write(InputStream is, OutputStream os, int bufferSize) throws IOException {
		long total = 0L;
		byte[] buff = new byte[bufferSize];
		while (is.available() > 0) {
			int read = is.read(buff, 0, buff.length);
			if (read > 0) {
				os.write(buff, 0, read);
				total += (long) read;
			}
		}
		return total;
	}
	/**读去流的文件*/
	public static String read(Reader reader) throws IOException {
		StringWriter writer = new StringWriter();
		String var2;
		try {
			write((Reader) reader, (Writer) writer);
			var2 = writer.getBuffer().toString();
		} finally {
			writer.close();
		}
		return var2;
	}
	/**写出文件*/
	public static long write(Writer writer, String string) throws IOException {
		StringReader reader = new StringReader(string);
		long var3;
		try {
			var3 = write((Reader) reader, (Writer) writer);
		} finally {
			reader.close();
		}
		return var3;
	}

	public static long write(Reader reader, Writer writer) throws IOException {
		return write((Reader) reader, (Writer) writer, BUFFER_SIZE);
	}

	public static long write(Reader reader, Writer writer, int bufferSize) throws IOException {
		long total = 0L;
		int read;
		for (char[] buf = new char[BUFFER_SIZE]; (read = reader.read(buf)) != -1; total += (long) read) {
			writer.write(buf, 0, read);
		}
		return total;
	}

	public static String[] readLines(File file) throws IOException {
		return file != null && file.exists() && file.canRead() ? readLines((InputStream) (new FileInputStream(file))) : new String[0];
	}

	public static String[] readLines(InputStream is) throws IOException {
		List<String> lines = new ArrayList();
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		String[] var4;
		try {
			String line;
			while ((line = reader.readLine()) != null) {
				lines.add(line);
			}
			var4 = (String[]) lines.toArray(new String[0]);
		} finally {
			reader.close();
		}
		return var4;
	}

	public static void writeLines(OutputStream os, String[] lines) throws IOException {
		PrintWriter writer = new PrintWriter(new OutputStreamWriter(os));
		try {
			String[] arr$ = lines;
			int len$ = lines.length;
			for (int i$ = 0; i$ < len$; ++i$) {
				String line = arr$[i$];
				writer.println(line);
			}
			writer.flush();
		} finally {
			writer.close();
		}
	}
	/**追加内容*/
	public static void writeLines(File file, String[] lines) throws IOException {
		if (file == null) {
			throw new IOException("File is null.");
		} else {
			writeLines((OutputStream) (new FileOutputStream(file)), lines);
		}
	}
	/**追加内容*/
	public static void appendLines(File file, String[] lines) throws IOException {
		if (file == null) {
			throw new IOException("File is null.");
		} else {
			writeLines((OutputStream) (new FileOutputStream(file, true)), lines);
		}
	}

	public static Byte[] bufferedIoFromURL(String baseUrl) {
		try {
			URL url = new URL(baseUrl);
			URLConnection connection = url.openConnection();
			//获得字节流
			InputStream inputStream = connection.getInputStream();
			//对文件
			BufferedInputStream in = new BufferedInputStream(inputStream);
			List<Byte> list = new ArrayList<Byte>();
			int n = -1;
			while ((n = in.read()) != -1) {
				list.add((byte) n);
			}
			int size = list.size();
			Byte[] by = new Byte[size];
			for (int i = 0; i < size; i++) {
				by[i] = list.get(i);
			}
			in.close();
			return by;
			/*URL urlConet = new URL(baseUrl);
			HttpURLConnection con = (HttpURLConnection)urlConet.openConnection();
			con.setRequestMethod("GET");
			con.setConnectTimeout(10 * 1000);
			InputStream inStream = con .getInputStream();    //通过输入流获取图片数据
			ByteArrayOutputStream outStream = new ByteArrayOutputStream();
			byte[] buffer = new byte[2048];
			int len = 0;
			while( (len=inStream.read(buffer)) != -1 ){
				outStream.write(buffer, 0, len);
			}
			inStream.close();
			byte[] data =  outStream.toByteArray();
			Byte[] bytes = new Byte[data.length];
			int i = 0;
			for (byte b : data) bytes[i++] = b; // Autoboxing
			return bytes;*/
		}catch (Exception e){
			log.error("获取升级包错误，URL为：{}",baseUrl);
			e.printStackTrace();
		}
		return null;
	}
	// https://safe.comtikaiot.com/data/
	public static void main(String[] args){
		//String or = "https://safe.comtikaiot.com/data/";
		String str = "https://safe.comtikaiot.com/data/ota/2020-09-05/ESC-04-SPⅠ-Host-19091003.bin";
		/*String s = str.replaceAll(or, "/data/");
		System.out.println(s);*/
		Byte[] bytes = bufferedIoFromURL(str);
		if(bytes != null)
			System.out.println(bytes.length);
		//str.replaceAll(str,)
	}
}
