package com.comtika.safe.core.protocol.utils;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.encrypt.HexUtil;
import com.comtika.safe.core.protocol.entity.PlugLink;
import com.comtika.safe.core.protocol.entity.WattMeterHelp;
import com.comtika.safe.core.protocol.entity.*;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.entity.charge.ChargeBase;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.charge.ChargeStart;
import com.comtika.safe.core.protocol.entity.device.DeviceParams;
import com.comtika.safe.core.protocol.entity.device.EquipNodeParams;
import com.comtika.safe.core.protocol.entity.node.*;
import com.comtika.safe.core.protocol.eunms.*;
import com.comtika.safe.core.protocol.eunms.charge.AmmeterType;
import com.comtika.safe.core.protocol.eunms.charge.ChargeEndType;
import com.comtika.safe.core.protocol.eunms.charge.ChargeResCode;
import com.comtika.safe.core.protocol.eunms.device.EquipWarnType;
import com.comtika.safe.core.protocol.eunms.device.EquipmentCode;
import com.comtika.safe.core.protocol.eunms.node.NodeStatus;

import java.util.*;


/**
 * @desc    :	查询/设置 数据转换类   byte[] ---> bean   / bean ---> byte[]
 * @author    :	actorLucky[494977551@qq.com]
 * @date    :	2018年5月19日 下午3:03:17
 */
@SuppressWarnings("rawtypes")
public class OrderDataConvertUtil {
    /**
     * 创建数据 通过ip与port
     * 使用场景：设置集中器远程更新IP和端口 FCH
     *
     * @param ip
     * @param port ip:"202.103.228.253",port:"8485"
     *             [15, 50, 48, 50, 46, 49, 48, 51, 46, 50, 50, 56, 46, 50, 53, 51, 4, 56, 52, 56, 53] //10进制
     *             [0f,32,30,32,2e,31,30,33,2e,32,32,38,2e,32,35,33,4,38,34,38,34,38,35] 16进制
     *             【0F 32 30 32 2E 31 30 33 2E 32 32 38 2E 32 35 33 04 38 34 38 35 】答案
     * @return
     */
    public static byte[] createByteByIpPort(String ip, String port) {
        if (StringUtil.isEmpty(ip) || StringUtil.isEmpty(port)) {
            return null;
        }
        byte[] bytes = ip.getBytes();
        byte[] bytes2 = port.getBytes();
        int lv1 = bytes.length;
        int lv2 = bytes2.length;
        byte[] res = new byte[lv1 + lv2 + 2];
        res[0] = (byte) lv1;
        res[lv1 + 1] = (byte) lv2;
        System.arraycopy(bytes, 0, res, 1, lv1);
        System.arraycopy(bytes2, 0, res, lv1 + 2, lv2);
        return res;
    }

    /**
     * 获取ip与端口map 通过查询byte数组 F9
     * 使用场景：查询集中器远程更新IP和端口 FDH
     *
     * @param by
     * @return
     */
    public static RespWrapper<Map<String, String>> getIpMapByByte(byte[] by) {
        if (by == null || by.length < 1) {
            return RespWrapper.makeFailResp("入参格式不对！");
        }
        try {
            Map<String, String> res = new HashMap<String, String>();
            byte[] ips = new byte[by[0]];
            System.arraycopy(by, 1, ips, 0, by[0]);
            int portL = by[by[0] + 1];
            byte[] ports = new byte[portL];
            System.arraycopy(by, by[0] + 2, ports, 0, portL);
            res.put(ProjectConstant.PORT, new String(ports));
            res.put(ProjectConstant.IP, new String(ips));
            //总长度为   portL+by[0]+2
            //res.put(ProjectConstant.ACKCODE, (int)by[portL+by[0]+2]+"");
            return RespWrapper.makeSuccessResp(res);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("参数解析出错！！");
        }
    }

    /**
     * 查询集中器软硬件信息 4AH
     * 5,1,24,2,1,1,24,5,34,1,0
     *
     * @param by
     * @return
     */
    public static RespWrapper<HardwareVersion> getHardwareStatus(byte[] by) {
        try {
            if (by == null || by.length != 11) {
                return RespWrapper.makeFailResp("入参格式不对,请检查！");
            }
            HardwareVersion hv = new HardwareVersion();
            hv.setCallLow(CallLowType.valueOf(by[0]));
            hv.setCallHigh(CallHighType.valueOf(by[1]));
            StringBuffer strh = new StringBuffer("20");
            StringBuffer strs = new StringBuffer("20");
            for (int i = 2; i < 6; i++) {
                String tmp = HexUtil.byte2HexStr(by[i]);
                if (i == 2) {
                    strh.append(tmp + "年");
                } else if (i == 3) {
                    strh.append(tmp + "月");
                } else if (i == 4) {
                    strh.append(tmp + "日");
                } else if (i == 5) {
                    strh.append(tmp + "版");
                }
            }
            for (int i = 6; i < 10; i++) {
                String tmp = HexUtil.byte2HexStr(by[i]);
                if (i == 6) {
                    strs.append(tmp + "年");
                } else if (i == 7) {
                    strs.append(tmp + "月");
                } else if (i == 8) {
                    strs.append(tmp + "日");
                } else if (i == 9) {
                    strs.append(tmp + "版");
                }
            }
            hv.setHVersion(strh.toString());
            hv.setSVersion(strs.toString());
            hv.setTtype(AmmeterType.valueOf(by[10]));
            return RespWrapper.makeSuccessResp(hv);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析数据失败！");
        }
    }

    /**
     * 配置节点参数  01H     isSmallPattern 是小端模式 true=是，false=大端模式    都是大端模式
     * <p>
     * 6Bs	NodeID	节点ID号
     * 2Bs	ChargeFull_Ith	充满电电流阈值，单位1mA（1~1000）
     * 1B	ChargeFull_ITth	充满电电流时长阈值，单位1min（1~255）
     * 2Bs	OverLoad_th	充电过载保护阈值，单位0.1A（1~168）
     * 2Bs	OverLoad_Tth	充电过载保护时长阈值，单位1ms（1~255）
     * 1B	Leak_Ith	漏电保护电流阈值，单位1 mA（1~255）
     * 1B	Leak_ITth	漏电保护电流时长阈值，单位1ms（1~255）
     * 1B	ChargeTimeoutth	充电超时阈值，单位1小时（1~255）
     * 1B	ChargeStart_Ith	充电启动时间 ，单位1sec（1~255）
     * 1B	Discon_Ith	开路电流 ，单位1mA（1~20）
     * 2Bs	Volt_Rated	额定电压--单位--1V（110,220)
     * 2Bs	Alarm	报警使能位：
     * Bit0：过压；Bit1：欠压；Bit2：过流；
     * Bit3：过载；Bit4：过温；
     * Bit5：负载连接失败
     * Bit6：漏电流保护；Bit7：接地故障；
     * Bit8：充电超时；   其它位保留。
     * 1B 	Plug_Nb 	插座编号,从0开始
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * <p>
     * 27个字节  修改版本后
     * 11	6Bs	NodeID	节点ID号
     * 17	2Bs	Current_Below	停充结算下限阈值，单位1mA（0~1000）
     * 19	1B	Temp_Above	温度保护上限阈值，单位1℃（60~120）
     * 20	2Bs	Current_Above	电流保护上限阈值，单位mA（0~5000）
     * 22	2Bs	Power_Above	功率保护上限阈值，单位0.1W(0~11000)
     * 24	1B
     * 25	1B
     * 26	1B
     * 27	1B
     * 28	1B
     * 29	2Bs	Voltage_Below	电压保护下限阈值，单位0.1V（800 ~1200）
     * 31	2Bs	Voltage_Above	电压保护上限阈值，单位0.1V（2200~2900）
     * 33	1B 	Plug_Nb 	插座编号,从0开始
     * 34	4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     *
     * @param dpc
     * @return
     */
    public static byte[] setNodeParams(DeviceParams dpc, boolean isSmallPattern, String sign) {
        try {
            byte[] res = new byte[27];
            System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(dpc.getNodeId()), 0, res, 0, 6);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeFullI(), 2), 0, res, 6, 2);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeFullIT(), 1), 0, res, 8, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE((int) dpc.getOverLoad(), 2), 0, res, 9, 2);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getOverLoadT(), 2), 0, res, 11, 2);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getLeakI(), 1), 0, res, 13, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getLeakIT(), 1), 0, res, 14, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeTimeout(), 1), 0, res, 15, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeStartI(), 1), 0, res, 16, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getDisconI(), 1), 0, res, 17, 1);
            System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getVoltRated(), 2), 0, res, 18, 2);
            FaultEnableReport faultEnable = dpc.getFaultEnable();
            if (faultEnable == null) {
                faultEnable = new FaultEnableReport(false);
            }
            byte[] b1 = new byte[]{(byte) faultEnable.getTooV().getId(), (byte) faultEnable.getOweV().getId(), (byte) faultEnable.getTooLoad().getId(), (byte) faultEnable.getTooI().getId(), (byte) faultEnable.getTooHot().getId(), (byte) faultEnable.getLoadConFail().getId(), (byte) faultEnable.getProtectI().getId(), (byte) faultEnable.getGroundFail().getId()};
            byte[] b2 = new byte[]{(byte) faultEnable.getChargeTimeout().getId(), 0, 0, 0, 0, 0, 0, 0};

            byte be1 = BaseTypeConvert.expectSizeBytes2Bte(b1);
            byte be2 = BaseTypeConvert.expectSizeBytes2Bte(b2);
            if (isSmallPattern) {
                res[20] = be1;
                res[21] = be2;
            } else {
                res[20] = be2;
                res[21] = be1;
            }
            res[22] = (byte) dpc.getPlugNo();
            System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, res, 23, 4);
            return res;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /***
     *
     * @Title: setNodeParams
     * @Description: 新版本设置参数
     * @param enp 节点配置参数
     * @param sign 通讯 签名
     * @return byte[]
     * @throws
     * @变更记录 2018年9月7日 下午3:15:03 actor_miss create
     */
    public static byte[] setNodeParamsFad(EquipNodeParams enp, String sign) throws Exception {
        /**
         * 27个字节  修改版本后
         0-5	6Bs	NodeID	节点ID号
         6-7	2Bs	Current_Below	停充结算下限阈值，单位1mA（0~1000）
         8	1B	Temp_Above	温度保护上限阈值，单位1℃（60~120）
         9-10	2Bs	Current_Above	电流保护上限阈值，单位mA（0~5000）
         11-12	2Bs	Power_Above	功率保护上限阈值，单位0.1W(0~11000)
         13	1B
         14	1B
         15	1B
         16	1B
         17	1B
         18-19	2Bs	Voltage_Below	电压保护下限阈值，单位0.1V（800 ~1200）
         20-21	2Bs	Voltage_Above	电压保护上限阈值，单位0.1V（2200~2900）
         22	1B 	Plug_Nb 	插座编号,从0开始
         23-26	4Bs	Signal_ID 	消息id号。接收和发送保持一致。
         */
        byte[] res = new byte[27];
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(enp.getNodeNo()), 0, res, 0, 6);//节点
        System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getMinI(), 2), 0, res, 6, 2);//停充
        //System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getTempAbove(),1), 0, res, 8, 1);//温度
        res[8] = enp.getMaxT().byteValue();
        System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getMaxI(), 2), 0, res, 9, 2);//电流上保
        System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getMaxP(), 2), 0, res, 11, 2);//功率上保
        //System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getLeakI(),1), 0, res, 13, 1);//24
        //System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getLeakIT(),1), 0, res, 14, 1);//25
        //System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeTimeout(),1), 0, res, 15, 1);//26
        //System.arraycopy(BaseTypeConvert.int2BytesBE(dpc.getChargeStartI(),1), 0, res, 16, 1);//27
        //System.arraycopy(BaseTypeConvert.int2BytesLE(dpc.getDisconI(),1), 0, res, 17, 1);//28
        System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getMinV(), 2), 0, res, 18, 2);
        System.arraycopy(BaseTypeConvert.int2BytesBE(enp.getMaxV(), 2), 0, res, 20, 2);
        res[22] = enp.getPlugNo().byteValue();
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, res, 23, 4);
        return res;
    }

    /**
     * 获取节点配置参数        isSmallPattern 是小端模式 true=是，false=大端模式
     * 6Bs	NodeID	节点ID号
     * 2Bs	ChargeFull_Ith	充满电电流阈值，单位1mA（1~1000）
     * 1B	ChargeFull_ITth	充满电电流时长阈值，单位1min（1~255）
     * 2Bs	OverLoad_th	充电过载保护阈值，单位0.1A（1~160）
     * 2Bs	OverLoad_Tth	充电过载保护时长阈值，单位1ms（1~255）
     * 1B	Leak_Ith	漏电保护电流阈值，单位1 mA（1~255）
     * 1B	Leak_ITth	漏电保护电流时长阈值，单位1ms（1~255）
     * 1B	ChargeTimeoutth	充电超时阈值，单位1小时（1~255）
     * 1B	ChargeStart_Ith	充电启动时间 ，单位1sec（1~255）
     * 1B	Discon_Ith	开路电流 ，单位1mA（1~15）
     * 2Bs	Volt_Rated	额定电压--单位--1V（110,220)
     * 2Bs	Alarm	报警使能位：
     * Bit0：过压；Bit1：欠压；Bit2：过流；
     * Bit3：过载；Bit4：过温；
     * Bit5：负载连接失败
     * Bit6：漏电流保护；Bit7：接地故障；
     * Bit8：充电超时；
     * 其它位保留。
     * 1B	Plug_Nb	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     * <p>
     * <p>
     * 修改后：
     * 11	6Bs	NodeID	节点ID号
     * 17	2Bs	ChargeFull_Ith	充满电电流阈值，单位1mA（1~1000）
     * 19	1B	Temp Above	温度保护上限阈值，单位1℃（60~120）
     * 20	2Bs	OverLoad_th	充电过载保护阈值，单位0.1A（1~35）
     * 22	2Bs	Power Above	功率保护上限阈值，单位0.1W(0~11000)
     * 24	1B
     * 25	1B
     * 26	1B
     * 27	1B
     * 28	1B
     * 29	2Bs	Voltage Below	电压保护下限阈值，单位0.1V（800 ~1200）
     * 31	2Bs	Voltage Above	电压保护上限阈值，单位0.1V（2200~2900）
     * 33	1B	Plug_Nb	插座编号
     * 34	4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 38	1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者 不 在线
     */
    public static RespWrapper<EquipNodeParams> findNodeParam(byte[] by, boolean isSmallPattern) {
        try {
            if (by == null || by.length != 28) {
                return RespWrapper.makeFailResp("检查数据格式！");
            }
            byte[] lv2 = new byte[2];
            byte[] lv6 = new byte[6];
            EquipNodeParams dp = new EquipNodeParams();
            System.arraycopy(by, 0, lv6, 0, 6);
            dp.setNodeNo(ProtocolIDUtil.decodeEvenProtocolID(lv6));//节点编号

            System.arraycopy(by, 6, lv2, 0, 2);
            dp.setMinI(BaseTypeConvert.bytes2IntBE(lv2));//电流下
            dp.setMaxT((int) by[8]);//温度
            System.arraycopy(by, 9, lv2, 0, 2);
            dp.setMaxI(BaseTypeConvert.bytes2IntBE(lv2));//电流上

            System.arraycopy(by, 11, lv2, 0, 2);
            dp.setMaxP(BaseTypeConvert.bytes2IntBE(lv2));//功率上

            //dp.setLeakI(by[13]);
            //dp.setLeakIT(by[14]);
            //dp.setChargeTimeout(by[15]);
            //dp.setChargeStartI(by[16]);
            //dp.setDisconI(by[17]);
            System.arraycopy(by, 18, lv2, 0, 2);
            dp.setMinV(BaseTypeConvert.bytes2IntBE(lv2));//电压下
            System.arraycopy(by, 20, lv2, 0, 2);
            dp.setMaxV(BaseTypeConvert.bytes2IntBE(lv2));//电压上
            //byte[] b1 = BaseTypeConvert.byte2EightBytes(by[20]);
            //byte[] b2 = BaseTypeConvert.byte2EightBytes(by[21]);
            //byte[] fb = new byte[16];
			/*if(isSmallPattern){
				System.arraycopy(b1, 0, fb, 0, 8);
				System.arraycopy(b2, 0, fb, 8, 8);
			}else{
				System.arraycopy(b2, 0, fb, 0, 8);
				System.arraycopy(b1, 0, fb, 8, 8);
			}
			dp.setFaultEnable(new FaultEnableReport(fb));*/
            dp.setPlugNo((int) by[22]);//插座
            byte[] lv4 = new byte[4];
            System.arraycopy(by, 23, lv4, 0, 4);
            //dp.putExtra(ProjectConstant.SIGN_ID, BaseTypeConvert.decodeHardwareSign(lv4));
            dp.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));//签名
            dp.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[27]));//操作结果
            return RespWrapper.makeSuccessResp(dp);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析有误！请检查！");
        }
    }

    /**
     * 设置节点RTC  03H
     * 6Bs	NodeID	节点ID号
     * 7Bs	Date	星期-年-月-日-时-分-秒 ；年=实际年-2000;十六进制格式
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     */
    public static byte[] setNodeRTC(String nodeNo, String sign) {
        byte[] res = new byte[17];
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(nodeNo), 0, res, 0, 6);
        System.arraycopy(BaseTypeConvert.createDateFormat(), 0, res, 6, 7);
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, res, 13, 4);
        return res;
    }

    /**
     * 查询节点状态及故障信息  10H
     * 6Bs	NodeID	节点ID号
     * 1B	Status	状态字：0-空闲，1-占用,2-正在充电中，3-充电完成，4-故障，5-维护
     * 2Bs	Plug	0-插座未插入，1—插座插入
     * 2Bs	Fault	故障字：Bit0：过压；Bit1：欠压；Bit2：过流；
     * Bit3：过载；Bit4：过温；
     * Bit5：负载连接失败(请先确保连接负载，再下订单)
     * Bit6：漏电流保护；Bit7：接地故障；
     * Bit8：充电超时；   其它位保留。
     * 1B	Plug_Nb	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     * <p>
     * 协议修改后：
     * 11	6Bs	NodeID	节点ID号
     * 17	1B	Status	状态字：0-空闲，1-占用，4-故障
     * 18	2Bs
     * 20	2Bs	Fault	故障字：Bit0：过压；Bit1：欠压；Bit2：过流；
     * Bit3：过载；Bit4：过温；其它位保留。
     * 22	1B	Plug_Nb	插座编号
     * 23	4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 27	1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者 不 在线
     *
     * @param by
     */
    public static RespWrapper<NodeInfo> getNodeInfo(byte[] by, boolean isSmallPattern) {
        try {
            if (by == null || by.length != 17) {
                return RespWrapper.makeFailResp("请检查入参是否正确！");
            }
            NodeInfo nodeInfo = new NodeInfo();
            byte[] nodeId = new byte[6];
            System.arraycopy(by, 0, nodeId, 0, 6);
            nodeInfo.setNodeId(ProtocolIDUtil.decodeEvenProtocolID(nodeId));
            nodeInfo.setNodeStatus(NodeStatus.valueOf(by[6]));
            byte[] tmpBy = new byte[16];
            /**
             * 7 8 是检查插座的 可以忽略
             */
            byte[] byte8BytesBE = BaseTypeConvert.byte2ExpectBytes(by[7], 8);
            byte[] byte2EightBytes = BaseTypeConvert.byte2ExpectBytes(by[8], 8);
            if (isSmallPattern) {
                System.arraycopy(byte8BytesBE, 0, tmpBy, 0, 8);
                System.arraycopy(byte2EightBytes, 0, tmpBy, 8, 8);
            } else {
                System.arraycopy(byte2EightBytes, 0, tmpBy, 0, 8);
                System.arraycopy(byte8BytesBE, 0, tmpBy, 8, 8);
            }
            nodeInfo.setPlug(new PlugLink(tmpBy));
            byte[] fault1 = BaseTypeConvert.byte2ExpectBytes(by[9], 8);
            byte[] fault2 = BaseTypeConvert.byte2ExpectBytes(by[10], 8);
            byte[] faults = new byte[16];
            if (isSmallPattern) {
                System.arraycopy(fault1, 0, faults, 0, 8);
                System.arraycopy(fault2, 0, faults, 8, 8);
            } else {
                System.arraycopy(fault2, 0, faults, 0, 8);
                System.arraycopy(fault1, 0, faults, 8, 8);
            }
            nodeInfo.setFaultReport(new FaultEnableReport(faults));
            nodeInfo.setPlugNb(by[11]);
            byte[] sign = new byte[4];
            System.arraycopy(by, 12, sign, 0, 4);
            //nodeInfo.putExtra(ProjectConstant.SIGN_ID, BaseTypeConvert.decodeHardwareSign(sign));
            nodeInfo.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(sign));
            nodeInfo.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[16]));
            return RespWrapper.makeSuccessResp(nodeInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析数据错误!");
        }
    }

    /**
     * 获取 主动获取充电数据 或 集中器上报数据
     * isReq = true 主机获取集中器的数据     / false 集中器主动上报
     *
     * @param by
     * @return 53byte
     * 6Bs	NodeID	节点ID号,86盒设备ID号
     * 8Bs	OderNb	订单号
     * 1B	DeviceID	设备码 (注释1)--90-2200W-PLC-86盒
     * 1B	T	温度 单位:1摄氏度，bit7=0正温度，1负温度；
     * 2Bs	V	电压 单位:0.1V
     * 4Bs	I	电流 单位:1mA
     * 4Bs	P	功率 单位: 0.001kW
     * 4Bs	EP	当前电量 单位: 0.01kWH
     * 4Bs	EP_Last	充电前的电量单位: 0.01kWH
     * <p>
     * 2Bs	Min	充电时长 单位: 1min
     * 2Bs	Ileak	漏电流 单位: 0.1mA
     * 2Bs	F	单位: 0.01Hz
     * 1B	PF	功率因素 单位: 0.01
     * 6Bs	Date	时间： 如果下发的订单号是正在充电，则返回当前时间；如果下发的订单号已经结束充电，则返回结束充电的时间。数据格式： 年-月-日-时-分-秒,实际年=年+2000；
     * 1B	Plug_Nb	插座编号 从0开始
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     * <p>
     * 集中器主动上报节点充电数据  53H
     * 6Bs	NodeID	节点ID
     * 8Bs	OderNb	订单号
     * 1B	DeviceID	设备码:(注释1)--91-500W-PLC-86盒
     * 1B	T	温度---单位:1摄氏度，bit7=0正温度，1负温度；
     * 2Bs	V	电压---单位:0.1V
     * 4Bs	I	电流---单位:1mA
     * 4Bs	P	功率---单位: 0.001kW
     * 4Bs	EP	当前电量--单位: 0.01kWH
     * 4Bs	EP_Last	充电前的电量-- 单位: 0.01kWH
     * 2Bs	Min	充电时长---单位: 1min
     * 2Bs	Ileak	漏电流---单位: 0.1mA
     * 2Bs	F	频率---单位: 0.01Hz
     * 1B	PF	功率因素--单位: 0.01
     * 1B	Plug_Nb	插座编号
     */
    public static RespWrapper<NodeChargeData> getNodeChargeData(byte[] by, boolean isReq) {
        int size = by.length;
        if (isReq) {
            if (size != 53) {
                return RespWrapper.makeFailResp("入参数据有误！");
            }
        } else {
            if (size != 42) {
                return RespWrapper.makeFailResp("入参数据有误！");
            }
        }
        try {
            NodeChargeData ncd = new NodeChargeData();
            byte[] lv2 = new byte[2];
            byte[] lv4 = new byte[4];
            byte[] lv6 = new byte[6];
            byte[] orderNo = new byte[8];
            System.arraycopy(by, 0, lv6, 0, 6);
            ncd.setNodeId(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            System.arraycopy(by, 6, orderNo, 0, 8);
            ncd.setOrderNo(ProtocolIDUtil.decodeEvenProtocolID(orderNo));

            ncd.setDeviceId(EquipmentCode.valueOf(by[14]));
            ncd.setT(BaseTypeConvert.byte2T(by[15]));
            System.arraycopy(by, 16, lv2, 0, 2);

            ncd.setEleV(BaseTypeConvert.bytes2IntBE(lv2));
            System.arraycopy(by, 18, lv4, 0, 4);

            ncd.setEleI(BaseTypeConvert.bytes2IntBE(lv4));

            System.arraycopy(by, 22, lv4, 0, 4);
            ncd.setEleP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 26, lv4, 0, 4);
            ncd.setEleEP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 30, lv4, 0, 4);
            ncd.setEleEPLast(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 34, lv2, 0, 2);
            ncd.setChargeTime(BaseTypeConvert.bytes2IntBE(lv2));
            System.arraycopy(by, 36, lv2, 0, 2);
            ncd.setEleILeak(BaseTypeConvert.bytes2IntBE(lv2));
            System.arraycopy(by, 38, lv2, 0, 2);
            ncd.setEleP(BaseTypeConvert.bytes2IntBE(lv2));
            ncd.setElePF(by[40]);
            if (isReq) {
                System.arraycopy(by, 41, lv6, 0, 6);
                ncd.setTime(BaseTypeConvert.getDateByBytes(lv6));
                ncd.setPlugNo(by[47]);
                System.arraycopy(by, 48, lv4, 0, 4);
                //ncd.putExtra(ProjectConstant.SIGN_ID, BaseTypeConvert.decodeHardwareSign(lv4));
                ncd.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));
                ncd.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[52]));
            } else {
                ncd.setPlugNo(by[41]);
            }
            return RespWrapper.makeSuccessResp(ncd);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析文件出错！");
        }

    }

    /**
     * 查询节点版本信息 12H
     * NodeID	节点ID号	6Bs
     * Software Info.	软件信息：年-月-日-版本，16进制 	4Bs
     * Hardware Info.	硬件信息：年-月-日-版本，16进制	4Bs
     * Signal_ID 	消息id号。接收和发送保持一致。 	4Bs
     * Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线 	1B
     *
     * @param by
     */
    public static RespWrapper<NodeVersion> getNodeVersion(byte[] by) {
        if (by == null || by.length != 19) {
            return RespWrapper.makeFailResp("请检查数据！");
        }
        try {
            byte[] lv6 = new byte[6];
            byte[] lv4 = new byte[4];
            NodeVersion nodeVersion = new NodeVersion();
            System.arraycopy(by, 0, lv6, 0, 6);
            nodeVersion.setNodeId(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            System.arraycopy(by, 6, lv4, 0, 4);
            nodeVersion.setSoftware(BaseTypeConvert.getNodeVersion(lv4, false));
            System.arraycopy(by, 10, lv4, 0, 4);
            nodeVersion.setHardware(BaseTypeConvert.getNodeVersion(lv4, true));
            System.arraycopy(by, 14, lv4, 0, 4);
            //nodeVersion.putExtra(ProjectConstant.SIGN_ID,BaseTypeConvert.decodeHardwareSign(lv4));
            nodeVersion.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));
            nodeVersion.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[18]));
            return RespWrapper.makeSuccessResp(nodeVersion);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析错误！请检查！");
        }
    }

    //68 10 00 00 00 00 00 68 80 14 12 10 00 00 00 01 00 18 05 22 01 18 05 10 01 01 23 45 67 01 D6 16
    //TODO
    public static void main(String[] args) throws Exception {
        /**
         * 过温
         */
        //byte[] res = new byte[]{0,0,0,65,9,-128,21,54,32,3,51,-106,-128,19,1,0,0,0,0,0,0,0,0,0,0,2,0,16,0,0,0,0,0};
        /**过压*/
        //byte[] res = new byte[]{0,0,0,65,9,-128,21,54,41,49,5,-121,48,67,1,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,10,84,0};
        /**过载*/
        //byte[] res = new byte[]{0,0,0,65,9,-128,21,54,48,115,20,41,32,70,1,0,0,0,0,1,0,0,0,0,0,0,0,8,36,18,34,0,6};
        /**欠压*/
		/*byte[] res = new byte[]{0,0,0,65,9,-128,21,54,41,37,115,37,80,66,1,0,0,0,0,0,0,0,0,0,0,6,0,2,0,0,0,0,0};

		RespWrapper<NodeReportData> nodeFaultSendMsg = nodeFaultSendMsg(res, false);
		System.out.println();
		System.out.println(nodeFaultSendMsg.getData().toJson());*/

        /**
         * maxI: 0
         * maxP: 0
         * maxT: 0
         * maxV: 2850
         * minI: 0
         * minV: 850
         * nodeNo: "001000031881"
         * plugNo: 1
         * terminalNo:
         */
		/*EquipNodeParams p = new EquipNodeParams();
		p.setTerminalNo("001000031880");
		p.setMaxI(800);
		p.setMaxT(60);
		p.setMaxV(2850);
		p.setMinV(850);
		p.setNodeNo("001000031881");
		p.setMaxP(0);
		p.setMinI(0);
		p.setPlugNo(1);
		String fadsSign = BaseTypeConvert.createFadsSign();

		byte[] bytes = setNodeParamsFad(p, fadsSign);
		System.out.println(Arrays.toString(bytes));*/
		/*byte by[] = new byte[]{0, 16, 0, 89, 35, -126, 0, 8, 65, 0, 5, 0, 8, 0, 0, 0, 0, 0, 3, 82, 11, 34, 7, -125, 80, 32, 3,1};
		RespWrapper<EquipNodeParams> nodeParam = findNodeParam(by, false);
		System.out.println(nodeParam.getData().toJson());*/
        //byte[] bt = new byte[]{05,01,0x18,0x10,0x19,01,0x19,04,0x20,0x16,00};
        //RespWrapper<HardwareVersion> st = getHardwareStatus(bt);
        //System.out.println(st.getMsg());
        //if(st.isSuccess())
        //    System.out.println(st.getData().toString());
        //{"maxV":2850,"terminalNo":"001000592380","nodeNo":"001000592382","maxT":65,"minI":8,"maxI":5,"minV":850,"plugNo":7,"maxP":8}
        //{"terminalNo":null,"maxV":2850,"nodeNo":"001000592382","maxT":65,"minI":8,"maxI":5,"ackCode":{"id":1,"name":"集中器受理成功/节点成功受理"},"minV":850,"signalId":"83502003","plugNo":7,"maxP":8}
        List<String> list = new ArrayList<>();
        list.add("001000592380");
        list.add("001000592381");
        byte[] bytes = whiteListToBytes(list);
        System.out.println(Arrays.toString(bytes));
        System.out.println(bytesToWhiteList(bytes));

    }

    /**
     * 查询节点数据
     * 6Bs	NodeID	节点ID号
     * 1B	Rec_Nb 	充电记录编号(0~7,0表示最近的充电记录)
     * 8Bs	OrderNum	订单号
     * 1B	FC 	结束类型:  01：故障结束；02：断开连接结束(插座被拔出)；
     * 03：充满电完成主动结束  04：完成设定充电结束(充电时间到，剩余设定充电电量为0)
     * 2Bs	Min 	充电时长---单位: 1min
     * 4Bs	EP 	充电结束电量--单位: 0.01kWH
     * 4Bs	EP_Last 	充电开始电量-- 单位: 0.01kWH
     * <p>
     * 6Bs	Date 	上次开始充电的时间 年-月-日-时-分-秒
     * 1B	Plug_Nb 	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     *
     * @param by
     * @return
     */
    public static RespWrapper<NodeChargeRecord> getNodeChargeRecord(byte[] by) {
        try {
            if (by == null || by.length != 38) {
                return RespWrapper.makeFailResp("请检查查询节点记录入参！");
            }
            byte[] lv6 = new byte[6];
            byte[] lv8 = new byte[8];
            byte[] lv4 = new byte[4];
            byte[] lv2 = new byte[2];
            System.arraycopy(by, 0, lv6, 0, 6);
            NodeChargeRecord nr = new NodeChargeRecord();
            nr.setNodeId(ProtocolIDUtil.decodeEvenProtocolID(lv6));

            nr.setRecordNo(by[6]);
            System.arraycopy(by, 7, lv8, 0, 8);
            nr.setOrderNo(ProtocolIDUtil.decodeEvenProtocolID(lv8));
            nr.setFc(ChargeEndType.valueOf(by[15]));
            System.arraycopy(by, 16, lv2, 0, 2);
            nr.setChargeTime(BaseTypeConvert.bytes2IntBE(lv2));
            System.arraycopy(by, 18, lv4, 0, 4);
            nr.setEleEndP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 22, lv4, 0, 4);
            nr.setEleStartP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 26, lv6, 0, 6);
            nr.setRecordTime(BaseTypeConvert.getDateByBytes(lv6));
            nr.setPlugNo(by[32]);
            System.arraycopy(by, 33, lv4, 0, 4);
            //nr.putExtra(ProjectConstant.SIGN_ID, BaseTypeConvert.decodeHardwareSign(lv4));
            nr.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));
            nr.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[37]));
            return RespWrapper.makeSuccessResp(nr);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("解析数据错误！");
        }
    }

    /**
     * 这是充电指令 的返回的数据
     * 开始充电指令    isSmallPattern 是小端模式 true=是，false=大端模式
     * 6Bs	NodeID	节点ID号
     * 8Bs	OrderNum	每次充电对应的订单号
     * 1B	Ret	返回码:0-订单下发成功，1-接收数据出错，2-设备忙，
     * 3-设备故障， 4-插座未插入
     * 2Bs	Fault	故障码:Bit0：过压；Bit1：欠压；Bit2：过流；Bit3：过载；Bit4：过温；Bit5：Bit5：负载连接失败(请先确保连接负载，再下订单)
     * Bit6：漏电流保护；Bit7：接地故障；
     * Bit8：充电超时；其它位保留。
     * 1B	Plug_Nb	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     */
    public static RespWrapper<ChargeBase> orderChargeStart(byte[] by, boolean isSmallPattern) {
        try {
            if (by == null || by.length != 23) {
                return RespWrapper.makeFailResp("请检查查询节点记录入参！");
            }
            ChargeBase md = new ChargeBase();
            byte[] lv4 = new byte[4];
            byte[] lv6 = new byte[6];
            byte[] lv8 = new byte[8];
            System.arraycopy(by, 0, lv6, 0, 6);
            md.setNodeNo(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            System.arraycopy(by, 6, lv8, 0, 8);
            md.setOrderNo(ProtocolIDUtil.decodeEvenProtocolID(lv8));
            md.setResCode(ChargeResCode.valueOf(by[14]));
            byte[] fault1 = BaseTypeConvert.byte2ExpectBytes(by[15], 8);
            byte[] fault2 = BaseTypeConvert.byte2ExpectBytes(by[16], 8);
            byte[] faults = new byte[16];
            if (isSmallPattern) {
                System.arraycopy(fault1, 0, faults, 0, 8);
                System.arraycopy(fault2, 0, faults, 8, 8);
            } else {
                System.arraycopy(fault2, 0, faults, 0, 8);
                System.arraycopy(fault1, 0, faults, 8, 8);
            }
            md.setFault(new FaultEnableReport(faults));
            md.setPlugNo(by[17]);
            System.arraycopy(by, 18, lv4, 0, 4);
            //md.putExtra(ProjectConstant.SIGN_ID, BaseTypeConvert.decodeHardwareSign(lv4));
            md.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));
            md.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[22]));
            return RespWrapper.makeSuccessResp(md);
        } catch (Exception e) {
            return RespWrapper.makeFailResp("下发充电指令时，解析数据出错！");
        }
    }

    /**
     * 结束充电指令 返回值
     * 6Bs	NodeID	节点ID号
     * 8Bs	OrderNum	订单号
     * 1B	Ret	返回码:0-结束订单下发成功，1-接收数据出错，2-设备忙，4-无此订单
     * 4Bs	EP	当前电量--0.01kWH
     * 4Bs	EP_Last	充电前的电量--0.01kWH
     * 2Bs	Min	充电时长--1min
     * 1B	Plug_Nb	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     * 1B	Ack_Code	应答码: 01H:节点成功受理，02H:命令或数据格式无效，03H:节点忙，04H:节点不存在或者不在线
     */
    public static RespWrapper<ChargeEnd> orderChargeEnd(byte[] by) {
        try {
            if (by.length != 31) {
                return RespWrapper.makeFailResp("入参格式不对！");
            }
            byte[] lv6 = new byte[6];
            byte[] lv8 = new byte[8];
            byte[] lv4 = new byte[4];
            byte[] lv2 = new byte[2];
            System.arraycopy(by, 0, lv6, 0, 6);
            System.arraycopy(by, 6, lv8, 0, 8);
            ChargeEnd chargeEnd = new ChargeEnd();
            chargeEnd.setNodeNo(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            chargeEnd.setOrderNo(ProtocolIDUtil.decodeEvenProtocolID(lv8));
            chargeEnd.setResCode(ChargeResCode.valueOf(by[14]));
            System.arraycopy(by, 15, lv4, 0, 4);
            chargeEnd.setEleEP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 19, lv4, 0, 4);
            chargeEnd.setEleEPLast(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 23, lv2, 0, 2);
            chargeEnd.setChargeTime(BaseTypeConvert.bytes2IntBE(lv2));
            chargeEnd.setPlugNo(by[25]);
            System.arraycopy(by, 26, lv4, 0, 4);
            chargeEnd.putExtra(ProjectConstant.SIGN_ID, ProtocolIDUtil.decodeEvenProtocolID(lv4));
            chargeEnd.putExtra(ProjectConstant.ACKCODE, AckCode.valueOf(by[30]));
            return RespWrapper.makeSuccessResp(chargeEnd);
        } catch (Exception e) {
            return RespWrapper.makeFailResp("结束充电操作，指令解析错误！");
        }
    }

    /**
     * 开始充电命令  20H 发送
     * 6Bs	NodeID	节点ID号
     * 8Bs	OrderNum	订单号
     * 1B	Mode	充电模式: 0-时间，1-金额，2-电量，3-充满, 4-固定时间
     * 2Bs	ChargeData	充电数据：单位分别是--时间(min),金额(RMB分)，电量0.01度(0.01kWh)，充满(xx xx)
     * 1B	Plug_Nb 	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     *
     * @return
     */

    public static byte[] setChargeStartSend(ChargeStart cs, String sign) {
        byte[] res = new byte[22];
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(cs.getNodeId()), 0, res, 0, 6);
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(cs.getOrderNo()), 0, res, 6, 8);
        res[14] = (byte) cs.getMode();
        System.arraycopy(BaseTypeConvert.int2BytesBE(cs.getValue(), 2), 0, res, 15, 2);
        res[17] = (byte) cs.getPlugNo();
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, res, 18, 4);
        return res;
    }

    /**
     * 结束订单
     * <p>
     * 6Bs	NodeID	节点ID号
     * 8Bs	OrderNum	订单号
     * 1B	Plug_Nb	插座编号
     * 4Bs	Signal_ID 	消息id号。接收和发送保持一致。
     */
    public static byte[] createEndChargeBytes(String nodeNo, String orderNo, int plugNo, String sign) {
        byte[] res = new byte[19];
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(nodeNo), 0, res, 0, 6);
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(orderNo), 0, res, 6, 8);
        res[14] = (byte) plugNo;
        System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, res, 15, 4);
        return res;
    }

    /**
     * 节点异常主动上报数据  50H  isSmallPattern 是小端模式 true=是，false=大端模式
     * 6Bs	NodeID	节点ID号   0,0,0,65,9,96
     * 8Bs	OrderNum	订单号   21,41,115,84,36,97,80,1
     * 1B	FC	结束类型:  01：故障结束；02：断开连接结束(插座被拔出)；   3
     * 03：充满电完成主动结束  04：完成设定充电结束(充电时间到，剩余设定充电电量为0)
     * 1B	Status	状态字:  0-空闲，1-占用,2-正在充电中，3-充满，4-故障，5-维护      0
     * 4Bs	EP	当前电量--单位: 0.01kWH    0,0,0,0
     * 4Bs	EP_Last	充电前的电量-- 单位: 0.01kWH   0,0,0,0
     * 2Bs	Min	充电时长---单位: 1min  0,1
     * <p>
     * 2Bs	Fault	故障字：Bit0：过压；Bit1：欠压；Bit2：过流；0,0
     * Bit3：过载；Bit4：过温；
     * Bit5：负载连接失败(请先确保连接负载，再下订单)
     * Bit6：漏电流保护；Bit7：接地故障；
     * Bit8：充电超时；   其它位保留。
     * <p>
     * 4Bs	FaultData	故障数据： 数据单位，过压、欠压--0.1V；过流--1mA；  0,0,0,0
     * 过载--0.001kW；过温--1摄氏度；漏电保护--0.1mA；负载连接异常--数据为0，无单位；
     * 1B	Plug_Nb	插座编号  0
     * 0,0,0,65,9,-128,  21,54,19,105,84,67,32,18,1,  0,   0,0,0,0 ,0,0,0,0, 0,9,  0,16,  0,0,0,0,   0
     * TODO
     */
    public static RespWrapper<NodeReportData> nodeFaultSendMsg(byte[] by, boolean isSmallPattern) {
        try {
            if (by == null || by.length != 33) {
                return RespWrapper.makeFailResp("传入参数有误！请检查！");
            }
            byte[] lv2 = new byte[2];
            byte[] lv4 = new byte[4];
            byte[] lv6 = new byte[6];
            byte[] lv8 = new byte[8];
            NodeReportData md = new NodeReportData();
            System.arraycopy(by, 0, lv6, 0, 6);
            md.setNodeId(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            System.arraycopy(by, 6, lv8, 0, 8);
            md.setOrderNo(ProtocolIDUtil.decodeEvenProtocolID(lv8));
            md.setEndType(ChargeEndType.valueOf(by[14]));
            md.setNodeStatus(NodeStatus.valueOf(by[15]));
            System.arraycopy(by, 16, lv4, 0, 4);
            md.setEleEP(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 20, lv4, 0, 4);
            md.setEleEPLast(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(by, 24, lv2, 0, 2);
            md.setChargeTime(BaseTypeConvert.bytes2IntBE(lv2));
            byte[] faults = new byte[16];
            byte[] fault1 = BaseTypeConvert.byte2ExpectBytes(by[26], 8);
            byte[] fault2 = BaseTypeConvert.byte2ExpectBytes(by[27], 8);
            if (isSmallPattern) {
                System.arraycopy(fault1, 0, faults, 0, 8);
                System.arraycopy(fault2, 0, faults, 8, 8);
            } else {
                System.arraycopy(fault2, 0, faults, 0, 8);
                System.arraycopy(fault1, 0, faults, 8, 8);
            }
            md.setFault(new FaultEnableReport(faults));
            System.arraycopy(by, 28, lv4, 0, 4);
            md.setFaultVaule(BaseTypeConvert.bytes2IntBE(lv4));
            //md.setFaultVaule();
            md.setPlugNo(by[32]);
            return RespWrapper.makeSuccessResp(md);
        } catch (Exception e) {
            return RespWrapper.makeFailResp("下发充电指令时，解析数据出错！" + e.getMessage());
        }
    }

    /**
     * 告警上报数据  是否是小端模式isSmallPattern true=是，false=大端
     * 6Bs	NodeID	节点ID号
     * 1B	Plug_Nb	插座编号
     * 2B	Err	过压Bit0 = 1,欠压Bit1 = 1,过流Bit2 = 1，过功Bit3 = 1,过温Bit4 = 1;为0则无故障；
     * 1B	T	温度---单位：1摄氏度，bit7=0正温度，1负温度
     * 2Bs	电压数值	电压---单位：0.1V
     * 4Bs	电流数值	电流---单位：1mA
     * 4Bs	功率数值	功率---单位：0.001kW
     */
    public static RespWrapper<NodeWarnData> convertWarnData(byte[] data,
                                                            boolean isSmallPattern) {
        try {
            if (data == null) return RespWrapper.makeFailResp("上报告警数据不能为空");
            int size = data.length;
            if (size != 20) return RespWrapper.makeFailResp("上报告警数据格式不对");
            NodeWarnData da = new NodeWarnData();
            byte[] lv6 = new byte[6];
            System.arraycopy(data, 0, lv6, 0, 6);
            da.setNodeNo(ProtocolIDUtil.decodeEvenProtocolID(lv6));
            da.setPlugNo(data[6]);
            byte[] lv2 = new byte[2];
            System.arraycopy(data, 7, lv2, 0, 2);
            /**这里需要增加对异常的数据处理*/
            da.setList(createWarn(lv2));
            da.setEquipT(Integer.valueOf(BaseTypeConvert.byte2T(data[9], false)));
            System.arraycopy(data, 10, lv2, 0, 2);
            da.setEleV(BaseTypeConvert.bytes2IntBE(lv2));
            byte[] lv4 = new byte[4];
            System.arraycopy(data, 12, lv4, 0, 4);
            da.setEleI(BaseTypeConvert.bytes2IntBE(lv4));
            System.arraycopy(data, 16, lv4, 0, 4);
            da.setEleP(BaseTypeConvert.bytes2IntBE(lv4));
            return RespWrapper.makeSuccessResp(da);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp(e.getMessage());
        }
    }

    /**
     * 创建告警数据
     */
    private static List<EquipWarnType> createWarn(byte[] lv2) {
        int bt = BaseTypeConvert.bytes2IntBE(lv2);
        List<EquipWarnType> list = new ArrayList<EquipWarnType>();
        if (getOffInt(bt, 0) == 1) {
            list.add(EquipWarnType.TOO_V);
        }
        if (getOffInt(bt, 1) == 1) {
            list.add(EquipWarnType.OWE_V);
        }
        if (getOffInt(bt, 2) == 1) {
            list.add(EquipWarnType.TOO_I);
        }
        if (getOffInt(bt, 3) == 1) {
            list.add(EquipWarnType.TOO_p);
        }
        if (getOffInt(bt, 4) == 1) {
            list.add(EquipWarnType.TOO_T);
        }
        if (list.size() > 0)
            return list;
        return null;
    }

    private static byte getOffInt(int bt, int off) {
        return (byte) (bt >> off & 0x1);
    }

    public static RespWrapper<WattMeterHelp> findEleTotalStatus(byte[] data) {
        if (data.length != 27)
            return RespWrapper.makeFailResp("数据长度不是27");
        WattMeterHelp help = new WattMeterHelp();
        byte[] lv2 = new byte[2];
        System.arraycopy(data, 0, lv2, 0, 2);
        help.setVa(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 2, lv2, 0, 2);
        help.setVb(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 4, lv2, 0, 2);
        help.setVc(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 6, lv2, 0, 2);
        help.setIa(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 8, lv2, 0, 2);
        help.setIb(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 10, lv2, 0, 2);
        help.setIc(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 12, lv2, 0, 2);
        help.setPa(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 14, lv2, 0, 2);
        help.setPb(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 16, lv2, 0, 2);
        help.setPc(BaseTypeConvert.bytes2IntBE(lv2));
        System.arraycopy(data, 18, lv2, 0, 2);
        help.setTotalP(BaseTypeConvert.bytes2IntBE(lv2));
        help.setPfa(BaseTypeConvert.byte2Int(data[20]));
        help.setPfb(BaseTypeConvert.byte2Int(data[21]));
        help.setPfc(BaseTypeConvert.byte2Int(data[22]));
        help.setPfs(BaseTypeConvert.byte2Int(data[23]));
        lv2 = new byte[3];
        System.arraycopy(data, 24, lv2, 0, 3);
        help.setEleTotal(BaseTypeConvert.bytes2IntBE(lv2));
        return RespWrapper.makeSuccessResp(help);
    }

    // 白名单处理
    public static List<String> bytesToWhiteList(byte[] data) {
        List<String> list = new ArrayList<>();
        int size = data.length;
        if (size == 1) return list;
        byte b1 = data[0];
        int len = b1 < 0 ? 256 + b1 : b1;
        if ((size - 1) % 6 != 0)
            return list;
        if ((size - 1) / 6 != len)
            return list;
        byte[] bytes = new byte[6];
        for (int i = 0; i < len; i++) {
            System.arraycopy(data, i * 6 + 1, bytes, 0, 6);
            list.add(ProtocolIDUtil.decodeEvenProtocolID(bytes));
        }
        return list;
    }

    public static byte[] whiteListToBytes(List<String> list) {
        if (list == null || list.isEmpty()) return new byte[0];
        int size = list.size();
        byte[] bytes = new byte[size * 6 + 1];
        bytes[0] = (byte) size;
        for (int i = 0; i < size; i++) {
            System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(list.get(i)), 0, bytes, 1 + i * 6, 6);
        }
        return bytes;
    }

}
