package com.comtika.safe.core.task;

import com.comtika.safe.dao.safe.IWarnRecordDao;
import com.comtika.safe.entity.warn.WarnRecord;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 告警处理数据
 */
@Component
@Log4j2
public class WarnHandleTask {
    @Autowired
    private IWarnRecordDao iWarnRecordDao;
    private static final int chceckRate = 5;//5分钟
    private static final long timeout_warn = 5*60*1000;//5分钟
    @Data
    class WarnCache{
        private WarnRecord record;
        private long time;
    }
    private static Map<String,WarnCache> WARN_CACHE_MAP = new ConcurrentHashMap<>();
    private static final ScheduledThreadPoolExecutor TIMER_CHECK_TERMINAL = new ScheduledThreadPoolExecutor(2, (r) -> new Thread(r, "Thread name:协议层，处理心跳与异常挂单线程"));

    @PostConstruct
    public void startCheckWarnTask(){
        // 这里添加到缓存中
        initCache();
        TIMER_CHECK_TERMINAL.scheduleAtFixedRate(() -> {
            timerWanHandleCore();
        }, chceckRate, chceckRate, TimeUnit.MINUTES);
    }

    private void initCache() {
        try {
            WarnRecord record = new WarnRecord();
            record.setWarnStatus(0);//告警状态
            long startTime = System.currentTimeMillis();
            iWarnRecordDao.template(record).forEach(s->{
                WarnCache warnCache = new WarnCache();
                warnCache.setRecord(s);
                warnCache.setTime(startTime);
                WARN_CACHE_MAP.put(createKey(s.getSerialNo(),s.getWarnType()),warnCache);
            });
        }catch (Exception e){
            e.printStackTrace();
            log.error("初始化告警数据错误：{}",e.getMessage());
        }

    }

    @PreDestroy
    public void stopCheckWarnTask(){
        TIMER_CHECK_TERMINAL.shutdown();
    }

    private void timerWanHandleCore() {
        List<String> delList = new ArrayList<>(8);
        long startTime = System.currentTimeMillis();
        WARN_CACHE_MAP.forEach((k,v)->{
            if (v.getTime() + timeout_warn < startTime) {
                // 这里要删除数据
                WarnRecord record = v.getRecord();
                boolean isFendMsg = record.getStatus() != null && record.getStatus() == 0;
                record.setWarnStatus(1);
                record.setStatus(null);// 这个状态不修改
                record.setUpdateTime(System.currentTimeMillis());
                iWarnRecordDao.updateTemplateById(record);
                delList.add(k);
                // todo 后期这里需要告警推送

            }
        });
        delList.forEach(s->{
            WARN_CACHE_MAP.remove(s);
        });
    }
    private static String createKey(String serialNo,int type){
        return String.format(WARN_KEY,serialNo,type);
    }
    private static String WARN_KEY = "%s-%s";
    public WarnRecord findWarnCache(String key){
        WarnCache warnCache = WARN_CACHE_MAP.get(key);
        if(warnCache != null) return warnCache.getRecord();
        return null;
    }
    public void updateWarnCache(String key,WarnRecord record){
        WarnCache cache = new WarnCache();
        cache.setRecord(record);
        cache.setTime(System.currentTimeMillis());
        WARN_CACHE_MAP.put(key,cache);
    }


}
