package com.comtika.safe.core.weChat;

import com.alibaba.fastjson.JSONObject;
import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespResult;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.http.HttpUtil;
import com.comtika.safe.config.WeChatProperties;
import com.comtika.safe.entity.warn.help.WarnMessageHelp;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 11:53 2019/11/13
 * @DESC: todo 这个是发送消息方法，后期需要处理
 **/
@Component
@Log4j2
public class WxChatUtil {
    @Autowired
    private WeChatProperties weChatProperties;
    private static final String TOKEN_KEY = "ACCESS_TOKEN";
    private static final String AC_PUSH_URL = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN";
    private static final String WX_TOKEN_URL = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET";
    // 发生公众号消息方法
    public JSONObject sendTemplateMsg(String accessToken, String json) {
        if(StringUtil.isEmpty(accessToken)) {
            log.error("token 为空~~");
            return null;
        }
        String requestUrl = AC_PUSH_URL.replace(TOKEN_KEY, accessToken);
        return JSONObject.parseObject(HttpUtil.doPost(requestUrl, json));
    }
    private static boolean isText = false;



    // 获取程序的openid
    public JSONObject findWeChatAuth2Data(String code, boolean isSmall) {
        String url = null;
        if (isSmall)
            url = ProjectConstant.ThirdConstant.WX_CODE_SESSION_URL.replace(ProjectConstant.ThirdConstant.WX_APPID, weChatProperties.getAppId()).replace(ProjectConstant.ThirdConstant.WX_SECRET, weChatProperties.getSecret()).replace(ProjectConstant.ThirdConstant.WX_JSCODE, code);
        else
            url = ProjectConstant.ThirdConstant.WX_AC_SESSION_URL.replace(ProjectConstant.ThirdConstant.WX_APPID, weChatProperties.getAcAppId()).replace(ProjectConstant.ThirdConstant.WX_SECRET, weChatProperties.getAcSecret()).replace(ProjectConstant.ThirdConstant.WX_JSCODE, code);
        JSONObject jsonResult = JSONObject.parseObject(HttpUtil.doGet(url));
        if (jsonResult.getIntValue("errcode") != 0) {
            RespResult.buidException(jsonResult.getIntValue("errcode") + ",errmsg" + jsonResult.getString("errmsg"));
        }
        return jsonResult;
    }

    public String getTocken(boolean isSmall){
        if (isSmall) {
            return getAccessToken(weChatProperties.getAppId(), weChatProperties.getSecret());
        }
        return getAccessToken(weChatProperties.getAcAppId(), weChatProperties.getAcSecret());

    }

    private String getAccessToken(String appId, String secret) {
        String requestUrl = WX_TOKEN_URL.replace("APPID", appId).replace("APPSECRET", secret);
        String s = HttpUtil.doPost(requestUrl);
        JSONObject jsonObject = JSONObject.parseObject(s);
        // 如果请求成功
        if (jsonObject != null) {
            return jsonObject.getString("access_token");
        }else System.out.println(jsonObject.toJSONString());
        return null;
    }

    // 发生消息到被访人中
    public RespWrapper<Boolean> sendMessageToBiVisit(String visitName, Long visitTime, String visitCause, String acOpenId) {
        return sendMessageToBiVisit(visitName,visitTime,visitCause,acOpenId,"");
    }
    public RespWrapper<Boolean> sendMessageToBiVisit(String visitName, Long visitTime, String visitCause, String acOpenId, String token) {
        if(StringUtil.isEmpty(acOpenId)) return RespWrapper.makeFailResp("微信公众号openId 不能为空");
        RespWrapper<String> vm = MessageDataHandle.createVisitMessage(visitName, visitTime, visitCause);
        if (!vm.isSuccess()) return RespWrapper.makeFailResp(vm.getMsg());
        //回调地址
        String json = MessageDataHandle.toJsonPN(true,null, acOpenId, AcTemplateMessage.BI_VISIT.templateId, vm.getData());
        if(!isText)
            token = getTocken(false);
        // 下面发生消息
        JSONObject jsonObject = sendTemplateMsg(token, json);
        log.info("发生结果为:{}" , jsonObject.toJSONString());
        return RespWrapper.makeSuccessResp(true);
    }
    public RespWrapper<Boolean> sendMessageToVisit(boolean isSuccess,String remark,String acOpenId,String token){
        if(StringUtil.isEmpty(acOpenId)) return RespWrapper.makeFailResp("微信公众号openId 不能为空");
        RespWrapper<String> vm = MessageDataHandle.createValidVisitMessage(isSuccess, remark);
        if (!vm.isSuccess()) return RespWrapper.makeFailResp(vm.getMsg());
        //回调地址
        String json = MessageDataHandle.toJsonPN(true,null, acOpenId, AcTemplateMessage.VISIT.templateId, vm.getData());
        if(!isText)
            token = getTocken(false);
        // 下面发生消息
        JSONObject jsonObject = sendTemplateMsg(token, json);
       log.info("发生结果为:{}" , jsonObject.toJSONString());
        return RespWrapper.makeSuccessResp(true);
    }
    public RespWrapper<Boolean> sendWarnMessageToUser(WarnMessageHelp help, List<String> acOpenIdList, String token){
        if(acOpenIdList.isEmpty()) return RespWrapper.makeFailResp("微信公众号openId 不能为空");
        RespWrapper<String> vm = createWarnMessage(help);
        if (!vm.isSuccess()) return RespWrapper.makeFailResp(vm.getMsg());
        /*if(StringUtil.isEmpty(token))
            token = getTocken(false);
        boolean isOne = true;*/
        for(String acOpenId:acOpenIdList){
            if(!isText){
                token = getTocken(false);
                if(StringUtil.isEmpty(token)){
                    log.error("获取token 失败，检查公众号是否配置");
                    return RespWrapper.makeFailResp("获取token 失败，检查公众号是否配置");
                }
            }
            //回调地址
            String json = MessageDataHandle.toJsonPN(false,null, acOpenId, AcTemplateMessage.WARN.templateId, vm.getData());
           // if(!isOne) token = getTocken(false);
            // 下面发生消息
            JSONObject jsonObject = sendTemplateMsg(token, json);
            System.out.println("发生结果为:" + jsonObject.toJSONString());
            // isOne = false;
        }
        //System.out.println("发生结果为:" + jsonObject.toJSONString());
        return RespWrapper.makeSuccessResp(true);
    }
    public static RespWrapper<String> createWarnMessage(WarnMessageHelp help){
        return MessageDataHandle.createWarnMessage(help);
    }
    public RespWrapper<Boolean> sendMessageToVisit(boolean isSuccess,String remark,String acOpenId){
        return sendMessageToVisit(isSuccess,remark,acOpenId,null);
    }
    public static void main(String[] args){
        String requestUrl = AC_PUSH_URL.replace(TOKEN_KEY, null);
        // WxChatUtil util = new WxChatUtil();
        // // String accessToken = util.getAccessToken("wxe05f9c9968b9bec9", "4526128c036c061202ee6e8051cb1a44");
        // String accessToken = util.getAccessToken("wxc915da2df176576a", "d91dd4fd9e6f39c344e069ef5250a39b");
        // // String accessToken = util.getAccessToken("wx385f55b4aef20405", "8636e519d8922784c7d10b7b729ce633");
        // // String accessToken = util.getAccessToken("wxc915da2df176576a", "2d03d05ca1064e4c880a3e094b6306ce");
        // System.out.println(accessToken);
        // //  util.sendMessageToBiVisit("张三",System.currentTimeMillis(),"测试","oz5lct2Z7wgVgnH8LOFB7A0TrbG8",accessToken);
        // // util.sendMessageToVisit(true,"不满足要求","oz5lct-zspfVoDoK_ftkNpxt4Oqc",accessToken);
        // WarnMessageHelp help = new WarnMessageHelp();
        // help.setSiteName("测试站点");
        // help.setTerminalNo("0123456789");
        // help.setTerminalType("三相电");
        // //help.setRemark("");
        // help.setWarnMsg("测试告警");
        // util.sendWarnMessageToUser(help, Arrays.asList("oFf-7wbPidWyi6JYx56svUrZefkI"),accessToken);
    }
    private enum AcTemplateMessage{
        BI_VISIT("T91U0IUos-MebRZki-NDf6TEZm14BiWcJPl1BA_X4Co"),
        VISIT("fGOLBXNhDi1KGCpgoGL27hQOfRidqWwJg4SZpf7F77A"),
        WARN("zr1-heR2-fAcTKXV_XM5W3WRHPnaWu7a9nvnLCzMALU");
        private String templateId;
        AcTemplateMessage(String templateId){
            this.templateId = templateId;
        }
    }
}
