package com.comtika.safe.entity.equip;

import java.util.ArrayList;
import java.util.List;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;

import lombok.Data;

/**
 * 批量新增设备入参实体类
 * @author Jimmy
 *
 */
@Data
public class AddEquipListEntity {
	
	private String startSerialNo; 		//批量导入设备 - 开始设备编码
	
	private String endSerialNo; 		//批量导入设备 - 结束设备编码
	
	private Long siteId;				// 站点id
	
	private Long equipProductTypeId;	//产品类型id
	
	private String remark;				//备注

	public RespWrapper<Object> ParameterValidation() {
		if(StringUtil.isEmpty(this.startSerialNo)) return RespWrapper.makeFailResp("未输入开始设备编码！");
		if(StringUtil.isEmpty(this.endSerialNo)) return RespWrapper.makeFailResp("未输入结束设备编码！");
		if(this.siteId == null) return RespWrapper.makeFailResp("未选择站点！");
		if(this.equipProductTypeId == null) return RespWrapper.makeFailResp("未选择产品类型！");
		//验证设备数量  -- 暂时不能超过100台
		try {
			Long startSerialNo = Long.parseLong(this.startSerialNo, 16);
			Long endSerialNo = Long.parseLong(this.endSerialNo, 16);
			if(endSerialNo < startSerialNo)return RespWrapper.makeFailResp("结束设备编码不能小于开始设备编码！");
			if((endSerialNo - startSerialNo) > 99) return RespWrapper.makeFailResp("批量新增设备暂时只支持100台以内！");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("输入的设备编码有误！");
		}
		return RespWrapper.makeSuccessResp("参数验证通过！");
	}
	
	
	public List<String> createSerialNo() {
		//将16进制转成10进制
		Long startSerialNo = Long.parseLong(this.startSerialNo, 16);
		Long endSerialNo = Long.parseLong(this.endSerialNo, 16);
		System.out.println("十六进制转成十进制结果startSerialNo："+startSerialNo +",endSerialNo:" + endSerialNo);
		List<String> list = new ArrayList<String>();
    	for( Long i = startSerialNo; i <= endSerialNo; i++ ){
    		//将10进制转成16进制
    		String serialNo = Long.toHexString(i).toUpperCase();
    		list.add(serialNo);
    	}
    	System.out.println("十进制转成十六进制结果："+String.join(",", list));
		return list;
	}
	
	public static void main(String[] args) {
		Long startSerialNo = Long.parseLong("10F09AA", 16);
		System.out.println(startSerialNo);
		String a = Long.toHexString(startSerialNo).toUpperCase();
		System.out.println(a);
	}
}
