package com.comtika.safe.entity.equip.terminal;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;

import lombok.Data;

/**
 * 
 * @Description:设备新增、修改参数实体--仅用于包装参数
 * @version V1.0
 */
@Data
public class EquipTerminalAddEntity {
	
	private Long terminalId;	//数据库表id

	private String terminalNo;// 设备编码

	private String nodeOneNo;// 子板1 id

	private String nodeTwoNo;// 子板2 id

	private String serialNo;// 设备流水号
	
	private Long siteId;// 站点id

    private Long equipProductTypeId;//产品类型id
    
    private Long merchantId;	//商户id

	private String remark;//备注

	public RespWrapper<Object> ParameterValidation() {
		if(StringUtil.isEmpty(this.terminalNo)) return RespWrapper.makeFailResp("设备ID未填写！");
		if(StringUtil.isEmpty(this.nodeOneNo)) return RespWrapper.makeFailResp("1子板未填写！");
		if(StringUtil.isEmpty(this.nodeTwoNo)) return RespWrapper.makeFailResp("1子板未填写！");
		if(StringUtil.isEmpty(this.serialNo)) return RespWrapper.makeFailResp("设备编码未填写！");
		if(this.siteId == null) return RespWrapper.makeFailResp("未选择站点！");
		if(this.equipProductTypeId == null) return RespWrapper.makeFailResp("未选择产品类型！");
		RespWrapper<Object> checkString = checkString(this.serialNo,this.terminalNo,this.nodeOneNo,this.nodeTwoNo);
		if(!checkString.isSuccess())return checkString;
		return RespWrapper.makeSuccessResp("参数验证通过！");
		
	}
	
	/***
     * 校验流水号是否符合规则  流水号 集中器编号  字板编号1 字板编号2
     */
    private RespWrapper<Object> checkString(String target,String...strs){
    	try {
    		int length = target.length();
        	if(length < 8 || length > 10){
        		return RespWrapper.makeFailResp("流水号不合规则！");
        	}
        	boolean bl = false;
        	for(int i=0;i<strs.length;i++){
        		if(strs[i].length() != 12){
        			bl = true;
        			break;
        		}
        		int indexOf = strs[i].indexOf(target);
        		if(indexOf < 0){
        			bl = true;
        			break;
        		}
        		String sub = strs[i].substring(indexOf+length, strs[i].length());
        		if(i == 0){
        			if(!"80".equals(sub)){
        				bl = true;
            			break;
        			}
        		}else if(i == 1){
        			if(!"81".equals(sub)){
        				bl = true;
            			break;
        			}
        		}else if(i == 2){
        			if(!"82".equals(sub)){
        				bl = true;
            			break;
        			}
        		}
        	}
        	if(bl){
        		return RespWrapper.makeFailResp("流水号不合规则！");
        	}else{
        		return RespWrapper.makeSuccessResp(true);
        	}
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("流水号不合规则！");
		}
    }
	


    
}