package com.comtika.safe.service;

import org.beetl.sql.core.engine.PageQuery;
import org.beetl.sql.core.mapper.BaseMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;


import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * 基于beetlSQL的基类实现
 */
//@Service
//@Transactional
@SuppressWarnings({ "unchecked", "hiding" })
public class BaseServiceImpl<T> implements BaseService<T>
{
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    protected BaseMapper<T> baseMapper;

    @Override
    public void saveByHiber(T entity) {
        baseMapper.insert(entity,true);
    }

    /**
     *
     * @Title: saveOrUpdateByHiber
     * @Description: TODO(实体不存在时，新增实体，存在时，更新实体)
     * void
     * @throws
     */
    public void saveOrUpdateByHiber(T entity) {
        T t = baseMapper.templateOne(entity);
        if(t == null) {
            baseMapper.insert(entity);
        }else {
            baseMapper.updateById(entity);
        }
    }

    public <T> T getByIdHiber(Serializable id) {
        return (T) baseMapper.single(id);
    }

    public <T> T getByIdUniqueHiber(Serializable id) {
        return (T) baseMapper.unique(id);
    }

    public void updateByHiber(T entity) {
        baseMapper.updateById(entity);
    }

    public void updateTemplateById(T entity) {
        baseMapper.updateTemplateById(entity);
    }

    public void deleteById(Serializable id) {
        baseMapper.deleteById(id);
    }

	public List<T> listAll(int pageStart, int pageSize) {
		int start = (pageStart - 1) * pageSize + 1;
		return baseMapper.all(start, pageSize);
	}

	public List<T> listAll() {
        return baseMapper.all();
    }

    public long count() {
        return baseMapper.allCount();
    }

    public List<T> listByHiber(T entity) {
        return baseMapper.template(entity);
    }

    public <T> T findByHiber(T entity) {
        return (T) baseMapper.templateOne(entity);
    }

    public int getPageStart(int pageStart, int pageSize) {
        return (pageStart - 1) * pageSize;
    }

    public PageQuery<T> pageQueryList(String sqlId,PageQuery<T> query,Class<T> clazz) {
        return baseMapper.getSQLManager().pageQuery(sqlId, clazz, query);
    }



    /**
     * @Title: getCacheObject
     * @Description: TODO(从缓存服务器读取对象)
     * @param key
     * @return Object
     * @throws @变更记录
     */
   /* protected Object getCacheObject(String key) {
        return redisApi.get(key);
    }*/

    /***删除黑名单的数据*/
   /* protected void deleteBlackListCache(String addrNo){
        redisApi.remove(ProjectConstant.BLACK_LIST_ADDRNO+addrNo);//删除黑名单
        try {
            Object obj = redisApi.get(ProjectConstant.BLACK_LIST_ADDRNO);
            if(obj != null){
                Map<String,Integer> map = (Map<String, Integer>) obj;
                if(map.containsKey(addrNo)){
                    map.remove(addrNo);
                    redisApi.set(ProjectConstant.BLACK_LIST_ADDRNO, map);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }*/

    /**
     *
     * @Title: getOrderId
     * @Description: TODO(生成单号--充值订单)
     * @param payTypeStr
     * @param busiStr
     * @return
     * String
     * @throws
     */
    /*public String getOrderId(String payTypeStr ,String busiStr) {
        return IDUtil.getRechargeOrderID(payTypeStr, busiStr);
    }*/

    /**
     * 批量新增实体
     * @param clazz
     * @param list
     */
    public void insertBatch(Class clazz, List<T> list) {
        baseMapper.getSQLManager().insertBatch(clazz, list);
    }

    /**
     * 批量修改实体
     * @param clazz
     * @param list
     */
    public void updateBatchTemplateById(Class clazz, List<T> list) {
        baseMapper.getSQLManager().updateBatchTemplateById(clazz, list);
    }
}
