package com.comtika.safe.service.equip.terminal.impl;

import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.utils.JXLUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.protocol.cache.SafeCache;
import com.comtika.safe.entity.equip.AddEquipListEntity;
import com.comtika.safe.entity.equip.help.TerminalExcelHelp;
import com.comtika.safe.service.warn.impl.WarnServiceImpl;

import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.equip.SocketStatus;
import com.comtika.safe.common.enums.equip.ota.OTAStatus;
import com.comtika.safe.common.enums.equip.terminal.TerminalStatusEnum;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.equip.product.ProductDao;
import com.comtika.safe.dao.equip.terminal.EquipNodeDao;
import com.comtika.safe.dao.equip.terminal.EquipSocketDao;
import com.comtika.safe.dao.equip.terminal.EquipTerminalDao;
import com.comtika.safe.dao.site.SiteDao;
import com.comtika.safe.entity.equip.product.CtkProductType;
import com.comtika.safe.entity.equip.terminal.CtkEquipNode;
import com.comtika.safe.entity.equip.terminal.CtkEquipSocket;
import com.comtika.safe.entity.equip.terminal.CtkEquipTerminal;
import com.comtika.safe.entity.equip.terminal.EquipTerminalAddEntity;
import com.comtika.safe.entity.site.CtkSite;
import com.comtika.safe.service.equip.terminal.EquipTerminalService;

@Service
public class EquipTerminalServiceImpl implements EquipTerminalService {

	@Autowired
	private ProductDao productDao;

	@Autowired
	private EquipTerminalDao equipTerminalDao;

	@Autowired
	private SiteDao siteDao;

	@Autowired
	private EquipNodeDao equipNodeDao;

	@Autowired
	private EquipSocketDao equipSocketDao;

	@Override
	public RespWrapper<Object> addEquipTerminal(EquipTerminalAddEntity equip,HttpServletRequest request) {
		try {
			//1.参数验证
			if(equip == null)return RespWrapper.makeFailResp("参数有误，请检查！！");
			RespWrapper<Object> parameterValidation = equip.ParameterValidation();
			if( !parameterValidation.isSuccess() ) return parameterValidation;

			//2.检查数据库是否存在设备编号或者设备ID
			List<Long> list = equipTerminalDao.recheckEquip(equip.getTerminalNo(),equip.getSerialNo());
			if( null != list && list.size() > 0) return RespWrapper.makeFailResp("设备编号或设备ID已存在，请检查！");

			//3.检查设备类型是否存在，以及查询该设备类型的回路数
			CtkProductType type = new CtkProductType();
			type.setId(equip.getEquipProductTypeId());
			type = productDao.templateOne(type);
			if( null == type) return RespWrapper.makeFailResp("产品类型不存在，请检查！");

			//4.查询站点，并且检查站点是否存在
			CtkSite ctkSite = new CtkSite();
			ctkSite.setSiteId(equip.getSiteId());
			ctkSite = siteDao.templateOne(ctkSite);
			if( null == ctkSite) return RespWrapper.makeFailResp("站点不存在，请检查！");

			//5.新增设备
			CtkEquipTerminal terminal = new CtkEquipTerminal();
			terminal.setMerchantId(ctkSite.getMerchantId());
            terminal.setTerminalNo(equip.getTerminalNo());
            terminal.setEquipProductTypeId(equip.getEquipProductTypeId());
            terminal.setSiteId(equip.getSiteId());
            terminal.setSerialNo(equip.getSerialNo());// 流水号
            terminal.setCreatTime(System.currentTimeMillis());
            terminal.setStatus(TerminalStatusEnum.IN_MAINTEN.getId());//初始化
            terminal.setOtaStatus(OTAStatus.OTA_SUCCESS.getId());
            terminal.setRemark(equip.getRemark());
            equipTerminalDao.insertTemplate(terminal, true);

            //6.新增子板
            List<CtkEquipNode> listNode = new ArrayList<CtkEquipNode>();
            CtkEquipNode node1 = new CtkEquipNode();
            node1.setEquipTerminalId(terminal.getTerminalId());
            node1.setNodeNo(equip.getNodeOneNo());
            CtkEquipNode node2 = new CtkEquipNode();
            node2.setEquipTerminalId(terminal.getTerminalId());
            node2.setNodeNo(equip.getNodeTwoNo());
            listNode.add(node1);
            listNode.add(node2);
            equipNodeDao.insertBatch(listNode, true);

            //7.新增回路
            //生成设备插座，协议规定，不管多少个子板，一个子板最多放8个插座。不满8个插座的，都放在子板1
            String serialStr = "";
            int num = type.getLoopNumber();
            List<CtkEquipSocket> socketList = new ArrayList<CtkEquipSocket>();
            CtkEquipSocket socket = null;
            for (int i = 1; i < num + 1; i++) {
                socket = new CtkEquipSocket();
                if( i < 9 ) {
                    socket.setEquipNodeId(node1.getId());
                    socket.setSocketNo( i - 1 );
                }else {
                    socket.setSocketNo( i - 9 );
                    socket.setEquipNodeId(node2.getId());
                }
                String dealCount = i < 10 ? ("0" + i) : i + "";
                serialStr = equip.getSerialNo() + dealCount;
                if(serialStr.length() < 12){
                	for(int p =0 ;p < 12 - serialStr.length(); p++){
                		serialStr = "0" + serialStr;
                	}
                }
                socket.setSerialNo(serialStr);
                socket.setEquipTerminalId(terminal.getTerminalId());
                socket.setSocketStatus(SocketStatus.IDLE.getId());//插座状态初始化？
                socketList.add(socket);
            }
            equipSocketDao.insertBatch(socketList);
			SafeCache.removeTerminalBaseInfo(equip.getTerminalNo());
		} catch (Exception e) {
			e.printStackTrace();
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "新增设备", e.getMessage());
			return RespWrapper.makeErrorResp("新增失败：" + e.getMessage());
		}
		return RespWrapper.makeSuccessResp("新增成功！");
	}
	
	@Override
	public RespWrapper<Object> addEquipTerminalList(AddEquipListEntity equip,HttpServletRequest request) {
		try {
			//1.参数验证
			if(equip == null)return RespWrapper.makeFailResp("参数有误，请检查！！");
			RespWrapper<Object> parameterValidation = equip.ParameterValidation();
			if( !parameterValidation.isSuccess() ) return parameterValidation;
			
			//2.生成所有设备编码
			List<String> serialNoList = equip.createSerialNo();
			
			//3.验证当前设备编码是否存在
			List<String> dataSerialNo = equipTerminalDao.querySerialNobySerialNoList(serialNoList);
			if(null != dataSerialNo && !dataSerialNo.isEmpty() && dataSerialNo.size() > 0){
				String data = String.join(",", dataSerialNo);
				return RespWrapper.makeFailResp("导入失败，已存在以下设备编码：" + data);
			}
			
			//4.检查设备类型是否存在，以及查询该设备类型的回路数
			CtkProductType type = new CtkProductType();
			type.setId(equip.getEquipProductTypeId());
			type = productDao.templateOne(type);
			if( null == type) return RespWrapper.makeFailResp("产品类型不存在，请检查！");

			//5.查询站点，并且检查站点是否存在
			CtkSite ctkSite = new CtkSite();
			ctkSite.setSiteId(equip.getSiteId());
			ctkSite = siteDao.templateOne(ctkSite);
			if( null == ctkSite) return RespWrapper.makeFailResp("站点不存在，请检查！");
			for(String serialNo : serialNoList){
                String terminalNo = serialNo + "80";
                //生成集中器编号
                if(serialNo.length() == 8){
                	terminalNo = "00" + terminalNo ;
                }else if(serialNo.length() == 9){
                	terminalNo = "0" + terminalNo;
                }
                //6.新增设备
    			CtkEquipTerminal terminal = new CtkEquipTerminal();
    			terminal.setMerchantId(ctkSite.getMerchantId());
                terminal.setTerminalNo(terminalNo);
                terminal.setEquipProductTypeId(equip.getEquipProductTypeId());
                terminal.setSiteId(equip.getSiteId());
                terminal.setSerialNo(serialNo);// 流水号
                terminal.setCreatTime(System.currentTimeMillis());
                terminal.setStatus(TerminalStatusEnum.IN_MAINTEN.getId());//初始化
                terminal.setOtaStatus(OTAStatus.OTA_SUCCESS.getId());
                terminal.setRemark(equip.getRemark());
                equipTerminalDao.insertTemplate(terminal, true);
                
                
                //生成两个子板编号
                String nodeOneNo = terminalNo.substring(0,terminalNo.length()-1) + "1";
                String nodeTwoNo = terminalNo.substring(0,terminalNo.length()-1) + "2";
                //6.新增子板
                List<CtkEquipNode> listNode = new ArrayList<CtkEquipNode>();
                CtkEquipNode node1 = new CtkEquipNode();
                node1.setEquipTerminalId(terminal.getTerminalId());
                node1.setNodeNo(nodeOneNo);
                CtkEquipNode node2 = new CtkEquipNode();
                node2.setEquipTerminalId(terminal.getTerminalId());
                node2.setNodeNo(nodeTwoNo);
                listNode.add(node1);
                listNode.add(node2);
                equipNodeDao.insertBatch(listNode, true);

                //7.新增回路
                //生成设备插座，协议规定，不管多少个子板，一个子板最多放8个插座。不满8个插座的，都放在子板1
                String serialStr = "";
                int num = type.getLoopNumber();
                List<CtkEquipSocket> socketList = new ArrayList<CtkEquipSocket>();
                CtkEquipSocket socket = null;
                for (int i = 1; i < num + 1; i++) {
                    socket = new CtkEquipSocket();
                    if( i < 9 ) {
                        socket.setEquipNodeId(node1.getId());
                        socket.setSocketNo( i - 1 );
                    }else {
                        socket.setSocketNo( i - 9 );
                        socket.setEquipNodeId(node2.getId());
                    }
                    String dealCount = i < 10 ? ("0" + i) : i + "";
                    serialStr = serialNo + dealCount;
                    if(serialStr.length() < 12){
                    	for(int p =0 ;p < 12 - serialStr.length(); p++){
                    		serialStr = "0" + serialStr;
                    	}
                    }
                    socket.setSerialNo(serialStr);
                    socket.setEquipTerminalId(terminal.getTerminalId());
                    socket.setSocketStatus(SocketStatus.IDLE.getId());//插座状态初始化？
                    socketList.add(socket);
                }
                equipSocketDao.insertBatch(socketList);
    			SafeCache.removeTerminalBaseInfo(terminalNo);
			}
		} catch (Exception e) {
			e.printStackTrace();
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "批量新增设备", e.getMessage());
			return RespWrapper.makeErrorResp("新增失败：" + e.getMessage());
		}
		return RespWrapper.makeSuccessResp("新增成功！");
	}
	
	@Override
	public RespWrapper<Object> findTerminalList(HttpServletRequest request,PageQuery<Map<String, Object>> pageQuery) {
		try {
			//获取前端参数
			if(StringUtil.isNotEmpty(request.getParameter("siteName")))pageQuery.setPara("siteName", request.getParameter("siteName"));
			if(StringUtil.isNotEmpty(request.getParameter("equipIccid")))pageQuery.setPara("equipIccid", request.getParameter("equipIccid"));
			if(StringUtil.isNotEmpty(request.getParameter("equipImei")))pageQuery.setPara("equipImei", request.getParameter("equipImei"));
			if(StringUtil.isNotEmpty(request.getParameter("terminalNo")))pageQuery.setPara("terminalNo", request.getParameter("terminalNo"));
			if(StringUtil.isNotEmpty(request.getParameter("status")))pageQuery.setPara("status", request.getParameter("status"));
			if(StringUtil.isNotEmpty(request.getParameter("equipProductTypeId")))pageQuery.setPara("equipProductTypeId", request.getParameter("equipProductTypeId"));
			if(StringUtil.isNotEmpty(request.getParameter("siteId")))pageQuery.setPara("siteId", request.getParameter("siteId"));
			//2.查询数据库
			return RespWrapper.makeSuccessResp(equipTerminalDao.findTerminalList(pageQuery));
		} catch (Exception e) {
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "设备列表", e.getMessage());
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	@Override
	public RespWrapper<Object> queryEquipTerminalById(Long terminalId,HttpServletRequest request) {
		try {
			if( null == terminalId){
				return RespWrapper.makeFailResp("参数有误，请检查！");
			}
			Map<String,Object> map = equipTerminalDao.queryEquipTerminalById(terminalId);
			int count = equipTerminalDao.findSocketCount(terminalId);
			map.put("loopCount",count);
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "设备详情", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	@Override
	public RespWrapper<Object> updateEquipTerminal(EquipTerminalAddEntity equip,HttpServletRequest request) {
		try {
			//1.参数验证
			if(equip == null)return RespWrapper.makeFailResp("参数有误，请检查！");
			RespWrapper<Object> parameterValidation = equip.ParameterValidation();
			if( !parameterValidation.isSuccess() ) return parameterValidation;
			//2.检查设备是否存在
			CtkEquipTerminal ctkEquipTerminal = new CtkEquipTerminal();
			ctkEquipTerminal.setTerminalId(equip.getTerminalId());
			ctkEquipTerminal = equipTerminalDao.templateOne(ctkEquipTerminal);
			if( null == ctkEquipTerminal) return RespWrapper.makeFailResp("设备不存在，请检查！");
			SafeCache.removeTerminalBaseInfo(equip.getTerminalNo());
			//3.检查站点是否发生了变化,如果发生了变化需要赋值
			if(! equip.getSiteId().equals(ctkEquipTerminal.getSiteId()) ){
				CtkSite ctkSite = new CtkSite();
				ctkSite.setSiteId(equip.getSiteId());
				ctkSite = siteDao.templateOne(ctkSite);
				if( null == ctkSite){
					return RespWrapper.makeFailResp("站点不存在，请检查！");
				}
				ctkEquipTerminal.setSiteId(equip.getSiteId());
				ctkEquipTerminal.setMerchantId(ctkSite.getMerchantId());
			}
			//4.检查序列号是否发生变化,如果发生了变化需要赋值,并且修改设备子板号
			CtkEquipNode ctkEquipNode = new CtkEquipNode();
			ctkEquipNode.setEquipTerminalId(equip.getTerminalId());
			List<CtkEquipNode> nodeList = null;
			if( !equip.getSerialNo().equals(ctkEquipTerminal.getSerialNo()) ){
				//检查新的序列号是否已被占用
				List<Long> list = equipTerminalDao.recheckEquip(equip.getTerminalNo(),equip.getSerialNo());
				if( null != list && list.size() > 0) return RespWrapper.makeFailResp("设备编号或设备ID已存在，请检查！");
				//查询子板
				ctkEquipTerminal.setSerialNo(equip.getSerialNo());
				ctkEquipTerminal.setTerminalNo(equip.getTerminalNo());
				nodeList = equipNodeDao.template(ctkEquipNode);//查询两个子板
				if (nodeList == null && nodeList.size() != 2) return RespWrapper.makeFailResp("该设备子板数量错误， 请联系管理员！");
				CtkEquipNode node1 = nodeList.get(0);
				node1.setNodeNo(equip.getNodeOneNo());// 子板1流水号
				equipNodeDao.updateTemplateById(node1);
				CtkEquipNode node2 = nodeList.get(1);
				node2.setNodeNo(equip.getNodeTwoNo());// 子板1流水号
				equipNodeDao.updateTemplateById(node2);
			}
			//5.检查设备类型是否发送变化
			if( !ctkEquipTerminal.getEquipProductTypeId().equals(equip.getEquipProductTypeId()) ){
				ctkEquipTerminal.setEquipProductTypeId(equip.getEquipProductTypeId());
				CtkProductType ctkProductType = new CtkProductType();
				ctkProductType.setId(equip.getEquipProductTypeId());
				ctkProductType = productDao.templateOne(ctkProductType);
				if( null == ctkProductType)return RespWrapper.makeFailResp("设备类型不存在，请检查！");
				//第四步如果没有查询子板，则这里查询两个子板
				nodeList = (nodeList == null) ? equipNodeDao.template(ctkEquipNode) : nodeList;
				if (nodeList == null && nodeList.size() != 2) return RespWrapper.makeFailResp("该设备子板数量错误， 请联系管理员！");
				//找出子板1以及子板2
				CtkEquipNode node1 = null;
                CtkEquipNode node2 = null;
                if(!nodeList.isEmpty() && nodeList.size() == 2) {
                    for (int i = 0; i < 2; i++) {
                        String str = nodeList.get(i).getNodeNo();
                        //截取最后一位，如果是1，那么是子板1.是2，就是子板2
                        if ("1".equals(str.substring(str.length() - 1))) {
                            node1 = nodeList.get(i);
                        } else {
                            node2 = nodeList.get(i);
                        }
                    }
                }
				//查询数据库先有的插座数量
				CtkEquipSocket ctkEquipSocket = new CtkEquipSocket();
				ctkEquipSocket.setEquipTerminalId(equip.getTerminalId());
				List<CtkEquipSocket> socketList = equipSocketDao.template(ctkEquipSocket);
				//如果查询出的插座数为null，或者比回路数少，则需要新增插座
				if( null == socketList || socketList.size() < ctkProductType.getLoopNumber() ){
					List<CtkEquipSocket> addSocketList = new ArrayList<CtkEquipSocket>();
					CtkEquipSocket addSocket = null;
					for (int i = socketList.size() + 1; i < ctkProductType.getLoopNumber() + 1 ; i++) {
						addSocket = new CtkEquipSocket();
						if( i < 9 ) {
							addSocket.setEquipNodeId(node1.getId());
							addSocket.setSocketNo( i - 1 );
                        }else {
                        	addSocket.setSocketNo( i - 9 );
                        	addSocket.setEquipNodeId(node2.getId());
                        }
						String dealCount = i < 10 ? ("0" + i) : i + "";
						addSocket.setSerialNo(equip.getSerialNo() + dealCount);
						addSocket.setEquipTerminalId(equip.getTerminalId());
						addSocket.setSocketStatus(SocketStatus.IDLE.getId());//插座状态初始化？
						addSocketList.add(addSocket);
					}
					equipSocketDao.insertBatch(addSocketList);
					//如果插座数变少了，则需要删除多余的插座数
				}else if( socketList.size() > ctkProductType.getLoopNumber() ){
					//如果回路数小于8，则需要删除子板2全部，以及部分子板1
					if(ctkProductType.getLoopNumber() < 8){
						for( int i = 0 ; i < socketList.size() ; i++ ){
							Long equipNodeId = socketList.get(i).getEquipNodeId();
							if( node2.getId().equals(equipNodeId) ){//如果是子板2，则删除
								equipSocketDao.deleteById(socketList.get(i).getId());
							}else if( socketList.get(i).getSocketNo() >= ctkProductType.getLoopNumber() ){
								//这里判断是子板1
								//例如ctkProductType.getLoopNumber() =5，则子板1需要保留 0~4号插座
								equipSocketDao.deleteById(socketList.get(i).getId());
							}
						}
					}else{
						for( int i = 0 ; i < socketList.size() ; i++ ){
							//这里回路数是大于等于8,则说明子板2全部删除或者删除部分
							Long equipNodeId = socketList.get(i).getEquipNodeId();
							if( node2.getId().equals(equipNodeId) && socketList.get(i).getSocketNo() >= (ctkProductType.getLoopNumber() - 8)){
								equipSocketDao.deleteById(socketList.get(i).getId());
							}
						}
					}
				}
			}
			if(!StringUtil.isEmpty(equip.getRemark())) ctkEquipTerminal.setRemark(equip.getRemark());
			int num = equipTerminalDao.updateTemplateById(ctkEquipTerminal);
			if( num == 1) return RespWrapper.makeSuccessResp("修改成功！");
		} catch (Exception e) {
			e.printStackTrace();
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "修改设备", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，修改是失败！");
		}
		return RespWrapper.makeFailResp("修改失败！");
	}

	@Override
	public RespWrapper<Object> queryLoopNumberByTerminalNo(String terminalNo,HttpServletRequest request) {
		try {
			if( StringUtil.isEmpty(terminalNo) )return RespWrapper.makeFailResp("设备编号不能为空，请检查！");
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("terminalNo", terminalNo);
			map = equipTerminalDao.queryLoopNumberByTerminalNo(map);
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			LuceneUtils.apiErrorLucene(ApiErrorType.equip.getType(), request, "根据设备编号查询回路数", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	@Override
	public RespWrapper fadFindTerminalList(TerminalExcelHelp help, UserCoreCache userCache) {
		PageQuery<Map<String, Object>> pageQuery = doFindTerminalList(help, userCache, false);
		return RespWrapper.makeSuccessResp(pageQuery);
	}

	@Override
	public RespWrapper terminalListExcel(TerminalExcelHelp help, UserCoreCache userCache, HttpServletResponse response) {
		PageQuery<Map<String, Object>> pageQuery = doFindTerminalList(help, userCache, true);
		List<String> columnList = new ArrayList<>();
		List<String> excelList = new ArrayList<>();
		addTerminalExcel(columnList,excelList);
		List<Map<String, Object>> list = pageQuery.getList();
		list.forEach(s->{
			handleTerminalStatus(s,"status");
		});
		String fileName = "设备列表导出";
		try {
			JXLUtil.execlWrite(list,excelList,columnList,fileName,response,fileName);
		}catch (Exception e){

			return RespWrapper.makeFailResp("导出实时数据错误");
		}
		return null;
	}

	private void handleTerminalStatus(Map<String, Object> s, String key) {
		Object o = s.get(key);
		if(o != null){
			Integer integer = Integer.valueOf(o.toString());
			if(integer == 1) s.put(key,"在线中");
			else s.put(key,"初始化");
		}
	}

	private void addTerminalExcel(List<String> columnList,List<String> excelList){
		columnList.add(JXLUtil.SERIAL_NO);
		excelList.add("序号");
		columnList.add("terminalNo");
		excelList.add("设备ID");
		columnList.add("productName");
		excelList.add("设备类型");
		columnList.add("siteName");
		excelList.add("站点名称");
		columnList.add("status");
		excelList.add("服务状态");
		columnList.add("equipImei");
		excelList.add("IMEI");
		columnList.add("equipIccid");
		excelList.add("ICCID");
		columnList.add("remark");
		excelList.add("备注");
	}

	private PageQuery<Map<String,Object>> doFindTerminalList(TerminalExcelHelp help,UserCoreCache userCache,boolean isExcel){
		Map<String,Object> condition = new HashMap<>(8);
		if(!ProjectConstant.ADMIN.equals(userCache.getLoginName())){
			condition.put("siteIds",userCache.getSiteData());
		}
		if(help.getStatus() != null) condition.put("status",help.getStatus());
		if(StringUtil.isNotEmpty(help.getSiteName())) condition.put("siteName",help.getSiteName());
		if(StringUtil.isNotEmpty(help.getTerminalNo()))condition.put("terminalNo",help.getTerminalNo());
		if(help.getTerminalType() != null) condition.put("terminalType",help.getTerminalType());
		if(StringUtil.isNotEmpty(help.getImei()))condition.put("imei",help.getImei());
		if(StringUtil.isNotEmpty(help.getIccid()))condition.put("iccid",help.getIccid());
		PageQuery<Map<String,Object>> pageQuery = new PageQuery<>();
		pageQuery.setParas(condition);
		if(isExcel){
			pageQuery.setPageNumber(1);
			pageQuery.setPageSize(WarnServiceImpl.excelMaxNum);
		}else {
			pageQuery.setPageNumber(help.getPageNum());
			pageQuery.setPageSize(help.getPageSize());
		}
		pageQuery.setOrderBy("cet.terminal_id desc");
		pageQuery = equipTerminalDao.fadFindTerminalList(pageQuery);
		return pageQuery;
	}

	@Override
	public RespWrapper<Object> deleteEquipTerminal(Long terminalId,HttpServletRequest request) {
		//此方法未实现
		return null;
	}

	

}
