package com.comtika.safe.service.homePage.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.entity.homePage.help.EleTotalHelp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.enums.equip.terminal.TerminalStatusEnum;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.dao.homePage.HomePageDao;
import com.comtika.safe.entity.homePage.EquipNumber;
import com.comtika.safe.entity.homePage.MerchantAndSiteNumber;
import com.comtika.safe.service.homePage.HomePageService;

@Service
public class HomePageServiceImpl implements HomePageService {

	@Autowired
	private HomePageDao homePageDao;

	@Override
	public RespWrapper<Object> queryMerchantAndSiteNumber(Map<String, Object> map) {
		try {
			MerchantAndSiteNumber number = homePageDao.queryMerchantAndSiteNumber(map);
			return RespWrapper.makeSuccessResp(number);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("查询失败，服务器错误！");
		}
	}

	@Override
	public RespWrapper<Object> queryEquipNumber(Map<String, Object> map) {
		try {
			EquipNumber number = new EquipNumber();
			//查询当前用户拥有哪些设备
			List<Map<String,Object>> list  = homePageDao.queryEquipNumber(map);
			if(list != null && !list.isEmpty() && list.size() > 0){
				number.setEquipNumber(list.size());
				for (Map<String, Object> map2 : list) {
					Integer status = Integer.parseInt(map2.get("status") + "");	//设备状态，0 -- 不在线，  1 --在线
					Integer productType = Integer.parseInt(map2.get("productType") + "");//0 -- 单项，1 -- 三项
					if( status == TerminalStatusEnum.ON_LINE.getId() ){//在线设备
						number.setEquipOnlineNumber(number.getEquipOnlineNumber() + 1);
					}
					if( productType == 0){
						number.setOneEquipNumber(number.getOneEquipNumber() + 1);
					}else if(productType == 1){
						number.setThreeEquipNumber(number.getThreeEquipNumber() + 1);
					}
				}
			}
			return RespWrapper.makeSuccessResp(number);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("查询失败，服务器错误！");
		}
	}

	@Override
	public RespWrapper<Object> querySite(Map<String, Object> map) {
		try {
			//1.查询该用户所有站点列表，包含商户名称
			List<Map<String,Object>> list  = homePageDao.querySite(map);
			if(list == null || list.isEmpty() || list.size() < 1){
				return RespWrapper.makeSuccessResp(new ArrayList<Map<String,Object>>());
			}
			for (Map<String, Object> listMap : list) {
				listMap.put("siteWarnNumber", 0);//赋值告警数量数据为0
				listMap.put("siteNotReleasedNumber", 0);//赋值未解除告警数量为0
				listMap.put("siteEquipStatus", 0);//站点告警状态 0 - 无警报， 1 - 有预警， 2 - 有报警
			}
			//2.查询该用户所有告警数据
			List<Map<String,Object>> siteWarn  = homePageDao.querySiteWarn(map);
			//3.数据合并
			if( siteWarn != null && !siteWarn.isEmpty() && siteWarn.size() > 0){
				for (Map<String, Object> listMap : list) {
					Long listSiteId = Long.parseLong(listMap.get("siteId") + "");
					for (Map<String, Object> siteWarnMap : siteWarn) {
						Long siteWarnMapSiteId = Long.parseLong(siteWarnMap.get("siteId") + "");
						if( listSiteId.equals(siteWarnMapSiteId) ){  //如果当前站点id相同
							//获取当前站点的告警数量并且赋值
							Integer siteWarnNumber = Integer.parseInt(listMap.get("siteWarnNumber") + "");
							listMap.put("siteWarnNumber", siteWarnNumber + 1);
							//获取当前未解除告警数量，并且判断当前for循环本次是否是未解除的
							Integer warnStatus = Integer.parseInt(siteWarnMap.get("warnStatus") + "");
							if( 0 == warnStatus){ //当前告警未解除
								Integer siteNotReleasedNumber = Integer.parseInt(listMap.get("siteNotReleasedNumber") + "");
								listMap.put("siteNotReleasedNumber", siteNotReleasedNumber + 1);
								//如果是未解除的，判断当前设备的告警等级
								Integer siteEquipStatus = Integer.parseInt(listMap.get("siteEquipStatus") + "");
								if( siteEquipStatus == 2) continue;//如果当前状态是报警，则就不会被覆盖
								//获取当前数据库中的告警状态,1  -- 预警, 0 -- 报警
								Integer preWarn = Integer.parseInt(siteWarnMap.get("preWarn") + "");
								if( 0 == preWarn ){
									listMap.put("siteEquipStatus", 2);//站点告警状态 0 - 无警报， 1 - 有预警， 2 - 有报警
								}else if( 1 == preWarn){
									listMap.put("siteEquipStatus", 1);//站点告警状态 0 - 无警报， 1 - 有预警， 2 - 有报警
								}
							}
						}
					}
				}
			}
			return RespWrapper.makeSuccessResp(list);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器异常，查询失败！");
		}

	}

	@Override
	public RespWrapper findHomeEleTotal(UserCoreCache userCache) {
		Map<String,Object> condition = new HashMap<>(4);
		if(!ProjectConstant.ADMIN.equals(userCache.getLoginName())){
			condition.put("siteIds",userCache.getSiteData());
		}
		// 所有的电量值
		EleTotalHelp help = homePageDao.findHomeEleTotal(condition);
		condition.clear();
		int totalEle = 0;
		int yesterdayEle = 0;
		if(help.getEndEle() != null && help.getStartEle() != null){
			totalEle = help.getEndEle() - help.getStartEle();
		}
		condition.put("startTime", DateTimeUtils.getStartTimeOfToday()-DateTimeUtils.ONE_DAY);
		EleTotalHelp yesterday = homePageDao.findHomeEleTotal(condition);
		if(yesterday.getEndEle() != null && yesterday.getStartEle() != null)
			yesterdayEle = yesterday.getEndEle()-yesterday.getStartEle();
		condition.put("totalEle",totalEle);
		condition.put("yesterdayEle",yesterdayEle);
		return RespWrapper.makeSuccessResp(condition);
	}

}
