package com.comtika.safe.service.log.impl;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.lucene.search.Query;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.SortField;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.lucene.LuceneQueryUtils;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.core.lucene.LuceneUtils.LuceneType;
import com.comtika.safe.entity.lucene.LuceneReturnEntity;
import com.comtika.safe.service.log.LogService;

@Service
public class LogServiceImpl implements LogService {

	@Override
	public RespWrapper<Object> queryLoginLog(Integer start,Integer size,String loginName, Long startTime,Long endTime) {
		Query time = null;
		try {
			if( start == null || size == null){
				return RespWrapper.makeFailResp("页数或页容量不能为空！");
			}
			//参数赋值
			Map<String,Object> map = new HashMap<String,Object>();
			if(StringUtil.isNotEmpty(loginName)){
				map.put("loginName", loginName);
			}
			//参数赋值，时间范围查询  -- 不知道为啥，如果不带条件，查询为空。所以默认赋值一个时间范围的日期
			if(startTime == null || endTime == null){
				startTime = 946656040000L;
				endTime = System.currentTimeMillis();
			}
			time = LuceneQueryUtils.timeRange("loginTime", startTime, endTime);
			//排序规则
			Sort sort =  LuceneQueryUtils.createSort("loginTime", SortField.Type.LONG, true);
			LuceneReturnEntity in = LuceneQueryUtils.queryLucene(LuceneUtils.indexDir + LuceneType.LOGIN_LOG , start, size, map, sort,time);
			return RespWrapper.makeSuccessResp(in);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
		
	}

	@Override
	public RespWrapper<Object> queryVisitLog(HttpServletRequest request) {
		Query time = null;
		try {
			Long startTime = 0L;
			Long endTime = 0L;
			if( request.getParameter("start") == null || request.getParameter("size") == null){
				return RespWrapper.makeFailResp("页数或页容量不能为空！");
			}
			Integer start = Integer.parseInt(request.getParameter("start"));
			Integer size = Integer.parseInt(request.getParameter("size"));
			//参数赋值
			Map<String,Object> map = new HashMap<String,Object>();
			if(StringUtil.isNotEmpty(request.getParameter("loginName"))){
				map.put("loginName", request.getParameter("loginName"));
			}
			if(StringUtil.isNotEmpty(request.getParameter("name"))){
				map.put("name", request.getParameter("name"));
			}
			if(StringUtil.isNotEmpty(request.getParameter("visitType"))){
				map.put("visitType", request.getParameter("visitType"));
			}
			//参数赋值，时间范围查询
			if(StringUtil.isNotEmpty(request.getParameter("startTime")) && StringUtil.isNotEmpty(request.getParameter("endTime"))){
				startTime = Long.parseLong(request.getParameter("startTime"));
				endTime = Long.parseLong(request.getParameter("endTime"));
				
			}else{
				startTime = 946656040000L;
				endTime = System.currentTimeMillis();
			}
			time = LuceneQueryUtils.timeRange("visitTime", startTime, endTime);
			//排序规则
			Sort sort =  LuceneQueryUtils.createSort("visitTime", SortField.Type.LONG, true);
			LuceneReturnEntity in = LuceneQueryUtils.queryLucene(LuceneUtils.indexDir + LuceneType.VISIT_LOG , start, size, map, sort,time);
			return RespWrapper.makeSuccessResp(in);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	@Override
	public RespWrapper<Object> queryErrorLog(HttpServletRequest request) {
		Query time = null;
		try {
			Long startTime = 0L;
			Long endTime = 0L;
			if( request.getParameter("start") == null || request.getParameter("size") == null){
				return RespWrapper.makeFailResp("页数或页容量不能为空！");
			}
			Integer start = Integer.parseInt(request.getParameter("start"));
			Integer size = Integer.parseInt(request.getParameter("size"));
			//参数赋值
			Map<String,Object> map = new HashMap<String,Object>();
			if(StringUtil.isNotEmpty(request.getParameter("loginName"))){
				map.put("loginName", request.getParameter("loginName"));
			}
			if(StringUtil.isNotEmpty(request.getParameter("name"))){
				map.put("name", request.getParameter("name"));
			}
			if(StringUtil.isNotEmpty(request.getParameter("errorType"))){
				map.put("errorType", request.getParameter("errorType"));
			}
			//参数赋值，时间范围查询
			if(StringUtil.isNotEmpty(request.getParameter("startTime")) && StringUtil.isNotEmpty(request.getParameter("endTime"))){
				startTime = Long.parseLong(request.getParameter("startTime"));
				endTime = Long.parseLong(request.getParameter("endTime"));
				
			}else{
				startTime = 946656040000L;
				endTime = System.currentTimeMillis();
			}
			time = LuceneQueryUtils.timeRange("errorTime", startTime, endTime);
			//排序规则
			Sort sort =  LuceneQueryUtils.createSort("errorTime", SortField.Type.LONG, true);
			LuceneReturnEntity in = LuceneQueryUtils.queryLucene(LuceneUtils.indexDir + LuceneType.ERROR_LOG , start, size, map, sort,time);
			return RespWrapper.makeSuccessResp(in);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

}
