package com.comtika.safe.service.message.impl;

import com.alibaba.fastjson.JSONObject;
import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.message.MessageUtils;
import com.comtika.safe.core.protocol.cache.SafeCache;
import com.comtika.safe.core.weChat.WxChatUtil;
import com.comtika.safe.dao.message.FadMessageConfDao;
import com.comtika.safe.dao.message.FadMessageHandleConfDao;
import com.comtika.safe.dao.message.MessagePushDao;
import com.comtika.safe.dao.wx.WeChatCodeDao;
import com.comtika.safe.entity.SysUser;
import com.comtika.safe.entity.message.FadMessageConf;
import com.comtika.safe.entity.message.FadMessageHandleConf;
import com.comtika.safe.entity.message.help.MessageAcceptHelp;
import com.comtika.safe.entity.message.help.MessageHandleHelp;
import com.comtika.safe.entity.message.help.SimpleUserInfoHelp;
import com.comtika.safe.entity.wx.WeChatCode;
import com.comtika.safe.entity.wx.WeChatHelp;
import com.comtika.safe.service.message.IMessageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class MessageServiceImpl implements IMessageService {
    /* @Autowired
     private RedisApi redisApi;*/
    private static final String SITE_CONF = "site_%s_comtika";

    @Autowired
    private FadMessageConfDao fadMessageConfDao;
    @Autowired
    private FadMessageHandleConfDao fadMessageHandleConfDao;

    @Autowired
    private MessagePushDao messagePushDao;
    @Autowired
    private WeChatCodeDao weChatCodeDao;

    @Autowired
    private WxChatUtil wxChatUtil;


    @Override
    public RespWrapper<Boolean> updateMessageHandleUser(Integer msgId, Integer userId) {
        FadMessageHandleConf conf = new FadMessageHandleConf();
        conf.setMesId(msgId);
        conf.setType(2);
        conf = fadMessageHandleConfDao.templateOne(conf);
        if (conf == null) return RespWrapper.makeFailResp("初始化数据错误");
        if (conf.getUserId().intValue() != userId) {
            SysUser merchantId = messagePushDao.findUserMerchantId(userId);
            conf.setUserId(userId);
            conf.setPhoneNo(merchantId.getLoginName());
            conf.setMerchantId(merchantId.getMerchantId().intValue());
            conf.setNickName(merchantId.getName());
            fadMessageHandleConfDao.updateTemplateById(conf);
            removeSiteMessage(conf.getSiteId());
        }
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> deleteMessageAcceptUser(Integer id, UserCoreCache userCache) {
        FadMessageHandleConf single = fadMessageHandleConfDao.single(id);
        if (single == null) return RespWrapper.makeFailResp("该唯一id 错误");
        if (single.getType() == 1 || 1 == userCache.getUserId()) {
            //直接处理
            fadMessageHandleConfDao.deleteById(id);
        } else {
            // 不是admin 时需呀校验该操作权限
            SysUser merchantId = messagePushDao.findUserMerchantId(userCache.getUserId().intValue());
            if (single.getMerchantId() != null && single.getMerchantId().intValue() == merchantId.getMerchantId())
                fadMessageHandleConfDao.deleteById(id);
            else return RespWrapper.makeFailResp("该商户没有权限删除该消息接收人");
        }
        removeSiteMessage(single.getSiteId());
        return RespWrapper.makeSuccessResp(true);
    }

    private void removeSiteMessage(Integer siteId) {
        SafeCache.ONE_HOUR_CACHE.remove(String.format(SITE_CONF, siteId));
    }

    @Override
    public RespWrapper<Boolean> fadMessageConfig(FadMessageConf help) {
        FadMessageConf single = fadMessageConfDao.single(help.getId());
        if (single == null) return RespWrapper.makeFailResp("唯一id 错误");
        single.setOfflineSwitch(help.getOfflineSwitch());
        single.setMainSwitch(help.getMainSwitch());
        single.setWxSwitch(help.getWxSwitch());
        fadMessageConfDao.updateTemplateById(single);
        // 下面处理一些数据啦
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public List<FadMessageHandleConf> findSiteMessageConfig(int siteId) {
        // 第一步，查询是否该站点是否配置
        return findSiteMeessage(siteId);
    }

    @Override
    public RespWrapper<Boolean> checkWeChatCode(String phoneNo) {
        WeChatCode chatCode = new WeChatCode();
        chatCode.setPhoneNo(phoneNo);
        if (weChatCodeDao.templateCount(chatCode) > 0) {
            return RespWrapper.makeSuccessResp(true);
        }
        return RespWrapper.makeResp(2, "未授权", null);
    }

    @Override
    public RespWrapper<Boolean> weChatAcCodeAuth(WeChatHelp help, UserCoreCache userCoreCache) {
       /* String code = SafeCache.findPhoneNoCode(help.getPhoneNo());
        if(code == null) return RespWrapper.makeFailResp("手机验证码已过期，请重新获取");
        if(!code.equals(help.getCode())) return RespWrapper.makeFailResp("手机验证码错误，请重新获取");*/
       // 没有手机号时，使用默认的手机号码
        String phoneNo = help.getPhoneNo();
        if (StringUtil.isEmpty(phoneNo)) {
            phoneNo = userCoreCache.getLoginName();
        }
        JSONObject weChatAuth2Data = wxChatUtil.findWeChatAuth2Data(help.getAcCode(), false);
        String openId = weChatAuth2Data.getString(ProjectConstant.ThirdConstant.OPENID);
        if (StringUtil.isEmpty(openId)) return RespWrapper.makeFailResp("获取微信公众号openId错误");
        WeChatCode chatCode = new WeChatCode();
        chatCode.setPhoneNo(phoneNo);
        chatCode = weChatCodeDao.templateOne(chatCode);
        if (chatCode == null) {
            chatCode = new WeChatCode();
            chatCode.setPhoneNo(phoneNo);
            chatCode.setOpenId(openId);
            weChatCodeDao.insert(chatCode);
        } else {
            chatCode.setOpenId(openId);
            weChatCodeDao.updateTemplateById(chatCode);
        }
        return RespWrapper.makeSuccessResp(true);
    }

    // 这里发生手机验证码
    @Override
    public RespWrapper<Boolean> sendPhoneCode(String phoneNo) {
        if (!ProjectConstant.PhonePattern.PHONE_NO.matcher(phoneNo).matches()) {
            return RespWrapper.makeFailResp("请检查手机号码");
        }
        String code = ProjectConstant.PhoneCodeRandom.getRandom();
        RespWrapper<Boolean> ck = MessageUtils.sendCode(phoneNo, code);
        if (!ck.isSuccess()) return ck;
        SafeCache.addPhoneNoCode(phoneNo, code);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> delUserAuth(UserCoreCache userCache) {
        String phoneNo = userCache.getLoginName();
        WeChatCode chatCode = new WeChatCode();
        chatCode.setPhoneNo(phoneNo);
        chatCode = weChatCodeDao.templateOne(chatCode);
        if (chatCode == null) return RespWrapper.makeFailResp("该用户暂还没有授权，不需要移除授权");
        weChatCodeDao.deleteById(chatCode.getId());
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> checkPhoneCode(String phoneNo, String code) {
        String _code = SafeCache.findPhoneNoCode(phoneNo);
        if (_code == null) return RespWrapper.makeFailResp("手机验证码已过期，请重新获取");
        if (!_code.equals(code)) return RespWrapper.makeFailResp("手机验证码错误，请重新获取");
        return RespWrapper.makeSuccessResp(true);
    }

    private List<FadMessageHandleConf> findSiteMeessage(int siteId) {
        String key = String.format(SITE_CONF, siteId);
        Object present = SafeCache.ONE_HOUR_CACHE.getIfPresent(key);
        if (present != null) return (List<FadMessageHandleConf>) present;
        FadMessageConf fadMessage = findFadMessage(siteId);
        // 查询配置数据
        List<FadMessageHandleConf> fadMessageHandleConf = findFadMessageHandleConf(fadMessage.getId(), siteId);
        if (fadMessageHandleConf != null)
            SafeCache.ONE_HOUR_CACHE.set(key, fadMessageHandleConf);
        return fadMessageHandleConf;
    }

    @Override
    public RespWrapper<Map<String, Object>> fadFindMessageConfig(Integer siteId) {
        // 第一步，查询是否该站点是否配置
        FadMessageConf fadMessage = findFadMessage(siteId);
        // 查询配置数据
        List<FadMessageHandleConf> list = findFadMessageHandleConf(fadMessage.getId(), siteId);
        if (list == null) return RespWrapper.makeFailResp("查询数据错误");
        Map<String, Object> map = new HashMap<>();
        map.put("conf", fadMessage);
        Map<String, Object> handleUser = new HashMap<>(2);
        List<Map<String, Object>> acceptUserList = new ArrayList<>(8);
        list.forEach(s -> {
            if (s.getType() == 1) {
                Map<String, Object> acUser = new HashMap<>(4);
                acUser.put("id", s.getId());
                acUser.put("name", s.getNickName());
                acUser.put("phoneNo", s.getPhoneNo());
                acceptUserList.add(acUser);
            } else {
                handleUser.put("id", s.getUserId());
                handleUser.put("name", s.getNickName());
                handleUser.put("phoneNo", s.getPhoneNo());
                handleUser.put("mesId", fadMessage.getId());
            }
        });
        map.put("acceptUserList", acceptUserList);
        map.put("handleUser", handleUser);
        return RespWrapper.makeSuccessResp(map);
    }

    private List<FadMessageHandleConf> findFadMessageHandleConf(Integer id, Integer siteId) {
        FadMessageHandleConf conf = new FadMessageHandleConf();
        conf.setMesId(id);
        List<FadMessageHandleConf> template = fadMessageHandleConfDao.template(conf);
        if (template.size() == 0) {
            // 这里需要新增
            MessageHandleHelp handleHelp = messagePushDao.findMessageAcceptList(siteId);
            Integer handleId = handleHelp.getMerchantAdmin();
            // if(handleId == null) handleHelp.getMerchantAdmin();
            if (handleId == null) return null;
            List<SimpleUserInfoHelp> list = messagePushDao.findSimpleUserInfo(handleHelp);
            for (SimpleUserInfoHelp help : list) {
                if (help.getId() == null) continue;
                /// String acOpenId = findAcOpenId(help.getId());
                String acOpenId = "";//todo 微信OpenId
                FadMessageHandleConf cof = new FadMessageHandleConf();
                cof.setMesId(id);
                cof.setType(1);
                cof.setNickName(help.getName());
                cof.setPhoneNo(help.getPhoneNo());
                cof.setUserId(help.getId());
                cof.setMerchantId(help.getMerchantId());
                cof.setAcOpenId(acOpenId);
                cof.setSiteId(siteId);
                template.add(cof);
                if (handleId == help.getId().intValue()) {
                    FadMessageHandleConf cof2 = new FadMessageHandleConf();
                    cof2.setMesId(id);
                    cof2.setType(2);
                    cof2.setNickName(help.getName());
                    cof2.setPhoneNo(help.getPhoneNo());
                    cof2.setUserId(help.getId());
                    cof2.setMerchantId(help.getMerchantId());
                    cof2.setAcOpenId(acOpenId);
                    cof2.setSiteId(siteId);
                    template.add(cof2);
                }
            }
            fadMessageHandleConfDao.insertBatch(template);
            template = fadMessageHandleConfDao.template(conf);
        } else {
           /* template.forEach(s->{
                if(StringUtil.isEmpty(s.getAcOpenId())){
                    String acOpenId = findAcOpenId(s.getUserId());
                    if(StringUtil.isNotEmpty(acOpenId)){
                        s.setAcOpenId(acOpenId);
                        fadMessageHandleConfDao.updateTemplateById(s);
                    }
                }
            });*/
        }
        return template;
    }

    private FadMessageConf findFadMessage(Integer siteId) {
        FadMessageConf conf = new FadMessageConf();
        conf.setSiteId(siteId);
        conf = fadMessageConfDao.templateOne(conf);
        if (conf == null) {
            conf = new FadMessageConf();
            conf.setSiteId(siteId);
            conf.setMainSwitch(false);
            conf.setOfflineSwitch(false);
            conf.setWxSwitch(true);
            fadMessageConfDao.insert(conf, true);
        }
        return conf;
    }

    @Override
    public RespWrapper<Boolean> addMessageAcceptList(MessageAcceptHelp help) {
        FadMessageConf fadMessage = findFadMessage(help.getSiteId());
        List<FadMessageHandleConf> fadMessageHandleConf = findFadMessageHandleConf(fadMessage.getId(), help.getSiteId());
        if (fadMessageHandleConf == null) return RespWrapper.makeFailResp("查询数据错误");
        List<Integer> userList = help.getUserList();
        Map<Integer, Boolean> map = new HashMap<>();
        fadMessageHandleConf.forEach(s -> {
            if (s.getType() == 1)
                map.put(s.getUserId(), true);
        });
        fadMessageHandleConf.clear();
        Set<Integer> list = new HashSet<>(userList.size());
        userList.forEach(s -> {
            if (!map.containsKey(s)) {
                list.add(s);
            }
        });
        userList.clear();
        if (!list.isEmpty()) {
            List<FadMessageHandleConf> template = new ArrayList<>();
            Map<String, Object> condition = new HashMap<>(4);
            condition.put("ids", list);
            List<SimpleUserInfoHelp> lt = messagePushDao.findUserList(condition);
            lt.forEach(s -> {
                FadMessageHandleConf cof = new FadMessageHandleConf();
                cof.setMesId(fadMessage.getId());
                cof.setType(1);
                cof.setNickName(s.getName());
                cof.setPhoneNo(s.getPhoneNo());
                cof.setUserId(s.getId());
                cof.setMerchantId(s.getMerchantId());
                cof.setAcOpenId(s.getAcOpenId());
                cof.setSiteId(help.getSiteId());
                template.add(cof);
            });
            if (!template.isEmpty()) {
                fadMessageHandleConfDao.insertBatch(template);
                removeSiteMessage(help.getSiteId());
            }
        }
        return RespWrapper.makeSuccessResp(true);
    }


}
