package com.comtika.safe.service.mobile.impl;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.dao.message.FadMessageConfDao;
import com.comtika.safe.entity.mobile.CommonHelp;
import com.comtika.safe.entity.mobile.EleTotalHelp;
import com.comtika.safe.entity.mobile.EleTotalQueryHelp;
import com.comtika.safe.entity.mobile.help.EleHelp;
import com.comtika.safe.entity.safe.TerminalNewestData;
import com.comtika.safe.service.mobile.IMobileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 15:24 2020/10/24
 * @DESC: 移动端数据统计
 **/
@Service
public class MobileServiceImpl implements IMobileService {
    private long handleStartTime(long time){
        return (time/1000)*1000;
    }

    @Autowired
    private FadMessageConfDao fadMessageConfDao;

    @Override
    public RespWrapper<Map<String, Object>> findUserSiteEleTotal(EleTotalQueryHelp help, UserCoreCache userCache) {
        Map<String,Object> condition = new HashMap<>(8);
        if (!ProjectConstant.ADMIN.equals(userCache.getLoginName())) {
            condition.put("siteIds", userCache.getSiteData());
        }
        if(help.getStartTime() != null && help.getEndTime() != null){
            condition.put("startTime",handleStartTime(help.getStartTime()));
            condition.put("endTime",help.getEndTime()+ DateTimeUtils.ONE_DAY);
        }
        // 1.查询数据
        List<EleTotalHelp> list = fadMessageConfDao.findUserSiteEleTotal(condition);
        Map<Integer,Integer> siteEleTotal = new HashMap<>(list.size());
        list.forEach(s->{
            siteEleTotal.put(s.getSiteId(),calEle(s.getEndEle() , s.getStartEle()));
        });
        // 下面还需要查询站点类别
        List<CommonHelp> siteList = fadMessageConfDao.findSiteList(condition);
        int totalEle = 0;
        List<Map<String,Object>> siteTotal = new ArrayList<>(siteList.size());
        for(CommonHelp hp: siteList){
            Integer id = hp.getId();
            Integer integer = siteEleTotal.get(id);
            int i = integer == null?0:integer;
            totalEle+=i;
            Map<String,Object> arg1 = new HashMap<>();
            arg1.put("totalEle",i);
            arg1.put("name",hp.getName());
            arg1.put("siteId",id);
            siteTotal.add(arg1);
        }
        condition.put("allTotalEle",totalEle);
        condition.put("list",siteTotal);
        return RespWrapper.makeSuccessResp(condition);
    }

    @Override
    public RespWrapper<Map<String, Object>> findTerminalEleTotal(EleTotalQueryHelp help, UserCoreCache userCache) {
        Map<String,Object> condition = new HashMap<>(8);
        Integer siteId = help.getSiteId();
        condition.put("siteId",siteId);
        if(help.getStartTime() != null && help.getEndTime() != null){
            condition.put("startTime",handleStartTime(help.getStartTime()));
            condition.put("endTime",help.getEndTime()+DateTimeUtils.ONE_DAY);
        }

        // todo 这里需要处理下，这样必须有数据时才能查询
        List<EleTotalHelp> sumTotalList = fadMessageConfDao.findTerminalEleTotal(condition);
        Map<Integer,Integer> terminalTotalMap = new HashMap<>(sumTotalList.size());
        sumTotalList.forEach(s->{
            terminalTotalMap.put(s.getTerminalId(),calEle(s.getEndEle() , s.getStartEle()));
        });
        int totalEle = 0;
        // 这里根据站点id 查询编号
        List<CommonHelp> terList = fadMessageConfDao.findTerminalCommonList(siteId);
        List<Map<String,Object>> siteTotal = new ArrayList<>(terList.size());
        for(CommonHelp ch:terList){
            Integer integer = terminalTotalMap.get(ch.getId());
            int i = integer == null?0:integer;
            totalEle+=i;
            Map<String,Object> arg1 = new HashMap<>();
            arg1.put("totalEle",i);
            arg1.put("terminalNo",ch.getName());
            arg1.put("terminalId",ch.getId());
            siteTotal.add(arg1);
        }
        condition.put("allTotalEle",totalEle);
        condition.put("list",siteTotal);
        return RespWrapper.makeSuccessResp(condition);
    }


    @Override
    public RespWrapper<Map<String, Object>> findSocketEleTotal(EleTotalQueryHelp help) {
        Map<String,Object> condition = new HashMap<>();
        condition.put("terminalId",help.getTerminalId());
        if(help.getStartTime() != null && help.getEndTime() != null){
            condition.put("startTime",handleStartTime(help.getStartTime()));
            condition.put("endTime",help.getEndTime()+DateTimeUtils.ONE_DAY);
        }

        List<EleTotalHelp> sumTotalList = fadMessageConfDao.findSocketEleTotal(condition);
        Map<String,Integer> socketListMap = new HashMap<>(sumTotalList.size());
        sumTotalList.forEach(s->{
            socketListMap.put(s.getSerialNo(),calEle(s.getEndEle() , s.getStartEle()));
        });
        int totalEle = 0;
        List<CommonHelp> socketList = fadMessageConfDao.findTerSocketList(help.getTerminalId());
        List<Map<String,Object>> siteTotal = new ArrayList<>(socketList.size());
        for(CommonHelp ch: socketList){
            Integer integer = socketListMap.get(ch.getName());
            int i = integer == null ? 0: integer;
            totalEle+=i;
            Map<String,Object> arg1 = new HashMap<>();
            arg1.put("totalEle",i);
            arg1.put("serialNo",ch.getName());
            // arg1.put("socketId",ch.getId());
            siteTotal.add(arg1);
        }
        condition.put("allTotalEle",totalEle);
        condition.put("list",siteTotal);
        return RespWrapper.makeSuccessResp(condition);
    }
    // 数据统计
    @Override
    public RespWrapper<Map<String, Object>> findTerminalDataTotal(String terminalNo) {
        List<TerminalNewestData> list = fadMessageConfDao.findTerminalDataTotal(terminalNo);
        List<TerminalNewestData> master = new ArrayList<>(2);
        List<TerminalNewestData> collect = list.stream().filter(s -> {
                    if (s.getIsMain()) {
                        master.add(s);
                        return false;
                    }
                    return true;
                }
        ).collect(Collectors.toList());
        Map<String,Object> res = new HashMap<>(4);
        res.put("list",collect);
        if(master.size() == 1){
            res.put("master",master.get(0));
        }else return RespWrapper.makeFailResp("设备主板不止一个，请检查设备编号");
        return RespWrapper.makeSuccessResp(res);
    }

    @Override
    public RespWrapper<TerminalNewestData> findTerminalNewDataInfo(Integer id) {
        TerminalNewestData data = fadMessageConfDao.findTerminalNewDataInfo(id);
        return RespWrapper.makeSuccessResp(data);
    }

    @Override
    public RespWrapper findTerminalEleTotalList(EleTotalQueryHelp help) {
        // 这里查询数据
        List<EleTotalHelp> sumTotalList = fadMessageConfDao.findTerminalEleTotalList(help.getTerminalId(),help.getStartTime(),help.getEndTime()+DateTimeUtils.ONE_DAY);
        Map<String,Integer> res = new HashMap<>();
        sumTotalList.forEach(s->{
            // 处理时间
            if(s.getCreateTime() == null) return;
            String key = DateTimeUtils.formatDate(new Date(s.getCreateTime()), DateTimeUtils.FORMAT_YMD);
            int inc = calEle(s.getEndEle(), s.getStartEle());
            if(res.containsKey(key)){
                res.put(key,res.get(key)+inc);
            }else {
                res.put(key,inc);
            }
        });
        // 处理数据
        List<String> keys = new ArrayList<>();
        long startTime = help.getStartTime();
        long endTime = help.getEndTime();
        while (startTime <= endTime){
            keys.add(DateTimeUtils.formatDate(new Date(startTime), DateTimeUtils.FORMAT_YMD));
            startTime += DateTimeUtils.ONE_DAY;
        }
        List<EleHelp> list = new ArrayList<>();
        int size = keys.size();
        for(int i=0;i<size;i++){
            String key = keys.get(i);
            list.add(new EleHelp(key,res.get(key)));
        }
        return RespWrapper.makeSuccessResp(list);
    }


    private int calEle(Integer end,Integer start){
        if(end == null) end = 0;
        if(start == null) start = 0;
        return end - start;
    }
}
