package com.comtika.safe.service.ota.impl;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.protocol.cache.ProtocolRedisCache;
import com.comtika.safe.dao.ota.IOtaDao;
import com.comtika.safe.dao.upload.IUploadDao;
import com.comtika.safe.entity.ota.OtaRecord;
import com.comtika.safe.entity.upload.UploadRecord;
import com.comtika.safe.service.ota.IOtaService;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class OtaServiceImpl implements IOtaService {

    @Autowired
    private IOtaDao iOtaDao;
    @Autowired
    private IUploadDao iUploadDao;

    @Override
    public RespWrapper<PageQuery<Map<String, Object>>> findFadVersionPage(Integer start, Integer size, String hsName, Long startTime, Long endTime) {
        if(start == null)start = 1;
        if(size == null)size = 10;
        Map<String,Object> condition = new HashMap<>(8);
        if(StringUtil.isNotEmpty(hsName)){
            condition.put("versionName",hsName);
        }
        if(startTime != null && endTime != null){
            condition.put("startTime",startTime);
            condition.put("endTime",endTime);
        }
        PageQuery<Map<String, Object>> pageQuery = new PageQuery<>();
        pageQuery.setParas(condition);
        pageQuery.setPageNumber(start);
        pageQuery.setPageSize(size);
        pageQuery.setOrderBy(" id desc");
        pageQuery = iOtaDao.findFadVersionPage(pageQuery);
        condition.clear();
        return RespWrapper.makeSuccessResp(pageQuery);
    }

    @Override
    public RespWrapper<Boolean> addOTAVersion(OtaRecord hs) {
        RespWrapper<Boolean> ck = checkOtaParams(hs, false);
        if(!ck.isSuccess())
            return ck;
        hs.setHsTime(System.currentTimeMillis());
        iOtaDao.insert(hs);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> updateOTAHsVersion(OtaRecord hs) {
        RespWrapper<Boolean> ck = checkOtaParams(hs, true);
        if(!ck.isSuccess())
            return ck;
        OtaRecord single = iOtaDao.single(hs.getId());
        if(single == null) return RespWrapper.makeFailResp("唯一id 错误");
        single.setHsExplain(hs.getHsExplain());
        single.setHsVersionName(hs.getHsVersionName());
        single.setOtaId(hs.getOtaId());
        iOtaDao.updateTemplateById(single);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<List<OtaRecord>> findAllOtaVersion() {
        return RespWrapper.makeSuccessResp(iOtaDao.all());
    }

    @Override
    public RespWrapper<Boolean> deleteOta(Integer id) {
        OtaRecord single = iOtaDao.single(id);
        if(single == null) return RespWrapper.makeFailResp("ota 唯一id 错误");
        UploadRecord single1 = iUploadDao.single(single.getOtaId());
        if(single1 == null) return RespWrapper.makeFailResp("");
        String fielPath = replace(single1.getBaseUrl());
        File file = new File(fielPath);
        file.deleteOnExit();
        iUploadDao.deleteById(single1.getId());
        iOtaDao.deleteById(id);
        ProtocolRedisCache.removeOTAData(single1.getId());
        return RespWrapper.makeSuccessResp(true);
    }

    private String replace(String baseUrl){
        if(baseUrl == null)return null;
        return baseUrl.replace("https://safe.comtikaiot.com","");
    }

    private RespWrapper<Boolean> checkOtaParams(OtaRecord record,boolean isUpdate){
        if(isUpdate){
            if (record.getId() == null) {
                return RespWrapper.makeFailResp("更新时，唯一id不能为空");
            }
        }
        if(StringUtil.isEmpty(record.getHsExplain()))
            return RespWrapper.makeFailResp("版本说明不能为空");
        if(StringUtil.isEmpty(record.getHsVersionName()))
            return RespWrapper.makeFailResp("版本名称不能为空");
        if(record.getOtaId() == null)
            return RespWrapper.makeFailResp("OTA升级包不能为空");
        return RespWrapper.makeSuccessResp(true);
    }
}
