package com.comtika.safe.service.protocol;


import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.core.protocol.entity.*;
import com.comtika.safe.core.protocol.entity.charge.ChargeBase;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.charge.ChargeStart;
import com.comtika.safe.core.protocol.entity.device.EquipNodeParams;
import com.comtika.safe.core.protocol.entity.node.NodeChargeData;
import com.comtika.safe.core.protocol.entity.node.NodeChargeRecord;
import com.comtika.safe.core.protocol.entity.node.NodeInfo;
import com.comtika.safe.core.protocol.entity.node.NodeVersion;
import com.comtika.safe.core.protocol.entity.ota.OtaUpHelp;
import com.comtika.safe.core.protocol.entity.ota.TerminalOTA;
import com.comtika.safe.core.protocol.entity.safe.BitHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskConfHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskHelp;
import com.comtika.safe.core.protocol.entity.safe.ThrParamHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerQueryHelp;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.eunms.netty.OrderCodeType;
import com.comtika.safe.core.protocol.eunms.charge.SystemControllerMode;
import com.comtika.safe.entity.report.AlwaysDataHelp;
import com.comtika.safe.entity.report.RepAlwaysData;
import com.comtika.safe.entity.safe.RepDataRecord;

import java.util.List;
import java.util.Map;

/**
 * @desc	:	充电服务层
 * @author	:	actorLucky[494977551@qq.com]
 * @date	:	2018年5月23日 下午3:57:24
 */
@SuppressWarnings("rawtypes")
public interface ProtocolService {

	RespWrapper<ChargeBase> startCharge(ChargeStart cs);

	/**
	 *根据节点id 订单号  插座编码  签名 删除
	 */
	RespWrapper<ChargeEnd> endCharge(String nodeNo, String orderNO, int plugNo, boolean isValidate);

	/**
	 *响应主动上报结束指令
	 */
	RespWrapper<Boolean> respActiveEndOper(String nodeNo, int respCode, int plugNo);
	/**
	 * @Description: 设置集中器ip与端口
	 */
	RespWrapper<Boolean> setupIpPort(String addrNo, String ip, String port,
                                     int oprationCode);
	/**
	 *
	 * @Description: 查询集中器ip与port
	 */
	RespWrapper<Map<String, String>> getTerminalIpPort(String addrNo, int operationCode);
	/**
	 *
	 * @Description: 查询集中器软硬件信息
	 */
	RespWrapper<HardwareVersion> getHardwarInfo(String addrNo);
	/**
	 *
	 * @Description: 设置节点参数
	 */
	RespWrapper<Boolean> setDeviceParams(EquipNodeParams dp, OrderCodeType controlCode);
	/**
	 *
	 * @Description: 设置节点工作模式
	 */
	RespWrapper<Boolean> setNodeWorkMode(String nodeId, int workMode, int controlCode);
	/**
	 *
	 * @Description: TODO(设置节点RTC  03H)
	 */
	RespWrapper<Boolean> setTerminalRTC(String nodeId, Long time, OrderCodeType controlCode);

	/**
	 *
	 * @Description: 查询节点版本信息 12H
	 * @param nodeId
	 * @return
	 * RespWrapper
	 */
	RespWrapper<NodeVersion> getNodeVersion(String nodeId);
	/**
	 *
	 * @Description: 查询节点充电记录 14H
	 */
	RespWrapper<NodeChargeRecord> getNodeChargeRecord(String nodeId, int resNo);

	/**
	 *查询节点状态及故障信息
	 */
	RespWrapper<NodeInfo> getNodeFalutInfo(String nodeNo, int plugNo);
	/**
	 *查询节点充电数据 11H
	 */
	RespWrapper<NodeChargeData> getNodeChargerInfo(String nodeNo, int plugNo);
	/**
	 *查询节点配置参数 13H
	 */
	RespWrapper<EquipNodeParams> getNodeConfParam(String terminalNo, String nodeNo, int plugNo);

	/**
	 * 系统控制
	 */
	RespWrapper<String> systemController(String addrNo, SystemControllerMode mode);
	/**准备硬件升级*/
	//RespWrapper<String> planUpdateTerminal(Integer hsVersionId, List<String> addrNo, Boolean isUpdate);

	/**智能卡响应*/
	RespWrapper<Boolean> answerEquipCard(String nodeNo, Integer answerCode,
                                         Integer plugNo, Integer balance);
	/**新智能卡*/
	RespWrapper<Boolean> fadTerminalCard(TerminalCard terminalCard);
	/**新版OTA*/
	RespWrapper<String> fadTerminalUpdateByOTA(TerminalOTA terminalOTA);


    RespWrapper<String> fadDeviceRestart(String deviceCode);

	RespWrapper<String> fadDeviceMac(String deviceCode);

	RespWrapper<Boolean> fadSetDeviceMac(String deviceCode, String macId);

	RespWrapper<String> fadFindDeviceTime(String deviceCode);

	RespWrapper<Boolean> fadSetDeviceTime(String deviceCode, Long time);

	RespWrapper<List<TerminalAddress>> fadFindTerminalAddressList();

	RespWrapper<Map<String, String>> fadFindTerminalAddress(String deviceCode);

	RespWrapper<Boolean> fadSetTerminalAddress(String deviceCode, Integer serviceId);

	RespWrapper<Object> fadAuxHandle(AuxHandleHelp auxHandleHelp);

    RespWrapper<Object> fadSendProtocolToTerminal(ProtocolHelp help);

    RespWrapper<WattMeterHelp> fadEleTotalStatus(String terminalNo);

    RespWrapper<Map<String, Object>> fadFindTerminalIccId(Integer id);

	RespWrapper<String> fadUpdateTerminalIccId(Integer id);

    RespWrapper<String> fadUpdateTerminalImei(Integer id);

    RespWrapper<Boolean> findDeviceWhiteStatus(String terminalNo);

	RespWrapper<List<String>> findDeviceWhiteListParams(String terminalNo);

	RespWrapper<List<String>> findDeviceWhiteListOnlineParams(String terminalNo);

	RespWrapper<Boolean> setDeviceWhiteListParams(String id);

	RespWrapper<String> fadSafeOTAUpdate(OtaUpHelp help);

	RespWrapper<DeviceParamsHelp> findPlugNoParams(ThrParamHelp help);

	RespWrapper<Boolean> setTerminalParams(ThrParamHelp help);

	RespWrapper<Boolean> cleanTerminalEle(String terminalNo, BitHelp bit);

	RespWrapper<Boolean> safeTerminalClose(String terminalNo, BitHelp bit);

	RespWrapper<Boolean> safeTerminalOn(String terminalNo, BitHelp bit);

	RespWrapper<List<SafeTaskHelp>> findSafeTaskFromTerminal(String terminalNo);

	RespWrapper<Boolean> setSafeTaskConf(String terminalNo, List<SafeTaskHelp> list);

	RespWrapper<Boolean> deleteSafeTask(String terminalNo,Integer taskNumber);

	RespWrapper<BitHelp> findSafeTerminalStatus(String terminalNo);

    RespWrapper<RepAlwaysData> findAlwaysData(AlwaysDataHelp help);

	RespWrapper<List<TimerQueryHelp>> findTimerParams(String terminalNo);

	RespWrapper<Boolean> setTimerParams(TimerHelp help);
}
