package com.comtika.safe.service.protocol.impl;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.encrypt.HexUtil;
import com.comtika.safe.core.protocol.cache.LocalProtocolCache;
import com.comtika.safe.core.protocol.cache.ProtocolRedisCache;
import com.comtika.safe.core.protocol.entity.*;
import com.comtika.safe.core.protocol.entity.charge.ChargeBase;
import com.comtika.safe.core.protocol.entity.charge.ChargeEnd;
import com.comtika.safe.core.protocol.entity.charge.ChargeStart;
import com.comtika.safe.core.protocol.entity.device.EquipNodeParams;
import com.comtika.safe.core.protocol.entity.netty.OrderEntity;
import com.comtika.safe.core.protocol.entity.netty.ResProtocol;
import com.comtika.safe.core.protocol.entity.node.NodeChargeData;
import com.comtika.safe.core.protocol.entity.node.NodeChargeRecord;
import com.comtika.safe.core.protocol.entity.node.NodeInfo;
import com.comtika.safe.core.protocol.entity.node.NodeVersion;
import com.comtika.safe.core.protocol.entity.ota.OtaUpHelp;
import com.comtika.safe.core.protocol.entity.ota.TerminalOTA;
import com.comtika.safe.core.protocol.entity.safe.BitHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskHelp;
import com.comtika.safe.core.protocol.entity.safe.ThrParamHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerHelp;
import com.comtika.safe.core.protocol.entity.timer.TimerQueryHelp;
import com.comtika.safe.core.protocol.entity.version.HardwareVersion;
import com.comtika.safe.core.protocol.entity.version.OTAUpgradeEntity;
import com.comtika.safe.core.protocol.eunms.*;
import com.comtika.safe.core.protocol.eunms.charge.AnswerCard;
import com.comtika.safe.core.protocol.eunms.charge.SystemControllerMode;
import com.comtika.safe.core.protocol.eunms.netty.OperationCode;
import com.comtika.safe.core.protocol.eunms.netty.OrderCodeType;
import com.comtika.safe.core.protocol.ota.ProtocolOTAEntity;
import com.comtika.safe.core.protocol.ota.ThreadHandleOTA;
import com.comtika.safe.core.protocol.responseUtil.FutureAdapter;
import com.comtika.safe.core.protocol.utils.*;
import com.comtika.safe.dao.equip.IDeviceWhiteDao;
import com.comtika.safe.dao.equip.ServiceAddressDao;
import com.comtika.safe.dao.equip.terminal.EquipTerminalDao;
import com.comtika.safe.dao.upload.IUploadDao;
import com.comtika.safe.entity.equip.terminal.CtkEquipTerminal;
import com.comtika.safe.entity.report.AlwaysDataHelp;
import com.comtika.safe.entity.report.RepAlwaysData;
import com.comtika.safe.entity.upload.UploadRecord;
import com.comtika.safe.service.protocol.ProtocolService;
import io.netty.channel.Channel;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName: ChargeSeriveImpl
 * @Description:协议实现层
 * @author zhaoLuYang
 * @date2018年5月24日 下午6:46:47
 * @version V1.0
 */
@Service
@Log4j2
public class ProtocolSeriveImpl implements ProtocolService {


	/**ota远程升级 多线程*/
	@Autowired
	private ThreadHandleOTA threadHandleOTA;

	@Autowired
	private IUploadDao iUploadDao;

	/**缓存 端点*/
	@Autowired
	private ProtocolRedisCache protocolRedisCache;
	@Autowired
	private ServiceAddressDao serviceAddressDao;

	@Autowired
	private EquipTerminalDao terminalDao;
	@Autowired
	private IDeviceWhiteDao iDeviceWhite;

	/***阻塞获取值  使用redis 链表阻塞会大量使用redis连接  现改为使用自己封装的异步回调
	 */
/*	private Object getRedisListValue(String key){
		return getRedisListValue(key,protocolConfig.getDelayTime());
	}*/
	/***
	 * 阻塞获取值
	 */
	/*private Object getRedisListValue(String key,Integer time){
		return redisApi.lPopInBlock(key, time, TimeUnit.SECONDS);
	}*/

	private <V> V get(String sign)throws Exception{
		if(StringUtil.isEmpty(sign)) return null;
		return (V)new FutureAdapter(sign).get();
	}
	private <V> V getDelay(String sign)throws Exception{
		if(StringUtil.isEmpty(sign)) return null;
		// 最长延迟10秒
		return (V)new FutureAdapter(sign).get(15,TimeUnit.SECONDS);
	}

	/*@Autowired
	private RedisApi redisApi;*/
	/**集中器没有登入*/
	private static final String ADDRNO_NO_LOGIN = "因网络原因，集中器已掉线，请稍后再试";
	/**集中器没有配置*/
	private static final String ADDRNO_NO_CONFIG = "该集中器还没有新增";

	/**发送指令失败*/
	private static final String ADDRNO_SEND_FAIL = "因网络原因，接收设备响应超时，请重试";

	/**
	 * 开始充电
	 */
	@Override
	public RespWrapper<ChargeBase> startCharge(ChargeStart cs) {
		//根据充电节点查到集中器的唯一标识  譬如addrNo
		/** 这里需查询数据库*/
		/*String addrNo = getAddrNoByNodeNo(cs.getNodeId());
		if(StringUtil.isEmpty(addrNo))return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		//TODO
		Channel ctx = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ctx == null)return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		String sign = BaseTypeConvert.createFadsSign();
		byte[] by = OrderDataConvertUtil.setChargeStartSend(cs, sign);
		handleSignSendMsgBefore(sign);
		ctx.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.CHARGE_START.getCode(), by));
		try {
			ResProtocol<ChargeBase> res =  getDelay(sign);
			if(res == null) return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))return RespWrapper.makeFailResp("签名校验失败！");
			if(res.getAckCode() == AckCode.SUCCESS){
				*//**成功这增加到充电订单中**//*
				protocolRedisCache.addChargeOrderNo(cs.getOrderNo());
				protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.UPDATE,cs.getOrderNo());
				return RespWrapper.makeSuccessResp(res.getData());
			}
			FaultEnableReport fault = res.getData().getFault();
			if(fault != null){
				String faultDiagnosis = FaultDiagnosis.faultDiagnosis(fault);
				return RespWrapper.makeFailResp(res.getAckCode().getName()+",故障：【"+faultDiagnosis+"】");
			}
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			log.error( "协议层,,充电开始失败:{}", e, addrNo);
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}*/
		return null;
	}
	/**先去掉以前的sign  再增加需要加入的值*/
	private void handleSignSendMsgBefore(String sign){
		//redisApi.remove(sign);
		//LocalProtocolCache.addSign(sign);
	}
	private void handleSignSendMsgAfter(String sign){
		//redisApi.remove(sign);
		//LocalProtocolCache.removeSign(sign);
	}
	private RespWrapper<String> checkChargeEndSataus(ResProtocol<ChargeEnd> resPro, String sign){
		if(resPro == null)return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
		if(!sign.equals(resPro.getSign()))return RespWrapper.makeFailResp("签名校验失败");
		if(resPro.getAckCode() == AckCode.SUCCESS)return RespWrapper.makeSuccessResp("ok");
		return RespWrapper.makeFailResp(resPro.getAckCode().getName());
	}
	/** 结束订单*/
	@Override
	public RespWrapper<ChargeEnd> endCharge(String nodeNo, String orderNo, int plugNo,boolean isValidate) {
		/*if(isValidate && !protocolRedisCache.checkChargeOrderNo(orderNo)) return RespWrapper.makeFailResp(ProjectConstant.THE_ORDER_IS_CLOSED);
		String str = "";
		String sign = "";
		ChargeEnd res = new ChargeEnd();
		String addrNo = getAddrNoByNodeNo(nodeNo);;
		if(StringUtil.isEmpty(addrNo)) return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		res.setNodeNo(nodeNo);
		res.setOrderNo(orderNo);
		res.setPlugNo(plugNo);
		res.setResCode(ChargeResCode.SUCCESS);
		try {
			*//**说明集中器掉线*//*
			//protocolRedisCache.checkTerminalOnlineList(orderNo) - > LocalProtocolCache.TERMINAL_ONLINE_LIST
			if(!LocalProtocolCache.TERMINAL_ONLINE_LIST.containsKey(addrNo) ){
				NodeChargeData userChargeData = protocolRedisCache.findUserChargeData(orderNo);
				if( userChargeData != null){
					//CalculateChargeData.updateFailChargerData(nodeChargeData,System.currentTimeMillis());
					//充电时间可是是
					res.setEleEP(userChargeData.getEleEP());
					res.setEleEPLast(userChargeData.getEleEPLast());
					res.setChargeTime(userChargeData.getChargeTime());
				}
				return RespWrapper.makeSuccessResp(res);
			}
			Channel channel = LocalProtocolCache.findChannelByTerminalNo(addrNo);
			if(channel == null) return RespWrapper.makeSuccessResp(res);
			sign = BaseTypeConvert.createFadsSign();
			byte[] by = OrderDataConvertUtil.createEndChargeBytes(nodeNo, orderNo, plugNo, sign);
			handleSignSendMsgBefore(sign);
			channel.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.CHARGE_END.getCode(), by));
			*//***结束充电只等5*//*
			ResProtocol<ChargeEnd> resPro = get(sign);
			if(isValidate && !protocolRedisCache.checkChargeOrderNo(orderNo)) return RespWrapper.makeFailResp(ProjectConstant.THE_ORDER_IS_CLOSED);
			RespWrapper<String> pt = checkChargeEndSataus(resPro, sign);
			if(pt.isSuccess()) return RespWrapper.makeSuccessResp(resPro.getData());
			else {
				str = pt.getMsg();
				NodeChargeData userChargeData = protocolRedisCache.findUserChargeData(orderNo);
				if( userChargeData != null){
					//CalculateChargeData.updateFailChargerData(nodeChargeData,System.currentTimeMillis());
					//充电时间可是是
					res.setEleEP(userChargeData.getEleEP());
					res.setEleEPLast(userChargeData.getEleEPLast());
					res.setChargeTime(userChargeData.getChargeTime());
				}
				return RespWrapper.makeSuccessResp(res);
			}
		} catch (Exception e) {
			log.error("协议层,集中器id:{},结束充电失败,原因是：{}",addrNo, e.getMessage());
			e.printStackTrace();
			return RespWrapper.makeSuccessResp(res);
		}finally {
			*//**移除用户在充电列表*//*
			protocolRedisCache.removeChargeOrderNo(orderNo);
			*//**修改用户简单数据*//*
			protocolRedisCache.handelUserSimpleChargeData(ProtocolRedisCache.ChargeDataHandleType.DEL,orderNo);
			*//**用户充电数据*//*
			protocolRedisCache.removeUserChargeData(orderNo);
			handleSignSendMsgAfter(sign);
		}*/
		return null;
	}

	@Override
	public RespWrapper<Boolean> respActiveEndOper(String nodeNo, int respCode, int plugNo) {
		try {
			if(StringUtil.isEmpty(nodeNo)) return RespWrapper.makeFailResp("节点编号不能为空");
			String addrNo = getAddrNoByNodeNo(nodeNo);
			if(StringUtil.isEmpty(addrNo)){
				return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
			}
			Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
			if(ioSession == null){
				return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
			}
			byte[] bt = new byte[8];
			System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(nodeNo),0,bt,0,6);
			bt[6] = (byte)respCode;
			bt[7] = (byte) plugNo;
			ioSession.write(new OrderEntity(addrNo,OrderCodeType.CON_SINGLE.getResCode(),OperationCode.NODE_EXCEPTION.getCode(),bt));
			return RespWrapper.makeSuccessResp(true);
		}catch (Exception e){
			e.printStackTrace();
			return RespWrapper.makeFailResp(e.getMessage());
		}
	}

	/**
	 * 设置ip与端口
	 */
	@Override
	public RespWrapper<Boolean> setupIpPort(String addrNo, String ip, String port,
			int oprationCode) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null){
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		}
		String sign = addrNo+oprationCode;
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), oprationCode, OrderDataConvertUtil.createByteByIpPort(ip, port)));
		try {
			ResProtocol res = getDelay(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign())){
				return RespWrapper.makeFailResp("签名校验失败");
			}
			if(res.getAckCode() == AckCode.SUCCESS){
				return RespWrapper.makeSuccessResp(true);
			}
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}


	/**
	 * 查询端口与ip
	 */
	@Override
	public RespWrapper<Map<String,String>> getTerminalIpPort(String addrNo, int operationCode) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null){
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		}
		String sign = addrNo+operationCode;
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), operationCode, new byte[]{}));
		try {
			ResProtocol<Map<String,String>> res  =  get(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign())){
				return RespWrapper.makeFailResp("签名校验失败");
			}
			return RespWrapper.makeSuccessResp(res.getData());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 * 获取软硬件版本号
	 */
	@Override
	public RespWrapper<HardwareVersion> getHardwarInfo(String addrNo) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null)return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		String sign = addrNo+OperationCode.GET_DETAIL_HD.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.GET_DETAIL_HD.getCode(), new byte[]{}));
		try {
			ResProtocol<HardwareVersion> res =  get(sign);
			if(res == null)return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))return RespWrapper.makeFailResp("签名校验失败");
			return RespWrapper.makeSuccessResp(res.getData());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 * 配置节点参数
	 *
	 */
	@Override
	public RespWrapper<Boolean> setDeviceParams(EquipNodeParams dp, OrderCodeType controlCode) {
		/**
		 * 根据节点编号查询 集中器编号
		 * 这里还需查询是否登入  TODO
		 */
		String addrNo = dp.getTerminalNo();
		if(StringUtil.isEmpty(addrNo)) addrNo = getAddrNoByNodeNo(dp.getNodeNo());
		if(StringUtil.isEmpty(addrNo)){
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		}
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null){
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		}
		String sign = BaseTypeConvert.createFadsSign();
		try {
			byte[] setNodeParams = OrderDataConvertUtil.setNodeParamsFad(dp,sign);
			handleSignSendMsgBefore(sign);
			ioSession.writeAndFlush(new OrderEntity(addrNo, controlCode.getReqCode(), OperationCode.SET_NODE_PARAM.getCode(), setNodeParams));
			//广播模式没有上报数据 不用等
			if(OrderCodeType.CON_BROADCAST == controlCode){
				return RespWrapper.makeSuccessResp(null);
			}
			ResProtocol<Integer> res =  get(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign())){
				return RespWrapper.makeFailResp("签名校验失败");
			}
			if(res.getAckCode() == AckCode.SUCCESS){
				if(res.getData() == dp.getPlugNo().intValue()){
					return RespWrapper.makeSuccessResp(true);
				}
				return RespWrapper.makeFailResp("返回的插座编号不一致！请检查！");
			}
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	/**
	 * 通过节点号 获取集中器编号
	 * 修改缓存 通过节点编号查询集中器方法
	 * 使用新缓存保存集中器与节点的值
	 */
	//TODO
	private String getAddrNoByNodeNo(String nodeNo){
		// 根据节点转成集中器
		return nodeNo.substring(0,10)+"80";
		// return protocolRedisCache.findTerminalNoByNodeNo(nodeNo);
		/*Map<String, String> terminalNodeMap = protocolRedisCache.findTerminalNodeMap();
		if(terminalNodeMap.containsKey(nodeNo))  return terminalNodeMap.get(nodeNo);
		Map<String,String> params = new HashMap<String, String>();
		params.put("nodeNo", nodeNo);
		CtkEquipTerminal tb = terminalDao.queryTerminalByNodeNo(params);
		if(tb == null) return null;
		String terminalNo = tb.getTerminalNo();
		terminalNodeMap.put(nodeNo,terminalNo);
		protocolRedisCache.updateTerminalNodeMap(terminalNodeMap);
		return terminalNo;*/
	}
	/**
	 * 设置节点工作模式
	 */
	@Override
	public RespWrapper<Boolean> setNodeWorkMode(String nodeNo,int workMode, int controlCode) {
		/**
		 *TODO 根据节点id 查询集中器id
		 */
		/*String addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo)){
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		}
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null){
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		}
		byte[] by = new byte[11];
		byte[] hexStr2Bytes = BaseTypeConvert.hexStr2Bytes(nodeNo);
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		by[6] = (byte) workMode;
		String sign = BaseTypeConvert.createFadsSign();
		byte[] createHardwareSign = ProtocolIDUtil.encodeEvenProtocolID(sign);
		System.arraycopy(createHardwareSign, 0, by, 7, 4);
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, controlCode, OperationCode.SET_NODE_PATTERN.getCode(), by));
		try {
			//广播模式没有上报数据 不用等
			if(OrderCodeType.CON_BROADCAST.getReqCode() == controlCode){
				return RespWrapper.makeSuccessResp(null);
			}
			ResProtocol res = get(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign())){
				return RespWrapper.makeFailResp("签名校验失败");
			}
			if(res.getAckCode() == AckCode.SUCCESS){
				return RespWrapper.makeSuccessResp(true);
			}
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}*/
		return RespWrapper.makeFailResp("暂时没有该模式");
	}
	/**
	 * 设置集中器RTC
	 */
	@Override
	public RespWrapper<Boolean> setTerminalRTC(String nodeNo, Long time,OrderCodeType controlCode) {
		String addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo)){
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		}
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null){
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		}
		byte[] by = new byte[17];
		String sign = BaseTypeConvert.createFadsSign();
		byte[] lv7 ;
		if(time == null || time == 0){
			lv7 = BaseTypeConvert.createDateFormat();
		}else{
			//下面是指定时间
			lv7 = BaseTypeConvert.createDateFormat(time);
		}
		byte[] hexStr2Bytes = BaseTypeConvert.hexStr2Bytes(nodeNo);
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		System.arraycopy(lv7, 0, by, 6, 7);
		System.arraycopy( ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 13, 4);

		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, controlCode.getReqCode(), OperationCode.SET_NODE_RTC.getCode(), by));
		try {
			//广播模式没有上报数据 不用等
			if(OrderCodeType.CON_BROADCAST == controlCode){
				return RespWrapper.makeSuccessResp(null);
			}
			ResProtocol res = get(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign())){
				return RespWrapper.makeFailResp("签名校验失败");
			}
			if(res.getAckCode() == AckCode.SUCCESS){
				return RespWrapper.makeSuccessResp(true);
			}
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	/**
	 * 查询节点状态及故障信息  10H
	 */
	@Override
	public RespWrapper<NodeInfo> getNodeFalutInfo(String nodeNo, int plugNo) {
		if(nodeNo.length() != 12)
			return RespWrapper.makeErrorResp("不合规则的字板编号！");
		String addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo))return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null) return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		String sign = BaseTypeConvert.createFadsSign();
		byte[] nodeNoBytes = ProtocolIDUtil.encodeEvenProtocolID(nodeNo);
		byte[] by = new byte[11];
		System.arraycopy(nodeNoBytes, 0, by, 0, 6);
		by[6] = (byte) plugNo;
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 7, 4);
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.GET_NODE_STATUS.getCode(), by));
		try {
			ResProtocol<NodeInfo> res = getDelay(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeRetry(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() == AckCode.SUCCESS)
				return RespWrapper.makeSuccessResp(res.getData());
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			log.error( "协议层,集中器id:{}，查询节点状态失败:{}", addrNo,e);
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 *  查询节点充电数据 11H
	 */
	@Override
	public RespWrapper<NodeChargeData> getNodeChargerInfo(String nodeNo, int plugNo) {
		/**
		 *TODO 根据节点id 查询集中器id
		 */
		String addrNo = getAddrNoByNodeNo(nodeNo);
		//TODO
		if(StringUtil.isEmpty(addrNo))
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null)
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		String sign = BaseTypeConvert.createFadsSign();
		byte[] nodeNoBytes = ProtocolIDUtil.encodeEvenProtocolID(nodeNo);
		byte[] by = new byte[11];
		System.arraycopy(nodeNoBytes, 0, by, 0, 6);
		by[6] = (byte) plugNo;
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 7, 4);
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.GET_NODE_CDCS.getCode(), by));
		try {
			ResProtocol<NodeChargeData> res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() == AckCode.SUCCESS)
				return RespWrapper.makeSuccessResp(res.getData());
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 * 查询节点版本信息 12H
	 */
	@Override
	public RespWrapper<NodeVersion> getNodeVersion(String nodeNo) {
		/**TODO 根据节点id 查询集中器id*/
		String addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo))
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null)
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		byte by[] = new byte[10];
		byte[] hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(nodeNo);
		String sign = BaseTypeConvert.createFadsSign();
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 6, 4);
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.GET_NODE_VERSION.getCode(), by));
		try {
			ResProtocol<NodeVersion> res = get(sign);
			if(res == null){//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			}
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() == AckCode.SUCCESS)
				return RespWrapper.makeSuccessResp(res.getData());
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 * 查询节点配置参数 13H
	 */
	@Override
	public RespWrapper<EquipNodeParams> getNodeConfParam(String terminalNo,String nodeNo, int plugNo) {
		String addrNo = terminalNo;
		if(StringUtil.isEmpty(addrNo))
			addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo))
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null)
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		byte by[] = new byte[11];
		byte[] hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(nodeNo);
		String sign = BaseTypeConvert.createFadsSign();
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		by[6] = (byte) plugNo;
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 7, 4);

		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.GET_NODE_PARAM.getCode(), by));
		try {
			ResProtocol<EquipNodeParams> res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() == AckCode.SUCCESS)
				return RespWrapper.makeSuccessResp(res.getData());
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	/**
	 * 查询节点充电记录 14H
	 */
	@Override
	public RespWrapper<NodeChargeRecord> getNodeChargeRecord(String nodeNo, int resNo) {
		/**
		 *TODO 根据节点id 查询集中器id
		 */
		/*String addrNo = getAddrNoByNodeNo(nodeNo);
		if(StringUtil.isEmpty(addrNo))
			return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		if(ioSession == null)
			return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		String sign = BaseTypeConvert.createFadsSign();
		byte[] hexStr2Bytes = BaseTypeConvert.hexStr2Bytes(nodeNo);
		byte[] by = new byte[11];
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		by[6] = (byte) resNo;
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 7, 4);
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(addrNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.GET_NODE_RECORD.getCode(), by));
		try {
			ResProtocol<NodeChargeRecord> res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() == AckCode.SUCCESS)
				return RespWrapper.makeSuccessResp(res.getData());
			return RespWrapper.makeFailResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}*/
		return RespWrapper.makeFailResp("暂时没有该模式");
	}

	/**
	 * 系统控制 F1
	 *
	 * 控制模式  16路只使用  1=集中器重启，2=集中器重启进入升级模式（只供特定的更新固件软件使用）
	 */
	@Override
	public RespWrapper<String> systemController(String addrNo, SystemControllerMode mode) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(addrNo);
		// if(session == null)
		// 	return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
		// String sign = addrNo+OperationCode.SYS_CONTROL.getCode();
		// OrderEntity entity = new OrderEntity(addrNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.SYS_CONTROL.getCode(), new byte[]{(byte) mode.getId()});
		// //获取前，先清空对应的值
		// handleSignSendMsgBefore(sign);
		// session.writeAndFlush(entity);
		// try {
		// 	ResProtocol<String> res = get(sign);
		// 	if(res == null)
		// 		return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
		// 	if(!sign.equals(res.getSign()))
		// 		return RespWrapper.makeFailResp("签名校验失败");
		// 	if(mode == SystemControllerMode.READ_TERMINAL_EDITION)
		// 		return RespWrapper.makeSuccessResp(res.getData());
		// 	return RespWrapper.makeSuccessResp(res.getAckCode().getName());
		// } catch (Exception e) {
		// 	e.printStackTrace();
		// 	return RespWrapper.makeFailResp("异常"+e.getMessage());
		// }finally {
		// 	handleSignSendMsgAfter(sign);
		// }
		return handleObject(addrNo,OperationCode.SYS_CONTROL,()->new byte[]{(byte) mode.getId()},false);
	}
	/**新版本OTA*/
	@Override
	public RespWrapper<String> fadTerminalUpdateByOTA(TerminalOTA terminalOTA) {
		/*try {
			CtkHsVersion single = terminalHSVersion.single(terminalOTA.getUpdateVersion());
			if(single == null) return RespWrapper.makeFailResp("根据版本号查询设备版本信息失败！");
			Byte[] bytes = protocolRedisCache.findTerminalOTAData(terminalOTA.getUpdateVersion());
			if(bytes == null || bytes.length < 2){
				bytes = IOUtils.bufferedIo(single.getUpgradeFile());
				protocolRedisCache.addTermianlOTAData(terminalOTA.getUpdateVersion(),bytes);
			}
			updateData(bytes,terminalOTA.getUpdateVersion(), terminalOTA.getTerminalList(),terminalOTA.isUpdate());
			if(terminalOTA.isUpdate())
				return RespWrapper.makeSuccessResp(OTAStatus.OTA_RUNING.getName());
			return RespWrapper.makeSuccessResp(OTAStatus.OTA_SEND.getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}*/
		return RespWrapper.makeFailResp("后面再写");
	}
	@Override
	public RespWrapper<String> fadSafeOTAUpdate(OtaUpHelp help) {
		if(help.getOtaId() == null)
			return RespWrapper.makeFailResp("OTA id 不能为空");
		List<String> terminalList = help.getTerminalList();
		if(terminalList.isEmpty())
			return RespWrapper.makeFailResp("OTA升级时，升级对象不能为空");
		try {
			// 查询最新版本d
			UploadRecord otaFile = iUploadDao.findFadOtaRecord(help.getOtaId());
			if(otaFile == null)
				return RespWrapper.makeFailResp("暂时没有OTA文件,请先上传OTA文件再升级");
			// 处理数据
			List<CtkEquipTerminal> otaList = new ArrayList<>(terminalList.size());
			for(String str:terminalList){
				CtkEquipTerminal terminal = terminalDao.findTerminalOtaVersion(str);
				if(terminal == null)continue;
				if( terminal.getTerminalId() != null && terminal.getOtaVersion() != null &&
					otaFile.getId().intValue() == terminal.getOtaVersion()){
					continue;
				}
				otaList.add(terminal);
			}
			if(otaList.isEmpty())
				return RespWrapper.makeFailResp("该集中器已是最新版本，无需升级");
			Byte[] bytes = protocolRedisCache.findTerminalOTAData(otaFile.getId());
			if(bytes == null || bytes.length < 2){
				//String s = otaFile.getBaseUrl().replaceAll(projectConfig.getUploadUrl(), "/data/");
				//log.error("下载的OTA路径为：{}",s);
				//bytes = IOUtils.bufferedIo(s);
				bytes = IOUtils.bufferedIoFromURL(otaFile.getBaseUrl());
				if(bytes == null || bytes.length < 2)
					return RespWrapper.makeFailResp("获取升级文件错误");
				protocolRedisCache.addTermianlOTAData(otaFile.getId(),bytes);
			}
			updateData(bytes,otaFile.getId(),otaList,true);
			return RespWrapper.makeSuccessResp("正在升级中");
		}catch (Exception e){
			log.error("OTA 升级出错：{}",e.getMessage());
		}
		return RespWrapper.makeFailResp("OTA升级失败");
	}

	@Override
	public RespWrapper<DeviceParamsHelp> findPlugNoParams(ThrParamHelp help) {
		int plugNo = help.getPlugNo();
		if(plugNo < 1 || plugNo > 16)return RespWrapper.makeFailResp("请检查插座编号");
		String terminalNo = help.getTerminalNo();
		Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		TerminalToNode node = new TerminalToNode(terminalNo,plugNo,help.getIsMain());
		// String sign = terminalNo+OperationCode.SAFE_THR_FIND.getCode();
		byte by[] = new byte[11];
		byte[] hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(node.nodeNo);
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		by[6] = (byte) node.plugNo;
		String sign = BaseTypeConvert.createFadsSign();
		byte[] bytes = ProtocolIDUtil.encodeEvenProtocolID(sign);
		System.arraycopy(bytes, 0, by, 7, 4);
		session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_THR_FIND.getCode(), by));
		return doHandleObject(sign);
	}
	// 查询实时数据
	@Override
	public RespWrapper<RepAlwaysData> findAlwaysData(AlwaysDataHelp help) {
		int plugNo = help.getPlugNo();
		String terminalNo = help.getTerminalNo();
		if(plugNo < 1 || plugNo > 16)return RespWrapper.makeFailResp("请检查插座编号");
		Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		TerminalToNode node = new TerminalToNode(terminalNo,plugNo,help.getIsMain());
		byte by[] = new byte[11];
		byte[] hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(node.nodeNo);
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		by[6] = (byte) node.plugNo;
		String sign = BaseTypeConvert.createFadsSign();
		byte[] bytes = ProtocolIDUtil.encodeEvenProtocolID(sign);
		System.arraycopy(bytes, 0, by, 7, 4);
		session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_DATA_FIND.getCode(), by));
		return doHandleObject(sign);
	}

	@Override
	public RespWrapper<List<TimerQueryHelp>> findTimerParams(String terminalNo) {
		return handleObject(terminalNo,OperationCode.CTJ_FIND_TIME_CONF,()->new byte[0],false);
	}

	@Override
	public RespWrapper<Boolean> setTimerParams(TimerHelp help) {
		return handleObject(help.getTerminalNo(),OperationCode.CTJ_CONF_TIME_HANDLE,()->help.creteBytes(),false);
	}


	@Override
	public RespWrapper<Boolean> setTerminalParams(ThrParamHelp help) {
		int plugNo = help.getPlugNo();
		String terminalNo = help.getTerminalNo();
		if(plugNo < 1 || plugNo > 16)return RespWrapper.makeFailResp("请检查插座编号");
		Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		TerminalToNode node = new TerminalToNode(terminalNo,plugNo,help.getIsMain());
		// String sign = terminalNo+OperationCode.SAFE_CONFIG_THR.getCode();
		// 这里拼接数据
		byte by[] = new byte[41];
		byte[] hexStr2Bytes;
		// 8F
		if(help.getIsMain() || !help.getAllTerminal()){
			hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(node.nodeNo);
		}else {
			String nodeNo = node.nodeNo;
			nodeNo = nodeNo.substring(0,nodeNo.length()-2)+"8F";
			hexStr2Bytes = ProtocolIDUtil.encodeEvenProtocolID(nodeNo);
		}
		System.arraycopy(hexStr2Bytes, 0, by, 0, 6);
		SafeDataConvertUtil.setTerminalParams(by,help.getParams());
		by[36] = (byte) node.plugNo;
		String sign = BaseTypeConvert.createFadsSign();
		System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(sign), 0, by, 37, 4);
		handleSignSendMsgBefore(sign);
		session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_CONFIG_THR.getCode(), by));
		return doHandleObject(sign);
	}

	@Override
	public RespWrapper<Boolean> cleanTerminalEle(String terminalNo,BitHelp bit) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.SAFE_ELE_DELETE.getCode();
		// handleSignSendMsgBefore(sign);
		// byte[] bytes = null;
		// try {
		// 	bytes = BitHelp.convertBytes(bit);
		// }catch (Exception e){
		// 	log.error("解析byte 出错：{}",e.getMessage());
		// 	return RespWrapper.makeFailResp("解析bit出错");
		// }
		// session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_ELE_DELETE.getCode(), bytes));
		// return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_ELE_DELETE,()->BitHelp.convertBytes(bit),false);
	}

	@Override
	public RespWrapper<Boolean> safeTerminalClose(String terminalNo,BitHelp bit) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.SAFE_SWITCH_OFF.getCode();
		// byte[] bytes = null;
		// try {
		// 	bytes = BitHelp.convertBytes(bit);
		// }catch (Exception e){
		// 	log.error("解析byte 出错：{}",e.getMessage());
		// 	return RespWrapper.makeFailResp("解析bit出错");
		// }
		// session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_SWITCH_OFF.getCode(), bytes));
		// return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_SWITCH_OFF,()->BitHelp.convertBytes(bit),false);
	}

	@Override
	public RespWrapper<Boolean> safeTerminalOn(String terminalNo, BitHelp bit) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.SAFE_SWITCH_ON.getCode();
		// byte[] bytes = null;
		// try {
		// 	 bytes = BitHelp.convertBytes(bit);
		// }catch (Exception e){
		// 	log.error("解析byte 出错：{}",e.getMessage());
		// 	return RespWrapper.makeFailResp("解析bit出错");
		// }
		// session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SINGLE.getReqCode(), OperationCode.SAFE_SWITCH_ON.getCode(), bytes));
		// return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_SWITCH_ON,()->BitHelp.convertBytes(bit),false);
	}

	@Override
	public RespWrapper<List<SafeTaskHelp>> findSafeTaskFromTerminal(String terminalNo) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.SAFE_TASK_FIND.getCode();
		// byte[] bytes = new byte[0];
		// session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.SAFE_TASK_FIND.getCode(), bytes));
		// return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_TASK_FIND,()->new byte[0],true);
	}

	@Override
	public RespWrapper<Boolean> setSafeTaskConf(String terminalNo, List<SafeTaskHelp> list) {
		// Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.SAFE_TASK_ADD.getCode();
		// byte[] bytes = null;
		// try {
		// 	bytes = SafeDataConvertUtil.safeTaskConvert(list);
		// }catch (Exception e){
		// 	log.error("解析定时任务错误：{}",e);
		// 	return RespWrapper.makeFailResp("解析定时任务错误");
		// }
		// session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.SAFE_TASK_ADD.getCode(), bytes));
		// return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_TASK_ADD,()->SafeDataConvertUtil.safeTaskConvert(list),true);
	}

	@Override
	public RespWrapper<Boolean> deleteSafeTask(String terminalNo,Integer taskNumber) {
		//Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		//if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		//String sign = terminalNo+OperationCode.SAFE_TASK_DELETE.getCode();
		//session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.SAFE_TASK_DELETE.getCode(), new byte[0]));
		//return doHandleObject(sign);
		return handleObject(terminalNo,OperationCode.SAFE_TASK_DELETE,()->findTask(taskNumber),false);
	}
	private byte[] findTask(Integer taskNumber){
		if(taskNumber == null) return new byte[0];
		byte[] bytes = new byte[1];
		bytes[0] = taskNumber.byteValue();
		return bytes;
	}

	@Override
	public RespWrapper<BitHelp> findSafeTerminalStatus(String terminalNo) {
		return handleObject(terminalNo,OperationCode.SAFE_SWITCH_FIND,()->new byte[0],false);
	}


	private <T> RespWrapper<T> handleObject(String terminalNo,OperationCode code,TranByte tranByte,boolean isLong){
		Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = terminalNo+code.getCode();
		byte[] bytes = null;
		try {
			bytes = tranByte.convert();
		}catch (Exception e){
			log.error("数据转byte数值错误：{}",e);
			return RespWrapper.makeFailResp("数据转byte数值错误");
		}
		session.writeAndFlush(new OrderEntity(terminalNo, isLong?OrderCodeType.CON_SUPER_LONG.getReqCode():OrderCodeType.CLIENT_PARAM_SET.getReqCode(), code.getCode(), bytes));
		return doHandleObject(sign);
	}
	private interface TranByte{
		byte[] convert() throws Exception;
	}

	private <T> RespWrapper<T> doHandleObject(String sign){
		try {
			handleSignSendMsgBefore(sign);
			ResProtocol<T> res = getDelay(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(ADDRNO_SEND_FAIL);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			if(res.getAckCode() != null){
				if(res.getAckCode() == AckCode.SUCCESS)
					return RespWrapper.makeSuccessResp(res.getData());
				return RespWrapper.makeFailResp(res.getAckCode().getName());
			}
			return RespWrapper.makeSuccessResp(res.getData());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	@Data
	public static class TerminalToNode{
		private String nodeNo;
		private int plugNo;
		public TerminalToNode(String arg1,int plugNo,boolean isMain){
			if(!isMain){
				String str = arg1.substring(0,arg1.length()-2);
				// 前端传值是 1-16 所有这里需要处理
				if(plugNo > 8){
					str += "82";
					plugNo-=8;
				}else str += "81";
				nodeNo = str;
			}else nodeNo = arg1;
			this.plugNo = plugNo-1;
		}
	}



	/**
	 * 系统控制 F1
	 * 控制模式  16路只使用  1=集中器重启，2=集中器重启进入升级模式（只供特定的更新固件软件使用）
	 */
	@Override
	public RespWrapper<String> fadDeviceRestart(String deviceCode) {
		Channel session = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = deviceCode+OperationCode.SYS_CONTROL.getCode();
		OrderEntity entity = new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.SYS_CONTROL.getCode(), new byte[]{1});
		//获取前，先清空对应的值
		handleSignSendMsgBefore(sign);
		session.writeAndFlush(entity);
		try {
			ResProtocol<String> res = get(sign);
			if(res == null)
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			if(!sign.equals(res.getSign()))
				return RespWrapper.makeFailResp("签名校验失败");
			return RespWrapper.makeSuccessResp(res.getAckCode().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
    private static final String TERMINAL_OFF_LINE = "设备离线中,等设备在线再试";
	private static final String TERMINAL_SUC_NO = "指令下发成功,响应超时;具体结果以实际为准。";
	@Override
	public RespWrapper<String> fadDeviceMac(String deviceCode) {
		// 获取服务器mac 地址
        Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
        if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
        String sign = deviceCode+OperationCode.FIND_TERMINAL_MAC.getCode();
        handleSignSendMsgBefore(sign);
        ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.FIND_TERMINAL_MAC.getCode(), new byte[]{}));
        try {
            ResProtocol<Boolean> res  =  get(sign);
            if(res == null)//说明没有取得值
                return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
            return RespWrapper.makeSuccessResp(deviceCode);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("异常"+e.getMessage());
        }finally {
            handleSignSendMsgAfter(sign);
        }
	}

	@Override
	public RespWrapper<Boolean> fadSetDeviceMac(String deviceCode, String macId) {
        Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
        if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
        String sign = deviceCode+OperationCode.UPDATE_TERMINAL_MAC.getCode();
        handleSignSendMsgBefore(sign);
        byte[] bytes = HexUtil.hexStr2Bytes(macId);
        ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.UPDATE_TERMINAL_MAC.getCode(), bytes));
        try {
            ResProtocol<Boolean> res  =  get(sign);
            if(res == null)//说明没有取得值
                return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
            return RespWrapper.makeSuccessResp(res.getData());
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("异常"+e.getMessage());
        }finally {
            handleSignSendMsgAfter(sign);
        }
	}

	@Override
	public RespWrapper<String> fadFindDeviceTime(String deviceCode) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
		if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = deviceCode+ OperationCode.FIND_TERMINAL_TIME.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.FIND_TERMINAL_TIME.getCode(), new byte[0]));
		try {
			ResProtocol<String> res  =  get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			return RespWrapper.makeSuccessResp(res.getData());
		}catch (Exception e){
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常:"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	@Override
	public RespWrapper<Boolean> fadSetDeviceTime(String deviceCode, Long time) {
        // Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
        // if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
        // String sign = deviceCode+OperationCode.UPDATE_TERMINAL_TIME.getCode();
        // handleSignSendMsgBefore(sign);
        // ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.UPDATE_TERMINAL_TIME.getCode(), BaseTypeConvert.createTerminalTime(time)));
		// try {
        //     ResProtocol<String> res  =  get(sign);
        //     if(res == null)//说明没有取得值
        //         return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
        //     return RespWrapper.makeSuccessResp(true);
        // }catch (Exception e){
		//     e.printStackTrace();
		//     return RespWrapper.makeFailResp("异常:"+e.getMessage());
        // }finally {
        //     handleSignSendMsgAfter(sign);
        // }
		return handleObject(deviceCode,OperationCode.UPDATE_TERMINAL_TIME,()->BaseTypeConvert.createTerminalTime(time),false);
	}
	// 查询所有服务地址
	@Override
	public RespWrapper<List<TerminalAddress>> fadFindTerminalAddressList() {
		return RespWrapper.makeSuccessResp(serviceAddressDao.all());
	}
    // 查询设备通讯地址
	@Override
	public RespWrapper<Map<String, String>> fadFindTerminalAddress(String deviceCode) {
		/*Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
		if(ioSession == null)
			return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = deviceCode+OperationCode.GET_HOST_IPPORT.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.GET_HOST_IPPORT.getCode(), new byte[]{}));
		try {
			ResProtocol<Map<String,String>> res  =  get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			if(!sign.equals(res.getSign()))return RespWrapper.makeFailResp("签名校验失败");
			return RespWrapper.makeSuccessResp(res.getData());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}*/
		return handleObject(deviceCode,OperationCode.GET_HOST_IPPORT,()->new byte[0],false);
	}
    // 设置设备通讯地址
	@Override
	public RespWrapper<Boolean> fadSetTerminalAddress(String deviceCode, Integer serviceId) {
		TerminalAddress single = serviceAddressDao.single(serviceId);
		if(single == null) return RespWrapper.makeFailResp("服务器地址错误");
        if(single.getPort() == null || StringUtil.isEmpty(single.getIp()))
            return RespWrapper.makeFailResp("该服务器ip与端口都不能为空");
        // Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(deviceCode);
        // if(ioSession == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
        // String sign = deviceCode+OperationCode.SET_HOST_IPPORT.getCode();
        // handleSignSendMsgBefore(sign);
        // ioSession.writeAndFlush(new OrderEntity(deviceCode, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.SET_HOST_IPPORT.getCode(), OrderDataConvertUtil.createByteByIpPort(single.getIp(), single.getPort().toString())));
        // try {
        //     ResProtocol res = get(sign);
        //     if(res == null)//说明没有取得值
        //         return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
        //     if(!sign.equals(res.getSign()))return RespWrapper.makeFailResp("签名校验失败");
        //     if(res.getAckCode() == AckCode.SUCCESS) return RespWrapper.makeSuccessResp(true);
        //     return RespWrapper.makeFailResp(res.getAckCode().getName());
        // } catch (Exception e) {
        //     e.printStackTrace();
        //     return RespWrapper.makeFailResp("异常"+e.getMessage());
        // }finally {
        //     handleSignSendMsgAfter(sign);
        // }
		return handleObject(deviceCode,OperationCode.SET_HOST_IPPORT,()-> OrderDataConvertUtil.createByteByIpPort(single.getIp(), single.getPort().toString()),false);
	}

	@Override
	public RespWrapper<Object> fadSendProtocolToTerminal(ProtocolHelp help) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(help.getTerminalNo());
		if(ioSession == null)
			return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		int cmd = BaseTypeConvert.hexToInt(help.getCmd());
		int operCode = BaseTypeConvert.hexToInt(help.getControllerCode());
		String sign = help.getTerminalNo()+cmd;
		handleSignSendMsgBefore(sign);
		byte[] bt = new byte[0];
		if(StringUtil.isNotEmpty(help.getData()))
			bt = HexUtil.hexStr2Bytes(help.getData());
		ioSession.writeAndFlush(new OrderEntity(help.getTerminalNo(), operCode, cmd,bt));
		try {
			ResProtocol res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			return RespWrapper.makeSuccessResp(res.getData());
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	@Override
	public RespWrapper<WattMeterHelp> fadEleTotalStatus(String terminalNo) {
		// Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		// if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminalNo+OperationCode.FIND_ELE_TOTAL.getCode();
		// handleSignSendMsgBefore(sign);
		// ioSession.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.FIND_ELE_TOTAL.getCode(),new byte[0]));
		// try {
		// 	ResProtocol<WattMeterHelp> res = get(sign);
		// 	if(res == null)//说明没有取得值
		// 		return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
		// 	if(!sign.equals(res.getSign()))return RespWrapper.makeFailResp("签名校验失败");
		// 	return RespWrapper.makeSuccessResp(res.getData());
		// } catch (Exception e) {
		// 	e.printStackTrace();
		// 	return RespWrapper.makeFailResp("异常"+e.getMessage());
		// }finally {
		// 	handleSignSendMsgAfter(sign);
		// }
		return handleObject(terminalNo,OperationCode.FIND_ELE_TOTAL,()->new byte[0],false);
	}
	@Override
	public RespWrapper<Object> fadAuxHandle(AuxHandleHelp auxHandleHelp) {
		String terminalNo = auxHandleHelp.getTerminalNo();
		Channel session = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(session == null) return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// 这里需要处理一下数据
		int code = OperationCode.HANDLE_AUX_STATUS.getCode();
		String sign = terminalNo+code+auxHandleHelp.getStatus();
		byte[] bytes = new byte[2];
		bytes[0] = auxHandleHelp.getStatus().byteValue();
		if(auxHandleHelp.getStatus() != 2){
			// 8个数组合成一个字节的哟
			bytes[1] = auxHandleHelp.bitToByte();
		}
		session.writeAndFlush(new OrderEntity(terminalNo, OrderCodeType.CLIENT_PARAM_SET.getReqCode(), code, bytes));
		return doHandleObject(sign);
	}

	@Override
	public RespWrapper<Map<String, Object>> fadFindTerminalIccId(Integer id) {
		Map<String,Object> map =  terminalDao.fadFindTerminalIccId(id);
		return RespWrapper.makeSuccessResp(map);
	}
	// 下面是查询方法
	@Override
	public RespWrapper<String> fadUpdateTerminalIccId(Integer id) {
		CtkEquipTerminal terminal = terminalDao.single(id);
		if(terminal == null || StringUtil.isEmpty(terminal.getTerminalNo()))
			return RespWrapper.makeFailResp("集中器唯一id 错误");
		// Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminal.getTerminalNo());
		// if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminal.getTerminalNo()+OperationCode.PROTO_ICCID_UP.getCode();
		// handleSignSendMsgBefore(sign);
		// // 发送iccid 查询
		// ioSession.write(new OrderEntity(terminal.getTerminalNo(), OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.PROTO_ICCID_UP.getCode(),new byte[0]));
		// // 发送imei 查询
		// //ioSession.write(new OrderEntity(terminal.getTerminalNo(), OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.PROTO_IMEI.getCode(),new byte[0]));
		// // TODO
		// try {
		// 	String res = get(sign);
		// 	if(res == null)//说明没有取得值
		// 		return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
		// 	return RespWrapper.makeSuccessResp(res);
		// } catch (Exception e) {
		// 	e.printStackTrace();
		// 	return RespWrapper.makeFailResp("异常"+e.getMessage());
		// }finally {
		// 	handleSignSendMsgAfter(sign);
		// }
		return handleObject(terminal.getTerminalNo(),OperationCode.PROTO_ICCID_UP,()->new byte[0],false);
	}

	@Override
	public RespWrapper<String> fadUpdateTerminalImei(Integer id) {
		CtkEquipTerminal terminal = terminalDao.single(id);
		if(terminal == null || StringUtil.isEmpty(terminal.getTerminalNo()))
			return RespWrapper.makeFailResp("集中器唯一id 错误");
		// Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminal.getTerminalNo());
		// if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		// String sign = terminal.getTerminalNo()+OperationCode.PROTO_IMEI_UP.getCode();
		// handleSignSendMsgBefore(sign);
		// // 发送iccid 查询
		// //ioSession.write(new OrderEntity(terminal.getTerminalNo(), OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.PROTO_ICCID_UP.getCode(),new byte[0]));
		// // 发送imei 查询
		// ioSession.write(new OrderEntity(terminal.getTerminalNo(), OrderCodeType.CLIENT_PARAM_SET.getReqCode(), OperationCode.PROTO_IMEI_UP.getCode(),new byte[0]));
		// // TODO
		// try {
		// 	String res = get(sign);
		// 	if(res == null)//说明没有取得值
		// 		return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
		// 	return RespWrapper.makeSuccessResp(res);
		// } catch (Exception e) {
		// 	e.printStackTrace();
		// 	return RespWrapper.makeFailResp("异常"+e.getMessage());
		// }finally {
		// 	handleSignSendMsgAfter(sign);
		// }
		return handleObject(terminal.getTerminalNo(),OperationCode.PROTO_IMEI_UP,()->new byte[0],false);
	}

	@Override
	public RespWrapper<Boolean> findDeviceWhiteStatus(String terminalNo) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = terminalNo+OperationCode.FIND_MATER_NODE_WHITE_LIST.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.write(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.FIND_MATER_NODE_WHITE_LIST.getCode(),new byte[0]));
		// TODO
		try {
			Boolean res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			if(res) return RespWrapper.makeSuccessResp(true);
			return RespWrapper.makeResp(-10,"不是主节点",null);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	@Override
	public RespWrapper<List<String>> findDeviceWhiteListParams(String terminalNo) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = terminalNo+OperationCode.FIND_TERMINAL_WHITE_LIST.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.write(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.FIND_TERMINAL_WHITE_LIST.getCode(),new byte[0]));
		// TODO
		try {
			List<String> res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			 return RespWrapper.makeSuccessResp(res);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}

	@Override
	public RespWrapper<List<String>> findDeviceWhiteListOnlineParams(String terminalNo) {
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = terminalNo+OperationCode.FIND_TERMINAL_ONLINE_WHITE_LIST.getCode();
		handleSignSendMsgBefore(sign);
		ioSession.write(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.FIND_TERMINAL_ONLINE_WHITE_LIST.getCode(),new byte[0]));
		// TODO
		try {
			List<String> res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			return RespWrapper.makeSuccessResp(res);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}
	// TODO
	@Override
	public RespWrapper<Boolean> setDeviceWhiteListParams(String id) {
		CtkEquipTerminal single = terminalDao.single(id);
		if(single == null) return RespWrapper.makeFailResp("设备唯一id 错误");
		List<String> list = iDeviceWhite.findTerminalWhite(id);
		if(list == null || list.isEmpty())
			return RespWrapper.makeFailResp("该设备暂时还没有配置白名单");
		String terminalNo = single.getTerminalNo();
		Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(terminalNo);
		if(ioSession == null)return RespWrapper.makeFailResp(TERMINAL_OFF_LINE);
		String sign = terminalNo+OperationCode.SET_MATER_NODE_WHITE_LIST.getCode();
		handleSignSendMsgBefore(sign);
		byte[] bytes = OrderDataConvertUtil.whiteListToBytes(list);
		ioSession.write(new OrderEntity(terminalNo, OrderCodeType.CON_SUPER_LONG.getReqCode(), OperationCode.SET_MATER_NODE_WHITE_LIST.getCode(),bytes));
		// TODO
		try {
			Boolean res = get(sign);
			if(res == null)//说明没有取得值
				return RespWrapper.makeFailResp(TERMINAL_SUC_NO);
			if(res) return RespWrapper.makeSuccessResp(true);
			return RespWrapper.makeFailResp("配置失败");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常"+e.getMessage());
		}finally {
			handleSignSendMsgAfter(sign);
		}
	}



	/***
	 * @Title: updateData
	 * @Description: TODO(使用多线程传输升级包)
	 * @param hsVersionId
	 * @param addrNos
	 * @throws Exception
	 * @throws
	 * @变更记录  2018年8月16日 下午6:11:36 zhaoLuYang create
	 */
	private void updateData(Byte[] bufferedIo,Integer hsVersionId, List<CtkEquipTerminal> addrNos,boolean isUpdate) throws Exception{
		if(bufferedIo == null || bufferedIo.length < 2){
			log.error("获取升级包数据错误");
			return;
		}
		int size = addrNos.size();
		if(size == 0)return;
		for(CtkEquipTerminal terminal:addrNos){
			ProtocolOTAEntity pt = new ProtocolOTAEntity();
			pt.setOtaPoint( new OTAUpgradeEntity());
			pt.setAddrNo(terminal.getTerminalNo());
			pt.setTerminal(terminal);
			pt.setBufferedIo(bufferedIo);
			pt.setHsId( hsVersionId);
			pt.setUpdate(isUpdate);
			threadHandleOTA.run(pt);
		}
	}
	private static Integer MAX_BALANCE = 0xFF;
	@Override
	public RespWrapper<Boolean> answerEquipCard(String nodeNo,
			Integer answerCode, Integer plugNo,Integer balance) {
		try {
			String addrNo = getAddrNoByNodeNo(nodeNo);
			if(StringUtil.isEmpty(addrNo)){
				return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
			}
			Channel ioSession = LocalProtocolCache.findChannelByTerminalNo(addrNo);
			if(ioSession == null){
				return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
			}
			//是否需要传金额
			boolean balance1 = AnswerCard.isBalance(answerCode);
			byte[] bytes = null;
			if(balance1 ){
				if(balance == null) return RespWrapper.makeFailResp("该模式必须报用户余额");
				if(balance > MAX_BALANCE) balance = MAX_BALANCE;
				bytes = BaseTypeConvert.int2BytesBE(balance, 2);
			}
			byte[] bt = new byte[10];
			System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(addrNo), 0, bt, 0, 6);
			bt[6] = answerCode.byteValue();
			bt[7] = plugNo.byteValue();
			if(bytes != null) System.arraycopy(bytes, 0, bt, 8, 2);

			OrderEntity entity = new OrderEntity(addrNo,OrderCodeType.CLIENT_TO_SERVER.getResCode(),OperationCode.AUTO_CARD.getCode(),bt);
			ioSession.writeAndFlush(entity);
			return RespWrapper.makeSuccessResp(true);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常："+e.getMessage());
		}
	}



	/**下面是响应智能卡新接口*/
	@Override
	public RespWrapper<Boolean> fadTerminalCard(TerminalCard terminalCard) {
		try {
			String addrNo = getAddrNoByNodeNo(terminalCard.getUniqueNo());
			if(StringUtil.isEmpty(addrNo))return RespWrapper.makeFailResp(ADDRNO_NO_CONFIG);
			Channel channel = LocalProtocolCache.findChannelByTerminalNo(addrNo);
			if(channel == null){
				return RespWrapper.makeFailResp(ADDRNO_NO_LOGIN);
			}
			byte[] bytes = null;
			Integer balance = terminalCard.getBalance();
			if(AnswerCard.isBalance(terminalCard.getAnswerCard()) ){
				if(balance == null) return RespWrapper.makeFailResp("响应智能卡时，在"+terminalCard.getAnswerCard().getName()+"效应，必须报用户余额");
				if(balance > MAX_BALANCE) balance = MAX_BALANCE;
				bytes = BaseTypeConvert.int2BytesBE(balance, 2);
			}
			byte[] bt = new byte[10];
			System.arraycopy(ProtocolIDUtil.encodeEvenProtocolID(addrNo), 0, bt, 0, 6);
			bt[6] = (byte)terminalCard.getAnswerCard().getCode();
			bt[7] = (byte)terminalCard.getPulgNo();
			if(bytes != null) System.arraycopy(bytes, 0, bt, 8, 2);
			OrderEntity entity = new OrderEntity(addrNo,OrderCodeType.CLIENT_TO_SERVER.getResCode(),OperationCode.AUTO_CARD.getCode(),bt);
			channel.writeAndFlush(entity);
			return RespWrapper.makeSuccessResp(true);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeFailResp("异常："+e.getMessage());
		}
	}


}
