package com.comtika.safe.service.safe.impl;

import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.core.message.MessageUtils;
import com.comtika.safe.core.protocol.cache.SafeCache;
import com.comtika.safe.core.protocol.entity.safe.TerminalSafeInfo;
import com.comtika.safe.core.protocol.entity.safe.WarnDataHelp;
import com.comtika.safe.core.protocol.entity.safe.WarnErrHelp;
import com.comtika.safe.core.task.WarnHandleTask;
import com.comtika.safe.core.weChat.WxChatUtil;
import com.comtika.safe.dao.message.FadMessageConfDao;
import com.comtika.safe.dao.message.FadMessageHandleConfDao;
import com.comtika.safe.dao.safe.INewestDataDao;
import com.comtika.safe.dao.safe.IRepRecordDao;
import com.comtika.safe.dao.safe.IWarnRecordDao;
import com.comtika.safe.dao.safe.TerminalEleTotalDao;
import com.comtika.safe.entity.message.FadMessageConf;
import com.comtika.safe.entity.message.FadMessageHandleConf;
import com.comtika.safe.entity.safe.RepDataRecord;
import com.comtika.safe.entity.safe.SafeNode;
import com.comtika.safe.entity.safe.TerminalEleTotal;
import com.comtika.safe.entity.safe.TerminalNewestData;
import com.comtika.safe.entity.warn.WarnRecord;
import com.comtika.safe.entity.warn.help.WarnMessageHelp;
import com.comtika.safe.service.message.IMessageService;
import com.comtika.safe.service.safe.IRepService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 14:10 2020/8/21
 * @DESC:
 **/
@Service
@Log4j2
public class RepServiceImpl implements IRepService {
    // 这个只是为了更新而使用的
    private static Map<String,Integer> TERMINAL_KEY_RECORD = new ConcurrentHashMap<>();
    @Autowired
    private WarnHandleTask warnHandleTask;

    @Autowired
    private IRepRecordDao iRepRecordDao;
    @Autowired
    private INewestDataDao iNewestDataDao;
    @Autowired
    private IWarnRecordDao iWarnRecordDao;
    @Autowired
    private FadMessageHandleConfDao fadMessageHandleConfDao;
    @Autowired
    private FadMessageConfDao fadMessageConfDao;
    @Autowired
    private IMessageService iMessageService;
    @Autowired
    private WxChatUtil wxChatUtil;
    @Autowired
    private TerminalEleTotalDao terminalEleTotalDao;

    // 增加记录
    @Override
    public void terminalRepData(RepDataRecord res) {
        String terminalNo = res.getTerminalNo();
        if(StringUtil.isEmpty(terminalNo)){
            log.error("上报数据集中器编号为空，抛弃该数据：{}",res);
            return;
        }
        // 这里需要根据数据处理一些数据
        res.setCreateTime(System.currentTimeMillis());
        // 下面需要根据
        TerminalSafeInfo base = findTerminalBaseInfo(res.getTerminalNo());
        if(base == null) return;
        res.setSiteId(base.getSiteId());
        res.setSiteName(base.getSiteName());
        res.setMerchantId(base.getMerchantId());
        res.setMerchantName(base.getMerchantName());
        res.setTypeId(base.getTypeId());
        res.setTypeName(base.getTypeName());
        SafeNode node = new SafeNode(res.getNodeNo(),res.getPlugNo());
        if(StringUtil.isEmpty(node.getSerialNo())) return;
        res.setSerialNo(node.getSerialNo());
        iRepRecordDao.insert(res,true);
        LuceneUtils.writeRepRecord(res);
        // 这里还需要更新数据
        doHandleUpdateRecord(res,base,node);
    }
    @Override
    public TerminalSafeInfo findTerminalBaseInfo(String terminalNo){
        TerminalSafeInfo base = SafeCache.findTerminalBaseInfo(terminalNo);
        if(base == null){
            // 查询
            base = iRepRecordDao.findTerminalBaseInfoFromDB(terminalNo);
            if(base == null){
                log.error("根据集中器编号：{},查询站点名称、商户名称错误",terminalNo);
                return null;
            }
            SafeCache.addTerminalBaseInfo(terminalNo,base);
        }
        if(base == null){
            log.error("查询设备基础信息失败");
            return null;
        }
        return base;
    }
    private boolean isSendMsg(FadMessageConf conf){
        return conf.getMainSwitch() || conf.getWxSwitch();
    }
    // 设备断网断电消息推送
    @Override
    public void sendOffLineTerminalMessage(String terminalNo) {
        log.error("设备调用告警，设备掉线编号为:{}",terminalNo);
        try {
            TerminalSafeInfo base = findTerminalBaseInfo(terminalNo);
            if(base == null) return;
            List<FadMessageHandleConf> messageList = findMessageList(base.getSiteId(), terminalNo);
            if(messageList == null)return;
            FadMessageConf conf = new FadMessageConf();
            conf.setSiteId(base.getSiteId());
            conf = fadMessageConfDao.templateOne(conf);
            if(conf != null && isSendMsg(conf) && conf.getOfflineSwitch()){
                long startTime = System.currentTimeMillis();
                // todo 这里只发送了短信消息~ 后期需要增加公众号消息
                log.error("发生断点断网告警通知....");
                List<String> collect = messageList.stream().map(s -> s.getPhoneNo()).distinct().collect(Collectors.toList());
                if(conf.getMainSwitch()){
                    collect.forEach(s->{
                        // 这个告警需要查询数据
                        MessageUtils.sendSafeMessage(s,base.getSiteName(),terminalNo,startTime,"设备掉电或掉网");
                    });
                }
                if(conf.getWxSwitch() && !collect.isEmpty()){
                    // 发生公众号消息
                    Map<String,Object> condition = new HashMap<>(4);
                    condition.put("phoneList",collect);
                    List<String> openList = iNewestDataDao.findOpenList(condition);
                    if (!openList.isEmpty()) {
                        WarnMessageHelp help = new WarnMessageHelp();//这里推送告警消息
                        help.setTerminalNo(terminalNo);
                        help.setTerminalType(base.getTypeName());
                        help.setSiteName(base.getSiteName());
                        help.setWarnMsg("设备掉电或掉网");
                        wxChatUtil.sendWarnMessageToUser(help,openList,null);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("发送设备掉线掉网消息出错了：{},集中器NO：{}",e.getMessage(),terminalNo);
            //e.printStackTrace();
        }
    }
    public static void main(String[] args){
        List<Integer> res = new ArrayList<>();
        res.add(1);
        res.add(3);
        res.add(2);
        res.add(1);
        res.stream().distinct().forEach(
                s-> System.out.println(s)
        );
        System.out.println(res);
    }
    private List<FadMessageHandleConf> findMessageList(int siteId,String terminalNo){
        List<FadMessageHandleConf> siteMessageCache = iMessageService.findSiteMessageConfig(siteId);
        if(siteMessageCache == null){
            SafeCache.isEmptyOrAdd(terminalNo,true);
            log.error("告警查询错误");
            return null;
        }
        return siteMessageCache;
    }

    @Override
    public void handleWarnData(WarnDataHelp dataHelp) {
        if (SafeCache.isEmptyOrAdd(dataHelp.getTerminalNo(),false)) {
            log.error("这个数据没有消息配置，所有这个告警不添加");
            return;
        }
        // 这里处理告警数据
        WarnErrHelp err = dataHelp.getErr();
        Map<Integer, String> error = err.findError();
        int size = error.size();
        if(size == 0){
            log.error("设备告警数据：{};但又没有告警",dataHelp);
            return;
        }
        TerminalSafeInfo base = findTerminalBaseInfo(dataHelp.getTerminalNo());
        if(base == null)return;
        List<FadMessageHandleConf> siteMessageCache = findMessageList(base.getSiteId(), dataHelp.getTerminalNo());
        if(siteMessageCache == null) return;
        // final List<FadMessageHandleConf> list = siteMessageCache;
        //Integer handleUserId = fadMessageHandleConfDao.findMessageHandleUser(base.getSiteId());
        List<Integer> collect = siteMessageCache.stream().filter(s -> s.getType() == 2).map(s -> s.getUserId()).collect(Collectors.toList());
        Integer handleUserId = null;
        if(collect.isEmpty()){
            // 这里根据站点id 查询商户管理人
            handleUserId = fadMessageHandleConfDao.findMerchantAdmin(base.getMerchantId());
            if(handleUserId == null)handleUserId = 1;
        }else handleUserId = collect.get(0);
        int handUserId = handleUserId;
        List<String> phoneNoList = siteMessageCache.stream().map(s -> s.getPhoneNo()).distinct().collect(Collectors.toList());
        // 告警人根据设备编号查询的数据
        SafeNode node = new SafeNode(dataHelp.getNodeNo(),dataHelp.getPlugNo());
        error.forEach((k,v)->{
            String serialNo = node.getSerialNo();
            WarnRecord warnRecord = findWarnCache(serialNo, k);
            if (warnRecord == null) {
                // 这里需要处理数据
                WarnRecord res = new WarnRecord();
                res.handleBaseInfo(dataHelp,k);
                res.setWarnType(k);
                res.setRemarks(v);
                res.setSiteId(base.getSiteId());
                res.setMerchantId(base.getMerchantId());
                res.setSiteName(base.getSiteName());
                res.setMerchantName(base.getMerchantName());
                res.setSerialNo(node.getSerialNo());
                res.setHandleUserId(handUserId);
                res.setTypeId(base.getTypeId());
                res.setTypeName(base.getTypeName());
                res.setWarnStatus(0);
                res.setStatus(0);
                res.setCreateTime(System.currentTimeMillis());
                res.setPreWarn(err.fadIsPreWarn(k));
                iWarnRecordDao.insert(res,true);
                updateWarnCache(serialNo,k,res);
                // 这里需要发送告警数据
                sendWarnMessage(res,phoneNoList,findPlugNo(node.getPlugNo())+v);
            }else {
                //warnRecord.handleBaseInfo(dataHelp,k);
                //warnRecord.setWarnType(k);
                //warnRecord.setRemarks(v);
                //warnRecord.setSiteId(base.getSiteId());
                //warnRecord.setMerchantId(base.getMerchantId());
                //warnRecord.setSiteName(base.getSiteName());
                //warnRecord.setMerchantName(base.getMerchantName());
                //warnRecord.setSerialNo(node.getSerialNo());
                //warnRecord.setHandleUserId(handUserId);
                //warnRecord.setTypeId(base.getTypeId());
                //warnRecord.setTypeName(base.getTypeName());
                //warnRecord.setPreWarn(err.fadIsPreWarn(k));
                warnRecord.setUpdateTime(System.currentTimeMillis());
                iWarnRecordDao.updateTemplateById(warnRecord);
                updateWarnCache(serialNo,k,warnRecord);
            }
        });
    }
    private String findPlugNo(int plugNo){
       if(plugNo == -1)
           return "主板";
        return "第"+plugNo+"回路";
    }
    private void sendWarnMessage(WarnRecord res,List<String> list,String content){
        if(list.isEmpty())return;
        log.error("准备发生设备告警消息，告警实体为：{}",res);
        FadMessageConf conf = new FadMessageConf();
        conf.setSiteId(res.getSiteId());
        conf = fadMessageConfDao.templateOne(conf);
        // 有配置文件，并且设置告警消息
        if(conf != null && isSendMsg(conf)){
            log.error("推送告警数据....");
            // 这里说明需要发送消息
            if(conf.getMainSwitch()){
                list.forEach(s->{
                    // 这个告警需要查询数据
                    MessageUtils.sendSafeMessage(s,res.getSiteName(),res.getTerminalNo(),res.getCreateTime(),content);
                });
            }
            // 这里需要根据手机号码，查询已经授权的公众号
            if(conf.getWxSwitch()){
                Map<String,Object> condition = new HashMap<>(4);
                condition.put("phoneList",list);
                List<String> openList = iNewestDataDao.findOpenList(condition);
                if (!openList.isEmpty()) {
                    WarnMessageHelp help = new WarnMessageHelp();//这里推送告警消息
                    help.setTerminalNo(res.getTerminalNo());
                    help.setTerminalType(res.getTypeName());
                    help.setSiteName(res.getSiteName());
                    help.setWarnMsg(content);
                    wxChatUtil.sendWarnMessageToUser(help,openList,null);
                }
            }

        }
    }


    private static String WARN_KEY = "%s-%s";
    // todo 这里还需有个定时器，用来处理清除告警，后面再处理
    private WarnRecord findWarnCache(String serialNo, int type){
        String key = String.format(WARN_KEY,serialNo,type);
        return warnHandleTask.findWarnCache(key);
    }
    @Override
    public void updateWarnCacheStatus(String serialNo, int type,Integer status){
        // 修改缓存状态
        Optional.ofNullable(findWarnCache(serialNo, type)).ifPresent(k->{
            k.setStatus(status);
        });
    }
    private void updateWarnCache(String serialNo,int type,WarnRecord record){
        String key = String.format(WARN_KEY,serialNo,type);
        warnHandleTask.updateWarnCache(key,record);
    }

    private void doHandleUpdateRecord(RepDataRecord res,TerminalSafeInfo base,SafeNode node) {
        try {
            String serialNo = node.getSerialNo();
            Integer id = TERMINAL_KEY_RECORD.get(serialNo);
            TerminalNewestData data = null;
            if(id == null){
                // 这里需要查询下，在判断
                // 根据节点编号查询是否有该条记录
                data = new TerminalNewestData();
                data.setSerialNo(serialNo);
                data = iNewestDataDao.templateOne(data);
                if(data != null){
                    id = data.getId();
                    TERMINAL_KEY_RECORD.put(serialNo,id);
                }
            }
            data = new TerminalNewestData(res);
            // 这里还需要处理一些数据
            data.setSerialNo(node.getSerialNo());
            data.setIsMain(node.isMain());
            data.setTypeId(base.getTypeId());
            data.setTypeName(base.getTypeName());
            // data.setCreateTime(System.currentTimeMillis());
            // 更新数据
            if(id != null){
                data.setId(id);
                iNewestDataDao.updateTemplateById(data);
            }else {
                iNewestDataDao.insert(data,true);
                TERMINAL_KEY_RECORD.put(serialNo,data.getId());
            }
            // 下面处理电量的数据
            handleEleTotal(data,base.getTerminalId());
        }catch (Exception e){
            e.printStackTrace();
            log.error("增加实时数据错误，原因：{}",e.getMessage());
        }

    }
    // 处理电量
    private void handleEleTotal(TerminalNewestData data,Integer terminalId) {
        if(!data.getIsMain()){
            long startTimeOfToday = DateTimeUtils.getStartTimeOfToday();//开始时间
            TerminalEleTotal teraminlEleTotal = SafeCache.findTerminalEleTotal(data.getId(), startTimeOfToday);
            if(teraminlEleTotal == null || teraminlEleTotal.getId() == null){
                TerminalEleTotal eleTotal = new TerminalEleTotal();
                eleTotal.setRepId(data.getId());
                eleTotal.setCreateTime(startTimeOfToday);
                eleTotal = terminalEleTotalDao.templateOne(eleTotal);
                if(eleTotal == null){
                    eleTotal = new TerminalEleTotal();
                    // todo 这里有十分种的偏差
                    eleTotal.setStartEle(data.getSubE());
                    eleTotal.setEndEle(data.getSubE());
                    eleTotal.setCreateTime(startTimeOfToday);
                    eleTotal.setUpdateTime(startTimeOfToday);
                    eleTotal.setDeviceId(data.getDeviceId());
                    eleTotal.setTerminalNo(data.getTerminalNo());
                    eleTotal.setSiteId(data.getSiteId());
                    eleTotal.setRepId(data.getId());
                    eleTotal.setTerminalId(terminalId);
                    eleTotal.setSerialNo(data.getSerialNo());
                    terminalEleTotalDao.insert(eleTotal,true);
                }else {
                    eleTotal.setEndEle(data.getSubE());
                    eleTotal.setUpdateTime(System.currentTimeMillis());
                    terminalEleTotalDao.updateTemplateById(eleTotal);
                }
                SafeCache.addTerminalEleTotal(data.getId(), startTimeOfToday,eleTotal);
                // 查询上一天的数据
                /*teraminlEleTotal = SafeCache.findTerminalEleTotal(data.getId(), startTimeOfToday-DateTimeUtils.ONE_DAY);
                if(teraminlEleTotal != null && teraminlEleTotal.getId() != null){
                    teraminlEleTotal.setEndEle(data.getSubE());
                    terminalEleTotalDao.updateTemplateById(teraminlEleTotal);
                }*/
            }else {
                teraminlEleTotal.setEndEle(data.getSubE());
                teraminlEleTotal.setUpdateTime(System.currentTimeMillis());
                terminalEleTotalDao.updateTemplateById(teraminlEleTotal);
            }
            /*TerminalEleTotal eleTotal = new TerminalEleTotal();
            eleTotal.setRepId(data.getId());
            eleTotal.setCreateTime(startTimeOfToday);
            eleTotal = terminalEleTotalDao.templateOne(eleTotal);
            if(eleTotal == null){
                eleTotal = new TerminalEleTotal();
                // todo 这里有十分种的偏差
                eleTotal.setStartEle(data.getSubE());
                eleTotal.setEndEle(data.getSubE());
                // eleTotal.setTotalEle(0);
                eleTotal.setCreateTime(startTimeOfToday);
                eleTotal.setUpdateTime(startTimeOfToday);
                eleTotal.setUpdateTime(DateTimeUtils.getEndTimeOfToday());//结束时间
                eleTotal.setDeviceId(data.getDeviceId());
                eleTotal.setTerminalNo(data.getTerminalNo());
                terminalEleTotalDao.insert(eleTotal);
            }else {
                eleTotal.setEndEle(data.getSubE());
                eleTotal.setUpdateTime(System.currentTimeMillis());
               *//* if (System.currentTimeMillis() > eleTotal.getUpdateTime()) {
                    eleTotal.setUpdateTime(DateTimeUtils.getEndTimeOfToday());//结束时间
                    eleTotal.setTotalEle(eleTotal.getTotalEle()+eleTotal.getCurEle()-eleTotal.getPastEle());
                    eleTotal.setPastEle(data.getSubE());
                }*//*
                terminalEleTotalDao.updateTemplateById(eleTotal);
            }*/
        }

    }
}
