package com.comtika.safe.service.site.impl;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.comtika.safe.common.utils.JXLUtil;
import com.comtika.safe.entity.site.help.SiteExcelHelp;
import com.comtika.safe.service.warn.impl.WarnServiceImpl;
import lombok.extern.log4j.Log4j2;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.sysuserdata.SysUserDataEnum;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.mserchant.CtkMerchantDao;
import com.comtika.safe.dao.site.SiteDao;
import com.comtika.safe.dao.sys.data.SysUserDataDao;
import com.comtika.safe.dao.sys.role.RoleDao;
import com.comtika.safe.dao.sys.user.UserDao;
import com.comtika.safe.entity.data.SysUserData;
import com.comtika.safe.entity.site.CtkSite;
import com.comtika.safe.service.site.SiteService;

@Service
@Log4j2
public class SiteServiceImpl implements SiteService {

	@Autowired
	private SiteDao siteDao;

	@Autowired
	private SysUserDataDao sysUserDataDao;

	/*@Autowired
	private CtkMerchantDao ctkMerchantDao;

	@Autowired
	private UserDao userDao;

	@Autowired
	private RoleDao roleDao;*/

	@Override
	public RespWrapper<Object> addSite(CtkSite ctkSite,UserCoreCache findUserCache,HttpServletRequest request) {
		try {
			//1.新增站点
			ctkSite.setCreateTime(System.currentTimeMillis());
			ctkSite.setCreateUserId(findUserCache.getUserId());
			ctkSite.setUpdateUserId(findUserCache.getUserId());
			siteDao.insertTemplate(ctkSite, true);
			findUserCache.getSiteData().add(ctkSite.getSiteId());//赋值新增站点的权限
			//将权限放进session中
            request.getSession().setAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN, findUserCache);
			/*//2.默认给商户管理员和市场专员
			CtkMerchant ctkMerchant = new CtkMerchant();
			ctkMerchant.setMerchantId(ctkSite.getMerchantId());
			ctkMerchant = ctkMerchantDao.templateOne(ctkMerchant);
			if( null == ctkMerchant) return RespWrapper.makeFailResp("商户不存在，站点新增失败！");
			//3.查询哪些系统用户拥有该商户的权限
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("dataId", ctkSite.getMerchantId());
			List<Map<String,Object>> listMap = sysUserDataDao.queryMerchantUserData(map);
			if( null == listMap || listMap.isEmpty() || listMap.size() < 1)return RespWrapper.makeSuccessResp("新增成功！");
			boolean a = false;//判断第三步中是否包含市场专员
			SysUserData sysUserData = null;
			List<SysUserData> dataList = new ArrayList<SysUserData>();
			for (Map<String, Object> map2 : listMap) {
				Long userId = Long.parseLong(map2.get("userId") + "");
				Long roleId = Long.parseLong(map2.get("roleId") + "");
				if(  userId.equals(ctkMerchant.getMarketerId()) ){//该集合中包含市场专员
					a  = true;
				}
				sysUserData = new SysUserData();
				sysUserData.setUserId(userId);
				sysUserData.setDataId(ctkSite.getSiteId());
				sysUserData.setDataType(SysUserDataEnum.SITE.getUserType());
				sysUserData.setRoleId(roleId);
				dataList.add(sysUserData);
			}
			//4.查询商户管理员角色id
			SysRole sysrole  = new SysRole();
			sysrole.setRoleName(beanInject.getMerchantRole());
			sysrole = roleDao.templateOne(sysrole);
			if( null == sysrole) return RespWrapper.makeFailResp("商户管理员角色不存在！"); */

			//2.查询哪些用户拥有这个商户的数据权限
			SysUserData sysUserData = new SysUserData();
			sysUserData.setDataId(ctkSite.getMerchantId());
			sysUserData.setDataType(SysUserDataEnum.MERCHANT.getUserType());
			List<SysUserData> list =sysUserDataDao.template(sysUserData);
			//3.查询出的结果会包含该商户管理员和拥有该商户数据权限的人员id
			if( null != list && !list.isEmpty() && list.size() > 0){
				List<SysUserData> dataList = new ArrayList<SysUserData>();
				SysUserData userData = null;
				for (SysUserData data : list) {
					userData = new SysUserData();
					userData.setUserId(data.getUserId());
					userData.setDataId(ctkSite.getSiteId());
					userData.setDataType(SysUserDataEnum.SITE.getUserType());
					userData.setRoleId(data.getRoleId());
					dataList.add(userData);
				}
				sysUserDataDao.insertBatch(dataList);
			}
			return RespWrapper.makeSuccessResp("新增成功！");
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.site.getType(), request, "新增站点", e.getMessage());
			return RespWrapper.makeErrorResp("新增失败，服务器错误！");
		}
	}

	@Override
	public RespWrapper<Object> updateSite(CtkSite ctkSite,UserCoreCache findUserCache,HttpServletRequest request) {
		try {
			if( null == ctkSite || null == ctkSite.getSiteId()){
				return RespWrapper.makeFailResp("参数有误，请检查！");
			}
			ctkSite.setUpdateUserId(findUserCache.getUserId());
			int num = siteDao.updateTemplateById(ctkSite);
			if(num == 1)return RespWrapper.makeSuccessResp("修改成功！");
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.site.getType(), request, "修改站点", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，修改失败！");
		}
		return RespWrapper.makeFailResp("修改失败！");
	}

	@Override
	public RespWrapper<Object> querySiteDetail(Long siteId,HttpServletRequest request) {
		try {
			if( null == siteId) return RespWrapper.makeFailResp("参数有误，站点id不能为空！");
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("siteId", siteId);
			return RespWrapper.makeSuccessResp(siteDao.querySiteDetail(map));
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.site.getType(), request, "站点详情", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}

	}


	@Override
	public RespWrapper<Object> list(HttpServletRequest request,PageQuery<Map<String, Object>> pageQuery) {
		try {
			setParameter(request,pageQuery);//参数赋值
			pageQuery = siteDao.querySiteList(pageQuery);
			return RespWrapper.makeSuccessResp(pageQuery);
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.site.getType(), request, "站点列表", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}

	}

	private void setParameter(HttpServletRequest request,PageQuery<Map<String, Object>> pageQuery) {
		if(!StringUtil.isEmpty(request.getParameter("merchantId"))){
			pageQuery.setPara("merchantId", request.getParameter("merchantId"));
		}
		if(!StringUtil.isEmpty(request.getParameter("siteName"))){
			pageQuery.setPara("siteName", request.getParameter("siteName"));
		}
		if(!StringUtil.isEmpty(request.getParameter("siteType"))){
			pageQuery.setPara("siteType", request.getParameter("siteType"));
		}
		if(!StringUtil.isEmpty(request.getParameter("siteStatus"))){
			pageQuery.setPara("siteStatus", request.getParameter("siteStatus"));
		}
		if(!StringUtil.isEmpty(request.getParameter("area")) && !"[]".equals(request.getParameter("area"))){
			pageQuery.setPara("area", request.getParameter("area"));
		}
		if(!StringUtil.isEmpty(request.getParameter("createTime")) && !StringUtil.isEmpty(request.getParameter("endDate"))){
			pageQuery.setPara("createTime", request.getParameter("createTime"));
			pageQuery.setPara("endDate", request.getParameter("endDate"));
		}
	}

	@Override
	public RespWrapper<Object> querySiteCollectionList(UserCoreCache userCoreCache,HttpServletRequest request) {
		try {
			Map<String,Object> map = new HashMap<String, Object>();
			if (userCoreCache != null && !ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())) {
				map.put("siteData", userCoreCache.getSiteData());
			}
			List<Map<String,Object>> list = siteDao.queryAllSite(map);
			return RespWrapper.makeSuccessResp(list);
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.site.getType(), request, "站点列表,查询下拉框", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	@Override
	public RespWrapper fadFindSiteList(SiteExcelHelp help, UserCoreCache userCache) {
		return RespWrapper.makeSuccessResp(doFindSiteList(help,userCache,false));
	}

	@Override
	public RespWrapper siteListExcel(SiteExcelHelp help, UserCoreCache userCache, HttpServletResponse response) {
		PageQuery<Map<String, Object>> pageQuery = doFindSiteList(help, userCache, true);
		List<String> columnList = new ArrayList<>();
		List<String> excelList = new ArrayList<>();
		addTerminalExcel(columnList,excelList);
		List<Map<String, Object>> list = pageQuery.getList();
		list.forEach(s->{
			// 站点状态、以及区域
			handleSiteStatus(s,"siteStatus");
			handleArea(s,"area");
		});
		String fileName = "站点列表导出";
		try {
			JXLUtil.execlWrite(list,excelList,columnList,fileName,response,fileName);
		}catch (Exception e){
			return RespWrapper.makeFailResp("导出实时数据错误");
		}
		return null;
	}
	// [{"code":"110000","name":"北京市"},{"code":"110100","name":"市辖区"},{"code":"110101","name":"东城区"}]
	public static void handleArea(Map<String, Object> s, String area) {
		Object o = s.get(area);
		if(o != null){
			try {
				JSONArray array = JSONObject.parseArray(o.toString());
				boolean isOne = true;
				StringBuilder sb = new StringBuilder();
				for(Object a:array){
					JSONObject jsonObject = (JSONObject) a;
					if(isOne){
						sb.append(jsonObject.getString("name"));
						isOne = false;
					}else {
						sb.append("-");
						sb.append(jsonObject.getString("name"));
					}
				}
				s.put(area,sb.toString());
			}catch (Exception e){
				log.error("转换地区格式错误,地区为：{},错误原因：{}",o,e.getMessage());
			}
		}
	}

	private void handleSiteStatus(Map<String, Object> s, String key) {
		Object o = s.get(key);
		if(o != null){
			Integer integer = Integer.valueOf(o.toString());
			switch (integer){
				case 1: s.put(key,"调试中");break;
				case 2: s.put(key,"正常运营");break;
				case 3: s.put(key,"发送告警");break;
				case 4: s.put(key,"故障停运");break;
				case 5: s.put(key,"停止运营");break;
				case 6:s.put(key,"运维检修");break;
				default:
					log.error("站点状态错误，状态为:{}",o);
					s.put(key,"正常运营");
			}
		}

	}

	private void addTerminalExcel(List<String> columnList,List<String> excelList){
		columnList.add(JXLUtil.SERIAL_NO);
		excelList.add("序号");
		columnList.add("siteName");
		excelList.add("站点名称");
		columnList.add("siteStatus");
		excelList.add("运营状态");
		columnList.add("area");
		excelList.add("区域");
		columnList.add("createTime");
		excelList.add("投运时间");
		columnList.add("merchantAbbr");
		excelList.add("所属主体");
		columnList.add("siteAddress");
		excelList.add("安装地址");
		columnList.add("contactPersonName");
		excelList.add("站点联系人");
		columnList.add("contactPersonPhone");
		excelList.add("联系人手机");
	}

	private PageQuery<Map<String,Object>> doFindSiteList(SiteExcelHelp help, UserCoreCache userCache,boolean isExcel){
		Map<String,Object> condition = new HashMap<>(8);
		if(!ProjectConstant.ADMIN.equals(userCache.getLoginName())){
			condition.put("siteIds",userCache.getSiteData());
		}
		if(help.getSiteStatus() != null)
			condition.put("siteStatus",help.getSiteStatus());
		if(help.getMerchantId() != null)
			condition.put("merchantId",help.getMerchantId());
		if(StringUtil.isNotEmpty(help.getSiteName()))
			condition.put("siteName",help.getSiteName());
		if(StringUtil.isNotEmpty(help.getArea())) condition.put("area",help.getArea());
		if(help.getStartTime() != null && help.getEndTime() != null){
			condition.put("startTime",help.getStartTime());
			condition.put("endTime",help.getEndTime());
		}
		PageQuery<Map<String,Object>> pageQuery = new PageQuery<>();
		pageQuery.setParas(condition);
		if(isExcel){
			pageQuery.setPageNumber(1);
			pageQuery.setPageSize(WarnServiceImpl.excelMaxNum);
		}else {
			pageQuery.setPageNumber(help.getPageNum());
			pageQuery.setPageSize(help.getPageSize());
		}
		pageQuery.setOrderBy("site_id desc");
		pageQuery = siteDao.fadFindSiteList(pageQuery);
		return pageQuery;
	}

}
