package com.comtika.safe.service.sys.org.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.user.UserTypeEnum;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.mserchant.CtkMerchantDao;
import com.comtika.safe.dao.site.SiteDao;
import com.comtika.safe.dao.sys.user.UserDao;
import com.comtika.safe.service.sys.org.OrgService;

@Service
public class OrgServiceImpl implements OrgService {
	
	@Autowired
	private CtkMerchantDao ctkMerchantDao;
	
	@Autowired
	private SiteDao siteDao;
	
	@Autowired
	private UserDao userDao;
	
	@Override
	public RespWrapper<Object> list(Boolean flag,HttpServletRequest request) {
		try {
			List<Map<String,Object>> list = new ArrayList<Map<String,Object>>();
			orgStructure(list);//构建主节点
			//1.查询所有商户
			List<Map<String,Object>> merchantList = ctkMerchantDao.queryMerchantAbbrList(new HashMap<String, Object>());
			addAllMerchant(list,merchantList);
			//2.查询所有的站点
			List<Map<String,Object>> siteList = siteDao.queryAllSite(new HashMap<String, Object>());
			addAllSite(list,siteList);
			if( null == flag || flag){//组织架构需要查询用户，但是角色数据权限不需要查询用户，所以加了参数flag进行判断
				//3.查询所有用户
				List<Map<String,Object>> userList = userDao.queryAllUser(new HashMap<String, Object>());
				addAllUser(list,userList);
			}
			return RespWrapper.makeSuccessResp(list);
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.org.getType(), request, "组织架构", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	
	/**
	 * 将用户转化成节点所需的参数
	 * @param list
	 * @param userList
	 */
	private void addAllUser(List<Map<String, Object>> list,List<Map<String, Object>> userList) {
		if( null == userList || userList.isEmpty() || userList.size() < 1) return;
		Map<String,Object> map = null;
		for (Map<String, Object> userMap : userList) {
			map = new HashMap<String, Object>();
			Integer userType = userMap.get("userType") == null ? 5 : Integer.parseInt(userMap.get("userType") + "");
			if(userType == UserTypeEnum.SYSTEM_USER.getId()){
				map.put("menuType", 3);
			}else if(userType == UserTypeEnum.MERCHANT_USER.getId()){
				map.put("menuType", 4);
			}else {
				map.put("menuType", 5);
			}
			map.put("parentId", "merchant_" + userMap.get("merchantId"));
			map.put("menuName", userMap.get("userName"));
			map.put("menuId", "user_" + userMap.get("userId"));
			list.add(map);
		}
	}

	/**
	 * 将站点转化成节点所需的参数
	 * @param list
	 * @param siteList
	 */
	private void addAllSite(List<Map<String, Object>> list,List<Map<String, Object>> siteList) {
		if( null == siteList || siteList.isEmpty() || siteList.size() < 1) return;
		Map<String,Object> map = null;
		for (Map<String, Object> siteMap : siteList) {
			map = new HashMap<String, Object>();
			map.put("parentId", "merchant_" + siteMap.get("merchantId"));
			map.put("menuName", siteMap.get("siteName"));
			map.put("menuType", 2);
			map.put("menuId", "site_" + siteMap.get("siteId"));
			list.add(map);
		}
	}


	/**
	 * 将商户转化成节点所需的参数
	 * @param list
	 * @param merchantList
	 */
	private void addAllMerchant(List<Map<String, Object>> list,List<Map<String, Object>> merchantList) {
		if( null == merchantList || merchantList.isEmpty() || merchantList.size() < 1) return;
		Map<String,Object> map = null;
		for (Map<String, Object> merchantMap : merchantList) {
			map = new HashMap<String, Object>();
			map.put("parentId", 0);
			map.put("menuName", merchantMap.get("merchantName")+"(" + merchantMap.get("userName") +")");
			map.put("menuType", 1);
			map.put("menuId", "merchant_" + merchantMap.get("merchantId"));
			list.add(map);
		}
	}
	
	/**
	 * 构建主节点
	 * @param list
	 */
	private void orgStructure(List<Map<String,Object>> list) {
		Map<String,Object> map = new HashMap<String,Object>();
		map.put("menuId", 0);
		map.put("menuName", "安全用电组织架构");
		map.put("menuType", 0);
		map.put("parentId", 0);
		list.add(map);
	}

}
