package com.comtika.safe.service.sys.permission;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.beetl.sql.core.BeetlSQLException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.enums.GeneralStatusEnum;
import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.sys.permission.PermissionDao;
import com.comtika.safe.entity.SysPermission;
import com.comtika.safe.service.BaseServiceImpl;


@Service
public class PermissionServiceImpl extends BaseServiceImpl<SysPermission> implements PermissionService
{

    @Autowired
    PermissionDao permissionDao;

    /**
     * @Title: insertPermission
     * @Description: 新增菜单
     */
    public RespWrapper<Object> insertPermission(SysPermission menu,HttpServletRequest request) {
    	try {
            menu.setStatus(GeneralStatusEnum.ENABLE.getId());
            permissionDao.insert(menu);
            return RespWrapper.makeSuccessResp("新增成功！");
        }catch(BeetlSQLException be) {
        	be.printStackTrace();
            return RespWrapper.makeFailResp("新增失败，权限名称已存在！");
        }catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "新增菜单", e.getMessage());
            return RespWrapper.makeErrorResp("新增失败，服务器错误！");
        }
    }
    
    /**
     * @Title: updatePermission
     * @Description: 修改菜单
     * @param permission
     * void
     * @throws
     */
    public RespWrapper<Object> updatePermission(SysPermission permission,HttpServletRequest request) {
        try {
            int num = permissionDao.updateTemplateById(permission);
            if( num == 1)return RespWrapper.makeSuccessResp("修改成功！");
            return RespWrapper.makeFailResp("修改失败！");
        }catch(BeetlSQLException be) {
        	be.printStackTrace();
        	return RespWrapper.makeFailResp("修改失败，权限名称已存在！");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "修改菜单", e.getMessage());
        	return RespWrapper.makeErrorResp("修改失败，服务器错误！");
        }
    }

    /**
     * @Title: queryList
     * @Description: 获取列表
     * @throws
     */
    public RespWrapper<Object> queryList(Map<String,Object> map,HttpServletRequest request) {
    	try {
            return RespWrapper.makeSuccessResp(permissionDao.queryList(map));
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "获取页面菜单列表", e.getMessage());
            return RespWrapper.makeErrorResp("服务器错误，查询失败!");
        }
    }

    
    

	@Override
	public RespWrapper<Object> pageRole(UserCoreCache userCoreCache,HttpServletRequest request) {
        try {
            Map<String, Object> map = new HashMap<String, Object>();
            boolean adminFlag = false;
            if(ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())) {
                adminFlag = true;
            }
            map.put("userId", userCoreCache.getUserId());
            /*boolean unitFlag = false;
            SysUser user = userService.getByIdHiber(userCoreCache.getUserId());
            if(user != null && UserInitEnum.INIT.getId() == user.getInitStatus()) {
                unitFlag = true;
            }*/
            List<Map<String, Object>> list = findByAdminUserId(map,adminFlag);
            System.out.println();
            return RespWrapper.makeSuccessResp(list);
        } catch (Exception e) {
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "获取当前用户页面权限", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
	}
	
	/**
     * 获取用户权限--同登录
     */
    public List<Map<String,Object>> findByAdminUserId(Map<String, Object> map,boolean adminFlag) {
        if(adminFlag) {
            return permissionDao.findByAdmin();
        }
        return permissionDao.findByUserId(map);
    }

	@Override
	public RespWrapper<Object> enableMenu(String menuId, Integer status,HttpServletRequest request) {
		try {
            JSONArray jsonArray = JSONArray.parseArray(menuId);
            String message = "";
            if (jsonArray != null && jsonArray.size()>0) {
                for (Object uid : jsonArray) {
                    if(uid != null) {
                        SysPermission menu = super.getByIdHiber(Long.parseLong(uid.toString()));
                        if(status == menu.getStatus()) {
                            message += status == 1 ? "【" + menu.getMenuName() + "】已启用的菜单无须重复启用！/n"
                                : "【" + menu.getMenuName() + "】已停用的菜单无须重复停用！/n";
                        }else {
                            menu.setStatus(status);
                            super.updateByHiber(menu);
                        }
                    }
                }
            }
            if(message.length()==0) {
                message = "操作成功！";
            }
            return RespWrapper.makeSuccessResp(message);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "启用/停用菜单权限", e.getMessage());
            return RespWrapper.makeErrorResp("操作失败：" + e.getMessage());
        }
	}

	@Override
	public RespWrapper<Object> menuDetail(long menuId,HttpServletRequest request) {
		try {
            return RespWrapper.makeSuccessResp(super.getByIdUniqueHiber(menuId));
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.menu.getType(), request, "获取菜单权限详情", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
	}
}
