package com.comtika.safe.service.sys.role;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.beetl.sql.core.BeetlSQLException;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.comtika.safe.common.enums.OperatEnum;
import com.comtika.safe.common.enums.RoleStatusEnum;
import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.sysuserdata.SysUserDataEnum;
import com.comtika.safe.common.enums.user.UserTypeEnum;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.sys.UserRoleDao;
import com.comtika.safe.dao.sys.data.SysUserDataDao;
import com.comtika.safe.dao.sys.permission.PermissionDao;
import com.comtika.safe.dao.sys.permission.PermissionRoleDao;
import com.comtika.safe.dao.sys.role.RoleDao;
import com.comtika.safe.dao.sys.user.UserDao;
import com.comtika.safe.entity.SysRolePermission;
import com.comtika.safe.entity.SysRoleUser;
import com.comtika.safe.entity.SysUser;
import com.comtika.safe.entity.data.SysUserData;
import com.comtika.safe.entity.role.ParaSysRole;
import com.comtika.safe.entity.role.SysRole;
import com.comtika.safe.service.BaseServiceImpl;
import com.comtika.safe.service.sys.org.OrgService;

@Service
public class RoleServiceImpl extends BaseServiceImpl<SysRole> implements RoleService
{	
	@Autowired
	private OrgService orgService;
	
    @Autowired
    private RoleDao roleDao;

    @Autowired
    private UserRoleDao userRoleDao;

    @Autowired
    private PermissionRoleDao permissionRoleDao;
    
    @Autowired
    private PermissionDao permissionDao;
    
    @Autowired
    private SysUserDataDao sysUserDataDao;
    
    @Autowired
    private UserDao userDao;
    
    public List<SysRole>  getPubRole() {
        SysRole role = new SysRole();
        role.setRoleName("ROLE_PUB");
        role.setRoleStatus(RoleStatusEnum.ENABLE.getId());
        return roleDao.template(role);
    }

    /**
     * @Title: addRole
     * @param role
     * @param userIds
     * @throws
     */
    public RespWrapper<Object> addRole(ParaSysRole addSysRole,HttpServletRequest request) {
        try {
        	SysRole role = new SysRole();
        	addSysRole.upSysRole(role);
            role.setRoleStatus(RoleStatusEnum.ENABLE.getId());
            role.setLastUpdateTime(System.currentTimeMillis());
            try {
                roleDao.insert(role, true);
            }catch(BeetlSQLException be) {
                throw new BeetlSQLException(5, "角色名称已存在！");
            }
            JSONArray userIds = JSONArray.parseArray(addSysRole.getUserIds());
            updateUserRole(role.getId(),userIds,false);
            return RespWrapper.makeSuccessResp("新增成功！");
        }catch(BeetlSQLException be) {
        	be.printStackTrace();
        	return RespWrapper.makeFailResp("新增失败，角色名称已存在！");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "新增角色", e.getMessage());
        	return RespWrapper.makeErrorResp("服务器错误！");
        }
		
    }

    /**
     * @Title: updateRole
     * @Description: TODO(修改角色)
     * @param role
     * @param userIds
     */
    public RespWrapper<Object> updateRole(ParaSysRole paraSysRole,HttpServletRequest request) {
        try {
        	SysRole role = new SysRole();
        	paraSysRole.upSysRole(role);
        	JSONArray jsonArray = JSONArray.parseArray(paraSysRole.getUserIds());
            role.setLastUpdateTime(System.currentTimeMillis());
            roleDao.updateTemplateById(role);
            updateUserRole(role.getId(),jsonArray,true);
            return RespWrapper.makeSuccessResp("修改成功！");
        }catch(BeetlSQLException be) {
        	be.printStackTrace();
            return RespWrapper.makeFailResp("修改失败，角色名称已存在！");
        }  catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "修改角色", e.getMessage());
        	return RespWrapper.makeErrorResp("修改失败，服务器错误！");
        }
    }
    
    /**
     * 
     * @Title: updateUserRole
     * @Description: TODO(保存角色-用户关系表)
     * @param roleId  角色id
     * @param userIds 用户id集合
     * @param clearFlag 是否清除原有用户-角色信息
     */
    private void updateUserRole(long roleId,JSONArray userIds,boolean clearFlag) {
        try {
            if(clearFlag) {
                //1.清空角色原有用户信息
                Map<String,Object> paraMap = new HashMap<String,Object>();
                paraMap.put("roleId", roleId);
                userRoleDao.deleteByRoleOrUserId(paraMap);
            }
            
            //2.保存新的角色-用户信息
            if (userIds != null && userIds.size()>0) {
                List<SysRoleUser> list = new ArrayList<SysRoleUser>();
                for (Object uid : userIds) {
                    if(uid != null) {
                        SysRoleUser userRole = new SysRoleUser();
                        userRole.setUserId(Long.parseLong(uid.toString()));
                        userRole.setRoleId(roleId);
                        list.add(userRole);
                    }
                }
                userRoleDao.insertBatch(list);
            }
        }catch(Exception e) {
            throw new RuntimeException(e);
        }
    }


    @Override
    public RespWrapper<Object> pageRoleConfig(long roleId, String menuIds,HttpServletRequest request) {
        try {
            // 清空原有配置
            Map<String, Object> paraMap = new HashMap<String, Object>();
            paraMap.put("roleId", roleId);
            permissionRoleDao.deleteByRoleOrPermissionId(paraMap);
            // 增加新的配置
            JSONArray array = JSONArray.parseArray(menuIds);
            List<SysRolePermission> list = new ArrayList<SysRolePermission>();
            if (array != null && array.size() > 0) {
                for (Object obj : array) {
                    if (obj != null && obj.toString().length() > 0) {
                        Long permissionId = Long.parseLong(obj.toString());
                        SysRolePermission permissionRole = new SysRolePermission();
                        permissionRole.setRoleId(roleId);
                        permissionRole.setPermissionId(permissionId);
                        list.add(permissionRole);
                    }
                }
                permissionRoleDao.insertBatch(list);
            }
            return RespWrapper.makeSuccessResp("页面权限配置成功");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "配置角色页面权限", e.getMessage());
        	return RespWrapper.makeFailResp("服务器错误！");
        }
    }
    
    @SuppressWarnings("unchecked")
	@Override
	public RespWrapper<Object> roleDataDetail(Long roleId, Long userId,HttpServletRequest request) {
    	try {
    		if( null == userId || null == roleId){
    			return RespWrapper.makeFailResp("参数有误，请检查！");
    		}
    		Map<String,Object> res = new HashMap<String, Object>();
    		res.put("orgDataList", new ArrayList<Map<String,Object>>());
    		//1.获取组织架构列表，不包含用户
    		List<Map<String,Object>> list = (List<Map<String, Object>>) orgService.list(false,request).getData();
    		res.put("orgList", list);
    		if( null == list || list.isEmpty() || list.size() < 1) return RespWrapper.makeSuccessResp(list);
            //2.查询该用户在该角色下的数据权限
            SysUserData sys = new SysUserData();
            sys.setUserId(userId);
            sys.setRoleId(roleId);
            List<Map<String,Object>> orgDataList = new ArrayList<Map<String,Object>>();
            List<SysUserData> listData = sysUserDataDao.template(sys);
            if( null != listData && !listData.isEmpty() && listData.size() > 0 ){
            	Map<String,Object> data = null;
            	for (SysUserData sysUserData : listData) {
            		data = new HashMap<String, Object>();
					if( sysUserData.getDataType() == SysUserDataEnum.MERCHANT.getUserType() ){
						data.put("parentId", "merchant_" + sysUserData.getDataId());
						data.put("status", 1);
					}
					if( sysUserData.getDataType() == SysUserDataEnum.SITE.getUserType() ){
						data.put("parentId", "site_" + sysUserData.getDataId());
						data.put("status", 1);
					}
					orgDataList.add(data);
				}
            }
            res.put("orgDataList", orgDataList);
            return RespWrapper.makeSuccessResp(res);
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "根据用户id和角色id查询数据权限详情", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
		
	}

    @Override
    public RespWrapper<Object> dataRoleConfig(Long roleId, Long userId, String orgIds,HttpServletRequest request) {
        try {
        	if(null == roleId || null ==userId ){
        		return RespWrapper.makeFailResp("参数有误，请检查！");
        	}
        	boolean flag = false;
        	SysUser sysUser = new SysUser();
        	sysUser.setUserId(userId);
        	sysUser = userDao.templateOne(sysUser);
        	if(sysUser.getUserType() == UserTypeEnum.SITE_USER.getId()) flag = true;//这里判断是否是站点用户
            // 1.清空原有配置
            Map<String, Object> paraMap = new HashMap<String, Object>();
            paraMap.put("roleId", roleId);
            paraMap.put("userId", userId);
            int num = sysUserDataDao.deleteSysUserDataDao(paraMap);
            System.out.println(num);
            // 2.新增配置
            JSONArray array = JSONArray.parseArray(orgIds);
            List<SysUserData> list = new ArrayList<SysUserData>();
            SysUserData sysUserData = null;
            if(null != array && array.size() > 0){
            	for (Object ob : array) {
            		sysUserData = new SysUserData();
            		String str = ob.toString();
					if(str.contains("merchant_")){
						if(flag) continue;  //如果是站点用户，则不需要添加商户权限
						sysUserData.setUserId(userId);
						Long dataId = Long.parseLong(str.replaceAll("merchant_", ""));
						sysUserData.setDataId(dataId);
						sysUserData.setRoleId(roleId);
						sysUserData.setDataType(SysUserDataEnum.MERCHANT.getUserType());
					}else if(str.contains("site_")){
						sysUserData.setUserId(userId);
						Long dataId = Long.parseLong(str.replaceAll("site_", ""));
						sysUserData.setDataId(dataId);
						sysUserData.setRoleId(roleId);
						sysUserData.setDataType(SysUserDataEnum.SITE.getUserType());
					}else{
						continue;
					}
					list.add(sysUserData);
				}
            }
            if(null != list && !list.isEmpty() && list.size() > 0 ){
            	sysUserDataDao.insertBatch(list);
            }
            return RespWrapper.makeSuccessResp("修改成功！");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "角色数据权限配置", e.getMessage());
        	return RespWrapper.makeErrorResp("服务器错误，修改失败！");
        }
		
    }
    
    public static void aa (String[] str){
    	for (String str1 : str) {
			if(str1.contains("merchant_")){
				System.out.println(Long.parseLong(str1.replaceAll("merchant_", "")));
			}else if(str1.contains("site_")){
			}
		}
    }
    
    public static void main(String[] args) {
		String[] a = {"merchant_333","site_298"};
		aa(a);
	}

    @Override
    public RespWrapper<Object> enableRole(String roleId,Integer status,HttpServletRequest request) {
        try {
        	String[] roles = roleId.split(",");
            if (roles != null && roles.length > 0) {
                List<Long> roleIds = new ArrayList<Long>();
                for (String idStr : roles) {
                    if (StringUtil.isNotEmpty(idStr)) {
                        Long roleId1 = Long.parseLong(idStr);
                        roleIds.add(roleId1);
                    }
                }
                if(status == OperatEnum.DISABLE.getId()) {
                    // 清空角色配置的数据权限
                    Map<String, Object> paraMap = new HashMap<String, Object>();
                    paraMap.put("roleIds", roleIds);
                    //orgRoleDao.deleteByRoleIdAndUserId(paraMap);
                }
                
                //更新角色状态
                Map<String,Object> param = new HashMap<String,Object>();
                param.put("roleIds", roleIds);
                param.put("status", status);
                roleDao.updateStatusByIds(param);
            }
            return RespWrapper.makeSuccessResp("操作成功");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "启用/停用角色", e.getMessage());
            return RespWrapper.makeErrorResp("操作失败：" + e.getMessage());
        }
    }

    @Override
    public RespWrapper<Object> getUserRoleAuth(Long userId,HttpServletRequest request) {
    	try {
            if(userId == null ) {
                return RespWrapper.makeFailResp("查询失败，用户id不能为空");
            }
            Map<String, Object> res = new HashMap<String, Object>();
            //1.查询该用户属于哪些角色
            List<Map<String, Object>> roles = roleDao.getRoleListByUserId(userId);
            res.put("roles", roles);
            if (roles != null && roles.size() > 0) {
            	//将该用户的角色列表放进set集合中
                List<Long> roleIds = new ArrayList<Long>();
                for (Map<String, Object> obj : roles) {
                    if (obj.get("id") != null) {
                        roleIds.add(Long.parseLong(obj.get("id").toString()));
                    }
                }
                
                //2.查询用户有哪些页面权限
                if (roleIds != null && roleIds.size() > 0) {
                    List<Map<String, Object>> permissionList = permissionDao.findByRoleIds(roleIds);
                    res.put("permissions", permissionList);
                }
                //3.查询用户商户数据权限
                List<Map<String, Object>> orgs = new ArrayList<Map<String,Object>>();
                List<Map<String, Object>> merchantListData = sysUserDataDao.queryMerchantListData(userId);
                if( null != merchantListData && !merchantListData.isEmpty() && merchantListData.size() > 0)
                	orgs.addAll(merchantListData);
                List<Map<String, Object>> siteListData = sysUserDataDao.querySiteListData(userId);
                if( null != siteListData && !siteListData.isEmpty() && siteListData.size() > 0)
                	orgs.addAll(siteListData);
                res.put("orgs", orgs);
            }
            return RespWrapper.makeSuccessResp(res);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "根据用户id获取其配置的角色权限信息", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
    }

	@Override
	public RespWrapper<Object> rolePageDetail(long roleId, Long userId,HttpServletRequest request) {
		try {
            Map<String,Object> res = new HashMap<String,Object>();
            SysRole role = super.getByIdUniqueHiber(roleId);
            res.put("role", role);
            //查角色下的人员
            Map<String,Object> param = new HashMap<String,Object>();
            param.put("roleId", roleId);
            List<Map<String,Object>> userList = roleDao.getUserListByParam(param);
            res.put("userList", userList);
            
            //查角色下的菜单权限
            List<Map<String,Object>> permissionList = getPermissionListByRoleId(roleId);
            res.put("menuList", permissionList);
            
            return RespWrapper.makeSuccessResp(res);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "获取角色页面权限", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
	}
	
	/**
     * 根据角色id获取其菜单权限
     */
    public List<Map<String, Object>> getPermissionListByRoleId(long roleId) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("roleId", roleId);
        return roleDao.getPermissionListByRoleId(map);
    }
	
	/**
     * 根据参数获取人员列表
     */
    @Override
    public RespWrapper<Object> getUserListByParam(String roleName, String roleId,HttpServletRequest request) {
    	try {
            Map<String,Object> param = new HashMap<String,Object>();
            if(StringUtil.isNotEmpty(roleName)) {
                param.put("roleName", roleName);
            }
            if(StringUtil.isNotEmpty(roleId)) {
                param.put("roleId", roleId);
            }
            //查角色下的人员
            List<Map<String,Object>> userList = roleDao.getUserListByParam(param);
            return RespWrapper.makeSuccessResp(userList);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "根据角色名称、角色id获取角色配置的市场人员列表", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
    }

	@Override
	public RespWrapper<Object> getList(PageQuery<Map<String, Object>> pageQuery, HttpServletRequest request) {
		try {
            // 查询条件
            Map<String, String> paraMap = new HashMap<String, String>();
            if (StringUtil.isNotEmpty(request.getParameter("roleName"))) {
                paraMap.put("roleName", request.getParameter("roleName").trim());
            }
            if (StringUtil.isNotEmpty(request.getParameter("description"))) {
                paraMap.put("description", request.getParameter("description").trim());
            }
            if (StringUtil.isNotEmpty(request.getParameter("status"))) {
                paraMap.put("status", request.getParameter("status").trim());
            }
            pageQuery.setParas(paraMap);
            pageQuery = queryList(pageQuery);
            
            //时间格式转换
            List<Map<String, Object>> userList = pageQuery.getList();
            if(userList != null && userList.size()>0) {
                for(Map<String, Object> detail:userList) {
                    if(detail.get("lastUpdateTime") != null) {
                        String lastUpdateTime = detail.get("lastUpdateTime").toString();
                        if(StringUtil.isNotEmpty(lastUpdateTime)) {
                            long updateTime = Long.parseLong(lastUpdateTime);
                            String updateTimeStr = DateTimeUtils.formatDate(new Date(updateTime), "yyyy-MM-dd HH:mm:ss");
                            detail.put("lastUpdateTime", updateTimeStr);
                        }
                    }
                }
            }
            return RespWrapper.makeSuccessResp(pageQuery);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "角色列表", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
	}
	
	public PageQuery<Map<String, Object>> queryList(PageQuery<Map<String, Object>> pageQuery) {
        return roleDao.queryList(pageQuery);
    }

	@Override
	public RespWrapper<Object> roleDetail(Long roleId,HttpServletRequest request) {
		try {
			Map<String,Object> res = new HashMap<String,Object>();
            SysRole role = super.getByIdUniqueHiber(roleId);
            res.put("role", role);
            //查角色下的人员
            Map<String,Object> param = new HashMap<String,Object>();
            param.put("roleId", roleId);
            List<Map<String,Object>> userList = roleDao.getUserListByParam(param);
            res.put("userList", userList);
            return RespWrapper.makeSuccessResp(res);
		} catch (Exception e) {
			e.printStackTrace();
			//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.role.getType(), request, "角色详情", e.getMessage());
			return RespWrapper.makeErrorResp("服务器错误，查询失败！");
		}
	}

	
}
