package com.comtika.safe.service.sys.user;


import org.beetl.sql.core.BeetlSQLException;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.enums.RegistSourceEnum;
import com.comtika.safe.common.enums.api.ApiErrorType;
import com.comtika.safe.common.enums.sysuserdata.SysUserDataEnum;
import com.comtika.safe.common.enums.user.UserStatusEnum;
import com.comtika.safe.common.exception.UserLoginException;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.encrypt.BASE64Coding;
import com.comtika.safe.common.utils.encrypt.MD5Coding;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.mserchant.CtkMerchantDao;
import com.comtika.safe.dao.sys.data.SysUserDataDao;
import com.comtika.safe.dao.sys.user.UserDao;
import com.comtika.safe.entity.SysUser;
import com.comtika.safe.entity.data.SysUserData;
import com.comtika.safe.entity.log.LoginLog;
import com.comtika.safe.entity.metchant.CtkMerchant;
import com.comtika.safe.entity.oath.UserLoginDTO;
import com.comtika.safe.service.BaseServiceImpl;
import com.comtika.safe.service.sys.role.RoleService;

import java.util.*;

import javax.servlet.http.HttpServletRequest;

@Service
public class UserServiceImpl extends BaseServiceImpl<SysUser> implements UserService{

    @Autowired
    private UserDao userDao;

    @Autowired
    private RoleService roleService;


    @Autowired
    private SysUserDataDao sysUserDataDao;

    @Autowired
    private CtkMerchantDao ctkMerchantDao;

    public RespWrapper<Object> insertUser(SysUser user,Long operatoRuserId,HttpServletRequest request) {
    	 try {
    		 user.setOperator(operatoRuserId);
    		 user.setLoginName(user.getPhoneNo());
    	     if(StringUtil.isEmpty(user.getUserName())) {

    	         user.setUserName(user.getName());
    	     }
    	     String md5TruePwd = MD5Coding.encodeAddSaltCustom(user.getPhoneNo(), MD5Coding.MD5SALT);//将密码md5加密
    	     user.setPassword(md5TruePwd);//默认电话号码为密码，加密
    	     user.setStatus(UserStatusEnum.ENABLE.getId());
    	     user.setRegistTime(System.currentTimeMillis());
    	     user.setRegistSource(RegistSourceEnum.ADD.getId());
    	     userDao.insertTemplate(user, true);
    	     if(user.getUserId() != null ){
    	    	 return RespWrapper.makeSuccessResp("新增成功！");
    	     }
         } catch (BeetlSQLException be) {
         	be.printStackTrace();
         	return RespWrapper.makeFailResp("新增失败,手机号已存在！");
         } catch (Exception e) {
         	e.printStackTrace();
         	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "新增用户", e.getMessage());
            return RespWrapper.makeErrorResp("新增失败：" + e.getMessage());
         }
        return RespWrapper.makeFailResp("新增失败！");
    }

    public RespWrapper<Object> login(String loginName, String password,HttpServletRequest request,String ip) {
    	try {
            if (StringUtil.isEmpty(loginName) || StringUtil.isEmpty(password)) {
                return RespWrapper.makeFailResp("用户名或密码为空！");
            }
            SysUser query = new SysUser();
            query.setLoginName(loginName);
            SysUser user = userDao.templateOne(query);
            if (null == user) return RespWrapper.makeFailResp("该用户不存在！");
            //判断用户是否有效
            if( user.getStatus() != UserStatusEnum.ENABLE.getId() ){
            	if(user.getStatus() == UserStatusEnum.AUDIT.getId()) {
                	return RespWrapper.makeFailResp("用户信息审核中！");
                }else if(user.getStatus() == UserStatusEnum.DISABLE.getId()) {
                	return RespWrapper.makeFailResp("用户已注销！");
                }
            	return RespWrapper.makeFailResp("未知状态，不允许登录！");
            }
            //解密前端传过来的密码
            String truePwd = getTurePwd(password);
            String md5TruePwd = MD5Coding.encodeAddSaltCustom(truePwd, MD5Coding.MD5SALT);//将密码md5加密
            if(!md5TruePwd.equals(user.getPassword())) return RespWrapper.makeFailResp("用户名或密码错误");
            //查询数据权限
            UserCoreCache userCoreCache = queryUserData(user);
            userCoreCache.setLoginName(loginName);
            //将权限放进session中
            request.getSession().setAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN, userCoreCache);

            //将用户的信息返回给前端
            UserLoginDTO userLoginDTO = new UserLoginDTO();
            SysUser loginUser = new SysUser();
            loginUser.setUserName(user.getUserName());
            loginUser.setLoginName(user.getLoginName());
            loginUser.setUserPhoto(user.getUserPhoto());
            loginUser.setUserSex(user.getUserSex());
            loginUser.setUserType(user.getUserType());
            userLoginDTO.setUser(loginUser);
            //查询商户信息
            CtkMerchant ctkMerchant = new CtkMerchant();
            ctkMerchant.setMerchantId(user.getMerchantId());
            ctkMerchant = ctkMerchantDao.templateOne(ctkMerchant);
            userLoginDTO.setDetail(ctkMerchant);

            //修改最后登录时间
            SysUser updateUser = new SysUser();
            updateUser.setUserId(user.getUserId());
            updateUser.setLastLoginTime(System.currentTimeMillis());
            userDao.updateTemplateById(updateUser);
            //写进登录日志
            LuceneUtils.insertLoginLog(new LoginLog(
            		loginName,
            		System.currentTimeMillis(),
            		"后台管理系统",
            		0,
            		ip
            		));
            return RespWrapper.makeSuccessResp(userLoginDTO);
        }catch (Exception e) {
            e.printStackTrace();
            //写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户登录", e.getMessage());
          //写进登录日志
            LuceneUtils.insertLoginLog(new LoginLog(
            		loginName,
            		System.currentTimeMillis(),
            		"后台管理系统",
            		1,
            		ip
            		));
            return RespWrapper.makeErrorResp("登录失败：" + e.getMessage());
        }
    }

    private UserCoreCache queryUserData(SysUser user) {
    	UserCoreCache userCoreCache = new UserCoreCache();
    	Set<Long> merchantData = new HashSet<>();//用户商户权限
    	Set<Long> siteData = new HashSet<>();//用户站点权限
    	SysUserData sysUserData = new SysUserData();
        sysUserData.setUserId(user.getUserId());
        List<SysUserData> list = sysUserDataDao.template(sysUserData);
        for (SysUserData data : list) {
			if(SysUserDataEnum.SITE.getUserType() ==  data.getDataType()){//赋值站点数据权限
				siteData.add(data.getDataId());
			}
			if(SysUserDataEnum.MERCHANT.getUserType() == data.getDataType()){//赋值商户数据权限
				merchantData.add(data.getDataId());
			}
		}
        //数据权限的问题，如果登录的人不是admin，判断set集合中是否有数据，如果没有，默认赋值一个"-100"，防止查询到其他数据
        if(!ProjectConstant.ADMIN.equals(user.getLoginName())){
        	if( merchantData.isEmpty() || merchantData.size() == 0){
        		merchantData.add(-100L);
        	}
        	if( siteData.isEmpty() || siteData.size() == 0){
        		siteData.add(-100L);
        	}
        }
        //创建数据权限对象
        userCoreCache.setSiteData(siteData);
        userCoreCache.setMerchantData(merchantData);
        userCoreCache.setUserId(user.getUserId());
        userCoreCache.setName(user.getName());
        userCoreCache.setLoginName(user.getLoginName());
        userCoreCache.setPhoneNo(user.getPhoneNo());
        userCoreCache.setMerchantId(user.getMerchantId());
		return userCoreCache;
	}

    public static void main(String[] args) {
    	System.out.println(getTurePwd("MTRsdHh0ZXN0MjAyMDAzMjB0YWls"));
	}

	/**
     *
     * @Title: getTurePwd
     * @Description: TODO(根据前端加密密码获取实际密码)
     * @param password
     */
    private static String getTurePwd(String password) {
        try {
            String passwordStr = BASE64Coding.decode(password);
            if(passwordStr.indexOf("l") != -1) {
                int index = passwordStr.indexOf("l");
                String lengthStr = passwordStr.substring(0, index);
                int length = Integer.parseInt(lengthStr);//实际密码长度
                int headLen = lengthStr.length()+1;  //长度占位+分隔符1位
                String pwd = passwordStr.substring(headLen, headLen+length);
                return pwd;
            }else {
                throw new RuntimeException("密码格式错误！");
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public RespWrapper<Object> updateUser(SysUser user,HttpServletRequest request) {
    	try {
    		//查询数据库原有的用户信息
    		SysUser entityData = new SysUser();
    		entityData.setUserId(user.getUserId());
    		entityData = userDao.templateOne(entityData);
    		if( null == entityData ){
    			return RespWrapper.makeFailResp("修改失败,用户不存在！");
    		}
    		if( entityData.getUserType() != user.getUserType()){
    			CtkMerchant ctkMerchant = new CtkMerchant();
        		ctkMerchant.setManagerId(user.getUserId());
        		ctkMerchant = ctkMerchantDao.templateOne(ctkMerchant);
        		if(null != ctkMerchant){
        			return RespWrapper.makeFailResp("修改失败,商户管理员不能修改用户类型！");
        		}
    		}
    		//保存修改
            SysUser entity = new SysUser();
            entity.setUserId(user.getUserId());
            entity.setName(user.getName());
            entity.setPhoneNo(user.getPhoneNo());
            //entity.setOrgId(user.getOrgId());
            entity.setEmail(user.getEmail());
            entity.setUserSex(user.getUserSex());
            entity.setMerchantId(user.getMerchantId());
            entity.setIdCard(user.getIdCard());
            if(StringUtil.isEmpty(user.getUserName())) {//用户名为空时，填充姓名
                entity.setUserName(user.getName());
            }else {
                entity.setUserName(user.getUserName());
            }
            entity.setUserType(user.getUserType());
            entity.setUserCode(user.getUserCode());
            int num = userDao.updateTemplateById(entity);//只修改需要修改的列
            if(num == 1){
            	return RespWrapper.makeSuccessResp("修改成功！");
            }
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "修改用户", e.getMessage());
            return RespWrapper.makeErrorResp("修改失败：" + e.getMessage());
        }

		return RespWrapper.makeFailResp("修改失败");
    }

    public RespWrapper<Object> queryList(PageQuery<Map<String, Object>> pageQuery,HttpServletRequest request) {
    	try {
            // 查询条件
            Map<String, String> paraMap = new HashMap<String, String>();
            if (StringUtil.isNotEmpty(request.getParameter("name"))) {
                paraMap.put("name", request.getParameter("name").trim());
            }
            if (StringUtil.isNotEmpty(request.getParameter("loginName"))) {
                paraMap.put("loginName", request.getParameter("loginName").trim());
            }
            if (StringUtil.isNotEmpty(request.getParameter("merchantName"))) {
                paraMap.put("merchantName", request.getParameter("merchantName").trim());
            }
            if (StringUtil.isNotEmpty(request.getParameter("status"))) {
                paraMap.put("status", request.getParameter("status").trim());
            }
            pageQuery.setParas(paraMap);

            pageQuery = userDao.queryList(pageQuery);

            //时间格式转换
            List<Map<String, Object>> userList = pageQuery.getList();
            if (userList != null && userList.size() > 0) {
                for (Map<String, Object> detail : userList) {
                    if (detail.get("lastLoginTime") != null) {
                        String lastLoginTime = detail.get("lastLoginTime").toString();
                        if (StringUtil.isNotEmpty(lastLoginTime)) {
                            long loginTime = Long.parseLong(lastLoginTime);
                            String loginTimeStr = DateTimeUtils.formatDate(new Date(loginTime), "yyyy-MM-dd HH:mm:ss");
                            detail.put("lastLoginTime", loginTimeStr);
                        }
                    }
                }
            }

            return RespWrapper.makeSuccessResp(pageQuery);
        } catch (Exception e) {
            e.printStackTrace();
          //写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户列表", e.getMessage());
            return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
        }
    }


    public RespWrapper<Object> getUserDetailById(Long userId,HttpServletRequest request) {
    	 try {
    		 Map<String,Object> map = new HashMap<String,Object>();
    	     map.put("userId", userId);
             Map<String, Object> detail = userDao.getUserDetailById(map);
             if (detail.get("lastLoginTime") != null) {
                 String lastLoginTime = detail.get("lastLoginTime").toString();
                 if (StringUtil.isNotEmpty(lastLoginTime)) {
                     long loginTime = Long.parseLong(lastLoginTime);
                     String loginTimeStr = DateTimeUtils.formatDate(new Date(loginTime), "yyyy-MM-dd HH:mm:ss");
                     detail.put("lastLoginTime", loginTimeStr);
                 }
             }
             return RespWrapper.makeSuccessResp(detail);
         } catch (Exception e) {
        	//写进异常日志
             LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户详情", e.getMessage());
         	 e.printStackTrace();
             return RespWrapper.makeErrorResp("查询失败：" + e.getMessage());
         }

    }

    @Override
    public RespWrapper<Object> logout(HttpServletRequest request) {
        try {
        	//将用户的数据权限清空
        	request.getSession().removeAttribute(ProjectConstant.COMTIKA_SAFE_TOKEN);
        	return RespWrapper.makeSuccessResp("注销成功！");
        } catch (Exception e) {
            e.printStackTrace();
            //写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户注销", e.getMessage());
            return RespWrapper.makeFailResp("注销失败！");
        }

    }

    @Override
    public RespWrapper<Object> isUniquePhoneNo(HttpServletRequest request) {
    	 try {
             String phoneNo = request.getParameter("phoneNo");
             if(StringUtil.isEmpty(phoneNo))return RespWrapper.makeFailResp("手机号码不能为空");
             SysUser query = new SysUser();
             query.setPhoneNo(phoneNo);
             long count = userDao.templateCount(query);
             boolean flag = (count > 0)  ? false : true;
             return RespWrapper.makeSuccessResp(flag);
         } catch (Exception e) {
         	e.printStackTrace();
         	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户手机号码唯一性检查", e.getMessage());
            return RespWrapper.makeErrorResp("服务器错误，查询失败！");
         }

    }


	@Override
	public RespWrapper<Object> enableUser(Long userId, Integer status,HttpServletRequest request) {
		try {
			//前端只能传启用或者停用的状态
			if(null == userId || null == status || !( status == UserStatusEnum.ENABLE.getId() || status == UserStatusEnum.DISABLE.getId()))
				return RespWrapper.makeFailResp("参数有误，请检查！");
			SysUser user = new SysUser();
			user.setUserId(userId);
			user = userDao.templateOne(user);
			if(null == user)return RespWrapper.makeFailResp("用户不存在，请检查！");
            SysUser userData = new SysUser();
            userData.setUserId(userId);
            userData.setStatus(status);
            int num = userDao.updateTemplateById(userData);
            if( num == 1)return RespWrapper.makeSuccessResp("操作成功！");
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户启用/停用", e.getMessage());
            return RespWrapper.makeErrorResp("操作失败！" );
        }
		 return RespWrapper.makeFailResp("操作失败！" );
	}

	@Override
	public RespWrapper<Object> updateUserByEntity(SysUser user,HttpServletRequest request) {
		try {
			int num = userDao.updateTemplateById(user);
            if( num == 1 )return RespWrapper.makeSuccessResp(user.getUserId());
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户修改", e.getMessage());
            return RespWrapper.makeErrorResp("服务器错误，修改用户失败！");
        }
		return RespWrapper.makeFailResp("修改失败！");
	}

	@Override
	public RespWrapper<Object> resetPassword(Long userId, String password,HttpServletRequest request) {
		try {
            SysUser user = super.getByIdHiber(userId);
            String encrypPassword = MD5Coding.encodeAddSaltCustom(password, MD5Coding.MD5SALT);//将密码md5加密
            user.setPassword(encrypPassword);
            super.updateByHiber(user);
            return RespWrapper.makeSuccessResp(userId);
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户重置密码", e.getMessage());
            return RespWrapper.makeErrorResp("重置密码失败：" + e.getMessage());
        }
	}


	@Override
	public RespWrapper<Object> updatePassword(String oldPassword,String newPassword, UserCoreCache userCoreCache
			,HttpServletRequest request) {
        try {
            SysUser user = super.getByIdHiber(userCoreCache.getUserId());
            String password = MD5Coding.encodeAddSaltCustom(oldPassword, MD5Coding.MD5SALT);//将密码md5加密
            if (password.equals(user.getPassword())) {
                //旧密码校验通过后，设置新的密码
                String encrypPassword = MD5Coding.encodeAddSaltCustom(newPassword, MD5Coding.MD5SALT);//将密码md5加密
                user.setPassword(encrypPassword);
                super.updateByHiber(user);
                return RespWrapper.makeSuccessResp(user.getUserId());
            } else {
                return RespWrapper.makeFailResp("旧密码输入有误，请重新输入！");
            }
        } catch (Exception e) {
        	e.printStackTrace();
        	//写进异常日志
            LuceneUtils.apiErrorLucene(ApiErrorType.user.getType(), request, "用户修改密码", e.getMessage());
            return RespWrapper.makeErrorResp("服务器错误，重置密码失败!");
        }
	}

	@Override
	public RespWrapper<Object> queryUserPersonalInformation(HttpServletRequest request, UserCoreCache userCoreCache) {
		try {
			Long userId = userCoreCache.getUserId();
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("userId", userId);
			map = userDao.queryUserPersonalInformation(map);
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请稍后重试！");
		}
	}



}
