package com.comtika.safe.service.task.impl;

import com.alibaba.fastjson.JSONObject;
import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.JXLUtil;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.protocol.entity.safe.BitHelp;
import com.comtika.safe.core.protocol.entity.safe.SafeTaskHelp;
import com.comtika.safe.core.protocol.entity.safe.TerminalSafeInfo;
import com.comtika.safe.dao.task.ITaskTemplateDao;
import com.comtika.safe.dao.task.TerminalTaskDao;
import com.comtika.safe.entity.task.TaskTemplate;
import com.comtika.safe.entity.task.TerminalTask;
import com.comtika.safe.entity.task.help.*;
import com.comtika.safe.service.protocol.ProtocolService;
import com.comtika.safe.service.safe.IRepService;
import com.comtika.safe.service.task.ITaskService;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Service
public class TaskServiceImpl implements ITaskService {
    @Autowired
    private TerminalTaskDao terminalTaskDao;
    @Autowired
    private IRepService iRepService;
    @Autowired
    private ProtocolService protocolService;

    @Autowired
    private ITaskTemplateDao iTaskTemplateDao;

    @Override
    public RespWrapper<PageQuery<Map<String, Object>>> findTaskPageList(TaskQueryHelp help, UserCoreCache userCache) {
        return doFindTaskPageList(help, userCache, false);
    }

    private RespWrapper<PageQuery<Map<String, Object>>> doFindTaskPageList(TaskQueryHelp help, UserCoreCache userCache, boolean isExcel) {
        Map<String, Object> condition = new HashMap<>(8);
        if (!ProjectConstant.ADMIN.equals(userCache.getLoginName())) {
            condition.put("siteIds", userCache.getSiteData());
        }
        if (help.getMerchantId() != null)
            condition.put("merchantId", help.getMerchantId());
        if (help.getSiteId() != null)
            condition.put("siteId", help.getSiteId());
        if (StringUtil.isNotEmpty(help.getTerminalNo()))
            condition.put("terminalNo", help.getTerminalNo());
        if (help.getTaskModel() != null)
            condition.put("taskModel", help.getTaskModel());
        PageQuery<Map<String, Object>> pageQuery = new PageQuery<>();
        if (isExcel) {
            pageQuery.setPageNumber(1);
            pageQuery.setPageSize(50000);
        } else {
            pageQuery.setPageNumber(help.getPageNum());
            pageQuery.setPageSize(help.getPageSize());
        }
        pageQuery.setParas(condition);
        pageQuery.setOrderBy("id desc");
        pageQuery = terminalTaskDao.findTaskPageList(pageQuery);
        return RespWrapper.makeSuccessResp(pageQuery);
    }

    @Override
    public RespWrapper<Boolean> addTaskHandle(TaskHelp help) {
        // 这里新增task
        TerminalSafeInfo base = iRepService.findTerminalBaseInfo(help.getTerminalNo());
        if (base == null) return RespWrapper.makeFailResp("获取当前设备商户/站点数据错误");
        // 这里根据设备编号查询数据
        Integer taskNumber = terminalTaskDao.findMaxTaskNumber(help.getTerminalNo(), base.getMerchantId());
        TerminalTask terminalTask = new TerminalTask();
        terminalTask.fadHandleData(help);
        terminalTask.setStatus(0);
        terminalTask.setMerchantId(base.getMerchantId());
        terminalTask.setSiteId(base.getSiteId());
        if (taskNumber == null || taskNumber < 250) {
            // 这里说明可以直接加任务
            if (taskNumber == null) taskNumber = 0;
            terminalTask.setTaskNumber(taskNumber + 1);
            terminalTaskDao.insert(terminalTask);
        } else {
            // 这里查询一个删除的数据
            TerminalTask task = terminalTaskDao.findDeleteData(help.getTerminalNo(), base.getMerchantId());
            if (task == null)
                return RespWrapper.makeFailResp("该设备定时任务已经满了，无法添加其他定时任务，请先删除或修改其他定时任务");
            terminalTask.setId(task.getId());
            terminalTaskDao.updateTemplateById(terminalTask);
        }
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> updateTaskHandle(TaskHelp help) {
        if (help.getId() == null) return RespWrapper.makeFailResp("唯一id 错误");
        TerminalTask single = terminalTaskDao.single(help.getId());
        if (single == null) return RespWrapper.makeFailResp("定时任务唯一id 错误");
        if (single.getDeleteStatus()) return RespWrapper.makeFailResp("该定时任务已经被删除无法修改");
        single.fadHandleData(help);
        // 如果不是未下发，则改为已更改
        if (single.getStatus() != 0)
            single.setStatus(2);
        terminalTaskDao.updateTemplateById(single);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<Boolean> deleteTask(Integer id) {
        TerminalTask single = terminalTaskDao.single(id);
        if (single == null) return RespWrapper.makeFailResp("定时任务唯一id 错误");
        if (single.getStatus() == 0) {//未下发状态
            // 自己删除任务
            single.setDeleteStatus(true);
            single.setUpdateTime(System.currentTimeMillis());
            terminalTaskDao.updateTemplateById(single);
            return RespWrapper.makeSuccessResp(true);
        }
        // 这里需要下发定时任务
        RespWrapper<Boolean> ck = protocolService.deleteSafeTask(single.getTerminalNo(), single.getTaskNumber());
        if (ck.isSuccess()) {
            single.setDeleteStatus(true);
            single.setUpdateTime(System.currentTimeMillis());
            terminalTaskDao.updateTemplateById(single);
            return RespWrapper.makeSuccessResp(true);
        }
        return RespWrapper.makeFailResp(ck.getMsg());
    }

    @Override
    public RespWrapper<TaskHelp> findTaskInfo(Integer id) {
        TerminalTask single = terminalTaskDao.single(id);
        if (single == null) return RespWrapper.makeFailResp("定时任务唯一id 错误");
        return RespWrapper.makeSuccessResp(single.convertToTaskHelp());
    }

    @Override
    public RespWrapper<Boolean> takeTask(Integer id) {
        TerminalTask single = terminalTaskDao.single(id);
        if (single == null) return RespWrapper.makeFailResp("定时任务唯一id 错误");
        List<SafeTaskHelp> list = new ArrayList<>(4);
        SafeTaskHelp help = new SafeTaskHelp();
        // help.setBit();
        help.setNumber(single.getTaskNumber());
        help.setContent(single.getContent());
        help.setTaskModel(single.getTaskModel());
        help.setLoop(single.getWeekTime() == null ? 0 : single.getWeekTime());
        String s = DateTimeUtils.formatDate(new Date(single.getStartTime()), DateTimeUtils.FORMAT_YMD);
        String[] split = s.split("-");
        help.setStartYear(split[0]);
        help.setStartMonth(Integer.valueOf(split[1]));
        help.setStartDay(Integer.valueOf(split[2]));
        String end = DateTimeUtils.formatDate(new Date(single.getEndTime()), DateTimeUtils.FORMAT_YMD);
        String[] endP = end.split("-");
        help.setEndYear(endP[0]);
        help.setEndMonth(Integer.valueOf(endP[1]));
        help.setEndDay(Integer.valueOf(endP[2]));

        String taskTime = single.getTaskTime();
        String[] _taskTime = taskTime.split(":");
        help.setTaskTime(Integer.valueOf(_taskTime[0]));
        help.setTaskMinute(Integer.valueOf(_taskTime[1]));

        BitHelp bit = new BitHelp();
        List<Integer> integers = JSONObject.parseArray(single.getBit(), Integer.class);
        bit.convertIntegerList(integers);
        help.setBit(bit);
        list.add(help);
        RespWrapper<Boolean> ck = protocolService.setSafeTaskConf(single.getTerminalNo(), list);
        if (ck.isSuccess()) {
            single.setStatus(1);
            single.setUpdateTime(System.currentTimeMillis());
            terminalTaskDao.updateTemplateById(single);
            return RespWrapper.makeSuccessResp(true);
        }
        return RespWrapper.makeFailResp(ck.getMsg());
    }

    @Override
    public RespWrapper<Boolean> excelTaskList(TaskQueryHelp task, UserCoreCache userCache, HttpServletResponse response) {
        RespWrapper<PageQuery<Map<String, Object>>> pk = doFindTaskPageList(task, userCache, true);
        if (!pk.isSuccess()) return RespWrapper.makeFailResp(pk.getMsg());
        List<Map<String, Object>> list = pk.getData().getList();
        List<String> excelList = new ArrayList<>();
        List<String> columnList = new ArrayList<>();
        doHandleTaskExcel(excelList, columnList);
        list.forEach(s -> {
            timeConvert(s, "startTime");
            timeConvert(s, "endTime");
            contentConvert(s, "content");
            statusConvert(s, "status");
            taskModelConvert(s, "taskModel", "weekTime");
        });
        String fileName = "定时任务导出";
        try {
            JXLUtil.execlWrite(list, excelList, columnList, fileName, response, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            return RespWrapper.makeFailResp("定时任务导出失败");
        }
        return null;
    }

    @Override
    public RespWrapper findTaskTemplateList(TaskTemplateQueryHelp help, UserCoreCache userCoreCache) {
        Map<String, Object> condition = new HashMap<>(8);
       /* if(!ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())){
            condition.put("merchantIds",userCoreCache.getMerchantData());
        }*/
        condition.put("userId", userCoreCache.getUserId());
        if (help.getMerchantId() != null) condition.put("merchantId", help.getMerchantId());
        if (help.getTaskModel() != null) condition.put("taskModel", help.getTaskModel());
        if (help.getTerminalType() != null) condition.put("terminalType", help.getTerminalType());
        if(StringUtil.isNotEmpty(help.getName())) condition.put("name",help.getName());
        PageQuery<Map<String, Object>> pageQuery = new PageQuery<>();
        pageQuery.setParas(condition);
        pageQuery.setPageNumber(help.getPageNum());
        pageQuery.setPageSize(help.getPageSize());
        pageQuery.setOrderBy("id desc");
        pageQuery = terminalTaskDao.findTaskTemplateList(pageQuery);
        return RespWrapper.makeSuccessResp(pageQuery);
    }

    @Override
    public RespWrapper addTaskTemplate(TaskTemplateHelp help, UserCoreCache userCache) {
        // 校验模版是否超标
        RespWrapper<Boolean> ck = checkUserTemplateTask(userCache);
        if(!ck.isSuccess()) return ck;

        TaskTemplate taskTemplate = new TaskTemplate();
        taskTemplate.fadHandleData(help);
        taskTemplate.setMerchantId(userCache.getMerchantId().intValue());
        taskTemplate.setCreateTime(System.currentTimeMillis());
        taskTemplate.setUserId(userCache.getUserId().intValue());
        iTaskTemplateDao.insert(taskTemplate);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper updateTaskTemplate(TaskTemplateHelp help, UserCoreCache userCache) {
        TaskTemplate single = iTaskTemplateDao.single(help.getId());
        if (single == null) return RespWrapper.makeFailResp("模版唯一id 错误");
        if (single.getUserId().intValue() != userCache.getUserId()) {
            return RespWrapper.makeFailResp("不属于一商户你没有权限修改");
        }
        single.fadHandleData(help);
        iTaskTemplateDao.updateById(single);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper deleteTaskTemplate(Integer id, UserCoreCache userCache) {
        TaskTemplate single = iTaskTemplateDao.single(id);
        if (single == null) return RespWrapper.makeFailResp("模版唯一id 错误");
        if (single.getUserId().intValue() != userCache.getUserId()) {
            return RespWrapper.makeFailResp("不属于一商户你没有权限修改");
        }
        iTaskTemplateDao.deleteById(id);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper findTaskTemplate(String terminalNo, UserCoreCache userCache) {
        // 这里需要根据设备编号，查询是什么类型
        Integer terminalType = iTaskTemplateDao.findTerminalTypeByNo(terminalNo);
        if (terminalType == null) return RespWrapper.makeFailResp("设备编码错误");
        TaskTemplate template = new TaskTemplate();
        template.setTerminalType(terminalType);
        template.setMerchantId(userCache.getMerchantId().intValue());
        List<TaskTemplate> list = iTaskTemplateDao.template(template);
        List<TaskTemplateHelp> res = new ArrayList<>(list.size());
        list.forEach(s -> {
            res.add(s.convertToTaskHelp());
        });
        return RespWrapper.makeSuccessResp(res);
    }

    @Override
    public RespWrapper configTaskTemplate(AddTaskByTemplateHelp help, UserCoreCache userCache) {
        if (help.getTemplateId() == null) return RespWrapper.makeFailResp("模板唯一id 不能为空");
        if (StringUtil.isEmpty(help.getTerminalNo())) return RespWrapper.makeFailResp("设备编号不能为空");
        TaskTemplate single = iTaskTemplateDao.single(help.getTemplateId());
        if (single == null) return RespWrapper.makeFailResp("模版唯一id 错误");
        if (single.getUserId() != userCache.getUserId().intValue()) {
            return RespWrapper.makeFailResp("不属于同一个建的模板，无法配置");
        }
        TaskHelp res = single.convertTask();
        res.setTerminalNo(help.getTerminalNo());
        res.setBitList(help.getBitList());
        return addTaskHandle(res);
    }

    @Override
    public RespWrapper findTaskTemplateByName(String name, UserCoreCache userCache) {
        Map<String, Object> condition = new HashMap<>(8);
       /* if(!ProjectConstant.ADMIN.equals(userCoreCache.getLoginName())){
            condition.put("merchantIds",userCoreCache.getMerchantData());
        }*/
        condition.put("userId", userCache.getUserId());
        if(StringUtil.isNotEmpty(name))
            condition.put("name",name);
        List<TaskTemplate> list = terminalTaskDao.findTaskTemplateListByTemplateName(condition);
        List<TaskTemplateHelp> res = new ArrayList<>(list.size());
        list.forEach(s -> {
            res.add(s.convertToTaskHelp());
        });
        return RespWrapper.makeSuccessResp(res);
    }

    @Override
    public RespWrapper<Boolean> checkUserTemplateTask(UserCoreCache userCache) {
        long userTaskCount = findUserTaskCount(userCache.getUserId().intValue());
        if(userTaskCount >= 60) return RespWrapper.makeResp(2,"定时任务模版超过60个，请先删除后再添加",null);
        return RespWrapper.makeSuccessResp(true);
    }

    private long findUserTaskCount(Integer userId){
       if(userId == null) return 0;
        TaskTemplate task = new TaskTemplate();
        task.setUserId(userId);
        return iTaskTemplateDao.templateCount(task);
    }


    private void taskModelConvert(Map<String, Object> s, String model, String wk) {
        Object o = s.get(model);
        if (o != null) {
            Integer integer = Integer.valueOf(o.toString());
            switch (integer) {
                case 2:
                    s.put(model, "每天执行");
                    break;
                case 3:
                    s.put(model, findWeek(s.get(wk)));
                    break;
                default:
                    s.put(model, "单次执行");
            }
        }
    }

    private String findWeek(Object ob) {
        if (ob != null) {
            Integer integer = Integer.valueOf(ob.toString());
            switch (integer) {
                case 1:
                    return "每周一执行";
                case 2:
                    return "每周二执行";
                case 3:
                    return "每周三执行";
                case 4:
                    return "每周四执行";
                case 5:
                    return "每周五执行";
                case 6:
                    return "每周六执行";
            }
        }
        return "每周天执行";
    }

    private void statusConvert(Map<String, Object> s, String key) {
        Object o = s.get(key);
        if (o != null) {
            Integer integer = Integer.valueOf(o.toString());
            if (integer == 0)
                s.put(key, "未下发");
            else if (integer == 1) s.put(key, "下发成功");
            else s.put(key, "已更改");
        }
    }

    private void contentConvert(Map<String, Object> s, String key) {
        Object o = s.get(key);
        if (o != null) {
            Integer integer = Integer.valueOf(o.toString());
            if (integer == 1)
                s.put(key, "打开继电器");
            else s.put(key, "关闭继电器");
        }
    }

    private void timeConvert(Map<String, Object> s, String key) {
        Object o = s.get(key);
        if (o != null) {
            Long aLong = Long.valueOf(o.toString());
            s.put(key, DateTimeUtils.formatDate(new Date(aLong), DateTimeUtils.FORMAT_YMD));
        }
    }

    private void doHandleTaskExcel(List<String> excelList, List<String> columnList) {
        excelList.add("序号");
        columnList.add(JXLUtil.SERIAL_NO);
        excelList.add("商户名称");
        columnList.add("merchantAbbr");
        excelList.add("站点名称");
        columnList.add("siteName");
        excelList.add("设备ID");
        columnList.add("terminalNo");
        excelList.add("任务号");
        columnList.add("taskNumber");
        excelList.add("执行方式");
        columnList.add("taskModel");
        excelList.add("执行时间");
        columnList.add("taskTime");
        excelList.add("执行内容");
        columnList.add("content");
        excelList.add("开始时间");
        columnList.add("startTime");
        excelList.add("结束时间");
        columnList.add("endTime");
        excelList.add("状态");
        columnList.add("status");
        excelList.add("执行回路");
        columnList.add("bit");
    }

}
