package com.comtika.safe.service.upload.impl;

import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.config.ProjectConfig;
import com.comtika.safe.core.VersionUtil;
import com.comtika.safe.dao.upload.IUploadDao;
import com.comtika.safe.entity.upload.UploadRecord;
import com.comtika.safe.service.upload.IUploadService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.UUID;

@Service
@Log4j2
public class UploadServiceImpl implements IUploadService {
    @Autowired
    private ProjectConfig projectConfig;

    @Autowired
    private IUploadDao iUploadDao;


    // 上传模块
    @Override
    public RespWrapper<Object> upload(MultipartFile file, Integer id) {
        FileOutputStream fileOutputStream = null;
        try {
            if(id == null)
                return RespWrapper.makeFailResp("上传类型不能为空");
            if(id == 4){
                RespWrapper<String> suffix = VersionUtil.findSuffix(file.getOriginalFilename());
                if(!suffix.isSuccess())
                    return RespWrapper.makeFailResp(suffix.getMsg());
                String fileSuffix = suffix.getData();//文件后缀
                String fileName = createFileName(fileSuffix);//文件名
                UploadRecord record = new UploadRecord();
                record.setCreateTime(System.currentTimeMillis());
                record.setOrgFileName(file.getOriginalFilename());
                String time = DateTimeUtils.formatDate(new Date(), DateTimeUtils.FORMAT_YMD);
                StringBuilder sb = new StringBuilder("images/"+time);
                File file1 = new File("/data/" + sb.toString());
                if(!file1.exists())file1.mkdirs();
                sb.append("/");
                sb.append(fileName);
                // 这里上传文件
                fileOutputStream = new FileOutputStream(new File("/data/" + sb.toString()));
                FileCopyUtils.copy(file.getInputStream(),fileOutputStream );
                // 这里准备增加访问数据
                String baseUrl = projectConfig.getUploadUrl()+sb.toString();
                record.setBaseUrl(baseUrl);
                iUploadDao.insert(record);
                return RespWrapper.makeSuccessResp(baseUrl);
            }else if(id == 1){
                String originalFilename = file.getOriginalFilename();
                if(VersionUtil.checkVersion(originalFilename))
                    return RespWrapper.makeFailResp("请检查版本号");
                UploadRecord record = new UploadRecord();
                record.setCreateTime(System.currentTimeMillis());
                record.setOrgFileName(file.getOriginalFilename());
                String time = DateTimeUtils.formatDate(new Date(), DateTimeUtils.FORMAT_YMD);
                StringBuilder sb = new StringBuilder("ota/"+time);
                File file1 = new File("/data/" + sb.toString());
                if(!file1.exists())file1.mkdirs();
                sb.append("/");
                sb.append(originalFilename);
                fileOutputStream = new FileOutputStream(new File("/data/" + sb.toString()));
                // 这里上传文件
                FileCopyUtils.copy(file.getInputStream(), fileOutputStream);
                String baseUrl = projectConfig.getUploadUrl()+sb.toString();
                record.setBaseUrl(baseUrl);
                iUploadDao.insert(record,true);
                return RespWrapper.makeSuccessResp(record.getId());
            }
        }catch (Exception e){
            log.error("上传报错，请检查：{}",e.getMessage());
            return RespWrapper.makeFailResp("上传错误：{}",e.getMessage());
        }finally {
            if(fileOutputStream != null){
                try {
                    fileOutputStream.close();
                }catch (Exception e){

                }
            }
        }
        return RespWrapper.makeFailResp("暂不支持该类型上传");
    }


    // 文件名称生产方法
    private String createFileName(String suffix){
        return UUID.randomUUID().toString().replaceAll("-","").concat(suffix);
    }
}
