package com.comtika.safe.service.warn.impl;

import com.comtika.safe.common.constant.ProjectConstant;
import com.comtika.safe.common.response.RespWrapper;
import com.comtika.safe.common.utils.DateTimeUtils;
import com.comtika.safe.common.utils.JXLUtil;
import com.comtika.safe.common.utils.StringUtil;
import com.comtika.safe.common.utils.TimeUtil;
import com.comtika.safe.core.UserCoreCache;
import com.comtika.safe.core.lucene.LuceneQueryUtils;
import com.comtika.safe.core.lucene.LuceneUtils;
import com.comtika.safe.dao.safe.INewestDataDao;
import com.comtika.safe.dao.safe.IWarnRecordDao;
import com.comtika.safe.dao.sys.IExcelFieldDao;
import com.comtika.safe.entity.lucene.LuceneReturnEntity;
import com.comtika.safe.entity.report.AlwayQueryHelp;
import com.comtika.safe.entity.report.NewestQueryHelp;
import com.comtika.safe.entity.report.WarnQueryHelp;
import com.comtika.safe.entity.sys.SysExcelField;
import com.comtika.safe.entity.warn.WarnRecord;
import com.comtika.safe.entity.warn.help.WarnHelp;
import com.comtika.safe.entity.warn.help.WarnHomeHelp;
import com.comtika.safe.entity.warn.help.WarnMonthHelp;
import com.comtika.safe.service.protocol.impl.ProtocolSeriveImpl;
import com.comtika.safe.service.safe.IRepService;
import com.comtika.safe.service.warn.IWarnService;
import lombok.extern.log4j.Log4j2;
import org.apache.lucene.search.*;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * @Author: miss[494977551@qq.com]
 * @Date: Create in 16:04 2020/8/24
 * @DESC:
 **/
@Service
@Log4j2
public class WarnServiceImpl implements IWarnService {
    @Autowired
    private INewestDataDao iNewestDataDao;
    @Autowired
    private IWarnRecordDao iWarnRecordDao;
    @Autowired
    private IExcelFieldDao iExcelFieldDao;
    public static final int excelMaxNum = 50000;
    @Autowired
    private IRepService iRepService;

    // 这里处理数据
    @Override
    public RespWrapper<LuceneReturnEntity> findReportDataList(AlwayQueryHelp help) {
        return findLuceneReturnEntity(help);
    }
    private RespWrapper<LuceneReturnEntity> findLuceneReturnEntity(AlwayQueryHelp help){
        Long endTime = help.getEndTime();
        Long startTime = help.getStartTime();
        if(startTime == null || endTime == null)
            return RespWrapper.makeFailResp("时间不能为空");
        endTime += DateTimeUtils.ONE_DAY;
        String terminalNo = help.getTerminalNo();
        ProtocolSeriveImpl.TerminalToNode node = new ProtocolSeriveImpl.TerminalToNode(terminalNo,help.getPlugNo(),help.getIsMain());
        String nodeNo = node.getNodeNo();
        int plugNo = node.getPlugNo();
        Query query = LuceneQueryUtils.equalsField("terminalNo", terminalNo);
        Sort sort = LuceneQueryUtils.createSort("createTime", SortField.Type.LONG, false);
        BooleanQuery.Builder builder = new BooleanQuery.Builder();
        builder.add(query, BooleanClause.Occur.MUST);
        if(!help.getIsMain()){
            builder.add(LuceneQueryUtils.equalsField("nodeNo", nodeNo), BooleanClause.Occur.MUST);
            builder.add(LuceneQueryUtils.equalsField("plugNo", plugNo+""), BooleanClause.Occur.MUST);
        }else {
            builder.add(LuceneQueryUtils.equalsField("nodeNo", terminalNo), BooleanClause.Occur.MUST);
        }
        // 下面这里需要查询时间
        builder.add(LuceneQueryUtils.timeRange("createTime", startTime, endTime),BooleanClause.Occur.MUST);
        BooleanQuery build = builder.build();
        // log.error("查询实时数据sql:{}",build.toString());
        // 查询5万条数据
        LuceneReturnEntity luceneReturnEntity = LuceneQueryUtils.fadQueryLucene(LuceneUtils.LuceneType.REP_DATA, 1, excelMaxNum, build, sort);
        return RespWrapper.makeSuccessResp(luceneReturnEntity);
    }


    @Override
    public RespWrapper<PageQuery<Map<String, Object>>> findNewestDataList(NewestQueryHelp help, UserCoreCache userCache) {
        return doFindNewestDataList(help,userCache,false);
    }
    private RespWrapper<PageQuery<Map<String, Object>>> doFindNewestDataList(NewestQueryHelp help, UserCoreCache userCache,boolean isExcel){
        Map<String,Object> condition = new HashMap<>(8);
        addAuth(condition,userCache);
        condition.put("isMain",help.getIsMain());
        if(help.getMerchantId() != null)
            condition.put("merchantId",help.getMerchantId());
        if(help.getSiteId() != null)
            condition.put("siteId",help.getSiteId());
        if(StringUtil.isNotEmpty(help.getTerminalNo()))
            condition.put("terminalNo",help.getTerminalNo());
        if(StringUtil.isNotEmpty(help.getSerialNo()))
            condition.put("serialNo",help.getSerialNo());
        if(help.getTypeId() != null)
            condition.put("typeId",help.getTypeId());
        PageQuery<Map<String,Object>> pageQuery = new PageQuery<>();
        pageQuery.setParas(condition);
        if(isExcel){
            pageQuery.setPageNumber(1);
            pageQuery.setPageSize(excelMaxNum);
        }else {
            pageQuery.setPageNumber(help.getPageNum());
            pageQuery.setPageSize(help.getPageSize());
        }
        pageQuery.setOrderBy(" id desc");
        pageQuery = iNewestDataDao.findNewestDataList(pageQuery);
        return RespWrapper.makeSuccessResp(pageQuery);
    }

    @Override
    public RespWrapper<PageQuery<Map<String, Object>>> findWarnPageList(WarnQueryHelp help, UserCoreCache userCache) {
        return findWarnToDB(help,userCache,false);
    }
    private RespWrapper<PageQuery<Map<String, Object>>> findWarnToDB(WarnQueryHelp help, UserCoreCache userCache,boolean isExcel){
        Map<String,Object> condition = new HashMap<>(8);
        addAuth(condition,userCache);
        if(help.getMerchantId() != null)
            condition.put("merchantId",help.getMerchantId());
        if(help.getSiteId() != null)
            condition.put("siteId",help.getSiteId());
        if(StringUtil.isNotEmpty(help.getTerminalNo()))
            condition.put("terminalNo",help.getTerminalNo());
        if(help.getTypeId() != null)
            condition.put("typeId",help.getTypeId());
        if(help.getWarnType() != null)
            condition.put("warnType",help.getWarnType());
        if(help.getStartTime() != null && help.getEndTime() != null){
            condition.put("startTime",help.getStartTime());
            condition.put("endTime",help.getEndTime()+DateTimeUtils.ONE_DAY);
        }else if(isExcel){
            return RespWrapper.makeFailResp("导出时时间不能为空~");
        }
        if(help.getStatus() != null)
            condition.put("status",help.getStatus());
        if(help.getWarnStatus() != null)
            condition.put("warnStatus",help.getWarnStatus());
        if(help.getPreWarn() != null){
            if(help.getPreWarn())condition.put("preWarn",1);
            else condition.put("preWarn",0);
        }
        PageQuery<Map<String,Object>> pageQuery = new PageQuery<>();
        pageQuery.setParas(condition);
        if(isExcel){
            pageQuery.setPageNumber(1);
            pageQuery.setPageSize(excelMaxNum);//
        }else {
            pageQuery.setPageNumber(help.getPageNum());
            pageQuery.setPageSize(help.getPageSize());
        }
        pageQuery.setOrderBy(" id desc");
        pageQuery = iWarnRecordDao.findWarnPageList(pageQuery);
        return RespWrapper.makeSuccessResp(pageQuery);
    }

    @Override
    public RespWrapper<Boolean> warnStatusHandle(UserCoreCache userCache,Integer warnId,Integer status) {
        Long userId = userCache.getUserId();
        WarnRecord single = iWarnRecordDao.single(warnId);
        if(single == null) return RespWrapper.makeFailResp("告警唯一id 错误");
        if(single.getHandleUserId() != null && single.getHandleUserId().intValue() != userId)
            return RespWrapper.makeFailResp("你没有权限标记该告警状态");
       /* if (single.getStatus() == 0) {
            single.setStatus(1);
        }else single.setStatus(0);*/
        single.setStatus(status);
        // todo 需要修改缓存状态
        iRepService.updateWarnCacheStatus(single.getSerialNo(),single.getWarnType(),status);
        iWarnRecordDao.updateTemplateById(single);
        return RespWrapper.makeSuccessResp(true);
    }
    private void addAuth(Map<String,Object> condition,UserCoreCache userCache){
        if (!ProjectConstant.ADMIN.equals(userCache.getLoginName())) {
            condition.put("siteIds",userCache.getSiteData());
        }
    }

    @Override
    public RespWrapper<Collection<WarnHelp>> findSiteWarnRanking(UserCoreCache userCache) {
        Map<String,Object> condition = new HashMap<>(4);
        addAuth(condition,userCache);
        List<WarnHelp> list = iWarnRecordDao.findSiteWarnRanking(condition);
        // 下面处理列表
        Map<Integer,WarnHelp> map = new HashMap<>();
        list.forEach(s->{
            Integer siteId = s.getSiteId();
            if (map.containsKey(siteId)) {
                WarnHelp warnHelp = map.get(siteId);
                if(s.getWarnStatus() == null || s.getWarnStatus() == 0){
                    warnHelp.doAddWarn(true);
                }else warnHelp.doAddWarn(false);
            }else {
                if(s.getWarnStatus() == null || s.getWarnStatus() == 0){
                    s.doAddWarn(true);
                }else s.doAddWarn(false);
                map.put(siteId,s);
            }
        });
        Collection<WarnHelp> values = map.values();
        return RespWrapper.makeSuccessResp(values);
    }
    // 获取过去12个月的告警
    @Override
    public RespWrapper<Map<String, Object>> findMonthWarnRanking(UserCoreCache userCache) {
        Map<String,Object> condition = new HashMap<>(4);
        addAuth(condition,userCache);
        Calendar cal = Calendar.getInstance();
        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH )+1;
        List<String> keys = new ArrayList<>();
        List<Integer> pre = new ArrayList<>();
        List<Integer> warn = new ArrayList<>();
        try {
            for(int i = month; i > month-12;i--){
                //如果月份变成了负数，年份会自动减1，月份就会变成正数
                cal.set(Calendar.MONTH, i-1);
                condition.put("startTime", TimeUtil.dateMonthToStamp(year,i));
                condition.put("endTime", TimeUtil.dateMonthLastDayToStamp(year,i));
                keys.add((cal.get(Calendar.MONTH ) +1)+"月");
                List<WarnMonthHelp> lt = iWarnRecordDao.findWarnMonthTotal(condition);
                int size = lt.size();
                if(size > 0){
                    if(size == 1){
                        WarnMonthHelp s = lt.get(0);
                        if (s.getPreWarn() == null || s.getPreWarn() == 1) {
                            pre.add(s.getCount());
                            warn.add(0);
                        }else {
                            warn.add(s.getCount());
                            pre.add(0);
                        }
                    }else {
                        lt.forEach(s->{
                            if (s.getPreWarn() == null || s.getPreWarn() == 1) {
                                pre.add(s.getCount());
                            }else if(s.getPreWarn() == 0) warn.add(s.getCount());
                        });
                    }
                }else {
                    pre.add(0);
                    warn.add(0);
                }
            }
        }catch (Exception e){
            log.error("查询12个告警错误");
            e.printStackTrace();
        }
        Map<String,Object> res = new HashMap<>();
        res.put("keys",keys);
        res.put("preWarn",pre);
        res.put("warn",warn);
        return RespWrapper.makeSuccessResp(res);
    }

    @Override
    public RespWrapper<Map<String, Object>> findWarnHomeTotal(UserCoreCache userCache) {
        Map<String,Object> condition = new HashMap<>(16);
        addAuth(condition,userCache);
        long startTime = System.currentTimeMillis();//开始时间
        long startTimeOfDay = DateTimeUtils.getStartTimeOfDay(new Date());//今天开始时间
        long _startTime = startTimeOfDay - DateTimeUtils.ONE_DAY;//前天的数据
        condition.put("startTime",_startTime);
        condition.put("endTime",startTime);
        // log.error("查询头条件：{}",condition);
        List<WarnHomeHelp> list = iWarnRecordDao.findWarnHomeTotal(condition);
        int preDay = 0;
        int warnDay = 0;
        int preTotal = 0;
        int warnTotal = 0;
        int todayWarnNum = 0;
        int yesWarnNum = 0;
        for(WarnHomeHelp wh:list){
            boolean toDay = wh.getCreateTime() > startTimeOfDay;//如果创建时间大于今天开始时间，则说明是今天的数据
            if(toDay){
                todayWarnNum++;
            }else yesWarnNum++;
            if (wh.getPreWarn() == 1) {//等于1时说明是预计
                // 这里说明是预警
                preTotal++;
                if(toDay) preDay++;
            }else {
                warnTotal++;
                if(toDay)
                    warnDay++;
            }
        }
        condition.put("warnStatus",1);
        int handleWarn = iWarnRecordDao.findWarnTotal(condition);// 已经处理的告警
        condition.put("warnStatus",0);
        int noHanleWarn = iWarnRecordDao.findWarnTotal(condition);//没有处理的告警
        condition.clear();
        condition.put("noHandleWarn",noHanleWarn);//未解除告警
        condition.put("handleWarn",handleWarn);//已解除告警
        condition.put("preTodayWarn",preDay);//今日预警数
        condition.put("preYesterdayWarn",preTotal-preDay);//昨日预警数
        condition.put("warnTodayWarn",warnDay);//今日告警数
        condition.put("warnYesterdayWarn",warnTotal-warnDay);//昨日告警数
        condition.put("todayWarnTotal",todayWarnNum);//今天警告数
        condition.put("yesterdayWarnTotal",yesWarnNum);//昨日告警数
        return RespWrapper.makeSuccessResp(condition);
    }

    @Override
    public RespWrapper<List<SysExcelField>> findExcelField(Integer id) {
        List<SysExcelField> list = iExcelFieldDao.findExcelList(id);
        return RespWrapper.makeSuccessResp(list);
    }

    @Override
    public RespWrapper<Boolean> warnExcelHandle(WarnQueryHelp help, UserCoreCache userCache, HttpServletResponse response) {
        RespWrapper<PageQuery<Map<String, Object>>> warnToDB = findWarnToDB(help, userCache, true);
        if(!warnToDB.isSuccess())
            return RespWrapper.makeFailResp(warnToDB.getMsg());
        PageQuery<Map<String, Object>> data = warnToDB.getData();
        List<Integer> ids = help.getExcelList();
        List<String> culumnList = new ArrayList<>();
        List<String> excelList = new ArrayList<>();
        addExcelField(culumnList,excelList,ids);
        // 处理数据
        List<Map<String, Object>> list = data.getList();
        long statTime = System.currentTimeMillis();
        list.forEach(s->{
            // 处理数据
            handleWarnStatus(s,"warnStatus");
            handleHandleStatus(s,"status");
            handleTime(s,"createTime",statTime);
            handleTime(s,"updateTime",statTime);
        });
        String fileName = "告警数据导出";
        try {
            JXLUtil.execlWrite(list,excelList,culumnList,fileName,response,fileName);
        }catch (Exception e){
            log.error("导出数据错误：{}，请求参数:{}",e.getMessage(),help);
            return RespWrapper.makeFailResp("导出数据错误");
        }
        return null;
    }

    @Override
    public RespWrapper<Boolean> newestDataExcel(NewestQueryHelp help, UserCoreCache userCache, HttpServletResponse response) {
        RespWrapper<PageQuery<Map<String, Object>>> db = doFindNewestDataList(help, userCache, true);
        if(!db.isSuccess()) return RespWrapper.makeFailResp(db.getMsg());
        List<Integer> ids = help.getExcelList();
        List<String> culumnList = new ArrayList<>();
        List<String> excelList = new ArrayList<>();
        addExcelField(culumnList,excelList,ids);
        // 处理数据
        List<Map<String, Object>> list = db.getData().getList();
        Boolean isMain = help.getIsMain();//
        doConvertData(list,isMain);
        String fileName = "实时数据";
        try {
            JXLUtil.execlWrite(list,excelList,culumnList,fileName,response,fileName);
        }catch (Exception e){
            log.error("导出实时数据错误：{}，请求参数:{}",e.getMessage(),help);
            return RespWrapper.makeFailResp("导出实时数据错误");
        }
        return null;
    }
    private void doConvertData(List<Map<String, Object>> list,Boolean isMain){
        long statTime = System.currentTimeMillis();
        list.forEach(s->{
            if(isMain){
                // 主机温度
                handleUnitConvert(s,"mainT",1,"℃");
            }else {
                // 从的数据多很多
                handlePlugState(s,"plugState");
                // 1.0/10+"V";
                handleUnitConvert(s,"subV",10,"V");
                handleUnitConvert(s,"subI",1,"mA");
                handleUnitConvert(s,"subLeakI",10,"mA");
                handleUnitConvert(s,"subT",1,"℃");
                handleUnitConvert(s,"subP",1000,"kW");
                handleUnitConvert(s,"subE",100,"kWH");
            }
            handleTime(s,"createTime",statTime);
        });
    }
    //实时数据导出
    @Override
    public RespWrapper<Boolean> excelReportDataList(AlwayQueryHelp help, HttpServletResponse response) {
        RespWrapper<LuceneReturnEntity> lr = findLuceneReturnEntity(help);
        if(!lr.isSuccess())return RespWrapper.makeFailResp(lr.getMsg());
        List<Map<String, Object>> list = lr.getData().getList();
        // 处理数据中
        List<String> excelList = new ArrayList<>();
        List<String> columnList = new ArrayList<>();
        Boolean isMain = help.getIsMain();//是否是主板
        // todo 这里需要处理一些数据
        doHandleExcelField(columnList,excelList,isMain);
        doConvertData(list,isMain);
        String prefix = handlePrefix(help.getTerminalNo(), help.getIsMain(), help.getPlugNo());
        String fileName = prefix+DateTimeUtils.formatDate(new Date(help.getStartTime()),DateTimeUtils.FORMAT_YMD)+"-"+DateTimeUtils.formatDate(new Date(help.getEndTime()),DateTimeUtils.FORMAT_YMD);
        try {
            JXLUtil.execlWrite(list,excelList,columnList,fileName,response,fileName);
        }catch (Exception e){
            log.error("导出实时数据错误：{}，请求参数:{}",e.getMessage(),help);
            return RespWrapper.makeFailResp("导出实时数据错误");
        }

        return null;
    }

    @Override
    public RespWrapper findWarnInfo(Integer id) {
        // WarnRecord single = iWarnRecordDao.single(id);
        Map<String,Object> map = iWarnRecordDao.findWarnRecordInfo(id);
        return RespWrapper.makeSuccessResp(map);
    }

    private String handlePrefix(String terminalNo, Boolean isMain,int plugNo) {
        if(isMain) return "主板("+terminalNo+")";
        // 这里需要处理下数据
        boolean isOne = true;
        if(plugNo > 8){
            plugNo -= 8;
            isOne = false;
        }
        String substring = terminalNo.substring(0, 10);
        if(isOne) substring+="81";
        else substring+="82";
        return "第"+plugNo+"回路("+substring+")";
    }


    private void doHandleExcelField(List<String> columnList, List<String> excelList, Boolean isMain) {
        columnList.add(JXLUtil.SERIAL_NO);
        excelList.add("序号");
        if(isMain){
            // 主板
            columnList.add("mainT");
            excelList.add("温度");
        }else {
            // 温度，电压，电流
            columnList.add("subT");
            excelList.add("温度");
            columnList.add("subV");
            excelList.add("电压");
            columnList.add("subI");
            excelList.add("电流");
            columnList.add("subP");
            excelList.add("功率");
            columnList.add("subE");
            excelList.add("电量");
            columnList.add("subLeakI");
            excelList.add("漏电流");
            columnList.add("plugState");
            excelList.add("继电器状态");
        }
        columnList.add("showTime");
        excelList.add("上报时间");

    }


    private void handleUnitConvert(Map<String,Object> s,String key,int offset,String unit){
        Object o = s.get(key);
        if(o != null){
            Integer integer = Integer.valueOf(o.toString());
            if(integer != null)
                s.put(key,integer*1.0/offset+unit);
        }
    }

    private void handlePlugState(Map<String, Object> s, String key) {
        Object o = s.get(key);
        if(o != null){
            Integer integer = Integer.valueOf(o.toString());
            if(integer == 0){
                s.put(key,"继电器断开");
            }else if(integer == 1){
                s.put(key,"继电器导通");
            }
        }
    }


    private void handleTime(Map<String, Object> s, String createTime,long startTime) {
        Object o = s.get(createTime);
        if (o != null) {
            startTime = Long.valueOf(o.toString());
            s.put(createTime,DateTimeUtils.formatDate(new Date(startTime),DateTimeUtils.FORMAT));
        }
    }

    private void handleHandleStatus(Map<String,Object> map,String status){
        Object o = map.get(status);
        if(o != null){
            Integer integer = Integer.valueOf(o.toString());
            if(integer == 0){
                map.put(status,"未处理");
                return;
            }else if(integer == 1){
                map.put(status,"处理中");
                return;
            }
        }
        map.put(status,"已处理");
    }
    private void handleWarnStatus(Map<String,Object> map,String warnStatus){
        Object o = map.get(warnStatus);
        if(o != null){
            Integer integer = Integer.valueOf(o.toString());
            if(integer == 0){
                map.put(warnStatus,"警告中");
                return;
            }
        }
        map.put(warnStatus,"已结束");
    }
    public void addExcelField(List<String> columnList,List<String> excelList,List<Integer> ids){
        columnList.add(JXLUtil.SERIAL_NO);
        excelList.add("序号");
        List<SysExcelField> list = iExcelFieldDao.findExcelListByIds(ids);
        list.forEach(s->{
            columnList.add(s.getColumnName());
            excelList.add(s.getExcelName());
        });
    }




    public static List<Integer> listConvert(Set<Long> set){
        List<Integer> res = new ArrayList<>(set.size());
        for(Long it:set){
            if(it != -100)
                res.add(it.intValue());
        }
        return res;
    }
    public static void doAddMerchantData(Set<Long> set,Map<String,Object> condition){
        List<Integer> list = listConvert(set);
        if(!list.isEmpty()) condition.put("merchantIds",list);
    }

}
